#include "GAchievementManager.h"

#include "GStaticData.h"
#include <hpllib/system/HPLFile.h>
#include <hpllib/general/HPLString.h>
#include <hpllib/system/HPLLogger.h>

using namespace std;
using namespace GEnum;

static const char* const ACHIEVEMENTS_FILE_PATH = "data/achievements.dat";
static const char* const COMMAND_DESCRIPTION = "Description";
static const char* const COMMAND_MAX_NUM = "MaxNum";
static const char* const COMMAND_IS_RESET_ON_START_STAGE = "Reset";
static const char* const COMMAND_COMMENT = "Comment";
static const char* const COMMAND_LABEL = "Label";
static const char* const SEPARATOR = "=";

namespace AchievementItem {
	enum {
		DESCRIPTION,
		MAX_NUM,
		IS_RESET_ON_START_STAGE,
		LABEL,
		COMMENT,
		MAX,
	};
}
GAchievementDataSet::GAchievementDataSet() {
}
void GAchievementDataSet::load() {
	// load achievements
	vector<string> vecLines = HPLFile::loadStringLinesFromFile( ACHIEVEMENT_FILE_PATH );

	GAchievementData data;
	int READY_NUM = 0;
	// K{ȗvf
	READY_NUM = 1 << AchievementItem::DESCRIPTION | 1 << AchievementItem::LABEL;
	int nCheckSum = 0;
	int nMax = static_cast<int>( vecLines.size() );
	for( int i = 0; i < nMax; i ++ ) {
		
		if( HPLFile::isCommentLine( & vecLines[i] ) ) {
			continue;
		}

		string str = vecLines[i];
		HPLLogger::HPL_LOGGER.write( "%d:%s", i, str.c_str() );
		
		if( vecLines[i].at(0) == '[' ) {

			if( (nCheckSum & READY_NUM ) == READY_NUM ) {
				// Kvȗvf낦

				// o^ǉ
				this->m_vecAchievementDatas.push_back( data );

				// clear
				data.clear();
				nCheckSum = 0;
			}
		} else {
			vector<string> items = HPLString::split( str, SEPARATOR );

			if( items[0].compare( COMMAND_DESCRIPTION ) == 0 ){

				data.m_vecDescriotions.push_back( items[1] );
				nCheckSum |= 1 << AchievementItem::DESCRIPTION;
			} else if( items[0].compare( COMMAND_MAX_NUM ) == 0 ) {

				data.m_nMaxNum = atoi( items[1].c_str() );
				nCheckSum |= 1 << AchievementItem::MAX_NUM;

			} else if( 0 == items[0].compare( COMMAND_IS_RESET_ON_START_STAGE ) ) {

				data.m_bIsResetOnStartStage = atoi( items[1].c_str() ) == 0 ? false : true;
				nCheckSum |= 1 << AchievementItem::IS_RESET_ON_START_STAGE;

			} else if( items[0].compare( COMMAND_LABEL ) == 0 ) {
				data.m_strLabel = items[1];
				nCheckSum |= 1 << AchievementItem::LABEL;

			} else if( items[0].compare( COMMAND_COMMENT ) == 0 ) {
				data.m_nCommentScenarioItemID = atoi( items[1].c_str());
				nCheckSum |= 1 << AchievementItem::COMMENT;

			} else {
				HPLError::assertIt( false,
					"Unexpected command[%s],line:%d,[%s]", items[0].c_str(), i, vecLines[i].c_str());

			}
		} 
	}
	if( (nCheckSum & READY_NUM ) == READY_NUM ) {
		// Kvȗvf낦

		// o^ǉ
		this->m_vecAchievementDatas.push_back( data );
	}
}

GAchievementManager::GAchievementManager() : HPLObjectManager(){

	this->m_AchievementDataSet.load();
}


void GAchievementManager::process(bool bIsMove, bool bIsAI ) {
	for( list<GAchievement*>::iterator it = this->getList()->begin();
		it != this->getList()->end();
		)
	{
		GAchievement* lpAchievement = *it;

		// ʊÔ̂삷B
		
		
		if( bIsMove ) {
			this->algorithm( lpAchievement );
		}

		if( lpAchievement->isAlive() ) {
			lpAchievement->draw();
			it ++;
		} else {
			delete *it;
			it = this->getList()->erase( it );
		}
	}
	if( this->getList()->size() == 0 ) {
		// ςݏグZbg
		this->m_nStackNumber = 0;
	}
}

void GAchievementManager::algorithm( GAchievement* lpAchievement ) {
	int st = lpAchievement->getState()->getState();
	switch( st ) {
	case StateType::STAND:
		{
			// ㏸
			HPLVector vVelocityAngle(0,-ACHIEVEMENT_FLOW_UP_SPEED );

			if( lpAchievement->getCenter().getY() <= lpAchievement->getGoalCenterY() ) {
				// Goalʒuz

				// Goalʒuɂ낦
				HPLVector vCenter = lpAchievement->getCenter();
				vCenter.setY(lpAchievement->getGoalCenterY() );
				lpAchievement->setCenter(vCenter);

				// Ƃ߂
				vVelocityAngle.set(0,0);

				// WALKɂ
				lpAchievement->changeState( StateType::WALK );

				lpAchievement->m_StayCounter.reset();
			}
			lpAchievement->setVelocityAngle(vVelocityAngle);
		}		

		break;
	case StateType::WALK:

		lpAchievement->setVelocityAngle(0,0);

		if( lpAchievement->m_StayCounter.step().isFinishAnimation ) {
			// DAMAEGɂ
			lpAchievement->changeState( StateType::DAMAGE );
		}
		break;
	case StateType::DAMAGE:
		{
			lpAchievement->setVelocityAngle(0,ACHIEVEMENT_FLOW_DOWN_SPEED);
			int nBeyondCenterY = (int)(
				getStaticData()->m_Setting.windowHeight + 
				getStaticData()->getSequence( SpriteType::COMMON, Sequence::Common::ACHIEVEMENT_FRAME )->getImageInformation(0)->srcRect.getHeight()  / 2+ 10 );
			if( lpAchievement->getCenter().getY() > nBeyondCenterY ) {
				// 
				lpAchievement->die( NULL );
			}
		}
		break;
	default:
		break;
	}
	HPLVector vCenter = lpAchievement->getCenter();
	vCenter.add( lpAchievement->getVelocityAngle() );
	lpAchievement->setCenter( vCenter );
}

GAchievement* GAchievementManager::create( int nType ) {
	GAchievement* lpAchievement = new GAchievement( nType );
	
	lpAchievement->setGoalCenterY( this->m_nStackNumber );


	// ʒu
	// window height + frame height / 2
	lpAchievement->setCenter(
		static_cast<float>( ACHIEVEMENT_X ) , 
		getStaticData()->m_Setting.windowHeight +
		getStaticData()->getSequence(
		SpriteType::COMMON, Sequence::Common::ACHIEVEMENT_FRAME )->getImageInformation(0)->srcRect.getHeight() / 2 );
	
	this->getList()->push_back( lpAchievement );
	//ς
	this->m_nStackNumber ++;
	return lpAchievement;
}

