#include "GEnemyManager.h"

#include "GStaticData.h"
#include <hpllib/math/HPLMath.h>
#include <hpllib/action/HPLActionMoveTool.h>
#include <hpllib/system/HPLFile.h>
#include <hpllib/general/HPLString.h>

using namespace std;
using namespace GEnum;

static const float AIKO_CHASE_ACCEL = 0.2f;
static const float AIKO_CHASE_MAX_SPEED = 4;

void GEnemyManager::doAikoAlgorithm( GEnemy* lpEnemy ) {
	int t = lpEnemy->getType();
	
	HPLShootingObject* lpTarget = lpEnemy->getTarget();
	HPLVector vTargetCenter = lpTarget->getCenter();
	
	HPLVector vSub(
		vTargetCenter.getX() - lpEnemy->getCenter().getX(),
		vTargetCenter.getY() - lpEnemy->getCenter().getY() );


	float fLevel = getStaticData()->m_Game.getLevel( LEVEL_EPSILON );

	float fLength = vSub.getLength();

	int st = lpEnemy->getState()->getState();
	float RESIST = getStaticData()->m_Setting.resist;
	float GRAVITY = getStaticData()->m_Setting.gravity;

	// chasing
	HPLVector vGoal;
	bool bIsChaser = false;
	float fAccel = AIKO_CHASE_ACCEL;//0.2f;
	float fMaxSpeed = AIKO_CHASE_MAX_SPEED;//4;

	GEnemyAIResult result;

	// Aj[V
	HPLSequence* lpSeq = lpEnemy->getSequence();
	int nNext = -1;
	HPLAttackInformation* lpInfo = lpEnemy->m_lpAttackInformation;

	if( lpInfo->m_WaitForStartCounter.step().isFinishAnimation ) {
		// ҂

		if( lpInfo->m_WaitForNextCounter.step().isFinishAnimation )
		{
			// U̍Ԃ̑ҋ@

			HPLAttackItem* lpItem = lpEnemy->m_lpAttackInformation->getAttackItem( lpEnemy->getAttackType());

			bool bIsShot = false;
			// ԕω
			switch( lpEnemy->getAttackType() ) {
			case AikoAttackType::ICE_SPIKE:		//! ACXXpCNiƂXjFSʃVbg
				switch( st ) {
				case StateType::ATTACK:	//! グ
					bIsShot= true;
					break;
				default:
					lpEnemy->changeState( StateType::ATTACK );
				}
				break;
			case AikoAttackType::ICICLE:
				switch( st ) {
				case StateType::ATTACK:	// 
					bIsShot= true;
					break;
				default:
					lpEnemy->changeState( StateType::ATTACK );
				}
				break;
			case AikoAttackType::TARGET_FROST:
				switch( st ) {
				case StateType::ATTACK_2:	// Ў
					bIsShot= true;
					break;
				default:
					lpEnemy->changeState( StateType::ATTACK );
				}
				break;
			case AikoAttackType::CALL_FROST:
				switch( st ) {
				case StateType::ATTACK_2:
					bIsShot= true;
					break;
				default:
					lpEnemy->changeState( StateType::ATTACK );
				}
				break;
			case AikoAttackType::ICE_SPIKE_TARGET:
				switch( st ) {
				case StateType::ATTACK_2:
					bIsShot= true;
					break;
				default:
					lpEnemy->changeState( StateType::ATTACK_2 );
				}
				break;
			case AikoAttackType::ICE_SOWRD:
				switch( st ) {
				case StateType::ATTACK_2:
					bIsShot= true;
					break;
				default:
					lpEnemy->changeState( StateType::ATTACK_2 );
				}
				break;
			case AikoAttackType::BLIZZARD:
				switch(st ) {
				case StateType::ATTACK:
					bIsShot = true;
					break;
				default:
					lpEnemy->changeState( StateType::ATTACK_2 );
				}
			}
			if( bIsShot ) {
				int nNextAttackType = -1;
				bool bIsFinishOneSet = false;	//! 1ZbgIH
				if( lpItem->m_ShotNumCounter.isAlive() ) {	// 1Zbg{

					if( lpItem->m_IntervalCounter.step().isFinishAnimation ) {	// finish interval

						// shot!
						getStaticData()->m_ShotFactory.shot( lpEnemy,
							lpEnemy->getAttackType(), 0, lpItem->m_ShotNumCounter.getStepCounter());

						lpItem->m_IntervalCounter.reset();
						if( lpItem->m_ShotNumCounter.step().isFinishAnimation ) {
							bIsFinishOneSet = true;
						}
					}
				} else {	// 1ZbgI
					bIsFinishOneSet = true;
				}
				if( bIsFinishOneSet ) {	// after end of one set
					if( lpItem->m_SetIntervalCounter.step().isFinishAnimation ) {	// finish interval between sets
						lpItem->m_IntervalCounter.reset();
						lpItem->m_SetIntervalCounter.reset();
						lpItem->m_ShotNumCounter.reset();
						if( lpItem->m_ShotNumCounter.step().isFinishAnimation ) {	// finish all sets
							lpItem->m_ShotNumCounter.reset();

							// next attacks
							// shot!
							switch( lpEnemy->getAttackType() ) {
							case AikoAttackType::ICE_SPIKE:
								nNextAttackType = AikoAttackType::ICICLE;
								break;
							case AikoAttackType::ICICLE:
								nNextAttackType = AikoAttackType::TARGET_FROST;
								break;
							case AikoAttackType::TARGET_FROST:
								nNextAttackType = AikoAttackType::CALL_FROST;
								break;
							case AikoAttackType::CALL_FROST:
								nNextAttackType = AikoAttackType::ICE_SOWRD;
								break;
							case AikoAttackType::ICE_SOWRD:
								nNextAttackType = AikoAttackType::ICE_SPIKE_TARGET;
								break;
							case AikoAttackType::ICE_SPIKE_TARGET:
								nNextAttackType = AikoAttackType::BLIZZARD;
								break;
							case AikoAttackType::BLIZZARD:
								nNextAttackType = AikoAttackType::ICE_SPIKE;
							}
						}
					}
				}	// is finish one set
				if( nNextAttackType >= 0 ) {
					lpItem->reset();

					HPLAttackInformation* lpInfo = lpEnemy->m_lpAttackInformation;
					// new attack item
					HPLAttackItem* lpItem = lpInfo->getAttackItem( nNextAttackType );
					if( lpEnemy->getAttackType() == AikoAttackType::TARGET_FROST ) {
						
					} else {
						lpInfo->m_WaitForNextCounter.reset();
						
					}
					//lpItem->reset();
					for( int i = 0; i < lpInfo->getAttackItemNum(); i ++ ) {
						lpInfo->getAttackItem(i)->reset();
					}

				}
			}	// is shot

		} else {	// waiting for next
			lpEnemy->changeState( StateType::FLYDOWN );
		}
	} else {	// waiting for start
	} 
}

