#include "GEnemyManager.h"

#include "GStaticData.h"
#include <hpllib/math/HPLMath.h>
#include <hpllib/action/HPLActionMoveTool.h>
#include <hpllib/action/HPLShootingMove.h>
#include <hpllib/system/HPLFile.h>
#include <hpllib/general/HPLString.h>
#include <hpllib/system/HPLLogger.h>

using namespace std;
const int GEnum::LAVA_BALL_SPEED_Y = 10;
using namespace GEnum;


void GEnemyManager::algorithm(GEnemy* lpEnemy) {
	// type
	int t = lpEnemy->getType();
	
	// target
	HPLShootingObject* lpTarget = lpEnemy->getTarget();
	// target-center
	HPLVector vTargetCenter = lpTarget->getCenter();
	
	// from enemy to target(player)
	HPLVector vSub(
		vTargetCenter.getX() - lpEnemy->getCenter().getX(),
		vTargetCenter.getY() - lpEnemy->getCenter().getY() );

	// aim direction
	HPLVector vAim(
		HPLMath::getSgn<float>( vSub.getX() ),
		HPLMath::getSgn<float>( vSub.getY() ) );
	vAim.normalize();

	// hit rect
	HPLRect rHitRect = lpEnemy->getHitRect();

	// speed rate
	float fSpeedRate = 1;	// TODO

	const float ENEMY_RECOGNIZE_LENGTH_THRESHOLD = 300.f;

	float fLevel = getStaticData()->m_Game.getLevel( LEVEL_EPSILON );

	float fLength = vSub.getLength();

	int st = lpEnemy->getState()->getState();

	// ^C
	int nTileType = TileType::NOSPECIAL;

	// ꕔ̓G̓^CԂCɂ
	switch( lpEnemy->getType() ) {
	case EnemyType::SHARK:
	case EnemyType::UNI:
		{
			HPLVector vCen = lpEnemy->getCenter();
			int nHeadTileID = 0;
			int nFootTileID = 0;
			getStaticData()->m_ObjectManager.getTileType(
				vCen.getX(), vCen.getY(), nTileType, nHeadTileID, nFootTileID );
			switch( nTileType ) {
			case TileType::ON_THE_WATER:	// 
			case TileType::SWIM:			// 
				lpEnemy->setObjectAttribute( ObjectAttribute::GRAVITY, false );
				break;
			default:
				lpEnemy->setObjectAttribute( ObjectAttribute::GRAVITY, true );
				// d͂󂯂
				break;
			}
		}
			
		break;
	}



	// WvXe[gĂ邩
	bool bHaveJumpState =
		t == EnemyType::TANYA ||
		t == EnemyType::TANYA_YOUJO ||
		t == EnemyType::TOZICO ||
		t == EnemyType::ELLIN ||
		t == EnemyType::SLIME_QUEEN;

	// CÂۂɓ肵Ȃǂ
	bool bIsNoWalkWhenRecognized = 
		t == EnemyType::SLIME_QUEEN ||
		t == EnemyType::ARYONA;

	// GtFNg̕`ONȂ甭
	getStaticData()->m_ObjectManager.createFireEffect( lpEnemy);
	
	if( bHaveJumpState ) {	// WvԗL
		
		if( lpEnemy->getState()->isFlyingState() ) {	// flying state

			if( lpEnemy->getTrigger()->getTrigger( TriggerType::ON_THE_GROUND ) ) {	// n
				// default next state->STAND
				int nNextState = StateType::STAND;
				if( bIsNoWalkWhenRecognized == false &&		// can walk when recognized
					lpEnemy->isRecognized() )				// recognized
				{
					nNextState = StateType::WALK;
				}

				// change next state
				lpEnemy->changeState( nNextState );

				switch( lpEnemy->getType() ) {
				case EnemyType::SLIME_QUEEN:	// queen
					{
						// n
						HPLRect rHitArea = lpEnemy->getHitArea();

						for( int i = 0; i < HPLMath::nextInt( SLIME_QUEEN_SMOKE_MIN, SLIME_QUEEN_SMOKE_MAX) * 3; i ++ ) {
							HPLVector vSmokeCenter(
								rHitArea.getX() + //rHitArea.getWidth() / 2 + 
								//lpEnemy->getFaceDirection() * SLIME_QUEEN_SMOKE_DELTA_X + 
								//HPLMath::nextFloat( - SLIME_QUEEN_SMOKE_ERR, SLIME_QUEEN_SMOKE_ERR ) ,
								HPLMath::nextFloat( 0, rHitArea.getWidth() ),
								rHitArea.getBottom() + HPLMath::nextFloat( -SLIME_QUEEN_SMOKE_ERR_Y,SLIME_QUEEN_SMOKE_ERR_Y) );
							GEffect* lpEffect = getStaticData()->createEffect( Sequence::Effect::EXPLOSION_SMOKE ,
								vSmokeCenter, vSmokeCenter, -1, -1 );
							lpEffect->setVelocityAngle( 0, - SMOKE_SPEED_Y );
						}
					}
					break;
				}
			}
		} else if( st == StateType::STAND || st == StateType::WALK ) {	// stand or walk
			if( false == lpEnemy->getTrigger()->getTrigger( TriggerType::ON_THE_GROUND ) ) {	// flying
				int nState = StateType::FLYUP;

				if( lpEnemy->getVelocityAngle().getY() >= getStaticData()->m_Setting.flydownSpeedThreshold ) {
					nState = StateType::FLYDOWN;
				}
				lpEnemy->changeState( nState );
			}
		}
	}

	if( st == StateType::DAMAGE ) {	// "damage" state
		if( lpEnemy->getTrigger()->getTrigger( TriggerType::ON_THE_GROUND ) ) {	// land on the ground

			// n

			// TODOȓG͓|ꂽԂɂȂiтA^[jj
			lpEnemy->changeState(StateType::STAND );
		}

	}
	bool bIsMoveDirection = false;			// isւ̂܂ܐi
	// ȉbMoveDirection == false̎
	bool bIsTurnEdge = false;	// RňԂHiftHgFԂȂi
	bool bIsTurnWall = true;	// ǂňԂHiftHgFԂ

	bool bIsMoveToPlayerGround = false;		// vC[̕ɋ߂Â()
	bool bIsFaceToPlayerAlways = false;		// ɃvC[̕
	bool bIsFaceByVelocity = false;			// ړʂɕ߂Bxʈȉ̏ꍇςȂ

	bool bIsJumpOverCollision = false;		// QWvŏz

	bool bIsChaser = false;					//! 󒆂𕂗VAS[|WVɈړ
	HPLVector vGoalPosition = lpTarget->getCenter();
	float fAccel = 0;
	float fMaxSpeed = 0;

	bool bIsAmpEnable = lpEnemy->m_lpAmplitude != NULL && st == StateType::STAND;				// ҋ@̂݁B̑̏ԁiUj㉺ꍇ͊eŐݒ肷
	const float DEFAULT_AMPLITUDE = 100;
	float fAmplitude = DEFAULT_AMPLITUDE;

	float RESIST = getStaticData()->m_Setting.resist;
	float AIR_RESIST = this->getEnemyAirResist(1);
	float GRAVITY = getStaticData()->m_Setting.gravity;

	// Aj[V
	HPLSequence* lpSeq = lpEnemy->getSequence();
	bool bIsNoMoving = false;	//! ړȂBiOɂړ͑j
	bool bIsStop = false;	//! ~sBvel.x=0
	
	bool bIsDieAfterFinishAnimation = false;		//! Aj[VIŎS

	if( lpEnemy->isRecognized() == false ) {	// not recognized
		int nRecognizeThreshold = (int)(ENEMY_RECOGNIZE_LENGTH_THRESHOLD + rHitRect.getWidth() / 2.f );

		const int BIG_RECOG_THREHOLD = 128;
		if( rHitRect.getWidth() >= BIG_RECOG_THREHOLD ){
			// ł
			nRecognizeThreshold *= 2;
		} else if( lpEnemy->getTrigger()->getTrigger( TriggerType::ON_THE_GROUND ) == false ) {
			// 󒆂ɂ
			nRecognizeThreshold *= 2;
		}
		// CÂĂȂ
		if( lpEnemy->m_bIsNoRecognize == false && fLength <=  nRecognizeThreshold) {
			
			lpEnemy->setRecognized( true );

		} else {
			switch( t ) {
			case EnemyType::NAMAKO:
			case EnemyType::SLIME:
			case EnemyType::CYCLON:
			//case EnemyType::SNAKE:
				bIsMoveDirection = true;		// is֌Ĉړ	
				bIsTurnEdge = true;			// RňԂ
				break;
			}
		}

		if( lpEnemy->isRecognized() ) {	// ̃^[ŋCÂ

			if( t == EnemyType::MUNIERU || t == EnemyType::EVI_HIME ) {
				// Ȃ
			} else {
				int nNextState = StateType::STAND;
				if( bIsNoWalkWhenRecognized == false) {
					// ɂȂ
					nNextState = StateType::WALK;
				}
				lpEnemy->changeState( nNextState ) ;
			}
		}

	} else {	// recognized

		//bool bReturnEdge = lpEnemy->m_aryEnemyAttributes[EnemyAttribute::RETURN_EDGE];	// RňԂ
		//bool bJumpNearBy = false;	// ߂ɗWv
		//bool bJumpOverBlock = false;	// rɏQ΃Wv
		//bool bNoMove = false;

		// ^[QbgɌvfł΁AR痎Ȃ͂Ȃ
		bIsTurnEdge = bIsMoveToPlayerGround && bIsFaceToPlayerAlways && lpEnemy->getEnemyAttribute(EnemyAttribute::RETURN_AT_EDGE );

		// CÂƂG̐ݒɕ키
		bIsJumpOverCollision = lpEnemy->getEnemyAttribute( EnemyAttribute::JUMP_OVER_COLLISION );


		switch( t ) {
		case EnemyType::NAMAKO:
			bIsMoveToPlayerGround = true;
			bIsFaceToPlayerAlways = true;
			fSpeedRate = 1.1f;
			break;
		case EnemyType::SLIME:
		case EnemyType::SLIME_BABY:
			bIsMoveToPlayerGround = true;
			bIsFaceToPlayerAlways = true;
			fSpeedRate = 1.0f;
			break;
		case EnemyType::BLUE_SLIME:
		case EnemyType::BLUE_SLIME_BABY:
			bIsMoveToPlayerGround = true;
			bIsFaceToPlayerAlways = true;
			fSpeedRate = 0.5f;

			if( fLength <= BLUE_SLIME_JUMP_DISTANCE_THRESHOLD) {
				// ȏ߂ÂWv
				// Wv
				this->jump( lpEnemy,
					HPLMath::getSgn( vSub.getX() ) * BLUE_SLIME_JUMP_SPEED_X,
					-BLUE_SLIME_JUMP_SPEED,
					false, false );

				// TODO READY_TO_JUMP
			}
			break;
		case EnemyType::GREEN_SLIME:
		case EnemyType::GREEN_SLIME_BABY:
			if( st == StateType::HOLD ) {
				bIsNoMoving = true;
			} else {
				bIsMoveToPlayerGround = true;
				bIsFaceToPlayerAlways = true;
				fSpeedRate = 1.2f;
			}
			break;
		case EnemyType::YELLOW_SLIME:
		case EnemyType::YELLOW_SLIME_BABY:
			//bIsMoveToPlayerGround = true;
			bIsFaceToPlayerAlways = true;
			bIsNoMoving = true;
			fSpeedRate = 0;

			if( lpEnemy->getType() == EnemyType::YELLOW_SLIME ) {
				// ԊuŔ
				switch( st ) {
				case StateType::STAND:
				case StateType::WALK:
					if( lpEnemy->getCounter(0 )->step().isFinishAnimation ) {
						// ҂
						// UɈڂ
						lpEnemy->changeState( StateType::ATTACK );
					}
					break;
				case StateType::ATTACK:		// 
					if( lpSeq->getState().isFinishAnimation ) {
						// ˏ
						// ˏԂɈڂ
						lpEnemy->changeState( StateType::ATTACK_2 );
					}
					break;
				case StateType::ATTACK_2:	//ˌ

					if( lpSeq->getState().isEndLoop ) {
						// 1V[PXɔ x 3
						getStaticData()->m_ShotFactory.shot( 
							lpEnemy, 0, WeaponDirection::FRONT );
					}
					if( lpSeq->getState().isFinishAnimation ) {
						// I

						// ʏ탂[hցB
						lpEnemy->changeState( StateType::STAND );
						// ΉȂԂɂȂ
						lpEnemy->setRecognized( false );

						lpEnemy->getCounter(0)->reset();
					}
					break;
				}
			}

			break;
		case EnemyType::EVI_HIME:
			//bIsMoveToPlayerGround = true;
			//bIsFaceToPlayerAlways = true;
			bIsNoMoving = true;
			fSpeedRate = 0.0f;
			break;
		case EnemyType::MUNIERU:
			//bIsMoveToPlayerGround = true;
			//bIsFaceToPlayerAlways = true;
			bIsNoMoving = true;
			fSpeedRate = 0.0f;
			break;
		case EnemyType::TANYA:
			//bIsMoveToPlayerGround = true;
			//bIsFaceToPlayerAlways = true;
			//fSpeedRate = lpEnemy->getVelocity();
		case EnemyType::TANYA_YOUJO:
			bIsMoveToPlayerGround = true;
			bIsFaceToPlayerAlways = true;
			fSpeedRate = lpEnemy->getVelocity();
			break;
		case EnemyType::HIDE_ENEMY_GENERATOR:
			fSpeedRate = 0;

			// Xebv
			if( lpEnemy->stepRespawnCounter() ) {
				HPLVector vCreateCenter = lpEnemy->getCenter();
				int nType = EnemyType::SLIME;
				HPLVector vCreateVelocityAngle(
					HPLMath::nextFloat( -GENERATOR_VEL_X_ERR, GENERATOR_VEL_X_ERR ),
					-HPLMath::nextFloat( GENERATOR_VEL_Y_MIN, GENERATOR_VEL_Y_MAX ) );

				int nColor = -1;
				// 
				switch( lpEnemy->getGenerateType() ) {
				case GeneratorType::TANYAS:
					nType = HPLMath::nextInt( 0, 2 ) == 0 ? EnemyType::TANYA: EnemyType::TANYA_YOUJO;
					break;
				case GeneratorType::ROCKS:
					nType = EnemyType::ROCK;
					vCreateCenter.add( HPLMath::nextFloat(-GENERATOR_X_ERR, GENERATOR_X_ERR ),
						(float)(-getStaticData()->m_Setting.windowHeight / 2) );
					break;
				case GeneratorType::YELLOW_BEES:
				case GeneratorType::RED_BEES:
				case GeneratorType::ALL_BEES:
					nColor = 0;
					switch( lpEnemy->getGenerateType() ) {
					case GeneratorType::RED_BEES:
						nColor = 1;
						break;
					case GeneratorType::ALL_BEES:
						nColor = HPLMath::nextInt(0,2);
					}
					nType = EnemyType::BEE;
					vCreateCenter.add( HPLMath::nextFloat(-GENERATOR_X_ERR, GENERATOR_X_ERR ),
						HPLMath::nextFloat(-GENERATOR_X_ERR, GENERATOR_X_ERR ) );
					
					break;
				case GeneratorType::LAVA_BALLS:
					{
						//nType = -1;

						HPLVector vCen(HPLMath::nextFloat(-GENERATOR_X_ERR, GENERATOR_X_ERR ),
						HPLMath::nextFloat(-GENERATOR_X_ERR, GENERATOR_X_ERR ));
						vCreateCenter.add( vCen );
						//vCen.add( lpEnemy->getCenter() );
						//GShot* lpShot = getStaticData()->m_ObjectManager.m_ShotManager.create( 
						//	Sequence::Shot::LAVA_BALL,
						//	vCen, lpEnemy->getID(), lpEnemy->getFaceDirection(),
						//	false, true);

						nType = EnemyType::LAVA_BALL;
						HPLVector v(0,-LAVA_BALL_SPEED_Y );
						vCreateVelocityAngle = v;
						//lpShot->setVelocityAngle(v);
					}
					break;
				case GeneratorType::LAVA_UNDER:
					{
						nType = EnemyType::LAVA_UNDER;
						vCreateCenter.add(16,16);
						vCreateVelocityAngle.set(0,0);
					}
					break;
				default:
					{
						// other = SLIMES
/*						static int GENERATE_SLIME_TYPES[] = {
							EnemyType::SLIME,
							EnemyType::BLUE_SLIME,
							EnemyType::GREEN_SLIME,
							EnemyType::YELLOW_SLIME,
							
						};*/
					}
					break;
				}
				if( nType >= 0 ) {
					GEnemy* lpNew = new GEnemy( nType );
					lpNew->setCenter( vCreateCenter );
					lpNew->setVelocityAngle( vCreateVelocityAngle );
					lpNew->m_vCenterPosition = vCreateCenter;
					lpNew->m_vAmplitudeCenter = vCreateCenter;

					if( nColor >= 0 ) {
						lpNew->setColor(nColor);
					}
					// ŏCÂ
					lpNew->setRecognized( true );
					//lpNew->setVelocityAngle
					this->m_vecAdds.push_back( lpNew );
				}
			}
			break;
		case EnemyType::MAZER:
			bIsChaser = true;
			bIsFaceToPlayerAlways = true;
			vGoalPosition.add( ( vAim.getX() > 0 ? -1 : 1 ) * MAZER_DISTANCE, 0 );
			fAccel = MAZER_ACCEL;
			fMaxSpeed = MAZER_MAX_SPEED;
			break;
		case EnemyType::GHOST:
			{
				const float GHOST_DISTANCE_X = 100;
				const float GHOST_DISTANCE_Y = 150;
				const float GHOST_ACCEL = 0.1f;
				const float GHOST_MAX_SPEED = 5.f;
				bIsChaser = true;
				bIsFaceToPlayerAlways = true;
				vGoalPosition.add( lpTarget->getFaceDirection() * -1 * GHOST_DISTANCE_X, - GHOST_DISTANCE_Y );
				fAccel = GHOST_ACCEL;
				fMaxSpeed = GHOST_MAX_SPEED;

				if( lpEnemy->getCounter(0)->step().isFinishAnimation ) {
					lpEnemy->getCounter(0)->reset();
					// create shot
					getStaticData()->m_ShotFactory.shot(
						lpEnemy, 0, 0 );
				}
			}
			break;
		case EnemyType::ARYONA:
			this->doAryonaAlgorithm( lpEnemy );
//			bIsFaceToPlayerAlways = true;
//			bIsMoveToPlayerGround = true;
//			bIsJumpOverCollision = true;
			break;
		case EnemyType::TOZICO:
			bIsFaceToPlayerAlways = true;
			// ̏ŃWv
			if( lpEnemy->getTrigger()->getTrigger( TriggerType::ON_THE_GROUND ) ) {
				HPLVector vVelocityAngle = lpEnemy->getVelocityAngle();
				vVelocityAngle.setY(-5);
				lpEnemy->setVelocityAngle( vVelocityAngle);
			}
			break;
		case EnemyType::LAMIA:
		case EnemyType::ZOMBIE:
			switch( st ) {
			case StateType::HOLD:
			case StateType::DAMAGE:
				break;
			default:
				bIsFaceToPlayerAlways = true;
				bIsMoveToPlayerGround = true;
			}
			break;
		case EnemyType::ELLIN:

			bIsFaceToPlayerAlways = true;
			bIsMoveToPlayerGround = true;
			bIsJumpOverCollision = true;
			break;
		case EnemyType::ARAKHNE:
			bIsFaceToPlayerAlways = true;
			if( fLength <= ENEMY_SHOT_DISTANCE_THRESHOLD ) {
				// ߂Â
				
				switch( st ) {
				case StateType::STAND:
				case StateType::WALK:
					// ߂܂悤Ƌ߂Â
					bIsMoveToPlayerGround = true;
					break;
				case StateType::HOLD_DOWN:
					bIsNoMoving = true;
					if( lpSeq->getState().isFinishAnimation ) {
						// holdɈڍs
						lpEnemy->changeState( StateType::HOLD );
					}
					break;
				case StateType::HOLD:
					// ߂܂Ă
					bIsNoMoving = true;
					break;
				}

			} else {
				// 
				// ˂
				if( lpEnemy->getCounter(0)->step().isFinishAnimation ) {
					getStaticData()->m_ShotFactory.shot( lpEnemy, 0, 0 );
					lpEnemy->getCounter(0)->reset();
				}
				bIsNoMoving = true;
				lpEnemy->changeState( StateType::STAND );
			}
			break;
		case EnemyType::ROPER:
		case EnemyType::IVY:
		case EnemyType::IVY_2:
			bIsNoMoving = true;
			bIsFaceToPlayerAlways = true;
			break;
		case EnemyType::SLIME_QUEEN:
			this->doSlimeQueenAlgorithm( lpEnemy );

			break;
		case EnemyType::ALRAUNE:
			this->doAlrauneAlgorithm( lpEnemy );
			break;
		case EnemyType::SUCCUBUS:
		case EnemyType::SUCCUBUS_MISAGO:
			bIsFaceToPlayerAlways = true;
			switch( st ) {
			case StateType::HOLD:
				break;
			default:
				bIsAmpEnable = false;
				bIsChaser = true;
				const float SUCCUBUS_ACCEL = 0.5f;
				const float SUCCUBUS_MAX_SPEED = 3.f;
				fAccel = SUCCUBUS_ACCEL;
				fMaxSpeed = SUCCUBUS_MAX_SPEED;
			}

			// goal position = target
			break;
		case EnemyType::SHIHAN:
			bIsFaceToPlayerAlways = true;
			break;
		case EnemyType::ROCK:
			{
				// n`ƐڐG]ς
				HPLActionMoveTool::stepRotate(
					lpEnemy, ROTATE_STOP_VELOCITY_THRESHOLD );


				// xȉɂȂ瓖蔻肪Ȃ
				// tɑxɖ߂炠蔻肪o
				if( lpEnemy->getObjectAttribute( ObjectAttribute::HIT_TO_PLAYER) ) {
					if( lpEnemy->getVelocityAngle().getLength() < ROCK_NO_HIT_THRESHOLD ) {
						lpEnemy->setObjectAttribute( ObjectAttribute::HIT_TO_PLAYER, false );
					}
				} else {
					if( lpEnemy->getVelocityAngle().getLength() > ROCK_NO_HIT_THRESHOLD ) {
						lpEnemy->setObjectAttribute( ObjectAttribute::HIT_TO_PLAYER, true );
					}
				}
			}
			break;
		case EnemyType::BEE:
			// 0:F	FVbĝ݁B݃A
			// 1:		FVbg͂ȂBڋ߂Ƃ

			bIsFaceToPlayerAlways = true;
			switch( st ) {
			case StateType::HOLD:
				{
					bIsAmpEnable = true;
				}
				break;
			default:
				switch( lpEnemy->getColor() ) {
				case EnemyBeeColor::YELLOW_V:
					{
						if( fLength <= ENEMY_SHOT_DISTANCE_THRESHOLD ) {
							// ㉺ɗhȂ
							bIsAmpEnable = false;
							// ڋ߁߂܂
							bIsChaser = true;
						} else {
							if( lpEnemy->getCounter(1)->step().isFinishAnimation ) {
								// ˏԂł͂Ȃ
								if( lpEnemy->getCounter( 0 )->step().isFinishAnimation ) {
									lpEnemy->getCounter( 0 )->reset();

									// ˏԂɂȂ
									lpEnemy->getCounter(1)->reset();

								} // ҋ@
							} else {
								if( lpEnemy->getCounter(1)->getState().isEndLoop ) {
									// shot
									getStaticData()->m_ShotFactory.shot( lpEnemy, 0, 0 );
								}
								if( lpEnemy->getCounter(1)->getState().isFinishAnimation ) {
									// _ivC[}Nxj
									float fDegree = vSub.getDegree();
									const float BEE_ERR_DEGREE = 60;
									fDegree += HPLMath::nextFloat(-BEE_ERR_DEGREE, BEE_ERR_DEGREE );
									HPLVector v;
									v.setByDegree( fDegree );
								}
							}
						}
					}
					break;
				case EnemyBeeColor::RED_B:	// color == 1 ԁFɐڋ߂ĕ߂܂ 
					bIsChaser = true;

					// Ȃ
					bIsAmpEnable = false;
				}
			}
			if( bIsChaser ) {
				const float BEE_ACCEL = 0.5f;
				const float BEE_MAX_SPEED = 3.f;
				fAccel = BEE_ACCEL;
				fMaxSpeed = BEE_MAX_SPEED;
			}
			break;
		case EnemyType::MINI_BEE:	//! ~jI
			{
				// chaser
				bIsChaser = true;
				fAccel = 0.3f;
				fMaxSpeed = 2.f;

				// look to player always
				bIsFaceToPlayerAlways = true;
			}
			break;
		case EnemyType::BEE_HIVE:
			{
				switch( lpEnemy->getState()->getState() ) {
				case StateType::DEAD:
					{
					}
					break;
				default:
					{
						// 0: wait for starting
						// 1: shot interval
						if( lpEnemy->getCounter(0)->step().isFinishAnimation ) {
							// 
							if( lpEnemy->getCounter(1)->step().isFinishAnimation ) {
								lpEnemy->getCounter(1)->reset();

								// shot!
								//getStaticData()->m_ShotFactory.shot( lpEnemy, 0, 0  );

								const int RADIUS = lpEnemy->getWidth() * 0.8f;
								HPLVector vCen;
								vCen.setByDegree( HPLMath::nextInt(0,360));
								vCen.multiple( RADIUS );
								vCen.add( lpEnemy->getCenter());
								// create mini bee
								GEnemy* lpAdd = this->createWithoutAdd( EnemyType::MINI_BEE,
									vCen, -1 );

								// velocity 
								const int VEL = 5 * fLevel;
								HPLVector v;
								v.setByDegree( HPLMath::nextInt(0,360));
								v.multiple( VEL );
								lpAdd->setVelocityAngle( v );

								// add
								this->m_vecAdds.push_back( lpAdd );
							}
						} else {
							// ܂ҋ@
						}
						lpEnemy->setVelocityAngle(0,0);
						bIsNoMoving = true;
					}
					break;
				}
			}
			break;
		case EnemyType::QBEE:
			// vC[̕
			bIsFaceToPlayerAlways = true;
			bIsNoMoving = true;

			if( lpEnemy->getCounter(1)->step().isFinishAnimation ) {
				// ˏԂł͂Ȃ
				if( lpEnemy->getCounter( 0 )->step().isFinishAnimation ) {
					lpEnemy->getCounter( 0 )->reset();

					// ˏԂɂȂ
					lpEnemy->getCounter(1)->reset();

					lpEnemy->changeState( StateType::ATTACK );
				} // ҋ@
			} else {
				if( lpEnemy->getCounter(1)->getState().isEndLoop ) {
					// ˃JE^
					switch( st ) {
					case StateType::ATTACK:
						if( lpSeq->getState().isFinishAnimation ) {
							// Aj[V

							// ->Standɖ߂
							lpEnemy->changeState( StateType::STAND );
							// shot
							//getStaticData()->m_ShotFactory.shot( lpEnemy, 0, 0 );

							// I
							GEnemy* lpBee = new GEnemy( EnemyType::BEE );
							lpBee->setup( EnemyType::BEE );
							HPLVector vCen;
							vCen.setByDegree( HPLMath::nextFloat(0,HPLMathConsts::DEGREE_FULL_ROUND ) );
							const int BEE_GENERATE_RADIUS = 100;
							vCen.multiple( (float)BEE_GENERATE_RADIUS );
							vCen.add( lpEnemy->getCenter() );

							lpBee->setCenter( vCen );
							lpBee->m_vCenterPosition = vCen;
							lpBee->m_vAmplitudeCenter = vCen;
							this->m_vecAdds.push_back( lpBee );
						} else {
							bIsAmpEnable = false;
						}
					}
				}
				if( lpEnemy->getCounter(1)->getState().isFinishAnimation ) {
					// 傤ǍSI
					lpEnemy->getCounter(1)->reset();
				}
			}
			break;
		case EnemyType::FUUN:
		case EnemyType::RAI:
			{
				int nMapIndex = getStaticData()->getMapIndex();
				switch( nMapIndex ) {
				case SpecialMapIndex::WIND_BOSS_2:
					{
						// ̏ŗhĂ
						// oAWJ
						bIsAmpEnable = true;
					}
					break;
				default:
					{
						bIsFaceToPlayerAlways = true;
						// 莞ԖɃvC[̏ւs
						if( lpEnemy->getCounter(FuuRaiCounterType::TURN_INTERVAL)->step().isFinishAnimation ) {
							lpEnemy->getCounter(FuuRaiCounterType::TURN_INTERVAL)->reset();

							// vC[̔wiΑj։ړ
							const int FRAMES_FOR_GOAL = lpEnemy->getCounter( FuuRaiCounterType::KEEP_MOVE_TERM)->getWaitFrameNum();
							// ݃vC[̉EɂȂ獶ցAւȂEֈړ
							const int MIN_DELTA_X = 100;		//l
							const int MAX_DELTA_X = 200;		//l`w
							const int MAX_DELTA_Y_ABOVE = 200;	//_l`w
							const int MAX_DELTA_Y_UNDER = 100;	//_l`w
							const int LEFT_RIGHT = (vSub.getX() > 0 ? 1 : -1);
							// S[n_
							HPLVector vGoalCenter( LEFT_RIGHT * HPLMath::nextInt(MIN_DELTA_X,MAX_DELTA_X),
								HPLMath::nextInt(-MAX_DELTA_Y_UNDER, MAX_DELTA_Y_ABOVE) );

							vGoalCenter.add( vTargetCenter );
							HPLVector vVelocityAngle;	// x
							float fReduceAccel = AIR_RESIST ;	// default reduce accel
							this->setVelocityByGoal(lpEnemy->getCenter(), vVelocityAngle, 
								vGoalCenter, FRAMES_FOR_GOAL, fReduceAccel);
//							HPLLogger::HPL_LOGGER.write( "ID[%d],Type[%d],Vel[%5.2f,%5.2f],Center[%5.2f,%5.2f],Reduce[%5.2f]", 
//								lpEnemy->getID(), lpEnemy->getType(), vVelocityAngle.getX(), vVelocityAngle.getY(),
//								lpEnemy->getCenter().getX(), lpEnemy->getCenter().getY(), lpEnemy->getReduceAccel());
							
//							HPLError::msg( "caption", MB_OK, "Vel:%d", (int)vVelocityAngle.getLength() );
							lpEnemy->setVelocityAngle( vVelocityAngle );
							lpEnemy->setReduceAccel( fReduceAccel );

							// ΂炭㉺^͂Ȃ
							lpEnemy->getCounter( FuuRaiCounterType::KEEP_MOVE_TERM )->reset();
							// Sʒu
							lpEnemy->m_lpAmplitude->reset();
						} else {
							if( lpEnemy->getCounter( FuuRaiCounterType::KEEP_MOVE_TERM )->step().isFinishAnimation ) {
								// after moving
								lpEnemy->disableReduceAccel();
								// ㉺^
								bIsAmpEnable = true;
							} else {	// moving
								// ㉺̒Sʒuɂ
								lpEnemy->m_vCenterPosition = lpEnemy->getCenter();
								lpEnemy->m_vAmplitudeCenter = lpEnemy->getCenter();

							}
						}
					}
				}

			}
			break;

		case EnemyType::EVIL_GATE:
			break;
		case EnemyType::MERAKO:
			this->doMerakoAlgorithm( lpEnemy );
			break;
		case EnemyType::MEDOUSA:
			{

				switch( st ) {
				case StateType::STAND:
					bIsFaceToPlayerAlways = true;
					break;
				default:
					bIsFaceByVelocity = true;
				}
				const float MEDOUSA_ACCEL = 0.1f;
				const float MEDOUSA_MAX_SPEED = 3.f;
				float fLength = rHitRect.getWidth() / 2 + 50;	// tꂸ
				const float MEDOUSA_GOAL_DELTA_Y = 30;
				switch( st ) {
				case StateType::WALK:
				{
					bIsChaser = true;
					// goal
					vGoalPosition = vTargetCenter;
					vGoalPosition.add( vAim.getX() * (-1) * fLength, - MEDOUSA_GOAL_DELTA_Y );
					// speed
					fMaxSpeed = MEDOUSA_MAX_SPEED;
					// accel
					fAccel = MEDOUSA_ACCEL;

					// 莞ԂAttackɁB
					if( lpEnemy->getCounter( MedousaCounterType::WAIT_FOR_NEXT )->step().isFinishAnimation ) {
						lpEnemy->changeState( StateType::ATTACK );
					}
				}
				break;
				case StateType::STAND:
					lpEnemy->changeState( StateType::WALK );
					break;
				case StateType::ATTACK:
					{
						int nNextAttackType = -1;
						switch( lpEnemy->getAttackType() ) {
						case 0:	// wr{qwrˏo
							if( lpEnemy->getCounter( MedousaCounterType::SNAKE_SHOT_INTERVAL )->step().isFinishAnimation ) {
								// VbgԊu

								lpEnemy->getCounter( MedousaCounterType::SNAKE_SHOT_INTERVAL )->reset();

								HPLSequenceState seqState = lpEnemy->getCounter( MedousaCounterType::SNAKE_COUNT )->step();

								if( seqState.isFinishAnimation ) {
									// I
									// ->̍Up^[iΉr[j
									nNextAttackType = lpEnemy->getAttackType() + 1;
								} else {
									// ֔
									getStaticData()->m_ShotFactory.shot(
										lpEnemy, lpEnemy->getAttackType(), -1 );
								}
							}
							if( lpSeq->getState().isFinishAnimation ) {
								lpSeq->resetCounter();
							}
							break;
						case 1:
							// V[PXI
							if( lpSeq->getState().isFinishAnimation ) {
								// Ήr[
							}
							break;
#ifdef _DEBUG
						default:
							ASSERT_IT_TYPE( lpEnemy->getAttackType(), "medousa attack type");
//							HPLError::assertIt( false );
#endif
						}
						if( nNextAttackType >= 0 ) {
							lpEnemy->setAttackType( nNextAttackType );
							lpEnemy->getCounter( MedousaCounterType::WAIT_FOR_NEXT )->reset();
							lpEnemy->getCounter( MedousaCounterType::SNAKE_SHOT_INTERVAL )->reset();
							lpEnemy->getCounter( MedousaCounterType::SNAKE_COUNT )->reset();
							lpEnemy->changeState( StateType::WALK );
						}
					}
					break;
				}
			}
			break;
		case EnemyType::WORM:
			{
				switch( st ) {
				case StateType::HOLD_DOWN:	// ݍݒ
					bIsNoMoving = true;
					if( lpEnemy->getCounter(0)->step().isFinishAnimation ) {
						// hold typeύX
						// hold Q[W
						this->resetTargetHoldCounter( lpEnemy->getTarget(), lpEnemy );
						lpEnemy->setHoldType(1);
						lpEnemy->changeState( StateType::HOLD );
						lpEnemy->getCounter(0)->reset();
					}
					break;
				case StateType::HOLD:
					bIsNoMoving = true;
					// ۂݍ񂾏
					if( lpSeq->getState().isFinishAnimation ) {
						lpSeq->resetCounter();
					}
					break;

				default:	// stand
					bIsFaceToPlayerAlways = true;
					bIsMoveToPlayerGround = true;
//					lpEnemy->getCounter(0)->end();
				}
			}
			break;
		case EnemyType::FIRE_ELEMENTAL:
			{
				HPLCounter* lpWaitCounter = lpEnemy->getCounter(FireElementalCounterType::WAIT_FOR_NEXT);
				if( lpWaitCounter->step().isFinishAnimation ) {
					// after waiting
					if( lpEnemy->getCounter( FireElementalCounterType::ATTACK_TERM )->step().isFinishAnimation ) {
						// after attacking
						// reset waiting counter
						lpWaitCounter->reset();

					} else {	// attacking
						bIsChaser = true;
						fAccel = 0.3f;
						fMaxSpeed = 3;
					}
				} else {	// 
					if( lpWaitCounter->getStepCounter() == lpWaitCounter->getWaitFrameNum() - 1 ) {
						lpEnemy->getCounter( FireElementalCounterType::ATTACK_TERM )->reset();
					}
				}
			}
			break;
		case EnemyType::FROG:
			bIsMoveToPlayerGround = true;
			bIsFaceByVelocity = true;
			break;
		case EnemyType::ROBOT:
			bIsFaceByVelocity = true;
			switch( st ) {
			case StateType::STAND:
				{
					bIsFaceToPlayerAlways = true;
					lpEnemy->changeState( StateType::WALK );
				}
				break;
			case StateType::WALK:
				{
					bIsMoveToPlayerGround = true;
					if( lpEnemy->getTrigger()->getTrigger( TriggerType::HIT_TO_THE_LEFT ) ||
						lpEnemy->getTrigger()->getTrigger( TriggerType::HIT_TO_THE_RIGHT ) )
					{
						// Gꂽ
						// Wv or ~TC
						if( HPLMath::nextInt(0,2) == 0 ) {
							getStaticData()->m_ShotFactory.shot( 
								lpEnemy, 1, 0 );
						} else {
							// jump
							lpEnemy->changeState( StateType::READY_TO_JUMP );
						}
						
					} else {
						// wait 
						if( lpEnemy->getCounter( RobotCounterType::WAIT_FOR_NEXT )->step().isFinishAnimation ) {
							lpEnemy->getCounter( RobotCounterType::WAIT_FOR_NEXT )->reset();

							// ፷mF
							const int BEAM_THRESHOLD = 100;
							// missile˂ɂȂm
							int RANDOM_THRESHOLD = vTargetCenter.getY() - lpEnemy->getCenter().getY() > BEAM_THRESHOLD ?
								70 : 50;	
							if( HPLMath::nextInt(0,100) < RANDOM_THRESHOLD  ) {
								lpEnemy->changeState( StateType::ATTACK_GUN_2 ); // missile
							} else {
								lpEnemy->changeState( StateType::ATTACK_GUN );	// beam
							}
						}
					}
				}
				break;
			case StateType::READY_TO_JUMP:
				{
					if( lpSeq->getState().isFinishAnimation ) {
						// jump!
						const int JUMP_SPEED = 8;
						HPLVector vVel = lpEnemy->getVelocityAngle();
						vVel.setY(-JUMP_SPEED );
						lpEnemy->setVelocityAngle( vVel );

						lpEnemy->changeState( StateType::FLYDOWN );
					}
				}
				break;
			case StateType::READY:
				{
					// n
					if( lpSeq->getState().isFinishAnimation ) {
						lpEnemy->changeState( StateType::STAND );
					}

				}
				break;
			case StateType::FLYDOWN:
				{
					// n
					if( lpEnemy->getTrigger()->getTrigger( TriggerType::ON_THE_GROUND ) ) {
						lpEnemy->changeState( StateType::READY );
					}
				}
				break;
			case StateType::ATTACK_GUN:
				{
					// N񔭎
					if( lpSeq->getState().isFinishAnimation ) {//lpEnemy->getCounter( RobotCounterType::BEAM_SHOT_INTERVAL )->step().isFinishAnimation ) {
						// 
						getStaticData()->m_ShotFactory.shot( lpEnemy, 0, 0 );

						if( lpEnemy->getCounter( RobotCounterType::BEAM_SHOT_COUNT )->step().isFinishAnimation ) {
							// ܂
							lpEnemy->changeState( lpEnemy->getTrigger()->getTrigger( TriggerType::ON_THE_GROUND ) ?
								StateType::STAND :
								StateType::FLYDOWN);

							lpEnemy->getCounter( RobotCounterType::BEAM_SHOT_COUNT )->reset();

						} else {
							// ܂
						}
						//lpEnemy->getCounter( RobotCounterType::BEAM_SHOT_INTERVAL )->reset();
						lpSeq->resetCounter();
					}
				}
				break;
			case StateType::ATTACK_GUN_2:
				{
					// N񔭎
					if( lpSeq->getState().isFinishAnimation ) {//lpEnemy->getCounter( RobotCounterType::MISSILE_SHOT_INTERVAL )->step().isFinishAnimation ) {

						// 
						getStaticData()->m_ShotFactory.shot( lpEnemy, 1, 0 );
						if( lpEnemy->getCounter( RobotCounterType::MISSILE_SHOT_COUNT )->step().isFinishAnimation ) {
							// ܂
							lpEnemy->changeState( lpEnemy->getTrigger()->getTrigger( TriggerType::ON_THE_GROUND ) ?
								StateType::STAND :
								StateType::FLYDOWN);

							lpEnemy->getCounter( RobotCounterType::MISSILE_SHOT_COUNT )->reset();
						} else {
							// ܂
						}
						//lpEnemy->getCounter( RobotCounterType::MISSILE_SHOT_INTERVAL )->reset();
						lpSeq->resetCounter();
					}
				}
				break;
			}
			break;
		case EnemyType::GOKI:
			bIsMoveToPlayerGround = true;
			bIsFaceByVelocity = true;
			if( lpEnemy->getCounter( 0)->step().isFinishAnimation ){
				lpEnemy->getCounter( 0)->reset();

				// jump!
				HPLVector vVel = lpEnemy->getVelocityAngle();
				const float JUMP_SPEED = 8;
				vVel.setY( -JUMP_SPEED);
				lpEnemy->setVelocityAngle(vVel);
				lpEnemy->changeState( StateType::FLYDOWN );
				lpEnemy->getTrigger()->setTrigger( TriggerType::ON_THE_GROUND, false );
			}
			break;
		case EnemyType::PIRANHA:
			bIsFaceByVelocity = true;
			bIsChaser = true;
			fAccel = 1.f;
			fMaxSpeed = fAccel;
			break;
		case EnemyType::JET_PORT:
			bIsNoMoving = true;
			break;
		case EnemyType::CYCLON:
			bIsFaceByVelocity = true;
			bIsMoveDirection = true;

			// Ăֈړ

			break;
		case EnemyType::ISO:
			bIsNoMoving = true;
			break;
		case EnemyType::SHARK:
			bIsMoveToPlayerGround = true;
			bIsFaceByVelocity = true;
			bIsMoveDirection = true;
			break;
		case EnemyType::MUTANT_GEL:
			bIsFaceByVelocity = true;
			bIsMoveDirection = true;
			break;
		case EnemyType::UNI:
			bIsFaceByVelocity = true;
			break;
		case EnemyType::DIRTY_CLOUD:
			// chaser
			{
				bIsChaser = true;
				fAccel = 0.2f;
				fMaxSpeed = 4.f;
				const float LENGTH_EPSIRON = 2;
				const int TOO_FAR_THRESHOLD = 200;	// 𒴂VڕWn_T

				// face to player always
				bIsFaceToPlayerAlways = true;

				// create new goal
				bool bIsCreateNewGoal = false;
				// enemyS[ւ̋
				float fDistanceToGoal = HPLVector::subtract( lpEnemy->m_vCenterPosition, lpEnemy->getCenter()).getLength();
				// enemy^[Qbgւ̋
				float fDistanceToTarget = vSub.getLength();

				if( lpEnemy->m_vCenterPosition.getLength() < LENGTH_EPSIRON ||	// first
					fDistanceToGoal < LENGTH_EPSIRON ||			// near
					fDistanceToTarget > TOO_FAR_THRESHOLD		// too far
					)
				{
					bIsCreateNewGoal = true;
				}
				// shot
				bool bIsShot = true;
				if( lpEnemy->getCounter( DirtyCloudCounterType::WAIT_FOR_START )->step().isFinishAnimation ) {
					if( lpEnemy->getCounter( DirtyCloudCounterType::WAIT_FOR_NEXT )->step().isFinishAnimation ) {
					} else {
						bIsShot = false;
					}
				} else {
					bIsShot = false;
				}
				// next attack
				int nNextAttack = -1;
				switch( lpEnemy->getAttackType() ) {
				case DirtyCloudAttackType::DROP:
					{
						// vC[̏㕔Ƀ_Ō
						if( bIsCreateNewGoal ) {
							const int ERR = 80;
							const int AVOBE = 50;
							lpEnemy->m_vCenterPosition.set(
								vTargetCenter.getX() + HPLMath::nextInt(-ERR,ERR ),
								vTargetCenter.getY() - AVOBE );
						}
						vGoalPosition = lpEnemy->m_vCenterPosition;
						
						if( bIsShot ) {
							const int DROP_X_THRESHOLD = 100;
							if( vSub.getY() > 0 &&						//
								abs(vSub.getX()) < DROP_X_THRESHOLD )	// near x
							{
								// I drop 
								if( lpEnemy->getCounter(DirtyCloudCounterType::DROP_INTERVAL)->step().isFinishAnimation ) {
									lpEnemy->getCounter(DirtyCloudCounterType::DROP_INTERVAL)->reset();

									// shot
									getStaticData()->m_ShotFactory.shot(lpEnemy,
										lpEnemy->getAttackType() , 0);

									if( lpEnemy->getCounter(DirtyCloudCounterType::DROP_COUNT)->step().isFinishAnimation ) {
										nNextAttack = DirtyCloudAttackType::WIND;
									}
								}
							}
						}

					}
					break;
				case DirtyCloudAttackType::WIND:
					{
						// vC[̍Ei߂jɃ_Ō
						if( bIsCreateNewGoal ) {
							const int ERR = 80;
							const int DISTANCE = 100;
							lpEnemy->m_vCenterPosition.set(
								vTargetCenter.getX() + DISTANCE * ( vSub.getX() > 0 ? 1 : -1 ), 
								vTargetCenter.getY() + HPLMath::nextInt(-ERR,ERR ) );
						}
						vGoalPosition = lpEnemy->m_vCenterPosition;

						const int WIND_Y_THRESHOLD = 50;
						if( bIsShot ) {
							if( vSub.getLength() < WIND_Y_THRESHOLD ) {
								// I wind 
								if( lpEnemy->getCounter(DirtyCloudCounterType::WIND_INTERVAL)->step().isFinishAnimation ) {
									lpEnemy->getCounter(DirtyCloudCounterType::WIND_INTERVAL)->reset();

									// shot
									getStaticData()->m_ShotFactory.shot(lpEnemy,
										lpEnemy->getAttackType() , 0);

									if( lpEnemy->getCounter(DirtyCloudCounterType::WIND_COUNT)->step().isFinishAnimation ) {
										nNextAttack = DirtyCloudAttackType::WIND;
									}
								}
							}
						}
					}
					break;
				}
				if( nNextAttack >= 0 ) {	// next attack type available
					int RESET_TARGETS[] = {
						DirtyCloudCounterType::WAIT_FOR_NEXT,
						DirtyCloudCounterType::DROP_COUNT,
						DirtyCloudCounterType::DROP_INTERVAL,
						DirtyCloudCounterType::WIND_COUNT,
						DirtyCloudCounterType::WIND_INTERVAL,
					};
					for( int i = 0; i < SIZEOF( RESET_TARGETS ); i ++ ) {
						// reset counter
						lpEnemy->getCounter(RESET_TARGETS[i])->reset();
					}

					lpEnemy->setAttackType( nNextAttack );
				}
			}
			break;
		case EnemyType::FAN_BIRD:
			{
				fAmplitude *= 1.5f;
				bIsAmpEnable = true;
				bIsFaceToPlayerAlways = true;
				// GtFNg𐶐
				float fDegree = (lpEnemy->getFaceDirection() == FaceDirection::LEFT ) ? 180.f : 0;
				getStaticData()->m_ObjectManager.m_EffectManager.createWind(
					lpEnemy->getCenter(), lpEnemy->getWidth(),
					lpEnemy->getHeight(),
					fDegree, 1 );
			}
			break;
		case EnemyType::JEALOUS_GHOST:
			break;
		case EnemyType::SORROW_CLOUDS:
			{
				switch( st ) {
				case StateType::DYING:
					{
						bIsAmpEnable = false;	// cancel amplitude
						HPLSequence* lpSeq = lpEnemy->getSequence();
						HPLSequenceState& seqState = lpSeq->getState();
						if( seqState.isFinishAnimation ) {	// finish dying animation
							lpEnemy->die(NULL);	// die!
						}
					}
					break;
				default:
					fAmplitude *= 0.7f;
					// amplitude
					bIsAmpEnable = true;
					break;
				}
			}
			break;
		case EnemyType::SHABONER:
			{
				HPLActionMoveTool::stepRotate(
					lpEnemy, 0 );
				if( lpEnemy->getCounter(0)->step().isFinishAnimation ) {
					lpEnemy->getCounter(0)->reset();
					// ˒

					//getStaticData()->m_ShotFactory.shot(
					//	lpEnemy, 0, 0 );

					// Shabon𐶐
					// ݂̕ɂ炵4ɔ˂
					getStaticData()->m_ShotFactory.shot( 
						lpEnemy, 0, 0 );
					/*int SHABON_NUM = 4;
					int SHABON_SPEED = 5;
					int INTERVAL_DEG = HPLMathConsts::DEGREE_FULL_ROUND / SHABON_NUM;
					float ERR = 5 * fLevel;
					const float LENGTH = 35;
					for( int i = 0; i < SHABON_NUM; i ++ ) {
						float fDeg = lpEnemy->getRotateDegree() + i * INTERVAL_DEG;
						HPLVector c;
						c.setByDegree(fDeg);
						c.multiple( LENGTH );
						c.add( lpEnemy->getCenter());
						GEnemy* lpShabon = getStaticData()->m_ObjectManager.m_EnemyManager.createWithoutAdd(
							EnemyType::SHABON, c, -1 );
						// random
						fDeg += HPLMath::nextFloat(-ERR,ERR);
						HPLVector v;
						v.setByDegree( fDeg );
						v.multiple( SHABON_SPEED) ;
						lpShabon->setVelocityAngle( v);
						this->m_vecAdds.push_back(lpShabon);
					}*/



					if( lpEnemy->getCounter(1)->step().isFinishAnimation ) {
						// ˏI

						// wait 
						lpEnemy->getCounter(0)->reset();
						lpEnemy->getCounter(1)->reset();
					}
				} 
			}
			break;
		case EnemyType::JACKPOT:
			{
				// 炶

				// ɒSW֖߂낤Ƃ
				const float TO_CENTER_SPEED = 3;
				HPLVector v = HPLVector::subtract(
					lpEnemy->m_vAmplitudeCenter, lpEnemy->getCenter() );
				if( v.getLength() <= TO_CENTER_SPEED ) {
					// ~܂
					lpEnemy->setCenter( lpEnemy->m_vAmplitudeCenter );
				} else {
					// S֌ē
					v.normalize();
					v.multiple( TO_CENTER_SPEED );
					lpEnemy->setVelocityAngle( v );
				}

				switch( st ) {
				case StateType::DAMAGE:
					if( lpSeq->getState().isFinishAnimation ) {
						// end damage animation
						// standɖ߂
						lpEnemy->changeState( StateType::STAND );
					}
					break;
				}
			}
			break;
		case EnemyType::CHASER:
			this->doChaserAlgorithm( lpEnemy );
			bIsAmpEnable = false;	// ƎchasingĂ
			break;
		case EnemyType::SUN:
			{
				bIsChaser= true;
				const float SUN_ACCEL = 0.4f;
				const float SUN_MAX_SPEED = 5;
				fAccel = SUN_ACCEL;
				fMaxSpeed = SUN_MAX_SPEED;
			}
			break;
		case EnemyType::LAVA:
			{
				lpEnemy->setVelocityAngle( 0,-LAVA_UP_SPEED);
			}
			break;
		case EnemyType::LAVA_UNDER:
			{
				if( lpEnemy->getCounter(0)->step().isFinishAnimation ) {
					lpEnemy->setVelocityAngle( 0,-LAVA_UP_SPEED);
				}
			}
			break;
		case EnemyType::LAVA_BALL:
			{
				if( lpEnemy->getCounter(0)->step().isFinishAnimation ) {
					lpEnemy->getCounter(0)->reset();

					HPLVector vCenter = lpEnemy->getCenter();
					HPLVector vAdd;
					vAdd.setByDegree( HPLMath::nextInt(0,360));
					HPLSequence* lpSeq = getStaticData()->getSequence(SpriteType::SHOT,
						Sequence::Shot::LAVA_BALL );
					HPLImageInformation* lpInfo = lpSeq->getImageInformation(0);
					vAdd.multiple( lpInfo->srcRect.getWidth() / 2 );
					vCenter.add( vAdd );
					// 
					GShot* lpShot = getStaticData()->m_ObjectManager.m_ShotManager.create(
						Sequence::Shot::FIRE_PARTICLE, vCenter,
						lpEnemy->getID(), lpEnemy->getFaceDirection(), false, true );
				}
			}
			break;
		case EnemyType::AIKO:
			this->doAikoAlgorithm(lpEnemy);
			break;
		case EnemyType::GHOST_PAIR:
			this->doGhostPairAlgorithm(lpEnemy );
		default:
			bIsFaceToPlayerAlways = true;
			break;
		}
	}

	if( bIsDieAfterFinishAnimation ) {	// die after animation
		if( lpSeq->getState().isFinishAnimation ) {
			lpEnemy->die( NULL );
		}
		return ;
	}


	if( lpEnemy->getState()->isDamageState() ) {	// damage state
		//&&
	//	false == lpEnemy->m_aryObjectAttribute[ObjectAttribute::GRAVITY] ) 
	//{
		// _[W

		lpEnemy->m_DamageCounter.step();

		if( lpEnemy->getSequence()->getState().isEndLoop || lpEnemy->m_DamageCounter.getState().isFinishAnimation) {
			// V[PXI
			// ʏԂ֖߂
			lpEnemy->changeState( StateType::STAND );
		}
	}
	// ړ

	// move direction
	int nMoveDirection = lpEnemy->getFaceDirection();

	if( bIsMoveDirection && bIsNoMoving == false) {	// isֈړ
		HPLVector vVelocityAngle = lpEnemy->getVelocityAngle();

		// ړ
		this->accelMoveX( vVelocityAngle,
			lpEnemy->getFaceDirection(),
			lpEnemy->getTrigger(),
			ENEMY_MOVE_ACCEL * fSpeedRate + RESIST,// * fLevel,
//			RESIST,
			ENEMY_SPEED_SLOW_X + RESIST, bIsJumpOverCollision == false );

		lpEnemy->setVelocityAngle( vVelocityAngle );

	}

	if( bIsMoveToPlayerGround && bIsNoMoving == false) {	// vC[ɋ߂Â

		if( fabs(vSub.getX() ) >= ENEMY_NEAR_THRESHOLD ) {
			this->moveToTarget( lpEnemy, fSpeedRate, bIsJumpOverCollision );
		} else {
			// ߂Â

			// Ԃɂ
			lpEnemy->changeState( StateType::STAND );
		}
	}

	if( bIsTurnEdge ) {	// RŔΑ
		if( lpEnemy->getTrigger()->getTrigger( TriggerType::NO_GROUND_AT_THE_FACE_DIRECTION ) ) {

			int nFaceDirection = lpEnemy->getFaceDirection();
			HPLVector vVelocityAngle = lpEnemy->getVelocityAngle();

			// ΂
			HPLActionMoveTool::turnX( nFaceDirection, vVelocityAngle );

			lpEnemy->setFaceDirection( nFaceDirection );
			lpEnemy->setVelocityAngle( vVelocityAngle );
		}
	}

	if( bIsJumpOverCollision ) {	// jump over collision
		bIsTurnEdge = false;
	}

	if( bIsTurnWall ) {	// ǂŔΕ
		HPLVector vVelocityAngle = lpEnemy->getVelocityAngle();
		int nFaceDirection = lpEnemy->getFaceDirection();

		HPLVector vAccel;
		HPLActionMoveTool::turnWhenHitX( lpEnemy->getTrigger(),
			&nFaceDirection, &vVelocityAngle, &vAccel );

		lpEnemy->setFaceDirection( nFaceDirection );
		lpEnemy->setVelocityAngle( vVelocityAngle );
	}

	// ǂɓWvŏz
	if( bIsJumpOverCollision &&	// jump over collision!
		( lpEnemy->getTrigger()->getTrigger( TriggerType::HIT_TO_THE_LEFT ) ||	// hit left
		lpEnemy->getTrigger()->getTrigger( TriggerType::HIT_TO_THE_RIGHT ) ))	// or hit right
	{
		this->jump( lpEnemy,
			//lpEnemy->getFaceDirection() * fabs( 
			lpEnemy->getVelocityAngle().getX()
			//)
			,
			-BLUE_SLIME_JUMP_SPEED,
			false, false );
	}
	if( bIsFaceToPlayerAlways ) {	// ɃvC[̕ifalseisj
		this->faceToTarget( lpEnemy );
	} else if( bIsFaceByVelocity ) {	// xɂĕς
		HPLActionMoveTool::faceByVelocity( lpEnemy, FACE_CHANGE_THRESHOLD );
	}

	if( bIsChaser ) {	// S[|WV։

		//this->chase( lpEnemy, vGoalPosition, fAccel, fMaxSpeed );			
		HPLShootingMove::doMissileHoming( lpEnemy, vGoalPosition, fAccel + AIR_RESIST, fMaxSpeed + AIR_RESIST );

	}
	if( bIsAmpEnable ) {	// ㉺^
		float fRate = lpEnemy->m_lpAmplitude->getStepCounterRate();
		// ʒup
		float fDegree = HPLMathConsts::DEGREE_FULL_ROUND * fRate;
		float fRad = HPLMath::getRadianFromDegree( fDegree );
		
		HPLVector vCenter(0, fAmplitude * sin(HPLMath::getRadianFromDegree( fDegree ) ));
		vCenter.add( lpEnemy->m_vAmplitudeCenter );
		lpEnemy->setCenter( vCenter );

		//const float DEFAULT_AMPLITUDE_SPEED = 5;
		//HPLVector vVel = lpEnemy->getVelocityAngle();

/*		float fEnemySkyResist = 0;
		if( lpEnemy->isUseReduceAccel() ) {
			fEnemySkyResist = lpEnemy->getReduceAccel();

		} else {
			fEnemySkyResist = this->getEnemyAirResist(1);
		}
		vVel.setY(fAmplitude / DEFAULT_AMPLITUDE * DEFAULT_AMPLITUDE_SPEED * cos( fRad ) +
			fEnemySkyResist);
		lpEnemy->setVelocityAngle( vVel );
		*/
//		lpEnemy->setVelocityAngle( 0,fAmplitude / DEFAULT_AMPLITUDE * DEFAULT_AMPLITUDE_SPEED * cos( fRad ) );

		if( lpEnemy->m_lpAmplitude->step().isFinishAnimation ) {
			lpEnemy->m_lpAmplitude->reset();
		}
	} else {	// not amplitude
		if( lpEnemy->m_lpAmplitude != NULL && false == bIsNoMoving) {
			// ㉺ɓ^Cv̓G

			// ݈ʒu𒆐SWɕۑĂ
			//lpEnemy->m_vCenterPosition = lpEnemy->getCenter();
			lpEnemy->m_vAmplitudeCenter = lpEnemy->getCenter();

			lpEnemy->m_lpAmplitude->reset();
		}
	}

	// ^[QbeBO
	if( getStaticData()->m_Game.isTheWorld() == false &&	// not the world
		lpEnemy->m_TargettedRemainCounter.getState().isFinishAnimation == false )	// animation not finish
	{
		lpEnemy->m_TargettedRemainCounter.step();
	}

	bool bIsReduceSpeed = false;
	if( bIsAmpEnable ) {
		bIsReduceSpeed = true;
	}
	switch( t ) {
	case EnemyType::FUUN:
	case EnemyType::RAI:
	case EnemyType::CHASER:
	case EnemyType::BEE:
	case EnemyType::SORROW_CLOUDS:
	case EnemyType::FAN_BIRD:
	case EnemyType::SUCCUBUS:
	case EnemyType::SUCCUBUS_MISAGO:
		{
			// (Kvł)
			bIsReduceSpeed = true;
			
		}
		break;
	}
	if( bIsReduceSpeed ) {
		this->checkAndReduceSpeed( lpEnemy );
	}

	//////////////////////////////////////////////////
	// 
	/*if( lpEnemy->getTrigger()->getTrigger( TriggerType::ON_THE_GROUND ) ) {
		HPLVector vCenter = lpEnemy->getCenter();
		HPLVector vVelocityAngle = lpEnemy->getVelocityAngle();

		vCenter.add( vVelocityAngle );
		lpEnemy->setCenter( vCenter );
	}*/
}

