#include "GEnemyManager.h"

#include "GStaticData.h"
#include <hpllib/math/HPLMath.h>
#include <hpllib/action/HPLActionMoveTool.h>
#include <hpllib/system/HPLFile.h>
#include <hpllib/general/HPLString.h>

using namespace std;
using namespace GEnum;


/** AElAI
  */
void GEnemyManager::doAryonaAlgorithm( GEnemy* lpEnemy ) {
	int t = lpEnemy->getType();

	// target ( player)
	HPLShootingObject* lpTarget = lpEnemy->getTarget();

	// target ( player) position
	HPLVector vTargetCenter = lpTarget->getCenter();

	// direction and length from enemy to target( player )
	HPLVector vSub(
		vTargetCenter.getX() - lpEnemy->getCenter().getX(),
		vTargetCenter.getY() - lpEnemy->getCenter().getY() );


	float fLevel = getStaticData()->m_Game.getLevel( LEVEL_EPSILON );

	float fLength = vSub.getLength();	// length to target

	int st = lpEnemy->getState()->getState();	// state
	float RESIST = getStaticData()->m_Setting.resist;	// resist
	float GRAVITY = getStaticData()->m_Setting.gravity;	// gravity

	// Aj[V
	HPLSequence* lpSeq = lpEnemy->getSequence();

	int nTileSize = getStaticData()->m_Setting.tileSize;	// tile size

	// block col num * tile
	int nBlockWidth = getStaticData()->m_BlockManager.getBlockMatrix()->getColumnSize() * 
		nTileSize;
	// block row num * tile
	int nBlockHeight = getStaticData()->m_BlockManager.getBlockMatrix()->getRowSize() * 
		nTileSize;

	bool bIsFaceToPlayer = false;	// face to player
	bool bIsMoveToPlayer = false;	// move to player
	int nNextAttackType = -1;	// next attack type
	
	switch( lpEnemy->getGenerateType() ) {	// switch generate type = star type
	case StageType::EVIL_STAR:
	case StageType::FIRE:
	case StageType::ROCK:
	case StageType::WATER:
	case StageType::TOZI_STAR:
	case StageType::ELECTRIC:
	case StageType::REST_STAR:
		// Ȃ

		// vC[̕
		bIsFaceToPlayer = true;
		break;

	case StageType::FOREST:
		{
			// forest
			bIsFaceToPlayer = true;

			// ŏ̑҂
			if( lpEnemy->getCounter(AryonaCounterType::WAIT_FOR_START )->step().isFinishAnimation ) {

				bool bIsWaitingYet = !lpEnemy->getCounter(AryonaCounterType::WAIT_FOR_NEXT )->getState().isFinishAnimation;;

				// ŃURԂ
				// ܂ł̑҂
				if( lpEnemy->getCounter(AryonaCounterType::WAIT_FOR_NEXT )->step().isFinishAnimation ) {
					// WAITI
					if( bIsWaitingYet ) {
						// ̃t[ŏI
						// ->uԈړ
						/*
						// next attack
						lpEnemy->getCounter( AryonaCounterType::ALL_ATTACK_PATTERN )->step();

						// reset wait next
						lpEnemy->getCounter(AryonaCounterType::WAIT_FOR_NEXT )->reset();
	*/
						float fX = 1;
						if( lpEnemy->getCenter().getX() < nBlockWidth / 2 ) {
							// E
							fX = 1;
						} else {
							fX = -1;
						}
						HPLVector vNextCenter( nBlockWidth / 2 + fX * nBlockWidth / 4.f, nBlockHeight / 2 );
						// effect 
						getStaticData()->m_ObjectManager.m_EffectManager.create( Sequence::Effect::WARP, vNextCenter, true );
						lpEnemy->setCenter( vNextCenter );
					}
					
					switch( lpEnemy->getAttackType() ) {//lpEnemy->getCounter( AryonaCounterType::ALL_ATTACK_PATTERN )->getStepCounter() ) {
					case AryonaAttackType::Forest::SUMMON_SLIME:
						// 
						switch( st ) {
						case StateType::ATTACK:
							if( lpEnemy->getCounter(AryonaCounterType::SUMMON_TIME )->getState().isFinishAnimation ) {
								// summon interval!

								// set as stand
								lpEnemy->changeState( StateType::STAND );

								// next attack
//								lpEnemy->getCounter( AryonaCounterType::ALL_ATTACK_PATTERN )->step();

								// reset wait next
								//lpEnemy->getCounter(AryonaCounterType::WAIT_FOR_NEXT )->reset();
								nNextAttackType = AryonaAttackType::Forest::SUMMON_SKYTHE;

							} else {
								lpEnemy->getCounter(AryonaCounterType::SUMMON_TIME )->step();
								if( lpEnemy->getCounter( AryonaCounterType::SUMMON_INTERVAL )->getState().isFinishAnimation ) {
									lpEnemy->getCounter( AryonaCounterType::SUMMON_INTERVAL )->reset();
									// 
									float fX = 1;
									if( lpEnemy->getCenter().getX() < nBlockWidth / 2 ) {
										// 
										fX = 1;
									} else {
										fX = -1;
									}
									HPLVector vNextCenter( nBlockWidth / 2 + fX * nBlockWidth / 4.f, nBlockHeight / 2 );
									// G
									GEnemy* lpAdd = this->createWithoutAdd(
										EnemyType::SLIME, vNextCenter, -1 );
									this->m_vecAdds.push_back( lpAdd );

								} else {
									lpEnemy->getCounter( AryonaCounterType::SUMMON_INTERVAL )->step();
								}
							}
							break;
						default:
							lpEnemy->changeState( StateType::ATTACK );
						}
						break;
					case AryonaAttackType::Forest::SUMMON_SKYTHE:
						switch( st ) {
						case StateType::ATTACK:
							if( lpEnemy->getCounter(AryonaCounterType::SUMMON_TIME_2 )->getState().isFinishAnimation ) {
								nNextAttackType = AryonaAttackType::Forest::SUMMON_SLIME;

							} else {
								lpEnemy->getCounter(AryonaCounterType::SUMMON_TIME_2 )->step();
								if( lpEnemy->getCounter( AryonaCounterType::SUMMON_INTERVAL_2 )->getState().isFinishAnimation ) {
									lpEnemy->getCounter( AryonaCounterType::SUMMON_INTERVAL_2 )->reset();

									// 
									float fX = 1;
									if( lpEnemy->getCenter().getX() < nBlockWidth / 2 ) {
										// 
										fX = 1;
									} else {
										fX = -1;
									}
									HPLVector vNextCenter( nBlockWidth / 2 + fX * nBlockWidth / 4.f, nBlockHeight / 2 );

									// VbgƃGtFNg
									GShot* lpShot = getStaticData()->m_ObjectManager.m_ShotManager.create( Sequence::Shot::SKYTHE,
										vNextCenter, lpEnemy->getID(), lpEnemy->getFaceDirection(),
										false, true);
									HPLVector vAim = HPLVector::subtract( lpTarget->getCenter(), vNextCenter );
									float fDeg = vAim.getDegree();
									const float SKYTHE_ERR_DEG = 5;
									fDeg += HPLMath::nextFloat(-SKYTHE_ERR_DEG,SKYTHE_ERR_DEG );
									vAim.setByDegree( fDeg );
									const float SKYTHE_SPEED = 5;
									const float SKYTHE_SPEED_ERR = 3;
									vAim.multiple( SKYTHE_SPEED + HPLMath::nextFloat(-SKYTHE_SPEED_ERR,SKYTHE_SPEED_ERR ) );
									lpShot->setVelocityAngle(vAim);
									
									// GtFNg
									getStaticData()->createEffect( Sequence::Effect::WARP,
										vNextCenter, vNextCenter, -1, -1 );

									/*
									// G
									GEnemy* lpAdd = this->createWithoutAdd(
										EnemyType::SLIME, vNextCenter, -1 );
									this->m_vecAdds.push_back( lpAdd );
									*/

								} else {
									lpEnemy->getCounter( AryonaCounterType::SUMMON_INTERVAL_2 )->step();
								}
							}
							break;
						default:
							lpEnemy->changeState( StateType::ATTACK );
						}
						break;
					}	// switch attack type
				} // waiting for next

				if( nNextAttackType >= 0 ) {
					int RESET_COUNTERS[] = {
						AryonaCounterType::SUMMON_TIME ,
						AryonaCounterType::SUMMON_INTERVAL,
						AryonaCounterType::SUMMON_TIME_2,
						AryonaCounterType::SUMMON_INTERVAL_2,
						AryonaCounterType::WAIT_FOR_NEXT,
					};
					for( int i = 0; i < SIZEOF( RESET_COUNTERS ); i ++ ) {
						lpEnemy->getCounter( RESET_COUNTERS[i])->reset();
					}

					// set as stand
					lpEnemy->changeState( StateType::STAND );
					// next attack type
					lpEnemy->setAttackType( nNextAttackType );
				}
/*				if( lpEnemy->getCounter(AryonaCounterType::ALL_ATTACK_PATTERN )->getState().isFinishAnimation ) {
					// 1T

					// ŏɖ߂
					lpEnemy->getCounter(AryonaCounterType::ALL_ATTACK_PATTERN )->reset();
				}*/
			}
		}
		break;
	case StageType::WIND:		// ̃A[i
		{
			// face to player always
			bIsFaceToPlayer = true;

			
			HPLMatrix<int>* lpMat = this->m_mapCoordPositions[EnemyPositionType::WIND_ARYONA];

			// 6ӏɃ_ňړAUd|ĂB
			if( lpEnemy->getCounter(AryonaCounterType::WAIT_FOR_START )->step().isFinishAnimation ) {
				// start waitI

				bool bIsWaitingYet = !lpEnemy->getCounter(AryonaCounterType::WAIT_FOR_NEXT )->getState().isFinishAnimation;;

				// ŃURԂ
				// ܂ł̑҂
				if( lpEnemy->getCounter(AryonaCounterType::WAIT_FOR_NEXT )->step().isFinishAnimation ) {
					// end of waiting for next

					if( bIsWaitingYet ) {
						// ̃t[ŏI


						// ړ
						int nIndex = HPLMath::nextInt( 0, lpMat->getRowSize() );	// ړ惊Xg̃CfbNX
						HPLVector vNextCenter;	// ̈ʒu

						// w肵CfbNX̏ꏊɈړ
						this->getEnemyCoordGoalPos( vNextCenter, 
							lpMat->get(0, nIndex ), lpMat->get( 1, nIndex ), nTileSize );

						// ړ
						lpEnemy->setCenter( vNextCenter );

						// effect 
						getStaticData()->m_ObjectManager.m_EffectManager.create( Sequence::Effect::WARP, vNextCenter, true );
					}

					// A[iʒuɂĈقȂ
					int nX = lpEnemy->m_nSummonPositionType;

					// ʒ荶AËʒu擾
					HPLVector vNextCenter( nBlockWidth / 2 + nX * nBlockWidth / 4.f, nBlockHeight / 2 );

					switch( lpEnemy->getAttackType() ) {//lpEnemy->getCounter( AryonaCounterType::ALL_ATTACK_PATTERN )->getStepCounter() ) {
					case AryonaAttackType::Wind::SUMMON_SHABON:	// ڂ񏢊
						switch( st ) {
						case StateType::ATTACK:
							if( lpEnemy->getCounter( AryonaCounterType::SUMMON_TIME )->step().isFinishAnimation ) {	// ^CI
								nNextAttackType = AryonaAttackType::Wind::SUMMON_SHABON;
							//	lpEnemy->getCounter( AryonaCounterType::ALL_ATTACK_PATTERN )->step();
							} else {	// p
								if( lpEnemy->getCounter( AryonaCounterType::SUMMON_INTERVAL )->step().isFinishAnimation ) {
									lpEnemy->getCounter( AryonaCounterType::SUMMON_INTERVAL )->reset();
									// create shabon
									GShot* lpShot = getStaticData()->m_ObjectManager.m_ShotManager.create(
										Sequence::Shot::SHABON,
										vNextCenter, lpEnemy->getID(), lpEnemy->getFaceDirection(),
										false, true);
									// _()
									int nDeg = HPLMath::nextInt( 200, 340 );
									HPLVector v;
									v.setByDegree( nDeg );
									v.multiple( SHABON_SPEED );
									lpShot->setVelocityAngle( v );
								}

								if( lpEnemy->getCounter( AryonaCounterType::SUMMON_TIME )->getStepCounter() ==
									lpEnemy->getCounter( AryonaCounterType::SUMMON_TIME )->getWaitFrameNum() / 2 )
								{
									HPLVector v = vNextCenter;
									v.add( nX * 120,0 );
									// Fan-bird쐬
									GEnemy* lpAdd = this->createWithoutAdd(
										EnemyType::FAN_BIRD, v, -1 );
									this->m_vecAdds.push_back( lpAdd );
									lpAdd->m_vCenterPosition = v;
									lpAdd->m_vAmplitudeCenter = v;

									// GtFNg
									getStaticData()->createEffect( Sequence::Effect::WARP,
										vNextCenter, vNextCenter, -1, -1 );
								}
							}
							break;
						default:
							lpEnemy->changeState( StateType::ATTACK );
						}
						break;
					case AryonaAttackType::Wind::SUMMON_SUN:
						switch( st ) {
						case StateType::ATTACK:
							{
								if( lpEnemy->getCounter( AryonaCounterType::SUMMON_TIME_2 )->step().isFinishAnimation ) {	// ^CI
									nNextAttackType = AryonaAttackType::Wind::SUMMON_SHABON;
									//lpEnemy->getCounter( AryonaCounterType::ALL_ATTACK_PATTERN )->step();
								} else {	// p
									if( lpEnemy->getCounter( AryonaCounterType::SUMMON_INTERVAL_2 )->step().isFinishAnimation ) {	// summon interval!
										// reset
										lpEnemy->getCounter( AryonaCounterType::SUMMON_INTERVAL_2 )->reset();

										// create sun
										GEnemy* lpAdd = this->createWithoutAdd( EnemyType::SUN, vNextCenter, -1 );
										
										// vC[ƔΑɈU
										HPLVector v = vSub;
										v.normalize();
										const int SPEED = 8;
										v.multiple( -1 * SPEED );
										lpAdd->setVelocityAngle( v );
										this->m_vecAdds.push_back( lpAdd ) ;

										// GtFNg
										getStaticData()->createEffect( Sequence::Effect::WARP,
											vNextCenter, vNextCenter, -1, -1 );
									}

								}
							}
							break;
						default:
							lpEnemy->changeState( StateType::ATTACK );
						}
					}
				}
			}

			if( nNextAttackType >= 0 ) { 
				int RESET_COUNTERS[] = {
					AryonaCounterType::SUMMON_INTERVAL,
					AryonaCounterType::SUMMON_INTERVAL_2,
					AryonaCounterType::SUMMON_TIME,
					AryonaCounterType::SUMMON_TIME_2,
					AryonaCounterType::WAIT_FOR_NEXT
				};
				for( int i = 0; i < SIZEOF( RESET_COUNTERS ); i ++ ) {
					lpEnemy->getCounter( RESET_COUNTERS[i])->reset();
				}
				lpEnemy->setAttackType( nNextAttackType );
				lpEnemy->changeState( StateType::STAND );

				// vC[̈ʒuɂāASummonʒu肷
				lpEnemy->m_nSummonPositionType = vTargetCenter.getX() < nBlockWidth / 2 ? 1:-1;
			}
/*			if( lpEnemy->getCounter(AryonaCounterType::ALL_ATTACK_PATTERN )->getState().isFinishAnimation ) {
				// 1T

				// ŏɖ߂
				lpEnemy->getCounter(AryonaCounterType::ALL_ATTACK_PATTERN )->reset();
			}*/
		}
		break;
	}

	if( bIsFaceToPlayer) {
		this->faceToTarget(lpEnemy );
	}

	if( bIsMoveToPlayer ) {
		this->moveToTarget(lpEnemy, 1, true);
	}

}
