#include "GEnemyManager.h"

#include "GStaticData.h"
#include <hpllib/math/HPLMath.h>
#include <hpllib/action/HPLActionMoveTool.h>
#include <hpllib/system/HPLFile.h>
#include <hpllib/general/HPLString.h>

using namespace std;
using namespace GEnum;

static const float AIKO_CHASE_ACCEL = 0.2f;
static const float AIKO_CHASE_MAX_SPEED = 4;

void GEnemyManager::doGhostPairAlgorithm( GEnemy* lpEnemy ) {
	int t = lpEnemy->getType();
	
	HPLShootingObject* lpTarget = lpEnemy->getTarget();
	HPLVector vTargetCenter = lpTarget->getCenter();
	
	HPLVector vSub(
		vTargetCenter.getX() - lpEnemy->getCenter().getX(),
		vTargetCenter.getY() - lpEnemy->getCenter().getY() );


	float fLevel = getStaticData()->m_Game.getLevel( LEVEL_EPSILON );

	float fLength = vSub.getLength();

	int st = lpEnemy->getState()->getState();
	float RESIST = getStaticData()->m_Setting.resist;
	float GRAVITY = getStaticData()->m_Setting.gravity;

	// chasing
	HPLVector vGoal;
	bool bIsChaser = false;
	float fAccel = AIKO_CHASE_ACCEL;//0.2f;
	float fMaxSpeed = AIKO_CHASE_MAX_SPEED;//4;

	// Aj[V
	HPLSequence* lpSeq = lpEnemy->getSequence();
	int nNext = -1;

	bool bIsFinishAnimation = lpSeq->getState().isFinishAnimation;

	HPLAttackInformation* lpInfo = lpEnemy->m_lpAttackInformation;
	if( lpInfo->m_WaitForStartCounter.step().isFinishAnimation ) {
		if( lpInfo->m_WaitForNextCounter.step().isFinishAnimation ) {

			bool bIsShot = false;
			switch( lpEnemy->getAttackType()) {
			case GhostPairAttackType::ICE_SPIKE:
				switch( st ) {
				case StateType::ATTACK:
					if( bIsFinishAnimation ) {	lpEnemy->changeState( StateType::ATTACK_2 );	}
					break;
				case StateType::ATTACK_2:
					bIsShot = true;
					break;
				default:
					lpEnemy->changeState( StateType::ATTACK );
				}
				break;
			case GhostPairAttackType::ICE_SPIKE_WINDER:
				switch( st ) {
				case StateType::ATTACK:
					if( bIsFinishAnimation ) {	lpEnemy->changeState( StateType::ATTACK_2 );	}
					break;
				case StateType::ATTACK_2:
					bIsShot = true;
					break;
				default:
					lpEnemy->changeState( StateType::ATTACK );
				}
				break;
			case GhostPairAttackType::ICICLE:
				switch( st ) {
				case StateType::ATTACK:
					if( bIsFinishAnimation ) {	lpEnemy->changeState( StateType::ATTACK_2 );	}
					break;
				case StateType::ATTACK_2:
					bIsShot = true;
					break;
				default:
					lpEnemy->changeState( StateType::ATTACK );
				}
				break;
			case GhostPairAttackType::HOMING:
				switch( st ) {
				case StateType::ATTACK:
					if( bIsFinishAnimation ) {	lpEnemy->changeState( StateType::ATTACK_2 );	}
					break;
				case StateType::ATTACK_2:
					bIsShot = true;
					break;
				default:
					lpEnemy->changeState( StateType::ATTACK );
				}
				break;
			case GhostPairAttackType::MIRROR:
				switch( st ) {
				case StateType::ATTACK:
					if( bIsFinishAnimation ) {	lpEnemy->changeState( StateType::ATTACK_2 );	}
					break;
				case StateType::ATTACK_2:
					bIsShot = true;
					break;
				default:
					lpEnemy->changeState( StateType::ATTACK );
				}
				break;
			case GhostPairAttackType::PON:
				switch( st ) {
				case StateType::ATTACK:
					if( bIsFinishAnimation ) {	lpEnemy->changeState( StateType::ATTACK_2 );	}
					break;
				case StateType::ATTACK_2:
					bIsShot = true;
					break;
				default:
					lpEnemy->changeState( StateType::ATTACK );
				}
				break;
			case GhostPairAttackType::SANDWITCH:
				switch( st ) {
				case StateType::ATTACK:
					if( bIsFinishAnimation ) {	lpEnemy->changeState( StateType::ATTACK_2 );	}
					break;
				case StateType::ATTACK_2:
					bIsShot = true;
					break;
				default:
					lpEnemy->changeState( StateType::ATTACK );
				}
				break;
			case GhostPairAttackType::SUMMON_ELEMENT:
				switch( st ) {
				case StateType::ATTACK:
					if( bIsFinishAnimation ) {	lpEnemy->changeState( StateType::ATTACK_2 );	}
					break;
				case StateType::ATTACK_2:
					bIsShot = true;
					break;
				default:
					lpEnemy->changeState( StateType::ATTACK );
				}
				break;
			case GhostPairAttackType::ICE:
				switch( st ) {
				case StateType::ATTACK:
					if( bIsFinishAnimation ) {	lpEnemy->changeState( StateType::ATTACK_2 );	}
					break;
				case StateType::ATTACK_2:
					bIsShot = true;
					break;
				default:
					lpEnemy->changeState( StateType::ATTACK );
				}
				break;
			}
			if( bIsShot ) {
				HPLAttackItem* lpItem = lpInfo->getAttackItem( lpEnemy->getAttackType ());

				bool bIsFinishOneSet = false;	//! 1ZbgIH
				int nNextAttackType = -1;
				if( lpItem->m_ShotNumCounter.isAlive() ) {	// 1Zbg{

					if( lpItem->m_IntervalCounter.step().isFinishAnimation ) {	// finish interval

						switch( lpEnemy->getAttackType() ) {
						case GhostPairAttackType::HOMING:
						case GhostPairAttackType::ICE:
						case GhostPairAttackType::ICE_SPIKE:
						case GhostPairAttackType::ICE_SPIKE_WINDER:
						case GhostPairAttackType::MIRROR:
						case GhostPairAttackType::PON:
							// shot!
							getStaticData()->m_ShotFactory.shot( lpEnemy,
								lpEnemy->getAttackType(), 0, lpItem->m_ShotNumCounter.getStepCounter());
							break;
						case GhostPairAttackType::ICICLE:
						case GhostPairAttackType::SANDWITCH:
						case GhostPairAttackType::SUMMON_ELEMENT:
							break;
						}

						lpItem->m_IntervalCounter.reset();
						if( lpItem->m_ShotNumCounter.step().isFinishAnimation ) {
							bIsFinishOneSet = true;
						}
					}
				} else {	// 1ZbgI
					bIsFinishOneSet = true;
				}
				if( bIsFinishOneSet ) {	// after end of one set
					if( lpItem->m_SetIntervalCounter.step().isFinishAnimation ) {	// finish interval between sets
						lpItem->m_IntervalCounter.reset();
						lpItem->m_SetIntervalCounter.reset();
						lpItem->m_ShotNumCounter.reset();
						if( lpItem->m_ShotNumCounter.step().isFinishAnimation ) {	// finish all sets
							lpItem->m_ShotNumCounter.reset();

							// next attacks
							// shot!
							switch( lpEnemy->getAttackType() ) {
							case GhostPairAttackType::HOMING:
								nNextAttackType = GhostPairAttackType::ICE_SPIKE;
								break;
							case GhostPairAttackType::ICE_SPIKE:
								nNextAttackType = GhostPairAttackType::ICE_SPIKE_WINDER;
								break;
							case GhostPairAttackType::ICE_SPIKE_WINDER:
								nNextAttackType = GhostPairAttackType::ICICLE;
								break;
							case GhostPairAttackType::ICICLE:
								nNextAttackType = GhostPairAttackType::MIRROR;
								break;
							case GhostPairAttackType::MIRROR:
								nNextAttackType = GhostPairAttackType::PON;
								break;
							case GhostPairAttackType::PON:
								nNextAttackType = GhostPairAttackType::SANDWITCH;
								break;
							case GhostPairAttackType::SANDWITCH:
								nNextAttackType = GhostPairAttackType::SUMMON_ELEMENT;
								break;
							case GhostPairAttackType::SUMMON_ELEMENT:
								nNextAttackType = GhostPairAttackType::ICE;
								break;
							case GhostPairAttackType::ICE:
								nNextAttackType = GhostPairAttackType::HOMING;
								break;
							}
						}
					}
				}	// is finish one set
				if( nNextAttackType >= 0 ) {
					lpInfo->m_WaitForNextCounter.reset();
					lpEnemy->setAttackType( nNextAttackType );
				}
			}
		} else {	// now waiting for next
			lpEnemy->changeState( StateType::STAND );
		}

	}

}
