#include "GEffect.h"

#include "GStaticData.h"
#include <hpllib/action/HPLCollisionCheckTool.h>
#include <hpllib/action/HPLActionMoveTool.h>
#include <hpllib/math/HPLMath.h>
#include <hpllib/general/HPLString.h>

using namespace std;
using namespace GEnum;

GEffect::GEffect(int t): HPLShootingObject(t){

	setObjectType(ObjectType::EFFECT);	// object type
	
	this->m_lpvTargetDelta = NULL;	// target delta
	
	// color
	this->m_lpColor = NULL;	// color
	
	// number
	this->m_nNumber = 0;

	this->m_lpIntervalCounter = NULL;

	this->m_nGenerateEffectType = -1;
	
	this->setup( t );
}
void GEffect::setup(int t ) {

	this->setType(t);	// type
	HPLSprite* lpSprite = new HPLSprite();	// sprite

	

	// add sequence
	lpSprite->setSequence( t, getStaticData()->getSprite( SpriteType::EFFECT )->getPair(t )->clone() );

	if( Sequence::Effect::EI_START <= t && t < Sequence::Effect::EI_MAX_NUM ) {
		// \GtFNgł
		for( int i = Sequence::Effect::EI_START; i < Sequence::Effect::EI_MAX_NUM; i ++ ) {
			if( t == i ) {	// equals
				continue;
			}
			switch( i ) {
			case Sequence::Effect::GENERATOR:
			case Sequence::Effect::BACK_CLOUD:
			case Sequence::Effect::BU_CLOUD:
				break;
			}
			lpSprite->setSequence( i, getStaticData()->getSprite( SpriteType::EFFECT )->getPair(i )->clone() );
		}
	}
	this->setSprite( lpSprite );
//	this->setSprite(getStaticData()->getSprite(SpriteType::EFFECT)->clone());

//	this->m_aryObjectAttribute[ ObjectAttribute::HIT_BLOCK_TO_DIE ] = false;
//	this->m_aryObjectAttribute[ ObjectAttribute::HIT_TO_BLOCK ] = false;

//	this->m_aryObjectAttribute[ObjectAttribute::GRAVITY ] = false;

//	this->setSize( 96, 96 );

	// get src rect
	HPLRect rSrcRect = this->getSequence( )->getImageInformation( 0 )->srcRect;
	// size
	this->setSize( rSrcRect.getWidth(), rSrcRect.getHeight() );
	// hit rect (default)
	this->setHitRect( 0,0,getWidth(),getHeight() );

	// for c
	this->m_nSpriteType = -1;
	this->m_nSequenceID = -1;
	this->m_nFrameIndex = -1;

	// creating effect id after dead
	this->m_nCreateEffectTypeAfterDead = -1;

	this->m_bIsFadeOut = false;		// tF[hAEgH	ftHgFXɏȂ
	this->m_bIsBack = false;		// OiH	ftHgFOi

	this->m_fScale = 1;
	
	switch( t ) {
	case Sequence::Effect::HIT:
		// do nothing
		break;
	case Sequence::Effect::WIND:
		this->setColorAsRandom();
	case Sequence::Effect::WIND_RAIL:
//		this->setSize( 2,2);
		this->setHitRect( 0,0,2,2);
		this->setSize(2,2);

		break;
	case Sequence::Effect::BLOOD_TINY:
		this->setObjectAttribute( ObjectAttribute::GRAVITY, true );
//		this->m_aryObjectAttribute[ObjectAttribute::GRAVITY ] = true;
		this->m_bIsFadeOut = true;
		break;
	case Sequence::Effect::BLOOD_SMALL:
		this->setObjectAttribute( ObjectAttribute::GRAVITY, true );
//		this->m_aryObjectAttribute[ObjectAttribute::GRAVITY ] = true;
		this->m_bIsFadeOut = true;
		break;
	case Sequence::Effect::BLOOD_BIG:
		{
			this->m_bIsFadeOut = true;
			this->setObjectAttribute( ObjectAttribute::HIT_TO_BLOCK, true );
			this->setObjectAttribute( ObjectAttribute::GRAVITY, true );
//			this->m_aryObjectAttribute[ ObjectAttribute::HIT_TO_BLOCK ] = true;
//			this->m_aryObjectAttribute[ObjectAttribute::GRAVITY ] = true;

	/*		int nW = 4;
			if( t == Sequence::Effect::BLOOD_SMALL ) {
				nW = 8;
			} else if( t == Sequence::Effect::BLOOD_BIG ) {
				nW = 16;
			}
			this->setSize(nW,nW);*/

			this->setHitRect( (float)(this->getWidth() / 2 - 1),
				(float)(this->getHeight() / 2 - 1), 2, 2 );

			
		}
		break;
	case Sequence::Effect::LADER:
		break;
	case Sequence::Effect::TARGETTING:
		this->m_bIsFadeOut = true;
		break;
	case Sequence::Effect::CURE:
		{
			this->m_bIsFadeOut = true;
			// xۑ
			this->setVelocity( static_cast<float>( -HPLMath::nextFloat( CURE_FLOW_UP_SPEED_MIN, CURE_FLOW_UP_SPEED_MAX ) ) );
		}
		break;
	case Sequence::Effect::CURE_STAR:

		{
			this->m_bIsBack = true;
			float fVelocity = HPLMath::nextFloat( CURE_STAR_SIZE_RATE_MIN, CURE_STAR_SIZE_RATE_MAX );
			// x=傫
			this->setVelocity( fVelocity );

			// x
			this->setVelocityAngle( -1 * fVelocity, 0 );
		}
		break;
	case Sequence::Effect::CURE_CHARGE:
		{
			this->m_bIsFadeOut = true;	// fade out!
		}
		break;
	case Sequence::Effect::MISSILE_CONTRAIL:
		break;
	case Sequence::Effect::EXPLOSION_SMOKE:
		{
			this->m_bIsFadeOut = true;
			// xݒ
//			this->setVelocity( -HPLMath::nextFloat( EXPLOSION_SMOKE_SPEED_MIN, EXPLOSION_SMOKE_SPEED_MAX ) );
		}
		break;
	case Sequence::Effect::ZANZO:
		this->m_bIsFadeOut = true;
		this->m_bIsBack = true;
		break;

	case Sequence::Effect::SLIME_CHIP:
	case Sequence::Effect::SLIME_CHIP_2:
		this->setObjectAttribute( ObjectAttribute::GRAVITY, true );
//		this->m_aryObjectAttribute[ObjectAttribute::GRAVITY ] = true;
		this->m_bIsFadeOut = true;
		break;
	case Sequence::Effect::VACUUM:
		this->m_bIsFadeOut = true;
		break;
	case Sequence::Effect::ROCK:
	case Sequence::Effect::WOOD:
	case Sequence::Effect::LEAF:
		this->setObjectAttribute( ObjectAttribute::GRAVITY, true );
//		this->m_aryObjectAttribute[ObjectAttribute::GRAVITY ] = true;
		this->m_bIsFadeOut = true;
		break;
	case Sequence::Effect::WARP:	// warp in effect
		this->m_bIsFadeOut = true;
		break;
	case Sequence::Effect::ICE_CONTRAIL:
		break;
	case Sequence::Effect::NUMBER:
		{
			this->setSize( getStaticData()->m_ObjectManager.m_EffectManager.NUM_WIDTH,
				getStaticData()->m_ObjectManager.m_EffectManager.NUM_HEIGHT );
		}
		break;
	case Sequence::Effect::CLOUD:
		this->m_bIsFadeOut = true;
		break;
	case Sequence::Effect::BU_CLOUD:
		{
			this->m_bIsBack = true;
			// hit to shot false
			// hit rect
			// mount type none
			// invinsible true
			// gravity false

			// 
			int BU_SPEED_X = 5;
			int BU_SPEED_Y = 10;
			this->setVelocityAngle( -BU_SPEED_X, -BU_SPEED_Y );
//			this->setObjectAttribute( ObjectAttribute::HIT_TO_BLOCK, false );
		}
		break;
	case Sequence::Effect::BACK_CLOUD:
		{
			this->m_bIsBack = true;
			this->setVelocityAngle( -5,0);
		}
		break;

	case Sequence::Effect::GENERATOR:
		{
			this->m_lpIntervalCounter = new HPLCounter();
		}
		break;
	case Sequence::Effect::LAVA:
	case Sequence::Effect::LAVA_UNDER:
		{
			this->setObjectAttribute( ObjectAttribute::HIT_TO_PLAYER, true );
			HPLImageInformation* lpImage = getStaticData()->getSequence( 
				SpriteType::EFFECT, Sequence::Effect::LAVA_UNDER )->getImageInformation(0);
			this->m_lpIntervalCounter = new HPLCounter(lpImage->srcRect.getHeight() / LAVA_UP_SPEED);
		}
		break;
	case Sequence::Effect::HEAD_SHOT:
		{
			this->m_bIsFadeOut = true;
		}
		break;
	default:
/*
#ifdef _DEBUG
		// emotion
		HPLError::assertIt( false,
			"R" );
#endif
			*/
		break;
	}




/*	if( t == Sequence::Effect::HIT ) {

	} else if( t == Sequence::Effect::CONSTRUCT ) {
		setSize(64,64);
		setHitRect(2,19,59,32);

		this->m_bIsHitToBlock = true;
	} else if( t == Sequence::Effect::ABESHI ) {
		setSize(160,160);
		setHitRect(0,0,160,160);
	}*/

//	this->m_nCreateEnemyType = 0;
//	this->m_nParentForceID = -1;
}
GEffect::~GEffect(){
	if( NULL != this->m_lpColor ) {
		delete this->m_lpColor;
		this->m_lpColor = NULL;
	}
	if( NULL != this->m_lpIntervalCounter ) {
		delete this->m_lpIntervalCounter;
		this->m_lpIntervalCounter = NULL;
	}
}


void GEffect::die(HPLShootingObject* damageSource){
	this->setAlive( false );

}

void GEffect::damage(float dmg, HPLShootingObject* damageSource){
}

/***************************************************************************************
  * `
  */
void GEffect::draw() {

	int t = this->getType();

	// XN[
	HPLScroller* scroller = getStaticData()->getScrollManager()->getScroller(0);

	// V[PXɕϊ
	HPLSequence* seq = this->getSequence();
	
	if(seq->getLoopNumMax() > 0 && seq->getState().isFinishAnimation){
		//////
		// Aj[VIŎS
		// AAj[VȂݒ̂̂͂̂܂
		this->die( NULL );

	}

	float left = this->getArea().getX() + scroller->getVirtualRect().getX();
	float top = this->getArea().getY() + scroller->getVirtualRect().getY();
	float rotateAngle = 0;
	int alpha = 255;

	float fScrollX = scroller->getVirtualRect().getX();
	float fScrollY = scroller->getVirtualRect().getY();
	float fCenterX = this->getCenter().getX() + fScrollX;
	float fCenterY = this->getCenter().getY() + fScrollY;
	float fZoom = scroller->m_fZoomRate;



	fCenterX *= fZoom;
	fCenterY *= fZoom;

	// zoom ύX
	if( t == Sequence::Effect::CURE_STAR ) {
		// قǋ߂傫
		fZoom *= this->getVelocity();
	} else if( t == Sequence::Effect::ZANZO ) {
		// XP[ŃTCYύX
		fZoom *= this->m_fScale;
	}

	if( t == Sequence::Effect::WIND ||
		t == Sequence::Effect::WIND_RAIL)
	{
#ifdef _DEBUG
		ASSERT_IT_NULL( this->m_lpColor );
//		HPLError::assertIt( this->m_lpColor != NULL );
#endif
		// GtFNg
		DrawFillBox(
			static_cast<int>( left ),
			static_cast<int>( top ),
			static_cast<int>( left + 2 ),
			static_cast<int>( top + 2 ), 
			GetColor(this->m_lpColor->r, this->m_lpColor->g, this->m_lpColor->b));
		
		/*SetDrawBright( this->m_WindColor.r, this->m_WindColor.g, this->m_WindColor.b);
		getStaticData()->m_Basic.m_Graphics.draw(
			static_cast<int>( fCenterX ), static_cast<int>( fCenterY ), seq,
			this->getFaceDirection(), rotateAngle, fZoom, alpha, alpha < 255 ? DX_BLENDMODE_ALPHA : DX_BLENDMODE_NOBLEND );
		SetDrawBright(255,255,255);*/
	} else if( t == Sequence::Effect::NUMBER ) {
/*		if( this->m_nSpriteType < 0 ) {
			return;
		}*/

#ifdef _DEBUG
		ASSERT_IT_NULL( this->m_lpColor );
//		HPLError::assertIt( this->m_lpColor != NULL );
#endif
		// \
		int nImageHandle = getStaticData()->getImageHandle(SpriteType::EFFECT,
			Sequence::Effect::NUMBER );
		SetDrawBright( this->m_lpColor->r, this->m_lpColor->g, this->m_lpColor->b);
		for( int i = 0; i < (int)this->m_vecNumberList.size(); i ++ ) {
			int nNum = this->m_vecNumberList[i];
			if( nNum < 0 ) {
				continue;
			}
			DrawRectGraph(fCenterX + i * 
				getStaticData()->m_ObjectManager.m_EffectManager.NUM_WIDTH, fCenterY,
				nNum * getStaticData()->m_ObjectManager.m_EffectManager.NUM_WIDTH,
				0,
				getStaticData()->m_ObjectManager.m_EffectManager.NUM_WIDTH,
				getStaticData()->m_ObjectManager.m_EffectManager.NUM_HEIGHT,
				nImageHandle, TRUE, FALSE );
		}
		SetDrawBright(255,255,255);

		// DrawStringł\Ă݂
#if 0//#ifdef _DEBUG
		DrawString( fCenterX + 100, fCenterY,
			HPLString::chrToStr( "%d", this->m_nNumber ).c_str(),
			GetColor(this->m_lpColor->r, this->m_lpColor->g, this->m_lpColor->b) );
#endif
		
	} else {
		bool bIsLast = seq->getFrameIndex() == seq->getNumberOfFrames() - 1;	//! Ō̃t[

		int nStepCounter = seq->getStepCounter();					// ݂̃Xebv
		int nMaxCounter = seq->getImageInformation()->waitFrame;	// t[
		float fRate = (float)nStepCounter / (float)nMaxCounter;

		if( this->m_bIsFadeOut) {
			if( t == Sequence::Effect::BLOOD_TINY ||
				t == Sequence::Effect::BLOOD_SMALL ||
				t == Sequence::Effect::BLOOD_BIG )
			{
				int nFadeOutStartStep = (int)(nMaxCounter * 4.f / 5.f);		// tF[hAEgJn鎞
				if( nStepCounter >= nFadeOutStartStep ) {
					alpha = 255 - static_cast<int>( 255.f * static_cast<float>(nStepCounter - nFadeOutStartStep) /
						static_cast<float>(nMaxCounter - nFadeOutStartStep) );
				}
			} else {
				// isxɍ킹ĕω
				alpha = (int)(255.f * (1-fRate) );
			}
		}

		bool bIsSpecialDraw = false;

		bool bIsChangedColor = false;
		int nR = 255;
		int nG = 255;
		int nB = 255;
		bool bIsBlackOut = false;
		bool bIsZoomOut = false;
		switch( t ) {
		case Sequence::Effect::ZANZO:
			{
				bIsChangedColor = true;
				int nBright = (int)(255.f * (1-fRate) * .5f );
				nR = this->m_lpColor->r;
				nG = this->m_lpColor->g;
				nB = this->m_lpColor->b;
			}
			break;
		case Sequence::Effect::SLIME_CHIP:
		case Sequence::Effect::SLIME_CHIP_2:
			{
				bIsChangedColor = true;
				nR = this->m_lpColor->r;
				nG = this->m_lpColor->g;
				nB = this->m_lpColor->b;
			}
			break;
		case Sequence::Effect::BACK_CLOUD:
		case Sequence::Effect::BU_CLOUD:
			{
				bIsChangedColor = true;
				bIsZoomOut = true;
				bIsBlackOut = true;
				alpha = 250;
			}
			break;
		case Sequence::Effect::LAVA_UNDER:
			{
				if( this->m_lpIntervalCounter->isAlive() ) {
					// 犄̕
					HPLRect rArea = this->getArea();
					float fRate = 1 - this->m_lpIntervalCounter->getStepCounterRate();
					int nMainTop = seq->getImageInformation()->srcRect.getHeight() * fRate;
					int nGraphHandle = getStaticData()->getImageHandle(
						SpriteType::EFFECT, Sequence::Effect::LAVA_UNDER );
					DrawRectGraph( rArea.getX() + fScrollX, nMainTop + rArea.getY() + fScrollY,
						0,nMainTop, rArea.getWidth(), rArea.getHeight(), nGraphHandle, TRUE,
						this->getFaceDirection() > 0 ? TRUE : FALSE );
					bIsSpecialDraw = true;
				}

				// alpha blend
				alpha = 230;
			}
			break;
		} 

		float fThreshold = 0.1f;
		float fFrameRate = seq->getStepCounter() / (float)seq->getImageInformation()->waitFrame;
		if( bIsZoomOut) {
			fZoom = 1 - fFrameRate;
		}
		if( bIsBlackOut ) {
			nR = (int)(255 * (1.f-fFrameRate));
			nG = (int)(255.f * (1.f-fFrameRate));
			nB = (int)(100.f * (1.f-fFrameRate))+155;
		}
		if( bIsChangedColor ) {
			SetDrawBright( nR, nG, nB );
		}
		if( this->m_nSpriteType >= 0 ) {
			if( alpha < 255 ) {
				SetDrawBlendMode( DX_BLENDMODE_ALPHA, alpha );
			}
			// ֕\
			DrawRotaGraph( (int)fCenterX, (int)fCenterY, fZoom, rotateAngle, 
				getStaticData()->getImageHandle(
				this->m_nSpriteType, this->m_nSequenceID, this->m_nFrameIndex ),
				TRUE, this->getFaceDirection() == FaceDirection::RIGHT );
			if( alpha < 255 ) {
				SetDrawBlendMode( DX_BLENDMODE_NOBLEND, 255 );
			}
		} else if( bIsSpecialDraw == false ){
			getStaticData()->m_Basic.m_Graphics.draw(
				static_cast<int>( fCenterX ), static_cast<int>( fCenterY ), seq,
				this->getFaceDirection(), rotateAngle, fZoom, alpha, alpha < 255 ? DX_BLENDMODE_ALPHA : DX_BLENDMODE_NOBLEND );
		}
		if( bIsChangedColor ) {
			SetDrawBright( 255, 255, 255 );
		}
		bool bIsDieAfterAnimation = false;
		if( bIsBlackOut ) {	// black out effect
			bIsDieAfterAnimation = true;
		}
		if( bIsDieAfterAnimation &&	// die after animation finished
			( ( seq->getState().isFinishAnimation ) ||	// finish animation
			(bIsZoomOut && fZoom <= fThreshold)))	// too small zoom
		{
			// 
			this->die(NULL );
		}
	}
		// Aj[VXebv
		// 
	if(getStaticData()->m_Game.isTheWorld() == false){
		seq->step();

		if( seq->getState().isEndLoop ) {
			// [v؂

			if( this->getType() == Sequence::Effect::WIND ) {
				// Fς
				this->setColorAsRandom();
			}
		}
	}


/*	if(this->getType() == Sequence::Effect::CONSTRUCT)
	{
		// [v
		// ->v\

		
		// ݂̃[v
		int loop = seq->getLoopNum();

		// S[v
		int maxLoop = seq->getLoopNumMax();

		// 
		float r = (float)loop / (float)maxLoop;

		// vt[̍
		HPLSequence* frame = getStaticData()->getSequence(GEnum::SpriteType::COMMON,
			GEnum::Common::WATCH_FRAME);
		float frameHeight = frame->getImageInformation(0)->srcRect.getHeight();

		// ̍
		HPLSequence* sand =  getStaticData()->getSequence(GEnum::SpriteType::COMMON,
			GEnum::Common::WATCH_SAND);
		float sandHeight = sand->getImageInformation(0)->srcRect.getHeight();

		// \ʒuݒ
		float frameX = left;
		float frameY = top;

		// ̕\ݒ
		float sandSrcY = (frameHeight - sandHeight) * r;
		sand->getImageInformation(0)->srcRect.setY(sandSrcY);

		// ̕\ʒuݒ
		float sandX = frameX;
		float sandY = frameY + (frameHeight - sandHeight) * r;

		// t[\
		int alpha = 200;

		if( SetDrawBlendMode( DX_BLENDMODE_ALPHA, alpha ) != 0 ) {
		}
		getStaticData()->m_Basic.m_Graphics.draw(
			fCenterX, fCenterY, frame,
			this->getFaceDirection(), rotateAngle, fZoom );
		if( SetDrawBlendMode( DX_BLENDMODE_ALPHA, alpha * (1.0-r) ) != 0 ) {
		}
		getStaticData()->m_Basic.m_Graphics.draw(
			fCenterX, fCenterY, sand,
			this->getFaceDirection(), rotateAngle, fZoom );
		if( SetDrawBlendMode( DX_BLENDMODE_NOBLEND, 255 ) != 0 ) {
		}
	}*/
}


HPLSequence* GEffect::getSequence(){
	int seqID = getType();
	HPLSequence* seq = 
		this->getSprite()->getPair(seqID)->left;
	return seq;

}

/** F_ɕύX
  */
void GEffect::setColorAsRandom() {
//	m_nRed = rand()%255;
//	m_nGreen = rand()%255;
//	m_nBlue = rand()%255;

	this->checkAndAllocColor();
	this->m_lpColor->set(HPLMath::nextInt(0,255),HPLMath::nextInt(0,255),HPLMath::nextInt(0,255));
//	this->m_nWindColor = GetColor(HPLMath::nextInt(0,255),HPLMath::nextInt(0,255),HPLMath::nextInt(0,255));
}


/** lݒ肷
  * @param[in]	bIsZero		]0Ŗ߂邩
  * @param[in]	nMaxPower	ő包
  */
void GEffect::setNumber( int nNumber, bool bIsZero, int nMaxPower ) {
	// int to array
	// ex. 132 -> {1,3,2}(int[])
	HPLMath::getNumArrayFromNumber( nNumber, bIsZero, nMaxPower, this->m_vecNumberList );
	this->m_nNumber = nNumber;

/*	this->m_vecNumberList.clear();

	
	///// initialize
	for( int i = 0; i < nMaxPower; i ++ ) {
		int nNum = -1;
		if( bIsZero ) {	// fill by zero
			nNum = 0;
		} else if( i == nMaxPower - 1 ) {	// set last to zero
			nNum = 0;
		}
		this->m_vecNumberList.push_back( nNum );
	}
	// from last to max power
	int nIndex = nMaxPower - 1;
	// copy
	int nTempNum = nNumber;

	while( nIndex >= 0 && nTempNum > 0 ) {	// index and copy val is valid
		this->m_vecNumberList[nIndex] = nTempNum % 10;
		nIndex --;
		nTempNum /= 10;
	}
	*/
}
void GEffect::setColor( int nCol) {
	this->m_bIsSetColor = true;
	HPLShootingObject::setColor( nCol );
}

int GEffect::getColor() {
	int nCol = HPLShootingObject::getColor();
#ifdef _DEBUG
	HPLError::assertIt( this->m_bIsSetColor, 
		"%s:%s(%d):J[ݒ肳ĂȂ[%d]",
		__FILE__,__FUNCTION__,__LINE__,nCol );
#endif
	return nCol;
}
