#include "GEffectManager.h"

#include "GStaticData.h"
#include <hpllib/action/HPLActionMoveTool.h>
#include <hpllib/math/HPLMath.h>
#include <hpllib/action/HPLShootingMove.h>

using namespace std;
using namespace GEnum;

GEffectManager::GEffectManager(): HPLObjectManager() {
}


void GEffectManager::process(bool bIsMove, bool bIsAI) {

	HPLRect merginedGameRange = getStaticData()->getMerginedGameRange();

	int nMapIndex = getStaticData()->getMapIndex();
	int nMapType = getStaticData()->getMapType(  );
	
	bool bSpecialStage = 
		getStaticData()->getMapType() == MapType::BOSS ||
		getStaticData()->getMapType() == MapType::MID_BOSS;

	// tile size
/*	int nTileSize = getStaticData()->m_Setting.tileSize;
	// tiled range
	HPLRect tiledRange(0,0,
		(float)nTileSize * getStaticData()->getBlockManager()->getBlockMatrix()->getColumnSize(),
		(float)nTileSize * getStaticData()->getBlockManager()->getBlockMatrix()->getRowSize() );
*/
	for( list<GEffect*>::iterator it = this->getList()->begin();
	it != this->getList()->end(); )
	{
		GEffect* lpEffect = *it;

		bool bIsSpecialType = false;
		switch( lpEffect->getType()) {
		case Sequence::Effect::LAVA:
		case Sequence::Effect::LAVA_UNDER:
			bIsSpecialType = true;
			break;
		case Sequence::Effect::GENERATOR:
			if( lpEffect->getGenerateEffectType() == Sequence::Effect::LAVA_UNDER) {
				bIsSpecialType = true;
			}
			break;
		case Sequence::Effect::SNOW_BALL:
			bIsSpecialType = true;
			break;
		}
		bool bIsInMerginedArea = merginedGameRange.isShare( lpEffect->getHitArea() );

		if( !bIsSpecialType && !bSpecialStage &&	// not special stage
			bIsInMerginedArea ==false)	// out of range
		{
			// dead
			lpEffect->die( NULL );
//		} else if( lpEffect->getCenter().getY() > tiledRange.getY()  ) {
			// dead
		}

		if( lpEffect->isAlive() ) {
			if( bIsMove ) {	
/*				switch( lpEffect->getType()) {
				case Sequence::Effect::LAVA:
				case Sequence::Effect::LAVA_UNDER:
					{
						HPLVector vCen = lpEffect->getCenter();
						vCen.add( lpEffect->getVelocityAngle());
						lpEffect->setCenter( vCen );
					}
					break;
				default:*/
					getStaticData()->collisionObject( lpEffect );
				//}

				this->algorithm(lpEffect);

				this->checkHitToPlayer( lpEffect );
			}
			
		}
		
		if( lpEffect->isAlive() ) {
			if( false == lpEffect->m_bIsBack && bIsInMerginedArea) {
				lpEffect->draw();
			}
			it ++;
		} else {
			if( merginedGameRange.isShare( lpEffect->getHitArea()) ) {
				int nCreateAfterDead = lpEffect->getCreateEffectTypeAfterDead();
				// in range
				if( nCreateAfterDead >= 0 ) {	// create effect after dead
					HPLVector vCenter = lpEffect->getCenter();

					GEffect* lpAdd = this->create( nCreateAfterDead, vCenter,
						false );
					this->m_vAdds.push_back( lpAdd );
				}
			}
			delete *it;
			it = this->getList()->erase(it);
		}
	}

	for( list<GEffect*>::iterator it = this->m_vAdds.begin(); it != this->m_vAdds.end(); it ++ ) {
		this->add( *it );
	}
	this->m_vAdds.clear();
}

/** 
  */
bool GEffectManager::checkHitToPlayer( GEffect* lpEffect ) {
	if( lpEffect->isAlive() == false ) {
		return false;
	}
		
	if( lpEffect->getObjectAttribute( ObjectAttribute::HIT_TO_PLAYER ) == false ) {
		return false;
	}

	// player
	GPlayer* lpPlayer = &getStaticData()->m_ObjectManager.m_Player;

	HPLRect rPlayerHitArea = lpPlayer->getHitArea();
	HPLRect rHitArea = lpEffect->getHitArea();
	int nNextState = StateType::DAMAGE;
	bool bIsHit = rHitArea.isShare( rPlayerHitArea );

	if( lpPlayer->isInvinsible() ) {	// invinsible
		if( lpEffect->getType() == Sequence::Effect::LAVA_UNDER ||
			lpEffect->getType() == Sequence::Effect::LAVA )
		{
			if( bIsHit ) {
				lpPlayer->changeState( StateType::SWIM );
			}

		}

		return false;
	}

	if( bIsHit ) {	// hit!
		int nDamageType = 0;
		float fPower = 0;
		// TODO damage type
		switch( lpEffect->getType() ) {
		case Sequence::Effect::LAVA_UNDER:
			// fire!
			nDamageType = HPLMath::setFlagByIndexInt( nDamageType, DamageType::FIRE, true );
			fPower = PLAYER_HP_MAX / 2;
			nNextState = StateType::SWIM;
			break;
		}
		
		lpPlayer->damage( fPower, lpEffect, true, nNextState );
	}
}



GEffect* GEffectManager::create(int nType, HPLVector vCenter, bool bIsAddToList) {
	GEffect* lpEffect = new GEffect( nType );
	lpEffect->setCenter( vCenter );
	if( bIsAddToList ) {
		this->getList()->push_back( lpEffect );
	}

	return lpEffect;
}


void GEffectManager::algorithm(GEffect* lpEffect) {
	int nType = lpEffect->getType();
	
	// ȓL
	bool bIsFlowUp = false;		// Xɏɏオ
	
	bool bIsChase = false;		// chaser
	float fAccel = 0.5f;		// accel
	float fMaxSpeed = 5;		// speed
	HPLVector vGoal;

	float RESIST = getStaticData()->m_Setting.resist;
	
	// ɂ݂̂u[L
	float fResistX = getStaticData()->m_Setting.resist * RESIST_IN_THE_AIR_RATE;

	switch( nType ) {
	case Sequence::Effect::CURE:
		{
			// ㏸ON
			bIsFlowUp = true;
		}
		break;
	case Sequence::Effect::ZANZO:
		{
			bIsFlowUp = true;
		}
		break;
	case Sequence::Effect::HEAD_SHOT:
		{
			bIsFlowUp = true;
		}
		break;
	case Sequence::Effect::CURE_CHARGE:
	case Sequence::Effect::CLOUD:
		{
			bIsChase = true;
		}
		break;
	case Sequence::Effect::BU_CLOUD:
		{
			HPLVector vVel = lpEffect->getVelocityAngle();
			// (Ŷ)
			vVel.setY( HPLActionMoveTool::reduceSpeed( 
				RESIST * RESIST_IN_THE_AIR_RATE / 3, vVel.getY() ) );
			lpEffect->setVelocityAngle( vVel );
		}
		break;
	case Sequence::Effect::GENERATOR:
		{
			HPLCounter* lpCounter = lpEffect->getIntervalCounter();
			int nGenerateType = lpEffect->getGenerateEffectType();
			ASSERT_IT_NULL( lpCounter );
//			HPLError::assertIt( lpCounter != NULL );
			HPLError::assertIt( nGenerateType >= 0,
				"%s:%s(%d):GenerateType is invalid[%d]", __FILE__,__FUNCTION__,__LINE__,nGenerateType);
			if( lpCounter->step().isFinishAnimation ) {
				// generate interval
				lpCounter->reset();
				
				/////// generate
				GEffect* lpNew = this->create( nGenerateType,
					lpEffect->getCenter(), false );
				this->m_vAdds.push_back( lpNew );
			}
		}
		break;
	case Sequence::Effect::LAVA:
		lpEffect->setVelocityAngle( 0,-LAVA_UP_SPEED);
		break;
	case Sequence::Effect::LAVA_UNDER:
		{
			if( lpEffect->getIntervalCounter()->step().isFinishAnimation ) {
				lpEffect->setVelocityAngle( 0,-LAVA_UP_SPEED);
			}
		}
		break;
	case Sequence::Effect::EXPLOSION_SMOKE:	//! 킩o/Vbgo鉌
		{
			// ł͑̂ŁAʂɏ㏸
			//bIsFlowUp = true;

			// x
			HPLVector vVel = lpEffect->getVelocityAngle();
			vVel.setX( HPLActionMoveTool::reduceSpeed(
				0.1f * RESIST * RESIST_IN_THE_AIR_RATE, vVel.getX() ) );

			const float EXPLOSION_SMOKE_ACCEL = 0.1f;
			const float EXPLOSION_SMOKE_MAX_SPEED = 8.f;
			// y㏸
			vVel.setY( HPLActionMoveTool::accelMove(
				vVel.getY(), -EXPLOSION_SMOKE_ACCEL, -EXPLOSION_SMOKE_MAX_SPEED ) );

			lpEffect->setVelocityAngle( vVel );
		}
		break;
	default:
		// do nothing
		break;
	}

	if( bIsChase ) {
		if( lpEffect->getTarget() != NULL ) {	// this effect has target!
			// goal is target center
			
			vGoal = lpEffect->getTarget()->getCenter();
			if( lpEffect->getTargetDelta() != NULL ) {
				vGoal.add(*lpEffect->getTargetDelta() );
			}

		} else {	// this effect has no target
			// goal is global target center TODO this is not good
			vGoal = this->m_vGlobalMoveTargetCenter;
		}
		// missile homing
		HPLShootingMove::doMissileHoming( lpEffect, vGoal, fAccel, fMaxSpeed );
	}
	if( bIsFlowUp ) {		// xɏ

		// xNg
		HPLVector vVelocityAngle = lpEffect->getVelocityAngle();

		// x
		HPLVector vAccel( 0, lpEffect->getVelocity() );
		vVelocityAngle.add( vAccel );

		
		// ͌
		vVelocityAngle.setX( HPLActionMoveTool::reduceSpeed(
			fResistX,	// 󒆂̌
			vVelocityAngle.getX())  );
		lpEffect->setVelocityAngle( vVelocityAngle );
	}
	// ړ ( collision()Ŏ{ς )
//	HPLVector vCenter = lpEffect->getCenter();
//	vCenter.add( lpEffect->getVelocityAngle() );
//	lpEffect->setCenter( vCenter );

}
void GEffectManager::drawBack() {
	for( list<GEffect*>::iterator it = this->getList()->begin();
	it != this->getList()->end(); it ++)
	{
		GEffect* lpEffect = *it;

		if( lpEffect->isAlive() ==false ) {
			continue;
		}
		if( false == lpEffect->m_bIsBack ) {
			continue;
		}

		lpEffect->draw();
	}
}


/** c𐶐
  */
GEffect* GEffectManager::createZanzo( int nSpriteType, int nSequenceID, int nFrameIndex,
	HPLShootingObject* lpObject, int nR,int nG, int nB )
{
	GEffect* lpEffect = this->create( Sequence::Effect::ZANZO,
		lpObject->getCenter(), true);

	lpEffect->m_nSpriteType = nSpriteType;
	lpEffect->m_nSequenceID = nSequenceID;
	lpEffect->m_nFrameIndex = nFrameIndex;
	lpEffect->setFaceDirection( lpObject->getFaceDirection() );
	lpEffect->setColor( nR,nG,nB );
	return lpEffect;
}

void GEffectManager::createBlood( HPLVector vBloodCenter ) {
	for( int i = 0; i < HPLMath::nextInt( BLOOD_RND_MIN, BLOOD_RND_MAX ); i ++ ) {
		HPLVector vCenter = vBloodCenter;
		// 덷
		vCenter.add( 
			static_cast<float>( HPLMath::nextInt(- BLOOD_DELTA_MAX_X, BLOOD_DELTA_MAX_X ) ),
			static_cast<float>( HPLMath::nextInt(-BLOOD_DELTA_MAX_Y,BLOOD_DELTA_MAX_Y ) + BLOOD_DELTA_Y_BASE ) );

		GEffect* lpEffect = getStaticData()->m_ObjectManager.m_EffectManager.create( 
			Sequence::Effect::BLOOD_TINY + HPLMath::nextInt(0,3), vCenter, true);

		HPLVector vVelocityAngle;
		vVelocityAngle.setByDegree( 270.f + static_cast<float>( HPLMath::nextDouble( -BLOOD_ERR_DEGREE, BLOOD_ERR_DEGREE ) ) );
		vVelocityAngle.multiple( static_cast<float>( HPLMath::nextDouble( BLOOD_SPEED_RND_MIN, BLOOD_SPEED_RND_MAX ) ) );
		lpEffect->setVelocityAngle( vVelocityAngle );
	}
}

/** GtFNg𕡐쐬܂
  * @param[in]	vRandomVector	̕Ƀ_ʒuɔ܂B
  * @param[in]	fRandomCenterMin
  * @param[in]	fRandomCenterMax
  * @param[in]	vRandomVector	̕Ƀ_ʒuɈړʂ𔭐܂
  * @param[in]	fRandomCenterMin
  * @param[in]	fRandomCenterMax
  * @retval		-1		
  * @retval		0ȏ	쐬GtFNg̐
  */
int GEffectManager::createEffects( int nType, const HPLVector& vFrom, const HPLVector& vTo,
	int nCreateEnemyType, int nForceID,
	const int nRandomPositionType,
	const float fRandomParams0,	const float fRandomParams1,
	const float fRandomParams2,	const float fRandomParams3,
	const int nRandomSpeedType,
	const float fRandomSpeedMin, const float fRandomSpeedMax,
	const float fRandomSpeed2Min, const float fRandomSpeed2Max,
	const int nNum)
{
	
	int nSucceededNum = 0;

	for( int i = 0; i < nNum; i ++ ) {
		// o^
		GEffect* lpEffect = getStaticData()->createEffect(
			nType, vFrom, vTo, nCreateEnemyType, nForceID );

		HPLVector vCenter = lpEffect->getCenter();

		HPLVector vRandomAddPosition;

		// _
		switch( nRandomPositionType ) {
		case RandomVectorType::NO_RANDOM:
			// do nothing
			break;
		case RandomVectorType::CIRCLE_RANDOM:
			{
				float fDegree = HPLMath::nextFloat( 0, HPLMathConsts::DEGREE_FULL_ROUND );
				vRandomAddPosition.setByDegree( fDegree );
				vRandomAddPosition.multiple( HPLMath::nextFloat( fRandomParams0, fRandomParams1 ) );
			}

			break;
		default:
			// error
			HPLError::assertIt( false,
				"mnRandomPositionType:%d", nRandomPositionType );
			break;
		}

		vCenter.add( vRandomAddPosition );
		lpEffect->setCenter( vCenter );

		// xݒ
		switch( nRandomSpeedType ) {
		case RandomVectorType::NO_RANDOM:
			// do nothing
			break;
		case RandomVectorType::CIRCLE_RANDOM:
			{
				HPLVector vVelocityAngle;
				float fDegree = HPLMath::nextFloat( 0, HPLMathConsts::DEGREE_FULL_ROUND );
				vVelocityAngle.setByDegree( fDegree );
				vVelocityAngle.multiple( HPLMath::nextFloat( fRandomSpeedMin, fRandomSpeedMax ) );
				lpEffect->setVelocityAngle( vVelocityAngle );
			}
			break;
		case RandomVectorType::RECTANGLE_RANDOM:
			{
				lpEffect->setVelocityAngle(HPLMath::nextFloat(fRandomSpeedMin, fRandomSpeedMax),
					HPLMath::nextFloat(fRandomSpeed2Min, fRandomSpeed2Max ) );
			}
			break;
		default:
			// error
			HPLError::assertIt( false,
				"mnRandomSpeedType:%d", nRandomSpeedType );
			break;
		}

		
		nSucceededNum ++;
	}
	return nSucceededNum;
}

int GEffectManager::createCure( HPLVector& vCenter, int nID, bool bIsOne) {
	int nNum = 1;
	if( !bIsOne ) {
		nNum = HPLMath::nextInt( CREATE_CURE_EFFECT_NUM_MIN, CREATE_CURE_EFFECT_NUM_MAX ) ;
	}
	return this->createEffects(
		Sequence::Effect::CURE, vCenter, vCenter,
		-1, nID, RandomVectorType::CIRCLE_RANDOM,
		CURE_EFFECT_CREATE_RADIUS_MIN, CURE_EFFECT_CREATE_RADIUS_MAX,
		-1,-1,
		RandomVectorType::CIRCLE_RANDOM, CURE_EFFECT_CREATE_VELOCITY_MIN, CURE_EFFECT_CREATE_VELOCITY_MAX, -1,-1,
		nNum
		);
}

/** GtFNg쐬
  */
int GEffectManager::createWind( HPLVector vCenter, int nWidth, int nHeight, float fDegree, float fSpeedRate ) {
	vCenter.add( HPLMath::nextInt(-nWidth/2, nWidth/2), HPLMath::nextInt( -nHeight/2, nHeight/2 ) );

	GEffect* lpEffect = getStaticData()->m_ObjectManager.m_EffectManager.create( Sequence::Effect::WIND, vCenter, true );

	// 
	HPLVector vVelocityAngle;
	vVelocityAngle.setByDegree( fDegree );
	vVelocityAngle.multiple( 
		static_cast<float>( WIND_SPEED_Y + HPLMath::nextDouble(-WIND_SPEED_DELTA_Y, WIND_SPEED_DELTA_Y ) ) );
	lpEffect->setVelocityAngle( vVelocityAngle );
	return 1;
}

/** 
  */
int GEffectManager::createNumber( HPLVector& vCenter, int nNum, bool bIsZero, int nMaxPower, int nR, int nG, int nB ) {
	vCenter.add( - NUM_WIDTH * nMaxPower ,0  );
	GEffect* lpEffect = this->create( Sequence::Effect::NUMBER, vCenter, true );
	lpEffect->setNumber( nNum, bIsZero, nMaxPower );
	lpEffect->setColor( nR,nG,nB);
	const float NUM_SPEED = 3;
	lpEffect->setVelocityAngle( 0, -NUM_SPEED );
	return 1;
}

/** `[W𔭐
  * @param[in]	lpTarget	^[QbgivC[ȊOȂǁA݂ȂꍇNULLj
  * @param[in]	vCenter		SW
  * @param[in]	nMinRadius/nMaxRadius
  */
int GEffectManager::createCharge( HPLShootingObject* lpTarget, HPLVector vCenter,
								 int nMinRadius, int nMaxRadius )
{
	HPLVector vOriginalCenter = vCenter;
	// diff
	HPLVector vDelta;
	// 
	vDelta.setByDegree( HPLMath::nextInt( 0, HPLMathConsts::DEGREE_FULL_ROUND ));
	// get ball seq for getting width
//	HPLSequence* lpBallSeq = getStaticData()->getSequence( SpriteType::SHOT,
//		Sequence::Shot::CHARGE_BOLT );
	// random radius
	vDelta.multiple(HPLMath::nextInt( nMinRadius, nMaxRadius));
	// move to hand position
	vCenter.add( vDelta );
	// create charge effect
	GEffect* lpEffect = getStaticData()->m_ObjectManager.m_EffectManager.create(
		Sequence::Effect::CURE_CHARGE, vCenter, true );
	if( lpTarget != NULL ) {
		lpEffect->setTarget( lpTarget);
		lpEffect->setTargetDelta(HPLVector::subtract(vOriginalCenter, lpTarget->getCenter()));
	}
	return 1;
}

int GEffectManager::createWeaponBackEffect( int nFaceDirection, const HPLVector& vCenter, int nEffectType ) {

	// ɉ𔭐
	const int MAX_SMOKE_NUM = 2;
	const int DELTA_X = 10;
	const int DELTA_Y = 5;
	const float ERR_Y = 1;
	const float SMOKE_SPEED = 5;
	for( int i = 0; i < MAX_SMOKE_NUM; i ++ ) {
		HPLVector vSmokePos( nFaceDirection * DELTA_X,
			DELTA_Y);
		vSmokePos.add( vCenter );
		HPLVector v( nFaceDirection * -1, HPLMath::nextFloat(-ERR_Y,ERR_Y ) );
		v.multiple( SMOKE_SPEED);

		GEffect* lpEffect = getStaticData()->createEffect( nEffectType,// Sequence::Effect::EXPLOSION_SMOKE,
			vSmokePos,
			vSmokePos, -1,-1);
		lpEffect->setVelocityAngle( v);
	}
	return MAX_SMOKE_NUM;
}
void GEffectManager::drawNumber( int nX, int nY, int nR, int nG, int nB, vector<int>& vecNumberList ) {
	int nImageHandle = getStaticData()->getImageHandle(SpriteType::EFFECT,
		Sequence::Effect::NUMBER );
	SetDrawBright( nR, nG, nB );//this->m_lpColor->r, this->m_lpColor->g, this->m_lpColor->b);
	for( int i = 0; i < (int)vecNumberList.size(); i ++ ) {
		int nNum = vecNumberList[i];
		if( nNum < 0 ) {
			continue;
		}
		DrawRectGraph(nX + i * 
			getStaticData()->m_ObjectManager.m_EffectManager.NUM_WIDTH,
			nY,
			nNum * getStaticData()->m_ObjectManager.m_EffectManager.NUM_WIDTH,
			0,
			getStaticData()->m_ObjectManager.m_EffectManager.NUM_WIDTH,
			getStaticData()->m_ObjectManager.m_EffectManager.NUM_HEIGHT,
			nImageHandle, TRUE, FALSE );
	}
	SetDrawBright(255,255,255);
}
