#include "GEnemyManager.h"

#include "GStaticData.h"
#include <hpllib/math/HPLMath.h>
#include <hpllib/action/HPLActionMoveTool.h>
#include <hpllib/action/HPLShootingMove.h>

#include <hpllib/system/HPLFile.h>
#include <hpllib/general/HPLString.h>

using namespace std;
using namespace GEnum;

void setMatrix( int ary[][2], HPLMatrix<int>* lpMat, int nCount ) {
	lpMat->changeSize( 2, nCount );
	for( int i = 0; i < nCount; i ++ ) {
		for( int j = 0; j < 2; j ++ ) {
			lpMat->set( j, i, ary[i][j] );
		}
	}
}

GEnemyManager::GEnemyManager(): HPLObjectManager() {
	// for parsing solid type as index number
	map<string, int> mapSolidTypeStringToIndex;

	// load solid type strings
	{
		vector<string> vecLines = HPLFile::loadStringLinesFromFile( "data/image/Enemy/SolidType.txt", true );
		int nCount = 0;
		for( int i = 0; i < (int)vecLines.size(); i ++ ) {
			mapSolidTypeStringToIndex[vecLines[i]] = nCount;
			nCount ++;
		}

	}

	using namespace EnemyCommand;
	// load settings
	{
		vector<string> vecLines = HPLFile::loadStringLinesFromFile( ENEMY_SETTING_FILE_PATH, true );

		GEnemySetting enemySetting;

		// K{ڊmFp
		int NEED_BIT = 0;
		for( int i = 0; i < NeedType::MAX_NUM; i ++ ) {
			NEED_BIT |= 1 << i;
		}

		int nNeedCount = 0;
		int nNum = 0;

		int ALL_SOLID_TYPE = 0;
		for( int i = 0; i < SolidType::MAX_NUM; i ++ ) {
			ALL_SOLID_TYPE |= 1 << i;
		}
		for( int i = 0; i < static_cast<int>( vecLines.size() ); i ++ ) {
			if( vecLines[i].at(0) == '[' ) {
				
				if( ( nNeedCount & NEED_BIT ) == NEED_BIT ) {
					// add
					this->m_vecEnemySettings[nNum] = enemySetting;
					enemySetting.clear();
					nNeedCount = 0;
				}
				nNum = atoi(vecLines[i].substr(1,vecLines[i].size() - 2).c_str());
			} else {
				vector<string> vecItems = HPLString::split( vecLines[i], "=" );
				if( vecItems.size() < 2 ) {
					HPLError::msg( "Warning", MB_OK, "Error:%s", vecLines[i].c_str());
					continue;
				}

				if( 0 == vecItems[0].compare(CMD_SPRITE_ID ) ) {
					enemySetting.m_nSpriteID = atoi( vecItems[1].c_str() );
					nNeedCount |= 1 << NeedType::SPRITE_ID;
				} else if( 0 == vecItems[0].compare( CMD_HIT_RECT ) ) {
					vector<string> vecRects = HPLString::split( vecItems[1], ",") ;
					enemySetting.m_HitRect.set(
						(float)atoi( vecRects[0].c_str() ),
						(float)atoi( vecRects[1].c_str() ),
						(float)atoi( vecRects[2].c_str() ),
						(float)atoi( vecRects[3].c_str() )
						);

					nNeedCount |= 1 << NeedType::HIT_RECT;
				} else if( 0 == vecItems[0].compare( CMD_HP_RATE ) ) {
					enemySetting.m_fHPRate = (float)atof(vecItems[1].c_str() );
					nNeedCount |= 1 << NeedType::HP_RATE;
				} else if( 0 == vecItems[0].compare( CMD_POWER_RATE ) ) {
					enemySetting.m_fPowerRate = (float)atof(vecItems[1].c_str() );
					nNeedCount |= 1 << NeedType::POWER_RATE;
				} else if( 0 == vecItems[0].compare( CMD_BLOW_RATE ) ) {
					enemySetting.m_fBlowRate = (float)atof(vecItems[1].c_str() );
					nNeedCount |= 1 << NeedType::BLOW_RATE;
				} else if( HPLString::equals( vecItems[0], CMD_SOLID_TYPE ) ) {
					string strValue = vecItems[1];
					// ","ŕ
					vector<string> vecSlashes = HPLString::split( strValue, ",");
					int nSolidType = -1;
					for( int j = 0; j < (int)vecSlashes.size(); j ++ ) {
						string strTrim = HPLString::trim( vecSlashes[j] );
						map<string,int>::iterator it = mapSolidTypeStringToIndex.find( strTrim );
						if( it != mapSolidTypeStringToIndex.end() ) {
							nSolidType = it->second;
							break;
						}

					}
					if( nSolidType < 0 ) {
						// ALL/NONE`FbN
						if( vecSlashes.size() == 1 ) {
							if( HPLString::equals( strValue, CMD_SOLID_TYPE_ALL ) ) {
								nSolidType = ALL_SOLID_TYPE;

							} else if( HPLString::equals( strValue, CMD_SOLID_TYPE_NONE ) ) {
								// default
								nSolidType = 0;
							}
						}
					}
					if( nSolidType < 0 ) {
						// L[͂̂ɁAݒ肪ȂȂ
					} else {
						enemySetting.m_nSolidableType = nSolidType;
					}
				} else {

					for( int j = 0; j < EnemyFlagType::MAX_NUM; j ++ ) {
						if( 0 == vecItems[0].compare( CMD_FLAGS[j] ) ) {
							enemySetting.m_aryFlags[j] = vecItems[1].compare( "true" ) == 0;
						}
					}
				}

			}
		} // for vecLines
		if( ( nNeedCount & NEED_BIT ) == NEED_BIT ) {
			// add
			this->m_vecEnemySettings[nNum] = enemySetting;
		} else {
			LOG_WARN(HPLString::chrToStr( "Too few neseccally arguments.[%d]", nNeedCount));
		}
	}

	// G̖Oǂݍ
	{
		m_vecEnemyNames = HPLFile::loadStringFromFile(
			"data/image/Enemy/EnemyLabel.txt",
			true );
		// `FbN
		HPLError::assertIt(
			m_vecEnemyNames.size() == EnemyType::MAX_NUM,
			"G̃xƓG̎ނvȂ");
	}

	HPLMatrix<int>* mat = NULL;
	// sunny
	{
		const int MAP_DELTA_X = 5;
		const int MAP_DELTA_Y = 30;
		// iqWŋL
		int RANDOM_MOVE_CELL_LOC[][2] = {
			{MAP_DELTA_X + 14,	MAP_DELTA_Y + 2},	// gbv
			{MAP_DELTA_X + 4,	MAP_DELTA_Y + 3},	// 
			{MAP_DELTA_X + 25,	MAP_DELTA_Y + 3},	// E
			{MAP_DELTA_X + 14,	MAP_DELTA_Y + 2},	// ʒ
			{MAP_DELTA_X + 6,	MAP_DELTA_Y + 12},
			{MAP_DELTA_X + 22,	MAP_DELTA_Y + 12},
			{MAP_DELTA_X + 19,	MAP_DELTA_Y + 6},
			{MAP_DELTA_X + 11,	MAP_DELTA_Y + 6},
		};
		mat = new HPLMatrix<int>();
		setMatrix( RANDOM_MOVE_CELL_LOC, mat, SIZEOF(RANDOM_MOVE_CELL_LOC ) );
		this->m_mapCoordPositions[EnemyPositionType::SUNNY] = mat;
	}

	// aryona
	//  wind
	{
		mat = new HPLMatrix<int>();
		int WIND_ARYONA_POSITIONS[][2] = {
			{5,3},{14,3},{23,3},
			{5,16},{14,16},{23,16},
		};
		setMatrix( WIND_ARYONA_POSITIONS, mat, SIZEOF(WIND_ARYONA_POSITIONS ) );
		this->m_mapCoordPositions[EnemyPositionType::WIND_ARYONA] = mat;
	}

	// chaser
	// boss01Xe[Wł̈ړʒu
	{
		int POSITIONS_1[][2] = {
			{5,15},
			{15,15},
			{25,15},
			{35,15},
		};
		mat = new HPLMatrix<int>();
		setMatrix( POSITIONS_1, mat, SIZEOF(POSITIONS_1 ) );
		this->m_mapCoordPositions[EnemyPositionType::CHASER_1] = mat;
	}

	// boss02Xe[Wł̈ړʒu
	{
		int POSITIONS[][2] = {
			{5,16},	
			{15,16},
			{25,16},
			{35,16},
			{45,16},
		};
		mat = new HPLMatrix<int>();
		setMatrix( POSITIONS, mat, SIZEOF(POSITIONS ) );
		this->m_mapCoordPositions[EnemyPositionType::CHASER_2] = mat;
	}

	// ghost pairړʒu
/*	{
		int POSITIONS[][2] = {
			{0,0},
			{0,0},
			{0,0},
			{0,0},
		};
	}*/

	// aiko ړʒu
	{
		int POSITIONS[][2] = {
			{4,4},
			{9,4},
			{14,4},
			{19,4},
			{24,4},

			{4,9},
			{9,9},
			{14,9},
			{19,9},
			{24,9},

			{4,16},
			{9,16},
			{14,16},
			{19,16},
			{24,16},
		};
		mat = new HPLMatrix<int>();
		setMatrix( POSITIONS, mat, SIZEOF(POSITIONS));
		this->m_mapCoordPositions[EnemyPositionType::AIKO] = mat;

		mat = new HPLMatrix<int>();
		setMatrix( POSITIONS, mat, SIZEOF(POSITIONS));
		this->m_mapCoordPositions[EnemyPositionType::GHOST_PAIR] = mat;
	}


	///////////
	// UC^[oݒ
	{
		char WAIT_FOR_START_LABEL[]="WaitForStart";
		char WAIT_FOR_NEXT_LABEL[] = "WaitForNext";

		vector<string> vecLines = HPLFile::loadStringFromFile( 
			"data/image/Enemy/EnemyAttack.txt", true );
		int nType = -1;

		HPLAttackInformation* lpInformation = NULL;

		int nCount = 0;
		for( int i = 0; i < (int)vecLines.size(); i ++ ) {
			string strLine = HPLString::trim(vecLines[i]);
			if( strLine[0] == '[' ) {
				// id
				int nID = atoi(strLine.substr( 1, strLine.length() - 2 ).c_str());
				if( nType >= 0 ) {
					this->m_mapAttackInformations[nType] = lpInformation;
				}
				lpInformation = new HPLAttackInformation();
				nType = nID;
			} else {	// not []
				vector<string> vecEquals = HPLString::split( 
					strLine, "=");
				if( vecEquals.size() < 2 ) {	// not equals separated
					// separate with ","
					vector<string> vecCommas = HPLString::split(
						strLine, ",");
					if( vecCommas.size() < HPL::AttackItemType::MAX_NUM ) {	// too few arguments
						HPLError::msg("error", MB_OK,
							"Too few arguments[type=%d][%s]",
							nType, strLine.c_str() );
						continue;
					}

					HPLAttackItem* lpItem = new HPLAttackItem();

					lpItem->m_IntervalCounter.setWaitFrameNum( atoi( vecCommas[0].c_str()));
					lpItem->m_ShotNumCounter.setWaitFrameNum( atoi( vecCommas[1].c_str()));
					lpItem->m_SetIntervalCounter.setWaitFrameNum( atoi( vecCommas[2].c_str()));
					lpItem->m_SetNumCounter.setWaitFrameNum( atoi( vecCommas[3].c_str()));

					lpInformation->addAttackItem( lpItem );

				} else {	// equals separated
					for( int j = 0; j < vecEquals.size(); j ++ ) {
						vecEquals[j] = HPLString::trim( vecEquals[j] );
					}
					if( HPLString::equals( vecEquals[0], WAIT_FOR_START_LABEL ) ) {	// start
						lpInformation->m_WaitForStartCounter.setWaitFrameNum( atoi(vecEquals[1].c_str()));
					} else if( HPLString::equals( vecEquals[0], WAIT_FOR_NEXT_LABEL ) ) {	// next
						lpInformation->m_WaitForStartCounter.setWaitFrameNum( atoi(vecEquals[1].c_str()));
					} else {
						HPLError::msg("error", MB_OK,
							"Unknown [type=%d][%s]",
							nType, strLine.c_str() );
						continue;
					}
				}	// equals/not euals

			}	// []/not []
		}	// each vecLines
		if( nType >= 0 ) {
			this->m_mapAttackInformations[nType] = lpInformation;
		}
	}
}

HPLMatrix<int>* GEnemyManager::getCoordPosition( int nEnemyPositionType ) {
	HPLError::assertIt( 
		this->m_mapCoordPositions.find( nEnemyPositionType ) != this->m_mapCoordPositions.end(),
		"Invalid enemy position type[%d]", nEnemyPositionType );

	return this->m_mapCoordPositions[nEnemyPositionType];

}

string GEnemyManager::getEnemyName( int nIndex ) {
#ifdef _DEBUG
	HPLError::assertIt(nIndex >= 0 && nIndex < (int)this->m_vecEnemyNames.size(),
		"G擾:Index out of bounds");
#endif
	return this->m_vecEnemyNames[nIndex];
}

void GEnemyManager::process(bool bIsMove, bool bIsAI) {

	GPlayer* lpPlayer = &getStaticData()->m_ObjectManager.m_Player;

	// vC[̃[_[̓蔻
	HPLRect rLaderHitArea = lpPlayer->getLaderHitArea();

	// ^[QbeBOCfbNXXebv
	this->m_nTargettedIndexStep = 0;

	// ^[QbeBOXgNA
	this->m_vecTargettedEnemy.clear();

	// ʊO̗P\ẗ
	HPLRect merginedGameRange = getStaticData()->getMerginedGameRange();

	// ̋E
	float fLimitUnder= getStaticData()->getLimitUnder();

	// ǉp
	this->m_vecAdds.clear();

	// x
	float fLevel = getStaticData()->m_Game.getLevel( LEVEL_EPSILON );

	// {XCto[o^p
	this->m_vecBossLifeBarRates.clear();

	// Q[W
	// ->XN[ɃvC[łԂŉ߁B
	HPLRect rGameRange = getStaticData()->getGameRange();

	int TILE_SIZE = getStaticData()->m_Setting.tileSize;

	// ubNW^C}gNX̌E
	HPLRect BLOCK_RANGE = getStaticData()->m_BlockManager.getBlockRange(TILE_SIZE);

	// ubNWF]
	HPLRect MERGINED_BLOCK_RANGE = getStaticData()->m_BlockManager.getMerginedBlockRange(
		getStaticData()->m_Setting.screenOutMergine * 2 , TILE_SIZE);

	// ʂȏ
	//  BOSSXe[Wł
	//  {XXe[Wł
	// (ʊO܂œȂǂ̏Ɏg)
	bool bSpecialStage = 
		getStaticData()->getMapType() == MapType::BOSS ||
		getStaticData()->getMapType() == MapType::MID_BOSS;

	// for achievement
	int nSlimeCount = 0;

	// ubNO̓xe
	bool bIsAllowBlockOutRange = false;
	if( bSpecialStage ) {
		// {XXe[W
		bIsAllowBlockOutRange = true;
	}

	// map index ( for aryona)
	int nMapIndex = getStaticData()->getMapIndex();
	int nStar = getStaticData()->m_nSelectedStar;
	int nPlace = getStaticData()->m_nSelectedPlace;
//	getStaticData()->m_Setting.maps.getStarAndPlaceFromMapIndex(
//		nMapIndex, nStar, nPlace );

	
	vector<int> vTypes;
	switch( nMapIndex ) {
	case SpecialMapIndex::WIND_BOSS_1:
		{
			vTypes.push_back( EnemyType::CHASER );
		}
		break;
	}
	if( (int)vTypes.size() > 0 ) {
		this->setupCountTargets(vTypes);
	}

	// brust to player
	const float ENEMY_TO_PLAYER_BRUST_RATE = 1.f;

	for( list<GEnemy*>::iterator it = this->getList()->begin();
	it != this->getList()->end(); )
	{
		GEnemy* lpEnemy = *it;

		// G̃qbg̈
		HPLRect hitArea = lpEnemy->getHitArea();
		HPLRect rArea = lpEnemy->getArea();

		// BOSSȂʊOłǂԂłCto[Xgɓo^
		if( lpEnemy->m_nClassType == EnemyClassType::EVIL_BOSS ) {
			float fRate = lpEnemy->getHP() / lpEnemy->getHPMax();
			BossHPData data;
			data.m_fRate = fRate;
			data.m_nBerserkLevel = lpEnemy->getBerserkLevel();
			this->m_vecBossLifeBarRates.push_back( data );
		}

		// check shot hit to enemy
		//this->checkHitToShot( lpEnemy );

		bool bIsRangeDead = false;	// ʊOɏo玀ʓG

		bool bIsSpecialOutRange = false;	// ʊOłĂ^Cvi^C͈͊OȂ玀ʁj

		bool bIsAreaInMergeinedRange = merginedGameRange.isShare( rArea );

		// V[PX
		HPLSequence* lpSeq = lpEnemy->getSequence();
		if( lpEnemy->getType() == EnemyType::ROCK ) {
			// 
			if( lpEnemy->getCenter().getY() > fLimitUnder ) {
				// z

				// ->S
				lpEnemy->die(NULL);
			}
		} else if( lpEnemy->getType() == EnemyType::LAVA ||
			lpEnemy->getType() == EnemyType::LAVA_UNDER ||
			(lpEnemy->getType() == EnemyType::HIDE_ENEMY_GENERATOR &&
			lpEnemy->getGenerateType() == GeneratorType::LAVA_UNDER))
		{
			// n֌W
			bIsRangeDead = false;
			bIsSpecialOutRange = true;
		} else if( false == bSpecialStage &&
			lpEnemy->m_nClassType != EnemyClassType::EVIL_BOSS &&
			merginedGameRange.isShare(hitArea) ==false
			)
		{
			// ʃXe[WiBOSSjł͂Ȃ
			// NX{XȊOł
			// ̈Oł

			if( bIsRangeDead || lpEnemy->getCenter().getY() > fLimitUnder ) {
				// ͈͊OɏoƎʃ^Cv
				// ̗̈𒴂

				// S
				lpEnemy->die(NULL);
			} else if( lpEnemy->getState()->isDeadState() ) {
				// SAj[V
				// 
				lpEnemy->setAlive( false );
			} else {
				// ̑
				// ʊOȂ疳
				if( lpEnemy->isHolding() ) {
					// ł
					this->freeHold( lpEnemy->getTarget(), lpEnemy );
				}
				it ++;
				// ({X͏O)
				// ({XXe[WȂǁAꕔXe[Wł͌ȂƂł𓮂)
				continue;
			}
		}
		if( lpEnemy->isAlive() && 
			lpEnemy->isHolding())
		{// || lpEnemy->getObjectAttribute( ObjectAttribute::GRAVITY ) == false )) {
			// GĂ
			// GvC[߂܂Ă
			// ʁivC[ĂEBhÉ͈joȂ悤ɂ
			HPLActionMoveTool::adjustHitAreaInRange( lpEnemy,
				rGameRange );
		}
		if( bIsAllowBlockOutRange || bIsSpecialOutRange ) {
			// ubŃ͈{At@o玀
			if( lpEnemy->m_nClassType != EnemyClassType::EVIL_BOSS )
			{
				// {XȊO
				if( MERGINED_BLOCK_RANGE.isShare( hitArea ) == false ) {
					// WO
					// ->(Ȃ)
					lpEnemy->setAlive( false );
				}
			}
		} else if( lpEnemy->m_nClassType != EnemyClassType::EVIL_BOSS &&
			lpEnemy->getObjectAttribute( ObjectAttribute::GRAVITY ) == false )
		{
			// ubNȎ݂e
			// 
			// {XȊO
			// ->ubN͈͊Oɂ͏oȂ
			HPLActionMoveTool::adjustHitAreaInRange( lpEnemy,
				BLOCK_RANGE );
		}
		///// JEg
		if( lpEnemy->isAlive() ) {
			int nType = lpEnemy->getType();
			if( this->isCountTarget( nType ) ) {
				this->addCountTarget( nType );
			}
		}
		if( bIsMove && lpEnemy->isAlive()) {
			// GĂ & \
			
			// XtH
			if( lpEnemy->m_SolidData.getType() != SolidType::NONE ) {
				// Xebvi߂
				lpEnemy->m_SolidData.step(fLevel < 1 ? 1 : (int)fLevel);
			}
			if( bIsAI && lpEnemy->m_SolidData.getType() == SolidType::NONE) {
				// solid type == none

				// 
				this->fan( lpEnemy, lpEnemy->getTarget(), ENEMY_TO_PLAYER_BRUST_RATE );
				
				// collision
				getStaticData()->collisionObject( lpEnemy );
				
				// AIL
				
				if( lpEnemy->getState()->isDeadState() ) { //dead,dying
					// S̍s

					// shakeH
					bool bIsShake = false;

					switch( lpEnemy->getType() ) {
					case EnemyType::SLIME_QUEEN:
						bIsShake = true;
						break;
					case EnemyType::ALRAUNE:
						bIsShake = true;
						break;
					case EnemyType::CHASER:
						bIsShake = true;
						// ʁi}bvjChaser
						{
							int nTileSize = getStaticData()->m_Setting.tileSize;
							HPLRect rBlockRange = getStaticData()->m_BlockManager.getBlockRange(nTileSize);
							// ړ
							HPLVector vGoal(rBlockRange.getWidth()/2, rBlockRange.getHeight()/2);
							this->chase( lpEnemy, vGoal, CHASER_CHASE_ACCEL, CHASER_CHASE_MAX_SPEED);
						}
						break;
					}
					if( bIsShake ) {
						if( getStaticData()->m_ShakeManager.isShake() == false ) {
							// nkNĂȂ
							// nkN
							getStaticData()->m_ShakeManager.startShake();
						}
					}

					if( lpSeq->getState().isFinishAnimation ) {
						// dead stateV[PXI

						//switch( lpEnemy->getType() ) {
						//default:
							// S
							lpEnemy->setAlive( false );
						//}
					}
				} else {	// not dead state
					
					// ʏ펞̍s
					this->algorithm(lpEnemy);

					for( vector<GArmManager*>::iterator ait = lpEnemy->m_vecArmManagers.begin();
						ait != lpEnemy->m_vecArmManagers.end(); ait ++ )
					{
						GArmManager* lpArmManager = *ait;
						if( lpArmManager->isEnable() ) {
							// Gs
							lpArmManager->algorithm();
						}
					}
				}
			}
			if( lpEnemy->isAlive() ) {
				if( lpEnemy->isInvinsible() ) {
					// GiJE^isj
					lpEnemy->invinsibleProcess();
				}

				// check hit
				bool bIsHitToTarget = this->checkHitToTarget( lpEnemy, &getStaticData()->m_ObjectManager.m_Player );
				bool bIsHitToCOM = false;
				if( getStaticData()->isCOMPlayer() ) {
					bIsHitToCOM = this->checkHitToTarget( lpEnemy, &getStaticData()->m_ObjectManager.m_COMPlayer );
				}

				if( (false == bIsHitToCOM && false == bIsHitToTarget ) ) { // no hit to main body
					for( vector<GArmManager*>::iterator ait = lpEnemy->m_vecArmManagers.begin();
						ait != lpEnemy->m_vecArmManagers.end(); ait ++ ) 
					{
						GArmManager* lpArmManager = *ait;
						if( lpArmManager->isEnable() ) {
							lpArmManager->checkHitToTarget();
						}
					}
				}

				// check enemy hit to shot
				bool bIsHitToShot = this->checkHitToShot( lpEnemy );

				// check arms hit to shot
				for( vector<GArmManager*>::iterator ait = lpEnemy->m_vecArmManagers.begin();
					ait != lpEnemy->m_vecArmManagers.end(); ait ++ ) 
				{
					GArmManager* lpArmManager = *ait;
					if( lpArmManager->isEnable()) {
						vector<HPLShootingObject*> vecHitArms;
						lpArmManager->checkHitToShot(vecHitArms);
					}
				}

				// check enemy hit to other
				bool bIsHitToOther = this->checkHitToOther( lpEnemy );

			}


		}
		// [_[蔻
		if( lpEnemy->isAlive() && lpPlayer->m_LaderSequence->getState().isFinishAnimation == false ) {
			if( rLaderHitArea.isShare( lpEnemy->getHitArea() ) ) {
				// qbg

				// ^[QbeBO
				lpEnemy->m_TargettedRemainCounter.resetCounter();
			}
		}
		
		if( lpEnemy->isAlive() ) {
			if( false == lpEnemy->m_bIsDrawBack && bIsAreaInMergeinedRange ) {
				lpEnemy->draw();
			}

			// ^[QbeBOĂΓo^
			if( lpEnemy->m_TargettedRemainCounter.getState().isFinishAnimation == false ) {
				this->m_vecTargettedEnemy.push_back( lpEnemy );
			}

			if( lpEnemy->isSlime()) {
				nSlimeCount ++;
			}
			it ++;
		} else {
			// S
			// SGtFNg

			int nScenarioID = -1;				// ViI
			int nDestroyBossStageType = -1;	// {XXe[W
			int nPlayerAddWeaponType = -1;

			// ʂɃGtFNg𔭐Ȃ
			bool bIsNoDeathEffectType = false;
			switch( lpEnemy->getType()) {
			case EnemyType::LAVA_BALL:
			case EnemyType::LAVA:
			case EnemyType::LAVA_UNDER:
				bIsNoDeathEffectType = true;
				break;
			}
			if( !bIsNoDeathEffectType &&
				getStaticData()->m_SceneManager.getSceneType() == SceneType::GAME && (	// game scene
				bSpecialStage ||						// specialstage
				merginedGameRange.isShare(hitArea) ))	// or in range
			{
				// RC
				int nCoinNum = (int)(lpEnemy->getHPMax() * fLevel / ENEMY_HP_BASE * COINS_PER_HP_BASE);
				getStaticData()->m_ObjectManager.m_ItemManager.createCoins(lpEnemy->getCenter(), nCoinNum );

				//int nNextScene = -1;
				//int nNextMap = -1;

				// {XG̓Xe[Wɍ킹SuperPower𐶐	
				if( lpEnemy->m_nClassType == EnemyClassType::EVIL_BOSS ) {	// boss is dead
					switch( lpEnemy->getType() ) {
					case EnemyType::CHASER:
						{
							// * Ӂ[Ƃ[TĔj󂷂
							// * ȂΉȂ
							for( list<GEnemy*>::iterator oit = this->getList()->begin();
								oit != this->getList()->end(); oit ++ )
							{
								GEnemy* lpOther = *oit;
								if( lpOther == lpEnemy ) {
									continue;
								}
								if( lpOther->isAlive() == false ) {
									continue;
								}
								switch( lpOther->getType() ) {
								case EnemyType::FUUN:
								case EnemyType::RAI:
									lpOther->die( NULL );
									break;
								}
							}
						}
						break;
					}

					// tF[hĂ
					// fade
					HPLFadeOut* lpFade = getStaticData()->m_BGManager.getFadeOut();
					if( lpFade->isFading() &&					// fading!
						lpFade->getStart() < lpFade->getEnd() )	// darken
					{
						// fade in ( stop darken and start lighten )
						lpFade->setup(CHASER_FADE_IN_FRAME_NUM,
							0,0,0,CHASER_FADE_DARKEN_ALPHA, CHASER_FADE_LIGHTEN_ALPHA, DX_BLENDMODE_ALPHA);
						lpFade->start();
						
					}
					// create boss ability item
					GItem* lpItem = getStaticData()->m_ObjectManager.m_ItemManager.create(
						Sequence::Item::BOSS_ABILITY, lpEnemy->getCenter() );
					// color
					lpItem->setColor( nStar);//STAR_COLOR[nStar][0],STAR_COLOR[nStar][1],STAR_COLOR[nStar][2] );
				}

//				int nCol = -1;

				// SɐGtFNg
				// GńASGEnemy::die()̔h
				// sĂ̂ŊiSAG͑S폜邽߁jB
				lpEnemy->createDeadEffect( );

				// SSE
				int nDeadSE = SE::SAKU;

				// ʏ
				switch( lpEnemy->getType() ) {
				case EnemyType::SLIME_QUEEN:
					// ̃}bv
					
					// FORESTVׂ悤ɂ
					getStaticData()->m_Save.m_aryStarState[StageType::FOREST] = PlaceType::REVEALED; 
					getStaticData()->m_Save.m_aryStarState[StageType::WIND] = PlaceType::REVEALED; 
					getStaticData()->m_Save.m_aryStarState[StageType::FIRE] = PlaceType::REVEALED; 
					getStaticData()->m_Save.m_aryStarState[StageType::WATER] = PlaceType::REVEALED; 
					getStaticData()->m_Save.m_aryStarState[StageType::REST_STAR] = PlaceType::REVEALED;
					getStaticData()->m_Save.save();

					// j
					nDestroyBossStageType = StageType::EVIL_STAR;

					// 
					getStaticData()->m_nPlayerPlaceIndex = 1;

					break;
				case EnemyType::QBEE:
					// ̃}bv
					getStaticData()->startFadeOutDX( -1, SpecialMapIndex::BEE_HOUSE );

					// vC[oCfbNX1i[jɂ
					getStaticData()->m_nPlayerPlaceIndex = 1;

					break;
				case EnemyType::ALRAUNE:
					nDestroyBossStageType = StageType::FOREST;
					break;
				case EnemyType::CHASER:
					nDestroyBossStageType = StageType::WIND;

					break;
				}

				if( nDeadSE >= 0 ) {
					// dead se
					getStaticData()->m_MixerManager.play( nDeadSE, false );
				}
			}

			if( 0 <= nDestroyBossStageType ) {
				// {XjtO
				getStaticData()->m_Save.setFlag(
					FlagType::DESTROY_BOSS,
					HPLMath::setFlagByIndexInt( getStaticData()->m_Save.getFlag(
					FlagType::DESTROY_BOSS ), nDestroyBossStageType, true ) );

				// game set!
				getStaticData()->m_lpGameSetSequence->resetCounter();

				// {X|
				for( int i = 0; i < getStaticData()->m_Setting.maps.getPlaceNum( nDestroyBossStageType); i ++ ) {
					// ePlace

					// placȅԎ擾
					int nPlaceState = getStaticData()->m_Save.getPlaceState(nDestroyBossStageType,i );
					if( nPlaceState == PlaceType::REVEALED ) {	// release
						// NAԂɕύX
						getStaticData()->m_Save.setPlaceState(nDestroyBossStageType,i,PlaceType::CLEARED);
					}
				}
				getStaticData()->m_Save.save();
			}
			delete *it;
			it = this->getList()->erase(it);

			/*if( 0 <= nPlayerAddWeaponType ) {
				// ǉL
				lpPlayer->addEquipWeapon( nPlayerAddWeaponType );
			}*/
			/*if( nNextMap >= 0 || nNextScene >= 0 ) {
				getStaticData()->startFadeOutDX(nNextScene, nNextMap );
			}*/
		}
	}

	for( vector<GEnemy*>::iterator it = m_vecAdds.begin();
		it != m_vecAdds.end(); it ++ ) 
	{
		GEnemy* lpAdd = *it;

		switch( lpAdd->getType() ) 
		{
		case EnemyType::LAVA_BALL:
		case EnemyType::MINI_BEE:
			// no effect
			break;
		default:
			if( merginedGameRange.isShare( lpAdd->getHitArea() ) ) {
				// ʓł
				// ǉGtFNg

				HPLVector vCenter = lpAdd->getCenter();
				getStaticData()->createEffect( Sequence::Effect::WARP,
					vCenter, vCenter, -1, -1 );

				
			}
		}
		this->getList()->push_back( lpAdd );
	}

	// no enemy ?
	if( this->getList()->size() == 0 ) {
		// no enemy
		switch( getStaticData()->getMapIndex() ) {
		case SpecialMapIndex::WIND_FUU_RAI:
			if( getStaticData()->isFadingOutDX() == false ) {
				// `FbN|Cg
				bool bIsChanged = getStaticData()->checkPoint( SpecialMapIndex::WIND_KAZE );
				if( bIsChanged ) {
					// save
					getStaticData()->m_Save.save();
				}
			}

			break;
		}
	}

	GAchievementItem* lpItem = getStaticData()->m_AchievementSave.getAchievementItem( AchievementType::MANY_SLIMES);
	if( lpItem->isFinish() == false ) {
		// achievement check
		if( false == getStaticData()->stepAndCheckAchievement( AchievementType::MANY_SLIMES, nSlimeCount, true ) ) {
			// BȂ
			// ->reset
			lpItem->setCounter(0 );
		}
	}
}

/** {XHP\
	*/
void GEnemyManager::drawBossHP() {
	// {X̃CtBAR\

	if( this->m_vecBossLifeBarRates.size() > 0 ) {
		// S
		int nAllWidth = getStaticData()->m_Setting.windowWidth - 2 * ENEMY_BOSS_LIFE_BAR_MERGINE;

		// Ԋu(X̕v)
		int nAllBarWidth = nAllWidth - (m_vecBossLifeBarRates.size() - 1) * ENEMY_BOSS_LIFE_BAR_INTERVAL_X;
		
		// 1̃o[̕
		int nWidthPerEnemy = nAllBarWidth / m_vecBossLifeBarRates.size();

		// Ŝ̍[
		int nLeft = getStaticData()->m_Setting.windowWidth / 2 - nAllWidth / 2;

		HPLSequence* lpBarFrameSeq = getStaticData()->getSequence( SpriteType::COMMON,
			Sequence::Common::BOSS_HP_FRAME );
		int y = (int)(
			getStaticData()->m_Setting.windowHeight - ENEMY_BOSS_LIFE_BAR_Y_FROM_BOTTOM - 
			lpBarFrameSeq->getImageInformation(0)->srcRect.getHeight() );

		int h = (int)lpBarFrameSeq->getImageInformation(0)->srcRect.getHeight();

		HPLSequence* lpBarSeq = getStaticData()->getSequence( SpriteType::COMMON,
			Sequence::Common::BOSS_HP );
		for( int i = 0; i < (int)m_vecBossLifeBarRates.size(); i ++ ) {
			int x = nLeft + i * (nWidthPerEnemy + ENEMY_BOSS_LIFE_BAR_INTERVAL_X);
			// data
			BossHPData& data =(BossHPData&)this->m_vecBossLifeBarRates[i];

			DrawExtendGraph(x,y,x + nWidthPerEnemy , y + h,
				getStaticData()->getImageHandle( SpriteType::COMMON, Sequence::Common::BOSS_HP_FRAME ), TRUE );
			
			bool bIsColorChange = false;
			if( data.m_nBerserkLevel > 0 ) {	// berserk!
				bIsColorChange = true;
				// change to red
				SetDrawBright( 255,100,100 );
			}
			DrawExtendGraph(x,y,(int)(x + nWidthPerEnemy * data.m_fRate ), y + h,
				getStaticData()->getImageHandle( SpriteType::COMMON, Sequence::Common::BOSS_HP ), TRUE );

			if( bIsColorChange ) {	// changed color
				// revert color
				SetDrawBright( 255,255,255 );
			}
		}
	}
}


/** Ă΂xɁA̓GI
  */
GEnemy* GEnemyManager::getStepTargettedEnemy() {
	if( this->m_vecTargettedEnemy.size() == 0 ) {
		return NULL;
	}
	GEnemy* lpEnemy = this->m_vecTargettedEnemy[ this->m_nTargettedIndexStep ];
	this->m_nTargettedIndexStep ++;
	if( this->m_nTargettedIndexStep >= (int)this->m_vecTargettedEnemy.size() ) {
		this->m_nTargettedIndexStep = 0;
	}
	return lpEnemy;
}
GEnemy* GEnemyManager::getRandomTargettedEnemy() {
	if( this->m_vecTargettedEnemy.size() == 0 ) {
		return NULL;
	}
	return this->m_vecTargettedEnemy[ HPLMath::nextInt( 0, this->m_vecTargettedEnemy.size() ) ];
}

GEnemy* GEnemyManager::create(int nType, HPLVector vCenter, int nGenerator) {
	GEnemy* lpEnemy = this->createWithoutAdd( nType, vCenter, nGenerator );

	this->getList()->push_back( lpEnemy );

	return lpEnemy;
}
/** 쐬邪AXgɂ͒ǉȂ(m_vAddsɂǉȂ)
  */
GEnemy* GEnemyManager::createWithoutAdd( int nType, HPLVector vCenter, int nGenerateType ) {
	GEnemy* lpEnemy = new GEnemy( nType );
	lpEnemy->setCenter( vCenter );

	lpEnemy->setGenerateType( nGenerateType );

	// !!! e͈͖mFI
	lpEnemy->m_vCenterPosition = vCenter;
	lpEnemy->m_vAmplitudeCenter = vCenter;
	return lpEnemy;

}



void GEnemyManager::faceToTarget( GEnemy* lpEnemy ) {
	HPLVector vTargetCenter = lpEnemy->getTarget()->getCenter();
	HPLVector vSub(
		vTargetCenter.getX() - lpEnemy->getCenter().getX(),
		vTargetCenter.getY() - lpEnemy->getCenter().getY() );
	HPLVector vAim(
		HPLMath::getSgn<float>( vSub.getX() ),
		HPLMath::getSgn<float>( vSub.getY() ) );
	vAim.normalize();
	if( vAim.getX() * lpEnemy->getFaceDirection() < 0 ) {
		lpEnemy->setFaceDirection( vAim.getX() > 0 ? FaceDirection::RIGHT : FaceDirection::LEFT );
	}
}

void GEnemyManager::moveToTarget( GEnemy* lpEnemy, float fSpeedRate, bool bIsJumpOverCollision ) {

	float fLevel = getStaticData()->m_Game.getLevel( LEVEL_EPSILON );
	float RESIST = getStaticData()->m_Setting.resist;
	
	HPLVector vTargetCenter = lpEnemy->getTarget()->getCenter();
	HPLVector vSub(
		vTargetCenter.getX() - lpEnemy->getCenter().getX(),
		vTargetCenter.getY() - lpEnemy->getCenter().getY() );
	HPLVector vAim(
		HPLMath::getSgn<float>( vSub.getX() ),
		HPLMath::getSgn<float>( vSub.getY() ) );
	vAim.normalize();
	HPLVector vVelocityAngle = lpEnemy->getVelocityAngle();

	//HPLVector
	//vVelocityAngle.setX( vAim.getX() * ENEMY_SPEED_SLOW_X * fSpeedRate * fLevel );

	// ړ
	this->accelMoveX( vVelocityAngle,
		static_cast<int>( HPLMath::getSgn( vAim.getX() ) ),
		lpEnemy->getTrigger(),
		ENEMY_MOVE_ACCEL * fSpeedRate * fLevel + RESIST,
//		RESIST,
		ENEMY_SPEED_SLOW_X + RESIST, bIsJumpOverCollision == false );
	lpEnemy->setVelocityAngle( vVelocityAngle );

	//HPLVector vCenter = lpEnemy->getCenter();
	//vCenter.add( vVelocityAngle );
	//lpEnemy->setCenter( vCenter );
	lpEnemy->changeState( StateType::WALK );
}

/** ړ
  */
void GEnemyManager::accelMoveX( HPLVector& vVelocityAngle, int nDirection, HPLTrigger* lpTrigger, float fAccel,
//		float fResist,
		float fMaxSpeed, bool bIsNoAirMoving ) 
{
/*	if( false == lpTrigger->getTrigger( TriggerType::ON_THE_GROUND ) ) {
		fAccel *= ENEMY_AIR_MOVE_ACCEL_RATE;
		fResist *= ENEMY_AIR_MOVE_ACCEL_RATE;
		if( bIsNoAirMoving ) return;
	} else {
		fAccel += fResist;
	}*/

	// ړ
	HPLActionMoveTool::accelWithLimitX(
		&vVelocityAngle,
		HPLVector( nDirection * fAccel,0),
		//HPLVector( fResist, 0),
		fMaxSpeed);

}



/**
  */
bool GEnemyManager::checkHitToShot( GEnemy* lpEnemy, int nParentFaceDirection ) {

	if( getStaticData()->m_Game.isScenario() ) {
		return false;
	}

	// skip
	if( lpEnemy->isAlive() == false ) {
		return false;
	}
	/*if( lpEnemy->isHitToShot() == false ) {
		return false;
	}*/
	if( lpEnemy->getObjectAttribute( ObjectAttribute::HIT_TO_SHOT ) == false )// &&
		//lpEnemy->getBrustType() == BrustType::NONE)
	{
		return false;
	}

	if( lpEnemy->getState()->isDeadState() ) {
		return false;
	}


	HPLRect rHitArea = lpEnemy->getHitArea();

	HPLRect rGameRange = getStaticData()->getMerginedGameRange();

	const float ENEMY_TO_SHOT_BRUST_RATE = 0.5f;

	GEnemySetting* lpSetting = this->getEnemySetting(lpEnemy->getType());

	for( list<GShot*>::iterator it = getStaticData()->m_ObjectManager.m_ShotManager.getList()->begin();
		it != getStaticData()->m_ObjectManager.m_ShotManager.getList()->end();
		it ++ )
	{
		GShot* lpShot = *it;

		// skip
		if( lpShot->isAlive() == false ) {
			continue;
		}
		if( lpShot->getState()->isDeadState() ) {
			continue;
		}

		/*if( lpShot->isHitToEnemy() == false ) {
			// GɓȂVbg
			continue;
		}*/


		// ͂΂(GɓȂꍇ)
		this->fan( lpEnemy, lpShot, ENEMY_TO_SHOT_BRUST_RATE );

		if( lpShot->getObjectAttribute( ObjectAttribute::HIT_TO_ENEMY ) == false )
		{
			continue;
		}
		bool bIsHit = false;
		if( ShotClassType::LASER == lpShot->getShotClassType() ) {
			// [U[^Cv
			if( HPLActionMoveTool::isHitObjectToLaser( lpShot, true,
				lpEnemy, lpEnemy->getHitRect().getWidth() / 2 ) )
			{
				// [U[ɓGqbg
				bIsHit = true;
			}
		} else {	// not lazer shot
			HPLRect rShotHitArea = lpShot->getHitArea();
			if( rHitArea.isShare( rShotHitArea ) ) {
				bIsHit = true;
			}
		}

		if( bIsHit ) {
			// qbg

			//HPLRect rHeadShotArea = lpEnemy->getHead

			// O\
			getStaticData()->m_ShowEnemyName.start( lpEnemy->getID(), lpEnemy->getType() );
			if( lpEnemy->m_bIsBlowByShot ) {
				// ΂^Cv
				HPLActionMoveTool::blowAway( lpShot, lpEnemy, 
					lpShot->m_fBlowSpeed );
				if( nParentFaceDirection != 0 ) {
					HPLVector vVelocityAngle = lpEnemy->getVelocityAngle();
					vVelocityAngle.setX( nParentFaceDirection * vVelocityAngle.getX() );
					lpEnemy->setVelocityAngle( vVelocityAngle );
				}
			}
			
/*			FIRE_TRAP,
			NATO_BULLET,

			HOMING_LASER,		//! xWFz[~O
			ELECTRIC_HOMING_LASER,*/

			if( lpEnemy->getEnemyAttribute( EnemyAttribute::INVINSIBLE) == false ) {
				// sł͂Ȃ
				// ->_[W

				// _[W
				float fDamage = lpShot->m_fPower;

				// wbhVbg
				if( lpShot->getDamageType( DamageType::SNIPER ) &&	// sniper eł
					lpEnemy->isWeakHitRect() )	// weak hit rect
				{
					HPLRect rHeadHitArea = lpEnemy->getWeakHitArea();
					if( rHeadHitArea.isShare( lpShot->getHitArea())) {	// share!

						const float HEAD_SHOT_DAMAGE_RATE = 3;	//3{_[W
						fDamage *= HEAD_SHOT_DAMAGE_RATE;

						// TODO:wbhVbgSE

						// o̔ʒu
						// head hit areȁ
						HPLVector vCen( rHeadHitArea.getX() + rHeadHitArea.getWidth() / 2, rHeadHitArea.getY() );

						// wbhVbgo
						getStaticData()->m_ObjectManager.m_EffectManager.create(
							Sequence::Effect::HEAD_SHOT, vCen, true );
					}

					//rHead
				}

				if( lpShot->getDamageType(DamageType::FIRE)) {
					// fire

					switch( lpEnemy->m_SolidData.getType() ) {
					case SolidType::CRYO:
						// solid 
						lpEnemy->m_SolidData.setType( SolidType::NONE );
						break;
					default:
						// ->܂ƂGtFNgij𔭐
						getStaticData()->m_ObjectManager.startFireEffect( 
							Sequence::Effect::FIRE, 
							lpEnemy );
					}

					if( lpEnemy->getStateFlag( StateFlagType::OILED ) ) {
						// OILED
						// _[Wǉ
						fDamage *= 2.f;
					}

				} else {
					const float CRYO_DAMAGE_RATE = 2;
					// ʏU
					switch( lpEnemy->m_SolidData.getType() ) {
					case SolidType::CRYO:
						// cryo
						// ->_[W2{
						fDamage *= CRYO_DAMAGE_RATE;
						break;
					}
				}
				if( lpShot->getType() == Sequence::Shot::EXPLOSION ) {
					// igU_[Wj
					fDamage *= EXPLOSION_DAMAGE_RAGE;
				}
				if( lpShot->getType() == Sequence::Shot::WATER &&
					lpEnemy->getType() == EnemyType::ALRAUNE ) 
				{
					// {AEl

					// 
					fDamage = -1 * fabs(fDamage);
					getStaticData()->stepAndCheckAchievement(
						AchievementType::ALRAUNE_WATER, 1, true );
				}

				if( lpShot->getType() == Sequence::Shot::MAYO ) {
					//Gړ𓾂
					lpShot->setVelocityAngle( lpEnemy->getVelocityAngle() );
					
					if( lpEnemy->m_nClassType == EnemyClassType::EVIL_ZAKO ) {
						// }l[Y{G
						lpEnemy->setStateFlag( StateFlagType::OILED, true );
					}
				}

				
				if( lpShot->getType() == Sequence::Shot::ICE ) {
					// ICE^Cv̍U
					// ->ꕔ̓GɂCRYO

					if( EnemyClassType::EVIL_ZAKO == lpEnemy->m_nClassType &&
						( HPLMath::getFlagByIndexInt( lpSetting->m_nSolidableType, DamageType::ICE)  ||	// VIɑϐ
						lpEnemy->isImmunity( DamageType::ICE ) == false ))	// VIɑϐ
					{
						// ZAKOɂ̂ݔ
						// CRYOϐ̂Gɂ͗Ȃ
						
						lpEnemy->m_SolidData.setType( SolidType::CRYO );
						// se
						getStaticData()->m_MixerManager.play( SE::CRYO, false );
					}
				}
				switch( getStaticData()->m_ObjectManager.getWeakImmuneType( lpShot, lpEnemy ) ) {
				case WeakImmuneType::WEAKNESS:
					fDamage *= WEAKNESS_DAMAGE_RATE;
					break;
				case WeakImmuneType::IMMUNITIES:
					fDamage *= IMMUNITIES_DAMAGE_RATE;
					// se (no duplicate)
					getStaticData()->m_MixerManager.play( SE::UNBREAKABLE, true );
					break;
				}
				lpEnemy->damage( fDamage, lpShot );
			}

			int nHitEffectID = Sequence::Effect::HIT;	// ڐGGtFNg
														// -1ō쐬Ȃ
			switch( lpShot->getType() ) {
			case Sequence::Shot::MAYO:
				// x0
				lpShot->setVelocityAngle(0,0);
				// GƂ̐ڐGȂ
				lpShot->setObjectAttribute( ObjectAttribute::HIT_TO_ENEMY , false );
				nHitEffectID = -1;
				break;
			}
			if( lpShot->getObjectAttribute( ObjectAttribute::HIT_OBJECT_TO_DIE) ) {
				lpShot->die( lpEnemy );
			}

			if( nHitEffectID >= 0 ) {
				HPLVector vCenter = lpShot->getCenter();
				if( lpShot->getShotClassType() == ShotClassType::LASER)
				{
					vCenter = lpEnemy->getCenter();
				}
				vCenter.add( HPLMath::nextFloat(-EFFECT_HIT_ERR, EFFECT_HIT_ERR ),
					HPLMath::nextFloat(-EFFECT_HIT_ERR,EFFECT_HIT_ERR) );
				// GtFNg
				getStaticData()->createEffect( nHitEffectID, 
					vCenter, lpEnemy->getCenter(), -1, -1 );
			}

			// ԕύX
			if( lpEnemy->getState()->isDeadState() == false && lpEnemy->isAlive() && false == lpEnemy->getEnemyAttribute(EnemyAttribute::INVINSIBLE ) ) {
				// dead stateȂ
				// ܂Ă
				// Gł͂Ȃ

				// _[WԂɂ

				if( lpEnemy->isHasDamageState() ) {
					lpEnemy->changeState( StateType::DAMAGE );
				}

				// [vԂȂGp̃JE^
				lpEnemy->m_DamageCounter.reset();
			}
			switch( lpEnemy->getType() ) {
			case EnemyType::SLIME_QUEEN:
				{

					// XCNB[

					if( lpEnemy->getColor() == 0 && lpEnemy->getCounter( SlimeQueenCounterType::WAIT_FOR_NEXT )->getState().isFinishAnimation) {


						// red
						// ҂Ԃł͂Ȃ

						// ԃXC

						if( lpEnemy->getCounter( SlimeQueenCounterType::CREATE_INTERVAL )->getState().isFinishAnimation ) {

							// REDXC
							GEnemy* lpNew = new GEnemy( EnemyType::SLIME );
							// ẽqbgӏɐ
							HPLVector vCenter = lpEnemy->getCenter();
							HPLVector vShotCenter = lpShot->getCenter();
							lpNew->setCenter( 
								(vCenter.getX() + vShotCenter.getX()) / 2, (vCenter.getY() + vShotCenter.getY()) / 2 );

							// 
							lpNew->setVelocityAngle(
								HPLMath::nextFloat( - SLIME_QUEEN_GENERATE_VEL_ERR_X, SLIME_QUEEN_GENERATE_VEL_ERR_X ),
								- HPLMath::nextFloat( SLIME_QUEEN_GENERATE_VEL_Y_MIN, SLIME_QUEEN_GENERATE_VEL_Y_MAX ) );

							// 
							this->m_vecAdds.push_back( lpNew );

							lpEnemy->getCounter( SlimeQueenCounterType::CREATE_INTERVAL )->reset();
						}

					}
				}
				break;
			case EnemyType::JACKPOT:
				{
					// IɒSW
					lpEnemy->setCenter( lpEnemy->m_vAmplitudeCenter );
					// ㉺ɗh炷
					const int JACK_POT_DAMAGE_AMP = 10;
					HPLVector v(0, (HPLMath::nextInt(0,2) * 2 - 1 ) * JACK_POT_DAMAGE_AMP );
					lpEnemy->setVelocityAngle(v );
					HPLVector c(lpEnemy->m_vAmplitudeCenter );
					c.add( v);
					lpEnemy->setCenter( c );
				}
				break;
			}

			// CÂ
			lpEnemy->setRecognized( true );
		}

	}
	return false;
}
/**
  */
bool GEnemyManager::checkHitToOther( GEnemy* lpEnemy ) {
	if( false == lpEnemy->getObjectAttribute( ObjectAttribute::HIT_TO_ENEMY ) ){	// GmœȂ
		return false;
	}
	// special skip
	switch( lpEnemy->getType() ) {
	case EnemyType::SUN:
		{
			HPLSequence* lpSeq = lpEnemy->getSequence();
			if( lpSeq->getStepCounter() != 0 ) {	// et[̍ŏ1FڈȊO̓XLbv
				return false;
			}
		}
		break;
	}

	// level
	float fLevel = getStaticData()->m_Game.getLevel( LEVEL_EPSILON );
	// damage
	float fDamage = lpEnemy->m_fPower / fLevel;

	bool bIsHit = false;
	for( list<GEnemy*>::iterator it = this->getList()->begin(); 
		it != this->getList()->end(); it ++ )
	{
		GEnemy* lpOther = *it;
		if( lpOther == lpEnemy ) {
			// 
			continue;
		}
		if( lpOther->getType() == lpEnemy->getType() ) {
			// 푰
			continue;
		}
		if( lpOther->isAlive() == false ) {
			continue;
		}

		HPLRect rTargetHitArea = lpOther->getHitArea();
		HPLRect rHitArea = lpEnemy->getHitArea();

		if( rHitArea.isShare( rTargetHitArea ) ) { // hit!
			// give damage ( invinsible/next and status are not functionable )
			getStaticData()->damageObject( lpOther, lpEnemy, fDamage, true, -1);

			// blood?
			
			bIsHit = true;
		}
		
	}
	return bIsHit;
}


/** Wv
  * @param[in]	vx,vy	Wvx
  * 
  */
void GEnemyManager::jump( GEnemy* lpEnemy, float vx, float vy, bool bIsJumpableInAir, bool bIsAccelType ) {
	if( bIsJumpableInAir == false && 
		lpEnemy->getTrigger()->getTrigger( TriggerType::ON_THE_GROUND ) == false ) 
	{
		// 󒆃WvsA󒆂ɂꍇ̓WvȂiłȂj
		return ;
	}
	if( bIsAccelType ) {
		HPLVector vVelocityAngle = lpEnemy->getVelocityAngle();
		vVelocityAngle.add( vx, vy );
		lpEnemy->setVelocityAngle( vVelocityAngle );
	} else {
		lpEnemy->setVelocityAngle( vx, vy );
	}
}

/** ubNɕ`
  */
void GEnemyManager::drawEnemyBack() {
	for( list<GEnemy*>::iterator it = this->getList()->begin();
		it != this->getList()->end(); it ++ )
	{
		GEnemy* lpEnemy = *it;
		if( lpEnemy == NULL || lpEnemy->isAlive() == false ) {
			// null ⎀łȂ\Ȃ
			continue;
		}
		if( lpEnemy->m_bIsDrawBack ) {
			// wi\^Cv

			// ->draw
			lpEnemy->draw();
		}
	}
}

/** ڕWChaseiVGpj
  */
void GEnemyManager::chase( GEnemy* lpEnemy, HPLVector vGoalPosition, float fAccel, float fMaxSpeed ) {
	// S[|WV(eGŐݒBftHg̓vC[̈ʒu)
	/*
	HPLVector vGoalSub(
		vGoalPosition.getX() - lpEnemy->getCenter().getX(),
		vGoalPosition.getY() - lpEnemy->getCenter().getY() );

	// K
	HPLVector vGoalAim = vGoalSub;
	vGoalAim.normalize();

	// ݑx
	HPLVector vVelocityAngle = lpEnemy->getVelocityAngle();
	
	// x
	vGoalAim.multiple( fAccel );

	vVelocityAngle.add( vGoalAim );

	if( fMaxSpeed > 0 && fabs( vVelocityAngle.getLength() ) > fMaxSpeed ) {
		vVelocityAngle.normalize();
		vVelocityAngle.multiple( fMaxSpeed );

	}

	lpEnemy->setVelocityAngle( vVelocityAngle );
	*/
	HPLShootingMove::doMissileHoming( lpEnemy, vGoalPosition, fAccel, fMaxSpeed );
}


bool GEnemyManager::checkAndReduceSpeed( GEnemy* lpEnemy ) {
	//lpEnemy->getObjectAttribute(ObjectAttribute::GRAVITY) == false ) {
	if(lpEnemy->getTrigger()->getTrigger( TriggerType::ON_THE_GROUND ) == false ) {	// in the air
		// 
		// ->኱Resist

		// RlvZ
		float fEnemyResist = 0;
		if( lpEnemy->isUseReduceAccel()) {
			fEnemyResist = lpEnemy->getReduceAccel();
		} else {
			// default reduce speed
			fEnemyResist = this->getEnemyAirResist(1);//RESIST * ENEMY_AIR_RESIST_RATE;
		}
		HPLVector v = lpEnemy->getVelocityAngle();
		// xvZ
		float fSpeed = HPLActionMoveTool::reduceSpeed(
			fEnemyResist, v.getLength() );
		v.normalize();
		v.multiple( fSpeed );
		lpEnemy->setVelocityAngle( v );
		return true;
	}
	return false;
}

namespace ChaserHandType {
	enum {
		RISE_UP,
		CHARGE,
		OTHER,
	};
}
void GEnemyManager::getChaserHandFrom( HPLVector& vFrom, int nFaceDirection, HPLVector& vCenter, int nAttackType ) {
	int nMapIndex = getStaticData()->getMapIndex();

	int nCenterType = ChaserHandType::OTHER;
	switch( nMapIndex ) {
	case SpecialMapIndex::WIND_BOSS_1:
		switch( nAttackType ) {
		case ChaserAttackType1::ELECT_SHOT_1:
			nCenterType = ChaserHandType::RISE_UP;
			break;
		case ChaserAttackType1::WIND_SHOT_1:
		case ChaserAttackType1::THUNDER_BALL_HOMING:
		case ChaserAttackType1::THUNDER_BALL_LINE:
			nCenterType = ChaserHandType::CHARGE;
			break;
		}
		break;
	case SpecialMapIndex::WIND_BOSS_2:
		switch( nAttackType ) {
		case ChaserAttackType::WIND_SHOT:
		case ChaserAttackType::ATTACK:
		case ChaserAttackType::BALL:
		case ChaserAttackType::SHABON_SHOT:
			{
				nCenterType = ChaserHandType::CHARGE;
			}
			break;
		default:
	/*	
		case ChaserAttackType::THUNDER_SHOT:
		case ChaserAttackType::ELEC_SHOT:
		case ChaserAttackType::SHABON_SHOT:*/
			nCenterType = ChaserHandType::RISE_UP;
			break;
	/*	case ChaserAttackType::ATTACK:
	#ifdef _DEBUG
			HPLError::assertIt(false, "҂ȂAttackType");
	#endif
			break;
		default:
			HPLError::assertIt(false, "sAttackType:%d", nAttackType);
	*/
		}
	}

	switch( nCenterType ) {
	case ChaserHandType::RISE_UP:
		{
			vFrom = vCenter;
			vFrom.add( nFaceDirection * CHASER_ATTACK_1_ARM_DELTA_X,
				CHASER_ATTACK_1_ARM_DELTA_Y );
		}
		break;
	case ChaserHandType::CHARGE:
		{
			vFrom = vCenter;
			vFrom.add( nFaceDirection * CHASER_ATTACK_2_DELTA_X,
				CHASER_ATTACK_2_DELTA_Y );
		}
		break;
	}
}
/** Ԋuł̔
  * lpIntervalCounter̃t[xlpShotCounter̃JEg񐔂̐삳B
  * @param[in,out]	lpIntervalCounter	VbgԊu
  * @param[in,out]	lpShotCounter		ˉ
  * @param[out]		bIsShot				˃^C~O
  * @param[out]		bIsEnd				SVbgI
  * 
  */
void GEnemyManager::commonIntervalShot( HPLCounter* lpIntervalCounter, int nIntervalStep,
   HPLCounter* lpShotCounter,
	bool& bIsShot, bool& bIsEnd ) 
{
	bIsShot = false;
	bIsEnd = false;

	if( lpIntervalCounter->step().isFinishAnimation ) {
		// shot
		bIsShot = true;
		// reset
		lpIntervalCounter->reset();

		if( lpShotCounter->step().isFinishAnimation ) {	// end of shot count
			bIsEnd = true;
		}
	}
}


/** Gɑ΂CR
  * @param[in]	fRate	Ɋωꍇ
  */
float GEnemyManager::getEnemyAirResist( float fRate ) {
	return (getStaticData()->m_Setting.resist * ENEMY_AIR_RESIST_RATE * fRate );
}

