#include "GItemManager.h"

#include "GStaticData.h"
#include <hpllib/math/HPLMath.h>
#include <hpllib/system/HPLFile.h>
#include <hpllib/general/HPLString.h>

namespace GEnum {
	namespace ItemSettingType {
		enum ItemSettingType_ {
			NAME,
			TYPE,
			CLASS,
			EMOTION,
			NEXT_STATE,
			CURE,
			FLAGS,
			POINT,
			MAX_NUM
		};
	}
}
using namespace std;
using namespace GEnum;

HPLColor* GItemManager::getCoinColor( int nType ) {
	HPLError::assertIt( this->m_mapCoinColor.find(nType) != this->m_mapCoinColor.end(),
		"Invalid Coin Type:%d", nType);

	return &this->m_mapCoinColor[nType];
}

/** 
  */
int GItemSetting::setup( string& strLine ) {
	vector<string> aryList = HPLString::split( strLine, ",");
	if( aryList.size() < ItemSettingType::MAX_NUM ) {
		HPLError::msg( "error", MB_OK,
			"vf̐Ȃ[%d]<[%d]:%s", aryList.size(), ItemSettingType::MAX_NUM, strLine.c_str() );
		return -1;
	}
	const char FLAG_LABELS[][20] = {
		"CURE",
		"STOCK",
		"UNIQUE",
	};
	const char CLASS_LABELS[][20] = {
		"USE",
		"STOCK",
		"WEAPON",
	};
	const char EMOTION_LABELS[][20] = {
		"NONE",
		"EI_SNAKE",
		"EI_HEART",
	};
	int EMOTION_MAPS[] = {
		-1,
		Sequence::Effect::EI_SNAKE,
		Sequence::Effect::EI_HEART,
	};
	const char STATE_LABELS[][20] = {
		"NONE",
		"CHECK",
	};
	int STATE_MAPS[] = {
		-1,
		StateType::CHECK,
	};
	
	for( int i = 0; i < ItemSettingType::MAX_NUM; i ++ ) {
		try {
			string str = aryList[i];
			str = HPLString::trim( str );
			switch( i ) {
			case ItemSettingType::NAME:
				this->m_strName = str;
				break;
			case ItemSettingType::TYPE:
				this->m_nType = atoi( str.c_str());
				break;
			case ItemSettingType::CLASS:
				{
					int nIndex = -1;
					for( int j = 0; j < ItemClassType::MAX_NUM; j ++ ) {
						if( HPLString::equals( str, CLASS_LABELS[j] ) ){
							nIndex = j;
							break;
						}
					}
					if( nIndex >= 0 && nIndex < ItemClassType::MAX_NUM) {
						this->m_nClassType = nIndex;
					} else {
						HPLError::msg( "Error", MB_OK, "Unknown class:[%s]:%s",
							str.c_str(), strLine.c_str());
					}
				}
				break;
			case ItemSettingType::EMOTION:
				{
					int nIndex = -1;
					for( int j = 0; j < SIZEOF(EMOTION_MAPS); j ++ ) {
						if( HPLString::equals( str, EMOTION_LABELS[j] ) ){
							nIndex = j;
							break;
						}
					}
					if( nIndex >= 0 && nIndex < SIZEOF(EMOTION_MAPS)) {
						this->m_nEmotionIcon = EMOTION_MAPS[nIndex];
					} else {
						HPLError::msg( "Error", MB_OK, "Unknown emotion icon:[%s]:%s",
							str.c_str(), strLine.c_str());
					}
				}
				break;
			case ItemSettingType::NEXT_STATE:
				{
					int nIndex = -1;
					for( int j = 0; j < SIZEOF(STATE_MAPS); j ++ ) {
						if( HPLString::equals( str, STATE_LABELS[j] ) ){
							nIndex = j;
							break;
						}
					}
					if( nIndex >= 0 && nIndex < SIZEOF(STATE_MAPS)) {
						this->m_nNextState = STATE_MAPS[nIndex];
					} else {
						HPLError::msg( "Error", MB_OK, "Unknown state:[%s]:%s",
							str.c_str(), strLine.c_str());
					}
				}
				break;
			case ItemSettingType::CURE:
				this->m_nCure = atoi( str.c_str());
				break;
			case ItemSettingType::FLAGS:
				{
					vector<string> aryItems = HPLString::split( str, "/");
					if( aryItems.size() < ItemFlagType::MAX_NUM) {
						HPLError::msg("Error", MB_OK,"Flag̐s[%d]<[%d]:%s:%s",
							aryItems.size(), ItemFlagType::MAX_NUM, str.c_str(),
							strLine.c_str());
						return -1;
					}
					for( int j = 0; j < ItemFlagType::MAX_NUM; j ++) {
						string strItem = HPLString::trim( aryItems[j] );
						this->m_aryFlags[j] = HPLString::equals(strItem, TRUE_STR);
					}
				}
				break;
			case ItemSettingType::POINT:
				{
					this->m_nPoint = atoi(str.c_str());
				}
				break;
			}
		} catch( ... ) {
			HPLError::msg( "Error", MB_OK,"ϊG[" );
			return -1;
		}
	}

	return 0;
}

void GItemManager::setup() {
	vector<string> vecLines = HPLFile::loadStringFromFile( "data/image/Items/ItemSettings.txt", true);
	ASSERT_IT_EQUAL( vecLines.size(), Sequence::Item::MAX_NUM );
	for( int i = 0; i < Sequence::Item::MAX_NUM; i ++ ) {
		this->m_arySettings[i].setup( vecLines[i] );
	}
}


GItemManager::GItemManager() : HPLObjectManager() {
	// ACeXg
	this->m_vecNames = HPLFile::loadStringFromFile( "data/image/Items/ItemLabel.txt", true );
	HPLError::assertIt( this->m_vecNames.size() == Sequence::Item::MAX_NUM,
		"ACe̎ނƃACeXgv܂(Names.size()=%d,ItemNum=%d)",
		this->m_vecNames.size(), Sequence::Item::MAX_NUM);
	// 
	m_vecItemDescriptions = HPLFile::loadStringLinesFromFile( 
		"data/image/Items/ItemDescription.txt", true);

	ASSERT_IT_EQUAL( m_vecItemDescriptions.size(), Sequence::Item::MAX_NUM );

	this->setup();

	int COLORS[][4] = {
		{Sequence::Item::FLYING_COIN,255,255,0},			// F
		{Sequence::Item::COIN_10,0,255,0},					// 
		{Sequence::Item::FLYING_COIN_10,0,255,0},
		{Sequence::Item::COIN_50,0,255,255},				// 
		{Sequence::Item::FLYING_COIN_50,0,255,255},
		{Sequence::Item::COIN_100,255,0,100},				// Ԏ
		{Sequence::Item::FLYING_COIN_100,255,0,100},
		{Sequence::Item::COIN_500,100,0,255},				// 
		{Sequence::Item::FLYING_COIN_500,100,0,255},
		{Sequence::Item::COIN_1000,255,0,255},				// 
		{Sequence::Item::FLYING_COIN_1000,255,0,255},
		{Sequence::Item::COIN_5000,100,100,255},			// v`iij
		{Sequence::Item::FLYING_COIN_5000,100,100,255},
		{Sequence::Item::COIN_10000,255,255,100},			// 
		{Sequence::Item::FLYING_COIN_10000,255,255,100},
	};
	// coin color
	for( int i = 0; i < SIZEOF(COLORS); i ++ ) {
		this->m_mapCoinColor[COLORS[i][0]] = HPLColor(COLORS[i][1],COLORS[i][1],COLORS[i][1] );
	}
}
string GItemManager::getDescription( int nIndex ) {
#ifdef _DEBUG
	ASSERT_IT_INDEX( nIndex, 0, this->m_vecNames.size() );
//	HPLError::assertIt( nIndex >= 0 && nIndex < this->m_vecNames.size() );
#endif
	string str = this->m_vecNames[nIndex];
	str += string(":" ) + this->m_vecItemDescriptions[nIndex];
	return str;
}
string GItemManager::getName( int nIndex ) {
#ifdef _DEBUG
	ASSERT_IT_INDEX( nIndex, 0, this->m_vecNames.size() );
//	HPLError::assertIt( nIndex >= 0 && nIndex < this->m_vecNames.size() );
#endif
	return this->m_vecNames[nIndex];
}

void GItemManager::process(bool bIsMove, bool bIsAI) {

	HPLRect merginedGameRange = getStaticData()->getMerginedGameRange();
	for( list<GItem*>::iterator it = this->getList()->begin();
	it != this->getList()->end(); )
	{
		GItem* lpItem = *it;

		if( merginedGameRange.isShare(lpItem->getHitArea()) ==false) {
			it ++;
			continue;
		}

		if( bIsMove ) {
			getStaticData()->collisionObject( lpItem );
			this->algorithm(lpItem);

			this->checkHitToPlayer( lpItem );
		}
		
		if( lpItem->isAlive() ) {
			lpItem->draw();
			it ++;
		} else {
			delete *it;
			it = this->getList()->erase(it);
		}
	}
}

GItem* GItemManager::create(int nType, HPLVector vCenter) {
	GItem* lpItem = new GItem( nType );
	lpItem->setCenter( vCenter );
	this->getList()->push_back( lpItem );

	return lpItem;
}


void GItemManager::algorithm(GItem* lpItem) {
	int nType = lpItem->getType();

	// ^C
	int nTileType = TileType::NOSPECIAL;

	// ꕔ̓G̓^CԂCɂ
	switch( lpItem->getType() ) {
	case Sequence::Item::EVI:
	case Sequence::Item::AJI:
		{
			HPLVector vCen = lpItem->getCenter();
			int nHeadTileID = 0;
			int nFootTileID = 0;
			getStaticData()->m_ObjectManager.getTileType(
				vCen.getX(), vCen.getY(), nTileType, nHeadTileID, nFootTileID );
			switch( nTileType ) {
			case TileType::ON_THE_WATER:
			case TileType::SWIM:
				lpItem->setObjectAttribute( ObjectAttribute::GRAVITY, false );
				break;
			default:
				lpItem->setObjectAttribute( ObjectAttribute::GRAVITY, true );
				// d͂󂯂
				break;
			}
		}
			
		break;
	case Sequence::Item::BOSS_ABILITY:
		// TODO PARTICLEGtFNgBF͎g̐F
		{
			// star index
			int nStar = getStaticData()->m_nSelectedStar;
			int nPlace = getStaticData()->m_nSelectedPlace;
			int nMapIndex = getStaticData()->getMapIndex();
//			getStaticData()->m_Setting.maps.getStarAndPlaceFromMapIndex( nMapIndex, nStar, nPlace );
			// Xe[WJ[ɃX
			int nR = GEnum::STAR_COLOR[nStar][0];
			int nG = GEnum::STAR_COLOR[nStar][1];
			int nB = GEnum::STAR_COLOR[nStar][2];
			// wind num
			const int WIND_NUM = 3;
			// wind speed
			const int WIND_SPEED = 2;
			// effect sequence
			HPLSequence* lpSeq = lpItem->getSequence();
			int nMaxRadius = lpSeq->getImageInformation()->srcRect.getWidth() / 2;
			for( int i = 0; i < WIND_NUM; i ++ ) {
				HPLVector vCenter = lpItem->getCenter();
				int nDeg = HPLMath::nextInt(0,HPLMathConsts::DEGREE_FULL_ROUND );
				HPLVector v;
				v.setByDegree( nDeg );
				int nRadius = HPLMath::nextInt(10, nMaxRadius );
				v.multiple( nRadius );
				vCenter.add( v );

				GEffect* lpWind = getStaticData()->m_ObjectManager.m_EffectManager.create(
					Sequence::Effect::WIND_RAIL, vCenter, true );
					// set velocity and direction (flow up )
				lpWind->setVelocityAngle( 0, -WIND_SPEED) ;
				// set color
				lpWind->setColor(nR,nG,nB);
			}
		}
		break;
	}

}

bool GItemManager::checkHitToPlayer( GItem* lpItem ) {

	if( lpItem->isAlive() == false ) {
		return false;
	}

	GPlayer* lpPlayer = &getStaticData()->m_ObjectManager.m_Player;
	if( lpPlayer->isAlive() == false ) {
		// łȂ
		return false;
	}

	HPLRect rHitArea = lpItem->getHitArea();
	HPLRect rPlayerHitAre = lpPlayer->getHitArea();
	if( rHitArea.isShare( rPlayerHitAre ) ) {
		// 
		lpItem->setAlive( false );

		// 擾
		lpPlayer->getItem( lpItem->getType(), 1 );

	}
	return false;
}

/** w肵ɑΉRC쐬
  */
void GItemManager::createCoins( HPLVector& vCreateCenter, int nNum ) {
	int nBlueNum = nNum / COINS_PER_BLUE;
	int nCoinNum = nNum - COINS_PER_BLUE * nBlueNum;

	if( nBlueNum > 0 ) {
		this->createCoins( vCreateCenter, Sequence::Item::BLUE_COIN, nBlueNum );
	}
	
	if( nCoinNum > 0 ) {
		this->createCoins( vCreateCenter, Sequence::Item::COIN, nCoinNum );
	}
}

void GItemManager::createCoins( HPLVector& vCreateCenter, int nItemType, int nNum ) {
	for( int i = 0; i < nNum; i ++ ) {
		GItem* lpItem = this->create( nItemType, vCreateCenter );
		lpItem->setVelocityAngle( HPLMath::nextFloat(- ITEM_CREATE_ERR_X, ITEM_CREATE_ERR_X ),
			-HPLMath::nextFloat(ITEM_CREATE_ERR_Y_MIN, ITEM_CREATE_ERR_Y_MAX ) );
	}
}

