#include "GLibraryScene.h"

#include "GStaticData.h"
#include <hpllib/general/HPLString.h>

#include "GPlayerManager.h"

namespace GLibrarySceneConsts {
	namespace LibScreenType {
		enum {
			SELECT_OBJECT_TYPE,
			SELECT_TYPE,
			SHOW,
			MAX_NUM
		};
	}

	static char OBJECT_TYPE_LABELS[][200] = {
		"Enemy",
		"Item",
//		"Neutral"
	};
	namespace ObjectTypeIndexType {
		enum {
			ENEMY,
			ITEM,
//			NEUTRAL,
			MAX_NUM
		};
	};
	static int OBJECT_TYPES[] = {
		ObjectType::ENEMY,
		ObjectType::ITEM,
//		ObjectType::NEUTRAL,
	};

	static const int ROWS_PER_PAGE = 15;
}

using namespace GLibrarySceneConsts;
using namespace GEnum;
using namespace std;

GLibraryScene::GLibraryScene() {
	const int WAIT_COUNTER = 200;
	m_NextSequenceWaitCounter.setWaitFrameNum( WAIT_COUNTER );

	this->m_nObjectType = ObjectType::ENEMY;
	this->m_nType = 1;
	this->m_nScreenType = LibScreenType::SELECT_OBJECT_TYPE;

	this->cursorY = 0;
	this->maxCursorY = this->getMaxCursorY();
	this->m_nObjectTypeCursor = -1;
	this->m_nTypeCursor = -1;
	this->m_nTypePage = -1;
	this->m_cntWaitToStart.setWaitFrameNum(20);
	this->m_cntWaitForNext.setWaitFrameNum(1);
}

int getMax( int nObjectType ) {
	switch( nObjectType ) {
	case ObjectType::ENEMY:
		return EnemyType::MAX_NUM;
	case ObjectType::ITEM:
		return Sequence::Item::MAX_NUM;
	default:
		ASSERT_IT_TYPE(nObjectType, "Invalid type");
	}
	return 1;
}
/**
  * őJ[\wBłɃJ[\ʒuiʊ֐̕Hj
  * y[Wő吔Őݒ
  */
int GLibraryScene::getMaxCursorY() {
	// y[Wԍ
	switch( this->m_nScreenType ) {
	case LibScreenType::SELECT_OBJECT_TYPE:
		this->m_nMaxPage = 1;
		return SIZEOF(OBJECT_TYPE_LABELS);
	case LibScreenType::SELECT_TYPE:
		{
			int nMax = getMax( this->m_nObjectType );
			this->m_nMaxPage = nMax / ROWS_PER_PAGE+1;
			if( ( nMax % ROWS_PER_PAGE ) != 0 ) {
				// [L
				// y[W𑝂₷
				this->m_nMaxPage ++ ;
			}
			int nCurrentPageMaxRow = ROWS_PER_PAGE;
			if( this->m_nPage == this->m_nMaxPage - 1 ) {
				// 傤ǍŏIy[W
				// [ɂ
				nCurrentPageMaxRow = nMax % ROWS_PER_PAGE;
			}
			return nCurrentPageMaxRow;

		}
	case LibScreenType::SHOW:
		return 1;
	}
	return 1;
}

GLibraryScene::~GLibraryScene() {
}

void GLibraryScene::init() {
	this->m_NextSequenceWaitCounter.reset();
	getStaticData()->m_MixerManager.playMusic(Music::BAD_END, true );

	this->m_nObjectType = ObjectType::ENEMY;

	this->m_nObjectType = 0;
	this->m_nType = 0;
//	this->m_nSequenceID = 0;
	this->m_nPage = 0;
	this->cursorY = 0;

	this->getMaxCursorY();
	this->m_cntWaitToStart.reset();
	this->m_cntWaitForNext.reset();

	getStaticData()->m_ObjectManager.clear();
	getStaticData()->setupMap( SpecialMapIndex::LIBRARY, true );

	getStaticData()->m_EventPackManager.load( "data/event/EnemyDescriptionEvigen.txt");

}
void GLibraryScene::quit() {
	getStaticData()->m_EventImage.deactivate();
	getStaticData()->m_EventPackManager.clear();

	// object clear
	getStaticData()->m_ObjectManager.clear();
}
/** CvZX
  */
void GLibraryScene::process() {
	getStaticData()->isEvent( EventStepType::DEFAULT, true, false);

	bool bIsSelect = false;
	bool bIsCancel = false;
	bool bIsUpNow = false;
	bool bIsDownNow = false;
	bool bIsLeftNow = false;
	bool bIsRightNow = false;

	/* vf				ViI		tF[hAEg
	   -J[\		o				x
	   -			x				x
	   -LZ		x				x

	   ViIShow[ĥݗLȂ̂ŁAShowȊÕ[h͉eȂ͂
	 */
	if( 
		getStaticData()->isFadingOutDX() == false )	// not fading out
	{
		bIsUpNow = getStaticData()->getInputManager()->isPressedNow( KeyMap::UP );
		bIsDownNow = getStaticData()->getInputManager()->isPressedNow( KeyMap::DOWN );
		bIsLeftNow = getStaticData()->getInputManager()->isPressedNow( KeyMap::LEFT );
		bIsRightNow = getStaticData()->getInputManager()->isPressedNow( KeyMap::RIGHT );
		const int TARGET_KEY[] = {
			KeyMap::UP,
			KeyMap::DOWN,
			KeyMap::LEFT,
			KeyMap::RIGHT,
		};
		int nIndex = -1;
		for( int i = 0; i < SIZEOF(TARGET_KEY); i ++ ) {
			if( getStaticData()->getInputManager()->isPressed( TARGET_KEY[i] ) ) {
				nIndex = i;
				break;
			}
		}
		if( nIndex >= 0 ) {	// 
			if( this->m_cntWaitToStart.step().isFinishAnimation ) {	// AړJnI
				if( this->m_cntWaitForNext.step().isFinishAnimation ) {	// Aړ̃EFCgI
					this->m_cntWaitForNext.reset();
					switch(nIndex ) {
					case KeyMap::UP:	bIsUpNow = true;				break;
					case KeyMap::DOWN:	bIsDownNow = true;				break;
					case KeyMap::LEFT:	bIsLeftNow = true;				break;
					case KeyMap::RIGHT:	bIsRightNow = true;				break;
					}
				}
			}
		} else {
			this->m_cntWaitForNext.reset();
			this->m_cntWaitToStart.reset();
		}

		bool bANow = false;
		bool bBNow = false;
		bool bESC = false;
		if( getStaticData()->m_Game.isScenario() == false ) {	// not scenario
			bANow = getStaticData()->getInputManager()->isPressedNow( KeyMap::A );
			bBNow = getStaticData()->getInputManager()->isPressedNow( KeyMap::B );

			// /LZ
	//		bool bR = CheckHitKey( KEY_INPUT_R ) != 0;
			bESC = CheckHitKey( KEY_INPUT_ESCAPE ) != 0;
		} else {	// in the scenario
			if( getStaticData()->getInputManager()->isPressedNow( KeyMap::B ) ) { // pressed cancel
				// stop scenario
				getStaticData()->endScenario();
			}
		}
		
		if( bANow ) {
			bIsSelect = true;
		} else if( bBNow || bESC ) {
			bIsCancel = true;
		}
	}

	bool bIsEnableCheckEvent = true;

	switch( this->m_nScreenType ) {
	case LibScreenType::SELECT_OBJECT_TYPE:	// object type
		{
			this->procMainObjectType();
			if( bIsSelect ) {				// IL[ꂽ
				this->procSelObjectType();
			} else if( bIsCancel ) {		// LZL[ꂽ
				// ^Cg֖߂
				getStaticData()->startFadeOutDX( SceneType::TITLE, -1 );

			} else {
				// J[\ړ
				this->commonArrow( bIsUpNow, bIsDownNow,
					this->cursorY, 0, this->maxCursorY, true );
			}
		}
		break;
	case LibScreenType::SELECT_TYPE:		// type
		{
			this->procMainType();
			if( bIsSelect ) {
				this->procSelType();
				// ŏ̃CxgXLbvȂ悤ɁA`FbNCxg𖳌ɂ
				bIsEnableCheckEvent = false;
			} else if( bIsCancel ) {
				// Object type select֖߂
				this->m_nScreenType = LibScreenType::SELECT_OBJECT_TYPE;
				// cursor𕜌
				this->cursorY = this->m_nObjectTypeCursor;
			} else {
				int nDY = 0;
				if( bIsUpNow ) {	nDY = -1;	}
				if( bIsDownNow ) {	nDY = 1;	}
				this->cursorY += nDY;
				if( this->cursorY < 0 ) {
					this->m_nPage --;
					if( this->m_nPage < 0 ) {
						this->m_nPage = this->m_nMaxPage - 1;
					}
					this->maxCursorY = this->getMaxCursorY();
					this->cursorY = this->maxCursorY - 1;
				}
				if( this->cursorY >= this->maxCursorY ) {
					this->m_nPage ++;
					if( this->m_nPage >= this->m_nMaxPage ) {
						this->m_nPage = 0;
					}
					this->cursorY = 0;
				}
//				this->commonArrow( bIsUpNow, bIsDownNow,
//					this->cursorY, 0, this->maxCursorY );
				if( nDY == 0 ) {
					// Ȅ
					if( this->commonArrow( bIsLeftNow, bIsRightNow,
						this->m_nPage, 0, this->m_nMaxPage, true ) == 1 ) 
					{
						this->cursorY = 0;
					}
					
				}
			}
		}
		break;
	case LibScreenType::SHOW:
		{
			this->procMainShow();
			if( bIsSelect ) {
				this->procSelShow();
			} else if( bIsCancel ) {
				// Object type select֖߂
				this->m_nScreenType = LibScreenType::SELECT_TYPE;
				// ݂̃^CvɁAy[Wԍ𕜌
				this->m_nPage = this->m_nType / ROWS_PER_PAGE;
				// ݂̃^CvɁAJ[\ʒu𕜌
				this->cursorY = this->m_nType % ROWS_PER_PAGE;
			} else {
				// max number of selected object type
				int nMax = getMax( this->m_nObjectType );
				// old type
				int nOldType = this->m_nType;
				// check whether key for changing type was pressed
				this->commonArrow( bIsLeftNow || bIsUpNow, bIsRightNow || bIsDownNow,
					m_nType, 0, nMax, true );

				if( nOldType != this->m_nType ) {	// type was changed

					if( this->m_nType >= 0 ) {

						this->startEvent();

						switch( this->m_nObjectType ) {
						case ObjectType::ENEMY:
							// GύX
							this->changeEnemy();
							break;
						case ObjectType::ITEM:
							this->changeItem();
							break;
//						case ObjectType::NEUTRAL:
//							break;
						}
					}
				}

				if( CheckHitKey( KEY_INPUT_R ) ) {
					this->resetPlayer();
				}

/*				this->m_nType += nDelta;

				if( this->m_nType < 0 ) {
					this->m_nType = nMax;
				} else if( this->m_nType >= nMax ) {
					this->m_nType = 0;
				}
				*/
			}
		}
		break;
	}
	if( bIsEnableCheckEvent ) {
		getStaticData()->checkEvent(false, EventStepType::DEFAULT,
			true, false);
	}
}
//////////////////////////////////////////////////////////////////////////////////////
// proc Object Type
const int INTERVAL_Y = 30;
const int START_X = 100;
const int START_Y = 100;
const int CURSOR_START_X = 50;
const int CURSOR_START_Y = 95;
const int TITLE_X = 100;
const int TITLE_Y = 50;
void GLibraryScene::procMainObjectType() {

	for( int i = 0; i < ObjectTypeIndexType::MAX_NUM; i ++ ) {
		int nX = 100;
		int nY = 100 + i * INTERVAL_Y;

		DrawString( nX, nY, OBJECT_TYPE_LABELS[i], GetColor(255,255,0));
			
	}

	// J[\`
	{
		int nX = CURSOR_START_X;
		int nY = CURSOR_START_Y + this->cursorY * INTERVAL_Y;
		getStaticData()->m_Basic.m_Graphics.draw(
			nX, nY, getStaticData()->getSequence( SpriteType::COMMON, Sequence::Common::CURSOR ));
	}
}

void GLibraryScene::procSelObjectType() {
	// I
	this->m_nScreenType = LibScreenType::SELECT_TYPE;

	// IuWFNg^CvƂĊo
	this->m_nObjectType = OBJECT_TYPES[this->cursorY];

	if( this->m_nObjectTypeCursor != this->cursorY ) {
		// ύX
		// ->J[\ʒuƃy[Wʒu擪ɎĂ
		this->m_nTypeCursor = 0;
		this->m_nTypePage = 0;
	}
	// ʒuoĂ
	this->m_nObjectTypeCursor = this->cursorY;

	// ősvZ
	this->maxCursorY = this->getMaxCursorY();

	// se Đ
	getStaticData()->m_MixerManager.play(SE::KETTEI,true);
}

//////////////////////////////////////////////////////////////////////////////////////
// proc Type
const int PAGE_X = 500;
const int PAGE_Y = 550;
void GLibraryScene::procMainType() {
	// ݂̃y[W̃Xg\s
	int nStartIndex = this->m_nPage * ROWS_PER_PAGE;
	int nEndIndex = (this->m_nPage +1)*ROWS_PER_PAGE;
	string strTitle("Object Type: ");
	switch( this->m_nObjectType ) {
	case ObjectType::ENEMY:
		nEndIndex = min( nEndIndex, EnemyType::MAX_NUM );
		strTitle += string("Enemy");
		break;
	case ObjectType::ITEM:
		nEndIndex = min( nEndIndex, Sequence::Item::MAX_NUM );
		strTitle += string("Item");
		break;
/*	case ObjectType::NEUTRAL:
		nEndIndex = min( nEndIndex, Sequence::Neutral::MAX_NUM );
		strTitle += string("Neutral");
		break;
		*/
	}
	this->maxCursorY = nEndIndex - nStartIndex;
	if( this->cursorY >= this->maxCursorY ) {
		this->cursorY = this->maxCursorY - 1;
	}
	// ^Cg\iF"Object Type: Enemy"j
	DrawString(TITLE_X, TITLE_Y, strTitle.c_str(), GetColor(255,255,0));

	// y[Wԍ
	DrawString( PAGE_X, PAGE_Y, HPLString::chrToStr(
		"Page [ %5d / %5d ]", this->m_nPage+1, this->m_nMaxPage ).c_str(), GetColor(200,200,255 ) );

/*	if( ObjectType::ENEMY != this->m_nObjectType ) {
		// EnemyȊO
		// Ȃ̂Return
		return;
	}*/
	// y[W̕`
	for( int i = nStartIndex; i < nEndIndex; i ++ ) {
		int nX = START_X;
		int nY = START_Y + (i-nStartIndex) * INTERVAL_Y;
		string strLine = HPLString::chrToStr( "[%5d] ", i );
		
		switch( this->m_nObjectType ) {
		case ObjectType::ENEMY:
			// G̎擾
			// TODO ԍ`FbN(getEnemyName̕łĂ邩OK?)
			strLine += getStaticData()->m_ObjectManager.m_EnemyManager.getEnemyName( i );
			break;
		case ObjectType::ITEM:
			strLine += getStaticData()->m_ObjectManager.m_ItemManager.getName(i);
			//
			break;
/*		case ObjectType::NEUTRAL:
			//
			strLine += getStaticData()->m_ObjectManager.m_NeutralManager.getName(i);
			break;*/
		}
		DrawString( nX, nY, strLine.c_str(), GetColor(255,255,255)); 
	}
	
	// J[\`
	{
		int nX = CURSOR_START_X;
		int nY = CURSOR_START_Y + this->cursorY * INTERVAL_Y;
		getStaticData()->m_Basic.m_Graphics.draw(
			nX, nY, getStaticData()->getSequence( SpriteType::COMMON, Sequence::Common::CURSOR ));
	}
}

void GLibraryScene::procSelType() {
	// SHOW[h
	this->m_nScreenType = LibScreenType::SHOW;

	// őʒu擾
//	this->maxCursorY = this->getMaxCursorY();

	// y[WITypeԍ𓾂
	this->m_nType = this->cursorY + this->m_nPage * ROWS_PER_PAGE;

	// y[WԍoĂ
	this->m_nTypePage = this->m_nPage;

	// se Đ
	getStaticData()->m_MixerManager.play(SE::KETTEI,true);

	switch( this->m_nObjectType ) {
	case ObjectType::ENEMY:
		// Gǉ
		this->changeEnemy();
	}

	// event start
	this->startEvent();
}

//////////////////////////////////////////////////////////////////////////////////////
// proc Show
static GPlayerManager playerManager;
void GLibraryScene::procMainShow() {
	HPLScroller* lpScroller = getStaticData()->m_ScrollManager.getScroller(0);
	// ^C\
	getStaticData()->m_BlockManager.draw(
		getStaticData()->m_Basic.m_Graphics, lpScroller, false, true );

	// vC[
	GPlayer* lpPlayer = &getStaticData()->m_ObjectManager.m_Player;

	// is move object (except effect )
	bool bIsMoveObject = getStaticData()->m_Game.isTheWorld() == false;

	if( bIsMoveObject ){
		// 
		HPLVector vVelocityAngle = lpPlayer->getVelocityAngle();
		HPLVector center = lpPlayer->getCenter();
		playerManager.collisionCheck(
			lpPlayer,
			&center, &vVelocityAngle);

		// ړ
		vVelocityAngle = lpPlayer->getVelocityAngle();
		playerManager.move( lpPlayer, &vVelocityAngle );
	}

	// draw player
	lpPlayer->draw();

	// draw & process enemy
	getStaticData()->m_ObjectManager.m_EnemyManager.process( bIsMoveObject, bIsMoveObject );

	// draw & process shot
	getStaticData()->m_ObjectManager.m_ShotManager.process( bIsMoveObject, bIsMoveObject );

	// draw & process effect
	getStaticData()->m_ObjectManager.m_EffectManager.process( true, true );

	switch( this->m_nObjectType ) {
	case ObjectType::ENEMY:
		if( getStaticData()->m_ObjectManager.m_EnemyManager.getList()->size() == 0 ) {	// G
			// Gǉ
			this->addEnemy();
		}
		break;
	case ObjectType::ITEM:
		if( getStaticData()->m_ObjectManager.m_ItemManager.getList()->size() == 0 ) {
			this->addItem();
		}
		break;
	}

	// }EXŃvC[ړł悤ɂ
	{
		HPLScroller* lpScroller = getStaticData()->m_ScrollManager.getScroller(0);
		int nMouseInput = GetMouseInput();
		// }EXW
		int nMouseX = 0;
		int nMouseY = 0;
		GetMousePoint( &nMouseX, &nMouseY );
		getStaticData()->m_ObjectManager.checkAndMoveByMouse( nMouseInput,
			MOUSE_INPUT_LEFT, nMouseX, nMouseY,
			lpPlayer, lpScroller );
	}

	// front block drawing
	getStaticData()->m_BlockManager.draw(
		getStaticData()->m_Basic.m_Graphics, lpScroller, true, true );

	// draw object name
	getStaticData()->m_ObjectManager.drawObjectName( LayoutType::TOP_LEFT,
		getStaticData()->m_Game.isTheWorld() == false,
		this->m_nObjectType, this->m_nType );
/*	{
		int nCenterX = getStaticData()->m_Setting.windowWidth / 2;

		// Gݒi摜pj
		GEnemySetting* lpEnemySetting = getStaticData()->m_ObjectManager.m_EnemyManager.getEnemySetting( this->m_nType );

		int nCenterY = getStaticData()->m_Setting.windowHeight - 200 - lpEnemySetting->m_HitRect.getHeight() / 2;

		if( lpEnemySetting->m_nSpriteID >= 0 ) {
			// V[PX擾
			HPLSequence* lpSeq = getStaticData()->getSequence(
				lpEnemySetting->m_nSpriteID,
				this->m_nSequenceID );

			getStaticData()->m_Basic.m_Graphics.draw(
				nCenterX, nCenterY, lpSeq, FaceDirection::LEFT, 0,1 );

			if( lpSeq->step().isFinishAnimation ) {
				lpSeq->resetCounter();
			}

			// 
			if( this->m_NextSequenceWaitCounter.step().isFinishAnimation ) {
				// reset
				this->m_NextSequenceWaitCounter.reset();

				// 
				this->m_nSequenceID ++;
				if( lpSeq->getNumOfInformations() <= this->m_nSequenceID ) {
					this->m_nSequenceID = 0;
				}
			}
		}
	}*/
	if(getStaticData()->m_EventImage.isActivated()){	// event - image
		// draw image
		HPLSequence* lpSeq = getStaticData()->getSequence( getStaticData()->m_EventImage.spriteID,
			getStaticData()->m_EventImage.seqID );
		getStaticData()->m_Basic.m_Graphics.draw( 
			0,0, lpSeq );

	}
}
//	this->m_nPage = 0;
//	this->cursorY = 0;


void GLibraryScene::procSelShow() {
//	this->maxCursorY = this->getMaxCursorY();
	// Ȃ
}

void GLibraryScene::changeEnemy() {
	// Gւ
	// G폜
	this->clearEnemies();

	// G
	this->addEnemy();

	this->resetPlayer();
}
void GLibraryScene::clearEnemies() {
	for( list<GEnemy*>::iterator it = getStaticData()->m_ObjectManager.m_EnemyManager.getList()->begin();
		it != getStaticData()->m_ObjectManager.m_EnemyManager.getList()->end();
		 )
	{
		GEnemy* lpEnemy = *it;
		// SGtFNg
		getStaticData()->m_ObjectManager.m_EffectManager.createBlood( lpEnemy->getCenter() );

		// 폜
		delete lpEnemy;
		it = getStaticData()->m_ObjectManager.m_EnemyManager.getList()->erase(it);
	}
}

void GLibraryScene::addEnemy() {
	getStaticData()->m_ObjectManager.m_EnemyManager.create( this->m_nType,
		HPLVector( (float)getStaticData()->m_Setting.windowWidth /2,
		(float)getStaticData()->m_Setting.windowHeight / 2), -1);
}

/////////////////////////
void GLibraryScene::changeItem() {
	this->clearItems();

	this->addItem();

	this->resetPlayer();
}
void GLibraryScene::clearItems() {
	getStaticData()->m_ObjectManager.m_ItemManager.clear();
	/*for( list<GItem*>::iterator it = getStaticData()->m_ObjectManager.m_ItemManager.getList()->begin();
		it != getStaticData()->m_ObjectManager.m_ItemManager.getList()->end();
		 )
	{
	}*/
}
void GLibraryScene::addItem() {
	getStaticData()->m_ObjectManager.m_ItemManager.create( this->m_nType,
		HPLVector( (float)getStaticData()->m_Setting.windowWidth /2,
		(float)getStaticData()->m_Setting.windowHeight / 2));
}

//////////////////////////////
void GLibraryScene::resetPlayer() {
	// vC[
	GPlayer* lpPlayer = &getStaticData()->m_ObjectManager.m_Player;

	// reset common
	lpPlayer->resetLibrary();
//	lpPlayer->setAlive( true);
}

void GLibraryScene::startEvent() {
	int nObjectTypeBias = 0;	// bias for event id

	switch( this->m_nObjectType ) {
	case ObjectType::ITEM:
		nObjectTypeBias = 1000;
		break;
//						case ObjectType::NEUTRAL:
//							nObjectTypeBias = 2000;
//							break;
	}
	getStaticData()->startEvent( this->m_nType + nObjectTypeBias, true);
}
