#include "GNeutralManager.h"

#include "GStaticData.h"

#include <hpllib/action/HPLActionMoveTool.h>
#include <hpllib/math/HPLMath.h>
#include <hpllib/system/HPLFile.h>
#include <hpllib/general/HPLString.h>
#include <hpllib/game/map/HPLMapData.h>

#include "GSelectStageScene.h"

namespace NeutralSettingType {
	enum {
		NAME,
		TYPE,
		HIT_RECT,
		HIT_TO_SHOT,
		INVINSIBLE,
		GRAVITY,
		MOUNT_TYPE,
		EXTRA_SEQ,
		MAX_NUM
	};
}

using namespace std;
using namespace GEnum;
using namespace HPL::MapData::Item;

const float MOVER_ROTATE_SPEED = 1;

GNeutralManager::GNeutralManager() : HPLObjectManager() {

	char NEUTRAL_SETTING_FILE_PATH[] = "data/image/Neutral/NeutralSetting.txt";
	int nRet = this->loadSettings( NEUTRAL_SETTING_FILE_PATH);
	HPLError::assertIt(nRet == 0,
		"Failed loading Neutral Settings.[Error Code:%d]", nRet );

	//char NEUTRAL_LABEL_FILE_PATH[] = "data/image/Neutral/NeutralLabel.txt";
	//HPLFile::loadStringLinesFromFile( 
}

/*
string GNeutralManager::getName( int nIndex ) {
	ASSERT_IT_INDEX(nIndex,0, Sequence::Neutral::MAX_NUM );
	return this->m_aryNames[nIndex];
}
*/

void GNeutralManager::process(bool bIsMove, bool bIsAI) {

	this->m_bIsChecked = false;

	this->m_vecAdds.clear();

	HPLRect merginedGameRange = getStaticData()->getMerginedGameRange();
	int TILE_SIZE = getStaticData()->m_Setting.tileSize;
	HPLMatrix<int>* mat = getStaticData()->m_BlockManager.getBlockMatrix();
	const int MERGINE = 200;
	// S}bv̈{]̗̈
	HPLRect merginedBlockRange( -MERGINE, -MERGINE,
		mat->getColumnSize() * TILE_SIZE + MERGINE * 2,
		mat->getRowSize() * TILE_SIZE + MERGINE * 2 );

	bool bSpecialStage = 
		getStaticData()->getMapType() == MapType::BOSS ||
		getStaticData()->getMapType() == MapType::MID_BOSS;
	for( list<GNeutral*>::iterator it = this->getList()->begin();
	it != this->getList()->end(); )
	{
		GNeutral* lpNeutral = *it;

		switch( lpNeutral->getType() ) {
		case Sequence::Neutral::LOCK_BLOCK:	// lock
			if( getStaticData()->m_Game.getTempFlagBit( TempFlagType::SWITCH_LOCK, lpNeutral->getColor() ) ) {	// switch삵Ă
				// switch on -> ȂȂ
				it ++;
				continue;
			}
			break;
		}
		
		HPLRect rHitArea = lpNeutral->getHitArea();
		
		bool bIsSpecialType = false;
		if( lpNeutral->getType() == Sequence::Neutral::EFFECT_GENERATOR ) {
			int nEffectType = lpNeutral->getPlacerParameter(0);	// effect type
			int nGenerateType = lpNeutral->getPlacerParameter(2);
			if( nEffectType == Sequence::Effect::GENERATOR && nGenerateType == Sequence::Effect::LAVA_UNDER) {
				bIsSpecialType = true ;
			}
		}
		if( lpNeutral->isMover() || bSpecialStage || bIsSpecialType ) {
			// 
		} else {
			if( merginedGameRange.isShare( rHitArea ) == false )	// Q[̈o
			{
				switch( lpNeutral->getType() ) {
				case Sequence::Neutral::GENERATOR:
//				case Sequence::Neutral::BU_CLOUD:
					// 
					break;
				default:
					it ++;
					////
					// ʊOȂ瓮삵Ȃ
					// (eɂȂ)
					continue;
				}
			}
		}
		if( false == merginedBlockRange.isShare( rHitArea )) {
			// ubN̈Oo
			//if( lpNeutral->getType() != Sequence::Neutral::BU_CLOUD ) //lpNeutral->getObjectAttribute( ObjectAttribute::HIT_BLOCK_TO_DIE ) &&
				// BU_CLOUDȊO
				//lpNeutral->isMover() && 
			{
				// Ɏ
				lpNeutral->die(NULL);
			}
		}

		if( lpNeutral->isAlive() && bIsMove ) {	// alive & movable

			CollisionState stateX, stateY;
			// collision check and move
			getStaticData()->collisionObject( lpNeutral, stateX, stateY );

			if( lpNeutral->getObjectAttribute( ObjectAttribute::HIT_TO_BLOCK ) &&	// hittable to block
				lpNeutral->getObjectAttribute( ObjectAttribute::HIT_BLOCK_TO_DIE ) &&	// die when hit to block
				( stateX.isHitX != HitDirectionX::NONE || stateY.isHitY != HitDirectionY::NONE ) )	// hit top/bottom
			{
				// ubNɃqbgBqbgƎ
				// ǂɃqbg
				// ->
				lpNeutral->die(NULL);
			}

			if( lpNeutral->isAlive() ) {	// alive

				if( bIsAI ) {
					this->algorithm(lpNeutral);	// do algorithm
				}

				switch( lpNeutral->getType() ) {
//				case Sequence::Neutral::BU_CLOUD:
//				case Sequence::Neutral::BACK_CLOUD:
				case Sequence::Neutral::GENERATOR:
					break;
				default:
					// vC[Ƃ̓蔻
					this->checkHitToPlayer( lpNeutral, &getStaticData()->m_ObjectManager.m_Player );
					if( getStaticData()->isCOMPlayer() ) {
						this->checkHitToPlayer( lpNeutral, &getStaticData()->m_ObjectManager.m_COMPlayer );
					}

					// VbgƂ̓蔻
					this->checkHitToShot( lpNeutral );

					// O
					getStaticData()->m_ObjectManager.moveObjectForce( lpNeutral );
				}
			}
		}

		
		if( lpNeutral->isAlive() ) {	// alive
			// draw neutral
			lpNeutral->draw();
			it ++;
		} else {	// dead

			// do something after dead
			{
				switch( lpNeutral->getType() ) {
				case Sequence::Neutral::ROCK:		// rocks
					{
						const int ROCK_NUM = 4;
						const float ROCK_EFFECT_SPEED_X = 5;
						const float ROCK_EFFECT_SPEED_Y = 8;
						const float ROCK_EFFECT_MAX_DISTANCE = 50;
						// create small rock effect
						getStaticData()->m_ObjectManager.m_EffectManager.createEffects(
							Sequence::Effect::ROCK, lpNeutral->getCenter(), lpNeutral->getCenter(),
							-1, -1, RandomVectorType::CIRCLE_RANDOM,
							0, ROCK_EFFECT_MAX_DISTANCE, -1, -1,
							RandomVectorType::RECTANGLE_RANDOM,
							-ROCK_EFFECT_SPEED_X, ROCK_EFFECT_SPEED_X,
							-ROCK_EFFECT_SPEED_Y, 0, ROCK_NUM );

					}
					break;
				case Sequence::Neutral::TREE:
				case Sequence::Neutral::ROUND_TRUNK:
					{
						const int EFFECT_TYPES[] = {
							Sequence::Effect::WOOD,
							Sequence::Effect::LEAF,
						};
						const int TREE_NUM = 2;
						const float TREE_EFFECT_SPEED_X = 5;
						const float TREE_EFFECT_SPEED_Y = 8;
						const float TREE_EFFECT_MAX_DISTANCE = 50;
						// create small wood chip effect
						for( int i = 0; i < sizeof(EFFECT_TYPES) / sizeof(EFFECT_TYPES[0] ); i ++ ) {
							getStaticData()->m_ObjectManager.m_EffectManager.createEffects(
								EFFECT_TYPES[i], lpNeutral->getCenter(), lpNeutral->getCenter(),
								-1, -1, RandomVectorType::CIRCLE_RANDOM,
								0, TREE_EFFECT_MAX_DISTANCE, -1, -1,
								RandomVectorType::RECTANGLE_RANDOM,
								-TREE_EFFECT_SPEED_X, TREE_EFFECT_SPEED_X,
								-TREE_EFFECT_SPEED_Y, 0, TREE_NUM );
						}
						// se
						getStaticData()->m_MixerManager.play(
							SE::WOOD_BREAK, false );
					}
					break;
				case Sequence::Neutral::DISPENSER_BU:
					{
						// pCibvʕo
						const int PINE_APPLE_MAX_NUM = 50;
						HPLVector vCenter = lpNeutral->getCenter();
						const int PINE_SPEED = 10;
						for( int i = 0; i < PINE_APPLE_MAX_NUM; i ++ ) {
							GShot* lpShot = 
								getStaticData()->m_ObjectManager.m_ShotManager.create(
								Sequence::Shot::PINE_APPLE_0, vCenter,
								lpNeutral->getID(), HPLMath::nextInt(0,2) == 1 ? FaceDirection::LEFT : FaceDirection::RIGHT,
								false, true);
							HPLVector v(HPLMath::nextFloat(-1, 1),
								HPLMath::nextFloat( -1, 0 ) );
							v.normalize();
							v.multiple( HPLMath::nextFloat(1,PINE_SPEED) );

							lpShot->setVelocityAngle( v );
						}
					}
					break;
				case Sequence::Neutral::JAMMING:
					{
						// 󂵂tO𗧂Ă
						getStaticData()->m_Save.setFlag( FlagType::VTOL_JAMMING, FlagType::JammingFlagType::DESTROYED );
					}
					break;
				}

			}
			
			delete *it;
			it = this->getList()->erase(it);
		}
	}

	// tuika
	for( list<GNeutral*>::iterator it = this->m_vecAdds.begin();
		it != this->m_vecAdds.end(); it ++ )
	{
		GNeutral* lpNeutral = *it;
		this->getList()->push_back( lpNeutral );
	}


	GPlayer* lpPlayer = &getStaticData()->m_ObjectManager.m_Player;

	if( lpPlayer->isChecking() && lpPlayer->getState()->getState() != StateType::SWIM) {
		// `FbN

		// `FbN[VɈڂ
		lpPlayer->changeState( StateType::CHECK ) ;

		/*if( this->m_bIsChecked == false ) {
			// mFΏۂ
			// "H"}[N
			getStaticData()->createPlayerEmotionIcon( Sequence::Effect::EI_HATENA );
		} */
	
	}
}

GNeutral* GNeutralManager::create(int nType, HPLVector vCenter, int nMapNumber, int nScenarioNumber) {
	GNeutral* lpNeutral = this->createWithoutAddList( nType, vCenter, nMapNumber, nScenarioNumber );
	this->getList()->push_back( lpNeutral );

	return lpNeutral;
}
GNeutral* GNeutralManager::createWithoutAddList( int nType, HPLVector vCenter, int nMapNumber, int nScenarioNumber ) {
	GNeutral* lpNeutral = new GNeutral( nType, nMapNumber, nScenarioNumber );
	lpNeutral->setCenter( vCenter );


	if( nType == Sequence::Neutral::FAN ||
		nType == Sequence::Neutral::KILLER_FAN ) 
	{
		// 
		float MOVE_DELTA[][2] = {
			{0,0},
			{-16,16},
			{0,0},
			{16,16},
		};
		HPLVector vCenter = lpNeutral->getCenter();
		vCenter.add( MOVE_DELTA[nMapNumber][0], MOVE_DELTA[nMapNumber][1] );
		lpNeutral->setCenter( vCenter );
		
	} else if( nType == Sequence::Neutral::MOVER ) {
		HPLVector vCenter = lpNeutral->getCenter();
		vCenter.add( -16,-16 );
		lpNeutral->setCenter( vCenter );
	}

	// center position
	lpNeutral->m_CenterPosition = vCenter;

	return lpNeutral;
}

/** m_WaitCounter
  * m_IntervalCounterԊu
  * @param[in]	nMinRadius/nMaxRadius	E
  * @retval true	IĂ
  */
bool explosionCycle( GNeutral* lpNeutral, int nMinRadius, int nMaxRadius ) {
	// NAĂȂ
	if( false == lpNeutral->m_WaitCounter.getState().isFinishAnimation ) {	// wait JE^IĂȂ
		if( lpNeutral->m_IntervalCounter.step().isFinishAnimation ) {	// C^[oɔ
			
			// ő唼a
			//float fRadiusMax = lpNeutral->getHitRect().getWidth() / 2;

			// ŏa
			//const float BARRIER_RADIUS_MIN = 10;

			HPLVector v;

			// _
			HPLMath::getRandomRangeVector( v, nMinRadius, nMaxRadius );
//			v.setByDegree( HPLMath::nextFloat(0,HPLMathConsts::DEGREE_FULL_ROUND ) );
//			v.multiple( HPLMath::nextFloat( BARRIER_RADIUS_MIN, fRadiusMax ) );
			v.add( lpNeutral->getCenter() );
			
			// 
			GShot* lpShot = getStaticData()->m_ObjectManager.m_ShotManager.create(
				Sequence::Shot::EXPLOSION, v, lpNeutral->getID(),
				FaceDirection::LEFT,
				false ,true);
			lpShot->setObjectAttribute( ObjectAttribute::HIT_TO_NEUTRAL, false );
//				lpShot->m_aryObjectAttribute[ObjectAttribute::HIT_TO_NEUTRAL] = false;

			// reset
			lpNeutral->m_IntervalCounter.reset();

			// step
			if( lpNeutral->m_WaitCounter.step().isFinishAnimation ) {
				// SI

				// ->SelectStarɈڂ
				//getStaticData()->startFadeOutDX( SceneType::SELECT_STAGE, -1 );

				// VтŏôőJڂȂ
				return true;
			}
		}
	} else {
		return true;
	}
	return false;
}

void GNeutralManager::algorithm(GNeutral* lpNeutral) {
	int nType = lpNeutral->getType();
	
	GPlayer* lpPlayer = &getStaticData()->m_ObjectManager.m_Player;
	HPLRect rMerginedRange = getStaticData()->getMerginedGameRange();

	HPLSequence* lpSeq = lpNeutral->getSequence();

	// x(not 0)
	float fLevel = getStaticData()->m_Game.getLevel( LEVEL_EPSILON );

	const float RESIST = getStaticData()->m_Setting.resist;
	
	// ܂ƂGtFNg\
	getStaticData()->m_ObjectManager.createFireEffect( lpNeutral);
	switch( nType ) {
	case Sequence::Neutral::MOVER:
	case Sequence::Neutral::RIDE_CLOUD:
	case Sequence::Neutral::ROUND_TRUNK:
	case Sequence::Neutral::ICE:
	case Sequence::Neutral::SLIDE_CLOUD:
	case Sequence::Neutral::BACK_CLOUD:
		// TODO isMoverłH
		switch( lpNeutral->m_nMoverType ) {
		case MoverType::RIGHT_LEFT:
		case MoverType::DOWN_UP:
		case MoverType::LEFT_RIGHT:
		case MoverType::UP_DOWN:
			{


				if( lpNeutral->m_WaitCounter.getState().isFinishAnimation ) {
					HPLRect vHitArea = lpNeutral->getHitArea();

					switch( nType ) {
					case Sequence::Neutral::ICE:
					case Sequence::Neutral::SLIDE_CLOUD:
					case Sequence::Neutral::BACK_CLOUD:
						break;
					default:
						if( lpNeutral->m_nNextDirection != 0 && rMerginedRange.isShare( vHitArea ) &&
							HPLMath::nextFloat(0,100) < 10 ) 
						{
							// ړ
							// ʓ
							// _

							// p[eBN

							HPLVector vCenter( HPLMath::nextFloat( vHitArea.getX(), vHitArea.getRight() ),
								HPLMath::nextFloat( vHitArea.getY(), vHitArea.getBottom() ) );

							GEffect* lpEffect = getStaticData()->createEffect( Sequence::Effect::WIND,
								vCenter, vCenter, -1, -1 );
						}
					}
					// EGCgԂł͂Ȃiړj
					if( lpNeutral->m_nNextDirection == -1 ) {
						// /ړ
						bool bIsHit = 
							lpNeutral->m_nMoverType == MoverType::RIGHT_LEFT && 
							lpNeutral->getTrigger()->getTrigger( TriggerType::HIT_TO_THE_LEFT )
							
							||

							lpNeutral->m_nMoverType == MoverType::LEFT_RIGHT && 
							lpNeutral->getTrigger()->getTrigger( TriggerType::HIT_TO_THE_LEFT )
							
							||

							lpNeutral->m_nMoverType == MoverType::DOWN_UP && 
							lpNeutral->getTrigger()->getTrigger( TriggerType::HIT_TO_THE_CEILING )

							||

							lpNeutral->m_nMoverType == MoverType::UP_DOWN && 
							lpNeutral->getTrigger()->getTrigger( TriggerType::HIT_TO_THE_CEILING );

						if( bIsHit ) {
							// /̃ubNɐڐG
							// ړtɂ
							lpNeutral->m_nNextDirection = 1;

							// JE^X^[g
							lpNeutral->m_WaitCounter.reset();

							// ړ0
							lpNeutral->setVelocityAngle(0,0);
						}
					} else if( lpNeutral->m_nNextDirection == 1 ) {
						// E/ړ

						bool bIsHit = 
							lpNeutral->m_nMoverType == MoverType::RIGHT_LEFT && 
							lpNeutral->getTrigger()->getTrigger( TriggerType::HIT_TO_THE_RIGHT )

							|| 

							lpNeutral->m_nMoverType == MoverType::LEFT_RIGHT && 
							lpNeutral->getTrigger()->getTrigger( TriggerType::HIT_TO_THE_RIGHT )
							
							||

							lpNeutral->m_nMoverType == MoverType::DOWN_UP && 
							lpNeutral->getTrigger()->getTrigger( TriggerType::ON_THE_GROUND )

							||

							lpNeutral->m_nMoverType == MoverType::UP_DOWN && 
							lpNeutral->getTrigger()->getTrigger( TriggerType::ON_THE_GROUND );

						if( bIsHit ) {
							// E/ɐڐG

							// ړtɂ
							lpNeutral->m_nNextDirection = -1;

							// JE^X^[g
							lpNeutral->m_WaitCounter.reset();

							// ړ0
							lpNeutral->setVelocityAngle(0,0);
						}
					} else {
						// ~
						// E/ֈړ
						lpNeutral->m_nNextDirection = 1;
					}
					if( lpNeutral->m_WaitCounter.getState().isFinishAnimation ) {
						//HPLVector vCenter = lpNeutral->getCenter();
						//HPLVector vAdd( lpNeutral->m_nNextDirection * MOVER_SPEED_X, 0 );
						//vCenter.add( vAdd );
						//lpNeutral->setCenter( vAdd );

						// EGCgI
						HPLVector vAdd;
						if( lpNeutral->m_nMoverType == MoverType::LEFT_RIGHT ||
							lpNeutral->m_nMoverType == MoverType::RIGHT_LEFT)
						{
							vAdd.set(1,0);
							float fSpeedX = MOVER_SPEED_X * fLevel;
							switch( lpNeutral->getType() ) {
							case Sequence::Neutral::MOVER:
								break;
							case Sequence::Neutral::RIDE_CLOUD:
								{
									const float RIDE_CLOUD_SPEEDX = fSpeedX / 1.2f;
									fSpeedX = RIDE_CLOUD_SPEEDX;
								}
								break;
							case Sequence::Neutral::ROUND_TRUNK:
								{
									const float ROUND_TRUNK_SPEEDX = fSpeedX * 5;
									fSpeedX = ROUND_TRUNK_SPEEDX;

								}
								break;
							case Sequence::Neutral::SLIDE_CLOUD:
							case Sequence::Neutral::BACK_CLOUD:
								{
									fSpeedX = SLIDE_CLOUD_SPEED;
								}
								break;
							}
							if( fSpeedX < 1 ) {
								fSpeedX = 1;
							}
							vAdd.multiple( lpNeutral->m_nNextDirection * fSpeedX );
						} else {
							vAdd.set(0,1);
							float fSpeedY = MOVER_SPEED_Y;
							switch( lpNeutral->getType() ) {
							case Sequence::Neutral::MOVER:
								break;
							case Sequence::Neutral::RIDE_CLOUD:
								{
									const float RIDE_CLOUD_SPEEDY = fSpeedY / 1.2f;
									fSpeedY = RIDE_CLOUD_SPEEDY;
								}
								break;
							case Sequence::Neutral::ROUND_TRUNK:
								{
									const float ROUND_TRUNK_SPEEDY = fSpeedY ;
									fSpeedY = ROUND_TRUNK_SPEEDY;
								}
								break;
							case Sequence::Neutral::SLIDE_CLOUD:
							case Sequence::Neutral::BACK_CLOUD:
								{
									fSpeedY = SLIDE_CLOUD_SPEED;
								}
								break;
							}
							if( fSpeedY < 1 ) {
								fSpeedY = 1;
							}
							vAdd.multiple( lpNeutral->m_nNextDirection * fSpeedY );
						}
						lpNeutral->setVelocityAngle( vAdd );
					}
				} else {
					// EGCg
					lpNeutral->m_WaitCounter.step();
				}
				break;
			}
		case MoverType::ROTATE:
		case MoverType::ROTATE_REVERSE:
			{
				const float MOVER_ROTATE_RADIUS = 200;

				// px
				float fRotateSpeed = MOVER_ROTATE_SPEED;
				if( lpNeutral->m_nMoverType == MoverType::ROTATE_REVERSE ) {
					fRotateSpeed *= -1;
				}

				float fDegree = lpNeutral->getRotateDegree() + fRotateSpeed;
				if( fDegree > HPLMathConsts::DEGREE_FULL_ROUND ) {
					fDegree -= HPLMathConsts::DEGREE_FULL_ROUND;
				} else if( fDegree < 0 ) {
					fDegree += HPLMathConsts::DEGREE_FULL_ROUND;
				}
				// ]
				lpNeutral->setRotateDegree( fDegree );


				// set center
				HPLVector vCenter;
				vCenter.setByDegree( fDegree ) ;
				vCenter.multiple( MOVER_ROTATE_RADIUS );
				vCenter.add( lpNeutral->m_CenterPosition );

				lpNeutral->setCenter( vCenter );

				// ]ɑxiɍڂĂIuWFNgpj
				HPLVector v;
				v.setByDegree( fDegree + (lpNeutral->m_nMoverType == MoverType::ROTATE ? 1: -1 ) * 90 );

				// px
				v.multiple( MOVER_ROTATE_RADIUS * tan(HPLMath::getRadianFromDegree( MOVER_ROTATE_SPEED ) ) );
				lpNeutral->setVelocityAngle(v );

				break;
			}
			break;
		case MoverType::FALL:
			{
				
				switch( lpNeutral->getState()->getState() ) {
				case StateType::STAND:
					if( lpNeutral->m_bIsPlayerMountOn ) {

						// 炷
						lpNeutral->changeState( StateType::ATTACK );
					}
					break;
				case StateType::ATTACK:
					if( lpSeq->getState().isFinishAnimation ) {
						// 
						lpNeutral->changeState( StateType::FLYDOWN );
						lpNeutral->setObjectAttribute( ObjectAttribute::GRAVITY, true );
//						lpNeutral->m_aryObjectAttribute[ ObjectAttribute::GRAVITY ] = true;
					}
					break;
				case StateType::FLYDOWN:
					break;
				}
			}
			break;
		default:
			break;
		}
		break;
	case Sequence::Neutral::FAN:
		{
			if( getStaticData()->m_Game.getTempFlagBit( TempFlagType::SWITCH_LOCK, 0 ) == false ) {
				// GtFNg𐶐
				float fDegree = (lpNeutral->getMapNumber() - 1 ) * 90.f;
				getStaticData()->m_ObjectManager.m_EffectManager.createWind(
					lpNeutral->getCenter(), lpNeutral->getWidth(),
					lpNeutral->getHeight(),
					fDegree, 1 );
			}

		}
		break;
	case Sequence::Neutral::KILLER_FAN:
		break;
	case Sequence::Neutral::BURNER:
		{
			// ΂𔭎

			// CounterŊԊu
			// HPLShootingObjectɃJE^]ĂȂmF

			
			if( lpNeutral->m_IntervalCounter.step().isFinishAnimation ) {
				
				lpNeutral->m_IntervalCounter.reset();

				GShot* lpShot = getStaticData()->m_ObjectManager.m_ShotManager.create(
					Sequence::Shot::FIRE_TRAP, lpNeutral->getCenter(),
					lpNeutral->getID(), lpNeutral->getFaceDirection(),
					false ,true);
				lpShot->setObjectAttribute( ObjectAttribute::HIT_TO_NEUTRAL, false );
//				lpShot->m_aryObjectAttribute[ObjectAttribute::HIT_TO_NEUTRAL] = false;
				
				HPLVector vCenter = lpNeutral->getCenter();

				// Vbǧ
				float fDegree = ( lpNeutral->getMapNumber() - 1 ) * 90.f;
				HPLVector vDir;
				vDir.setByDegree( fDegree );
				vDir.multiple(
					lpShot->getSequence()->getImageInformation(0)->srcRect.getHeight() / 2 +
					lpNeutral->getSequence()->getImageInformation(0)->srcRect.getHeight() / 2 );
				vCenter.add( vDir );

				lpShot->setCenter( vCenter);

				// x0

				// pxݒ
				lpShot->setDirection( lpNeutral->getMapNumber() );
				
				
			}
		}
		break;
	case Sequence::Neutral::KILLER_TANSU:

		if( lpNeutral->getState()->getState() == StateType::ATTACK ) {

			if( lpNeutral->getSequence()->getState().isFinishAnimation ) {
				// Aj[VI

				// Hׂ
				lpNeutral->changeState( StateType::ATE_00 );
			}
		}
		break;
	case Sequence::Neutral::FUYU:
		
		if( lpNeutral->getState()->getState() == StateType::STAND ) {
			// ʏ
			HPLSequence* lpSeq = lpNeutral->getSequence();
			HPLImageInformation* lpInfo = lpSeq->getImageInformation();

			float fDegree = HPLMathConsts::DEGREE_FULL_ROUND * lpSeq->getStepCounter() / (float)lpInfo->waitFrame;
			HPLVector vCenter = lpNeutral->m_CenterPosition;
			vCenter.add(
				0,
				FUYU_MAX_LENGTH * sin(HPLMath::getRadianFromDegree(fDegree)) );
			lpNeutral->setCenter( vCenter );

		} else if( lpNeutral->getState()->getState() == StateType::WALK) {
			// vC[ƔΑֈړ
			float fDX = HPLMath::getSgn<float>(lpNeutral->getCenter().getX() - lpPlayer->getCenter().getX() );
			int nDX = ((int)fDX == 0) ? -1 : (int)fDX ;
			
			HPLVector vVelocity = lpNeutral->getVelocityAngle();

			vVelocity.setX( nDX * FUYU_AFTER_RIDE_SPEED );

			// ։
			vVelocity.add( 0, -FUYU_AFTER_RIDE_FLOW_UP_SPEED );
			lpNeutral->setVelocityAngle(vVelocity);

			if( lpNeutral->getCenter().getY() < FUYU_DISAPPEAR_POSITION_Y ) {
				// select star ֐i
				getStaticData()->startFadeOutDX( SceneType::SELECT_STAGE, -1 );

				if( getStaticData()->m_Save.getFlag( FlagType::START) <= StartType::UNTIL_FUYU_EVIL ) {
					// flagς
					getStaticData()->m_Save.setFlag( FlagType::START, StartType::STARS );

					getStaticData()->m_Save.save();
				}
			}

			// vC[ƓWɂ
			lpPlayer->setCenter( lpNeutral->getCenter() );

		}
		break;
	case Sequence::Neutral::GENERATOR:
		{
			if( lpNeutral->m_IntervalCounter.getState().isFinishAnimation ) {
				// counter end
				// generate another neutral
				HPLVector vCen = lpNeutral->getCenter();
				int nNewType = lpNeutral->getPlacerParameter( ParameterType::Neutral::TYPE );
				int nEffectType = -1;

				switch( nNewType ) {
				case Sequence::Neutral::BU_CLOUD:
					// Ɖo
					{
						const int ERR_Y = 3;
						const int DELTA_Y = 30 + HPLMath::nextInt(-ERR_Y,ERR_Y);
						const int ERR_X = 3;
						vCen.add(HPLMath::nextInt(-ERR_X,ERR_X),DELTA_Y);
						nEffectType = Sequence::Effect::BU_CLOUD;
					}
					break;
				case Sequence::Neutral::BACK_CLOUD:
					{
						nEffectType = Sequence::Effect::BACK_CLOUD;
					}
					break;
				}

				if( nEffectType >= 0 ) {
					GEffect* lpNew = getStaticData()->m_ObjectManager.m_EffectManager.create(
						nEffectType, vCen, true );

					if( nNewType != Sequence::Neutral::BU_CLOUD ) {
						int nDX = 0;
						int nDY = 0;
						switch( lpNeutral->getPlacerParameter( ParameterType::Neutral::WARP_MAP_INDEX) ) {
						case MoverType::LEFT_RIGHT:	nDX = -1;	break;
						case MoverType::RIGHT_LEFT:	nDX = 1;	break;
						case MoverType::UP_DOWN:	nDY = -1;	break;
						case MoverType::DOWN_UP:	nDY = 1;	break;
						}
						if( nDX != 0 || nDY != 0 ) {
							HPLVector v(nDX, nDY );
							float SPEED = SLIDE_CLOUD_SPEED;
							v.multiple( SPEED );
							lpNew->setVelocityAngle( v );
						}
					}
				}else {
					GNeutral* lpNew = getStaticData()->m_ObjectManager.m_NeutralManager.createWithoutAddList(
						nNewType,
						vCen,
						lpNeutral->getPlacerParameter( ParameterType::Neutral::WARP_MAP_INDEX ),
						lpNeutral->getPlacerParameter( ParameterType::Neutral::WARP_SCENE_INDEX ));
					// ڐG
					lpNew->setObjectAttribute( ObjectAttribute::HIT_TO_BLOCK, true );
					// ڐGDIE
					lpNew->setObjectAttribute( ObjectAttribute::HIT_BLOCK_TO_DIE, true );
					this->addToAdds( lpNew );
				}

				// reset counter
				lpNeutral->m_IntervalCounter.reset();

			} else {
				lpNeutral->m_IntervalCounter.step();
			}

		}
		break;
	case Sequence::Neutral::EFFECT_GENERATOR:
		{
			int nEffectType = lpNeutral->getPlacerParameter(0);	// effect type
			HPLVector vCen = lpNeutral->getCenter();
			// create effect
			GEffect * lpEffect = getStaticData()->m_ObjectManager.m_EffectManager.create(
				nEffectType, vCen, true );
			if( nEffectType == Sequence::Effect::GENERATOR ) {
				lpEffect->setIntervalCounter(lpNeutral->getPlacerParameter(1));	// interval 
				lpEffect->getIntervalCounter()->end();
				int nGenEffectType = lpNeutral->getPlacerParameter(2);
				switch( nGenEffectType ) {
				case Sequence::Effect::LAVA_UNDER:
					vCen.add(16,0);
					lpEffect->setCenter( vCen );
					break;
				}	
				lpEffect->setGenerateEffectType( nGenEffectType );	// generate effect 
			}
			lpNeutral->setAlive( false);
		}
		break;
	case Sequence::Neutral::BARRIER:		// oA
		switch( lpNeutral->getState()->getState() ) {
		case StateType::DEAD:			// SԁI
			if( HPLMath::getFlagByIndexInt(
				getStaticData()->m_Save.getFlag( FlagType::DESTROY_BARRIER ),	
				getStaticData()->m_nSelectedStar ) == false )	// not destroyed this star's barrier
			{
				// NAĂȂ
				if( false == lpNeutral->m_WaitCounter.getState().isFinishAnimation ) {	// wait JE^IĂȂ
					if( lpNeutral->m_IntervalCounter.step().isFinishAnimation ) {	// C^[oɔ
						
						// ő唼a
						float fRadiusMax = lpNeutral->getHitRect().getWidth() / 2;

						// ŏa
						const float BARRIER_RADIUS_MIN = 10;

						explosionCycle( lpNeutral, BARRIER_RADIUS_MIN, fRadiusMax );
/*						HPLVector v;

						// _
						v.setByDegree( HPLMath::nextFloat(0,HPLMathConsts::DEGREE_FULL_ROUND ) );
						v.multiple( HPLMath::nextFloat( BARRIER_RADIUS_MIN, fRadiusMax ) );
						v.add( lpNeutral->getCenter() );
						
						// 
						GShot* lpShot = getStaticData()->m_ObjectManager.m_ShotManager.create(
							Sequence::Shot::EXPLOSION, v, lpNeutral->getID(),
							FaceDirection::LEFT,
							false ,true);
						lpShot->setObjectAttribute( ObjectAttribute::HIT_TO_NEUTRAL, false );
		//				lpShot->m_aryObjectAttribute[ObjectAttribute::HIT_TO_NEUTRAL] = false;

						// reset
						lpNeutral->m_IntervalCounter.reset();

						// step
						if( lpNeutral->m_WaitCounter.step().isFinishAnimation ) {
							// SI

							// ->SelectStarɈڂ
							//getStaticData()->startFadeOutDX( SceneType::SELECT_STAGE, -1 );

							// VтŏôőJڂȂ
						}
						*/
					}
				}
			}
			break;
		default:
			break;
		}
		break;
	case Sequence::Neutral::ALLY_FLOWER:		//! seed琶܂ꂽԁBB
		{
			HPLSequence* lpSeq = lpNeutral->getSequence();

			// saita 
			bool bIsFlowered = lpSeq->getFrameIndex() == lpSeq->getNumberOfFrames() - 1;
			
			// ԊuŗtςƉԕ΂܂
			if( bIsFlowered && lpNeutral->m_IntervalCounter.step().isFinishAnimation ) {

				lpNeutral->m_IntervalCounter.reset();

				bool bIsHitToPlayer = false;
				if( getStaticData()->m_OptionData.getLevelIndex() == LevelType::MAX_NUM - 1 ) {
					// MAXx
					// ->蔻L
					bIsHitToPlayer = true;
				}
				// 
				// create leafs
				{
					const int LEAF_MAX = 6;
					float fRotateDeg = lpNeutral->getRotateDegree();
					const float LEAF_SPEED = 10;
					HPLVector vCenter = lpNeutral->getCenter();
					for( int i = 0; i < LEAF_MAX; i ++ ) {
						GShot* lpShot = getStaticData()->m_ObjectManager.m_ShotManager.create(
							Sequence::Shot::LEAF, vCenter, lpNeutral->getID(),
							FaceDirection::LEFT,
							false ,true);
						float fDeg = fRotateDeg + (float)i * 
							HPLMathConsts::DEGREE_FULL_ROUND / LEAF_MAX;
						HPLVector v;
						v.setByDegree( fDeg );
						v.multiple( LEAF_SPEED );
						lpShot->setVelocityAngle( v );

						lpShot->setObjectAttribute( ObjectAttribute::HIT_TO_ENEMY, true );
						lpShot->setObjectAttribute( ObjectAttribute::HIT_TO_PLAYER, bIsHitToPlayer );
						lpShot->setObjectAttribute( ObjectAttribute::HIT_OBJECT_TO_DIE, true );	// 
					}
					// rotate angles
					const float ROTATE_SPEED = 3;
					lpNeutral->setRotateDegree( fRotateDeg + ROTATE_SPEED );
				}

				// kahun
				{
					HPLVector vCenter = lpNeutral->getCenter();
					const int KAHUN_TOP = 200;
					vCenter.add( 0, - KAHUN_TOP );
					const int KAHUN_NUM = 3;
					const int KAHUN_SPEED = 5;
					for( int i = 0; i < KAHUN_NUM; i ++ ) {
						GShot* lpShot = getStaticData()->m_ObjectManager.m_ShotManager.create(
							Sequence::Shot::KAHUN, vCenter, lpNeutral->getID(),
							FaceDirection::LEFT,
							false ,true);
						float fDeg = 270 + (float)(i - KAHUN_NUM / 2 ) * 
							120 / KAHUN_NUM;
						HPLVector v;
						v.setByDegree( fDeg );
						v.multiple( KAHUN_SPEED );
						lpShot->setVelocityAngle( v );

						lpShot->setObjectAttribute( ObjectAttribute::HIT_TO_ENEMY, true );
						lpShot->setObjectAttribute( ObjectAttribute::HIT_TO_PLAYER, bIsHitToPlayer );
						lpShot->setObjectAttribute( ObjectAttribute::HIT_OBJECT_TO_DIE, true );	// 
					}
				}
			}
		}
		break;
	case Sequence::Neutral::EVIGEN_PARTS_ARM:
	case Sequence::Neutral::EVIGEN_PARTS_BODY:
	case Sequence::Neutral::EVIGEN_PARTS_HEAD:
	case Sequence::Neutral::EVIGEN_PARTS_LEG:
		{
			// n`ƐڐG]ς

			if( lpNeutral->getTrigger()->getTrigger( TriggerType::ON_THE_GROUND ) ) {
				// n
				// ->^ɂȂ
				lpNeutral->setRotateDegree( 90 );
			} else {
				// ]
				HPLActionMoveTool::stepRotate(
					lpNeutral, ROTATE_STOP_VELOCITY_THRESHOLD );
			}

		}
		break;
	case Sequence::Neutral::JAMMING:
		{
			switch( lpNeutral->getState()->getState() ) {
			case StateType::STAND:
				if( lpNeutral->m_IntervalCounter.step().isFinishAnimation ) {

					lpNeutral->m_IntervalCounter.reset();

					const int BEAM_MAX = 1;
					HPLVector vCenter = lpNeutral->getCenter();
					const int DEG_ERR = 5;
					const float SPEED_ERR = 4;
					
					for( int i = 0; i < BEAM_MAX; i ++ ) {
						GShot* lpShot = getStaticData()->m_ObjectManager.m_ShotManager.create(
							Sequence::Shot::EVIL_BEAM, vCenter, lpNeutral->getID(),
							FaceDirection::LEFT,
							false ,true);
						float fDeg = 270 + HPLMath::nextInt(-DEG_ERR,DEG_ERR);
						HPLVector v;
						v.setByDegree( fDeg );
						v.multiple( EVIL_BEAM_SPEED + HPLMath::nextInt(-SPEED_ERR,SPEED_ERR) );
						lpShot->setVelocityAngle( v );

						lpShot->setObjectAttribute( ObjectAttribute::HIT_TO_ENEMY, false );
						lpShot->setObjectAttribute( ObjectAttribute::HIT_TO_PLAYER, true );
						lpShot->setObjectAttribute( ObjectAttribute::HIT_OBJECT_TO_DIE, true );	// 
					}
				}
				break;
			case StateType::DEAD:
				{
					// 
					bool bIsEnd = explosionCycle( lpNeutral, 0, 50 );

					if( bIsEnd ) {	// I
						// o
						HPLCounter* lpCounter = lpNeutral->getSmokeIntervalCounter();
						if( lpCounter->step().isFinishAnimation ) {
							lpCounter->reset();
							// smoke
							HPLVector vRange;
							HPLMath::getRandomRangeVector( vRange, 0,50);
							vRange.add( lpNeutral->getCenter());
							getStaticData()->m_ObjectManager.m_EffectManager.create(
								Sequence::Effect::EXPLOSION_SMOKE, vRange, true );
						}
					}
				}
				break;
			}
		}
		break;
	default:
		break;

	}

}


/** Ƀ_ɔ˂
  */
GNeutral* GNeutralManager::createRandomUpVelocityAngle( int nType, HPLVector vCenter ) {
	GNeutral* lpNeutral = this->create(
		nType, vCenter,
		-1, -1 );

	const float OBJ_ERR_X = 5;
	const float OBJ_ERR_Y = 8;
	// 
	lpNeutral->setVelocityAngle(
		HPLMath::nextFloat(-OBJ_ERR_X, OBJ_ERR_X),
		HPLMath::nextFloat(-OBJ_ERR_Y, 0 ) );
	return lpNeutral;
}

int GNeutralManager::loadSettings( const char* lpszFilePath ) {

	// RgLœǂݍށiG[̃CwÊ߁j
	vector<string> vecLines = HPLFile::loadStringFromFile( lpszFilePath, false );

	int nCount = 0;
	int nErrorCount = 0;

	for( int i = 0; i < (int)vecLines.size(); i ++ ) {
		string strLine = vecLines[i];
		if( HPLFile::isCommentLine( strLine ) ) {
			continue;
		}

		int nRet = this->parseFromLine( &this->m_aryNeutralSettings[nCount], strLine );
		nCount ++;
		if( nRet < 0 ) {
			HPLError::msg( "Error", MB_OK,
				"[%d]sڃtH[}bgG[[%s],Error Code=[%d]", i+1,strLine.c_str(), nRet );
			nErrorCount ++;
		}
	}

	if( nCount != Sequence::Neutral::MAX_NUM ) {
		HPLError::msg("Error",MB_OK,
			"Setting num:[%d] != Neutral num:[%d]", nCount, Sequence::Neutral::MAX_NUM );
		nErrorCount ++;
	}
	return nErrorCount;
}


/** 񂩂e
  * @retval	0	I
  */
int GNeutralManager::parseFromLine( GNeutralSetting* lpSetting, std::string& strLine ) {
	vector<string> aryList = HPLString::split( strLine , ",");

	if( aryList.size() < NeutralSettingType::MAX_NUM ) {
		HPLError::msg( "error", MB_OK,
			"vf̐Ȃ[%d]", aryList.size() );
		return -1;
	}
 	for( int i = 0; i < NeutralSettingType::MAX_NUM && i < (int)aryList.size(); i ++ ) {
		try {
			string str = aryList[i];
			// Ő󔒂폜
			str = HPLString::trim( str );
			switch( i ) {
			case NeutralSettingType::NAME:	//name
				// do nothing
				lpSetting->m_strName = str;
				break;
			case NeutralSettingType::TYPE:	//
				lpSetting->m_nType = atoi(str.c_str());
				break;
			case NeutralSettingType::HIT_RECT:
				{
					vector<string> aryItems = HPLString::split( str, "/");
					if( aryItems.size() < SIZEOF(lpSetting->m_aryHitRect) ) {
						HPLError::msg("Error", MB_OK,"vfs" );
						return -1;
					}
					for( int j = 0; j < (int)aryItems.size(); j ++ ) {
						lpSetting->m_aryHitRect[j] = atoi(aryItems[j].c_str() );
					}
				}
				break;
			case NeutralSettingType::HIT_TO_SHOT:
				lpSetting->m_bIsHitToShot = HPLString::equals(str, TRUE_STR);
				break;
			case NeutralSettingType::INVINSIBLE:
				lpSetting->m_bIsInvinsible = HPLString::equals(str, TRUE_STR);
				break;
			case NeutralSettingType::GRAVITY:
				lpSetting->m_bIsGravity = HPLString::equals(str, TRUE_STR);
				break;
			case NeutralSettingType::MOUNT_TYPE:
				if( HPLString::equals( str, "NONE") ) {
					lpSetting->m_nMountType = MountType::NONE;//atoi(str.c_str());
				} else if( HPLString::equals( str, "THROW") ) {
					lpSetting->m_nMountType = MountType::THROW;
				} else if( HPLString::equals( str, "BLOCK")) {
					lpSetting->m_nMountType = MountType::BLOCK;
				} else {
					HPLError::msg("Error", MB_OK,"sȃ^Cv[%s]",str.c_str() );
					return -1;
				}
				break;
			case NeutralSettingType::EXTRA_SEQ:
				{
					vector<string> aryItems = HPLString::split( str, "/");
					for( int j = 0; j < (int)aryItems.size(); j ++ ) {
						int nID = atoi(aryItems[j].c_str() );
						if( nID < 0 ) {
							break;
						}
						lpSetting->m_vecExtraSequenceIDs.push_back( nID );
					}
				}
				break;
			}
		} catch( ... ) {
			HPLError::msg( "Error", MB_OK,"ϊG[" );
			return -1;
		}
	}
	return 0;
}
