#include "GOptionData.h"

#include <fstream>
#include <string>
#include "hpllib/general/HPLString.h"
#include "hpllib/system/HPLFile.h"
#include "hpllib/math/HPLMath.h"

#include "GStaticData.h"


const char* const OPTION_FILE_PATH = "data/GameOption.dat";
const char* const COMMAND_OPTION_FLAGS = "FLAGS";
const char* const COMMAND_LEVEL = "LEVEL";
const char* const COMMAND_MAX_PLAYER_REMAIN = "MAX_PLAYER_REMAIN";
const char* const COMMAND_PLAY_TIME = "PLAY_TIME";

const char* const COMMAND_KILL_ENEMY = "KILL_ENEMY";
const char* const COMMAND_GET_ITEM = "GET_ITEM";
const char* const COMMAND_HIGH_SCORE = "HIGH_SCORE";

using namespace std;
using namespace GEnum;

GOptionData::GOptionData() {
	this->setAsDefault();
	
}

void GOptionData::setAsDefault() {
	// level
	this->m_nLevelIndex = LevelType::NORMAL;

	// max player remain
	this->m_nMaxPlayerRemain = DEFAULT_REMAIN_NUM;

	// flags
	memset( this->m_aryOptionFlags, 0, sizeof(int) * MAX_FLAGS ); 
	this->setOptionFlag( OptionFlagType::MUSIC_VOLUME, 255 );
	this->setOptionFlag( OptionFlagType::SE_VOLUME, 255 );
	this->setOptionFlag( OptionFlagType::VOICE_VOLUME , 255 );
	this->setOptionFlag( OptionFlagType::MAX_SHORTCUT_NUM, 3 );
	this->setOptionFlag( OptionFlagType::NO_NEAR_DEAD_SPEAK, 1 );

	for( int i = 0; i < SIZEOF(this->m_aryKillEnemies); i ++ ) {
		this->m_aryKillEnemies[i] = 0;
	}
	for( int i = 0; i < SIZEOF(this->m_aryGetItems ); i ++ ) {
		this->m_aryGetItems[i] = 0;
	}
	this->m_lPlayTime = 0;
	this->m_lHighScore = 100000;

}

int GOptionData::save() {
	ofstream ofs( OPTION_FILE_PATH );
	if( !ofs.is_open() ) {
		return -1;
	}

	// flags
	ofs << COMMAND_OPTION_FLAGS << "=" << HPLString::getListStringFromArray( MAX_FLAGS, this->m_aryOptionFlags ) << endl;

	// level
	ofs << COMMAND_LEVEL << "=" << HPLString::chrToStr( "%d", this->m_nLevelIndex ) << endl;

	// max player remain
	ofs << COMMAND_MAX_PLAYER_REMAIN << "=" << HPLString::chrToStr( "%d", this->m_nMaxPlayerRemain ) << endl;

	ofs << COMMAND_KILL_ENEMY << "=" << HPLString::getListStringFromArray( SIZEOF(this->m_aryKillEnemies ),
		this->m_aryKillEnemies ) << endl;

	ofs << COMMAND_GET_ITEM << "=" << HPLString::getListStringFromArray( SIZEOF( this->m_aryGetItems ),
		this->m_aryGetItems ) << endl;

	ofs << COMMAND_PLAY_TIME << "=" << this->m_lPlayTime << endl;
	ofs << COMMAND_HIGH_SCORE << "=" << this->m_lHighScore << endl;

	ofs.close();
	return 0;
}
int GOptionData::load() {

	// ݃`FbN
	ifstream ifs( OPTION_FILE_PATH );
	if( !ifs.is_open() ) {
		return -1;
	}
	ifs.close();

	// Rg"#"XLbv
	vector<string> vecList = HPLFile::loadStringLinesFromFile( OPTION_FILE_PATH, true );

	for( vector<string>::iterator it = vecList.begin(); 
		it != vecList.end();
		it ++ )
	{
		string str = *it;

		vector<string> items = HPLString::split( str, "=" );
		if( items.size() < 2 ) {
			HPLError::msg( "WARNING", MB_OK, "tH[}bgG[BXLbv܂[%s]", str.c_str());
			continue;
		}
		if( items[0].compare( COMMAND_OPTION_FLAGS ) == 0 ) {
			vector<string> flags = HPLString::split( items[1], "," );
			if( MAX_FLAGS < static_cast<int>( flags.size() ) ) {
				HPLError::msg( "WARNING", MB_OK, "Z[üꕔ̃tO񂪌[Flags=%d<SaveFlags=%d][%s]",
					MAX_FLAGS, flags.size(), str.c_str() );
			}
			for( int i = 0; i < SIZEOF(m_aryOptionFlags) && i < (int)flags.size(); i ++ ) {
				this->m_aryOptionFlags[i] = atoi( flags[i].c_str() );
			}
		} else if( items[0].compare( COMMAND_LEVEL ) == 0 ) {
			this->m_nLevelIndex = atoi( items[1].c_str() );
		} else if( items[0].compare( COMMAND_MAX_PLAYER_REMAIN ) == 0 ) {
			this->m_nMaxPlayerRemain = atoi( items[1].c_str() );
		} else if( items[0].compare( COMMAND_KILL_ENEMY ) == 0 ) {
			vector<string> nums = HPLString::split( items[1], ",");
			int nMax = SIZEOF( this->m_aryKillEnemies );
			if( nMax != (int)nums.size() ) {
				HPLError::msg( "WARNING", MB_OK, "GJEg̎ސۂ̐ƈقȂ[G̎=%d <> f[^̐=%d][%s]",
					nMax, (int)nums.size(), str.c_str() );
			}
			for( int i = 0; i < (int)nums.size() && i < nMax; i ++ ) {
				this->m_aryKillEnemies[i] = atoi(nums[i].c_str());
			}
		} else if( items[0].compare( COMMAND_GET_ITEM ) == 0 ) {
			vector<string> nums = HPLString::split( items[1], ",");
			int nMax = SIZEOF( this->m_aryGetItems );
			if( nMax != (int)nums.size() ) {
				HPLError::msg( "WARNING", MB_OK, "ACeJEg̎ސۂ̐ƈقȂ[ACe̎=%d <> f[^̐=%d][%s]",
					nMax, (int)nums.size(), str.c_str() );
			}
			for( int i = 0; i < (int)nums.size() && i < nMax; i ++ ) {
				this->m_aryGetItems[i] = atoi(nums[i].c_str());
			}
		} else if( items[0].compare( COMMAND_PLAY_TIME ) == 0 ) {
			this->setPlayTime( atol( items[1].c_str()));
		} else if( HPLString::equals( items[0], COMMAND_HIGH_SCORE ) ) {
			this->checkAndSetHighScore( atol( items[0].c_str()));
		} else {
			// x
			HPLError::msg( "Warning", MB_OK,
				"łȂsłBXLbv܂[%s]",
				str.c_str() );
		}
	}
	return 0;
}
bool GOptionData::checkAndSetHighScore( long lScore ) {
	bool bIsOver = this->m_lHighScore < lScore;
	if( bIsOver ) {
		this->m_lHighScore = lScore;
	}
	return bIsOver;
}

void GOptionData::setOptionFlagByIndex( int nIndex, int nBitIndex, bool bFlag ) {
	this->m_aryOptionFlags[nIndex] = 
		HPLMath::setFlagByIndexInt( this->m_aryOptionFlags[nIndex],
		nBitIndex, bFlag );
}
bool GOptionData::getOptionFlagByIndex( int nIndex, int nBitIndex) {
	return HPLMath::getFlagByIndexInt( 
		this->m_aryOptionFlags[nIndex], nBitIndex );
}

int GOptionData::getKillEnemy( int nEnemyType ) {
#ifdef _DEBUG
	ASSERT_IT_INDEX( nEnemyType, 0, SIZEOF( this->m_aryKillEnemies ) );
//	HPLError::assertIt( nEnemyType >= 0 && nEnemyType <  );
#endif
	return this->m_aryKillEnemies[nEnemyType];
}
int GOptionData::getGetItem( int nItemType ) {
#ifdef _DEBUG
	ASSERT_IT_INDEX( nItemType, 0, SIZEOF( this->m_aryGetItems ) );
//	HPLError::assertIt( nItemType >= 0 && nItemType < SIZEOF( this->m_aryGetItems ) );
#endif
	return this->m_aryGetItems[nItemType];
}
int GOptionData::setKillEnemy( int nEnemyType, int nNum ) {
	if( nNum < 0 ) {
		nNum = 0;
	}
#ifdef _DEBUG
	ASSERT_IT_INDEX( nEnemyType, 0, SIZEOF( this->m_aryKillEnemies ) );
//	HPLError::assertIt( nEnemyType >= 0 && nEnemyType < SIZEOF( this->m_aryKillEnemies ) );
#endif
	this->m_aryKillEnemies[nEnemyType] = nNum;
	return nNum;
}
int GOptionData::setGetItem( int nItemType, int nNum ) {
	if( nNum < 0 ) {
		nNum = 0;
	}
#ifdef _DEBUG
	ASSERT_IT_INDEX( nItemType, 0, SIZEOF( this->m_aryGetItems ) );
//	HPLError::assertIt( nItemType >= 0 && nItemType < SIZEOF( this->m_aryGetItems ) );
#endif
	this->m_aryGetItems[nItemType] = nNum;
	return nNum;
}
