#include "GOptionScene.h"

#include <limits.h>

#include "GStaticData.h"
#include <hpllib/general/HPLString.h>
#include <hpllib/math/HPLMath.h>

using namespace GEnum;
using namespace std;

namespace OKCancelType {
	enum {
		OK,
		CANCEL,
		MAX_NUM
	};
}

GOptionScene::GOptionScene() {
	memset(this->m_aryOptions, 0, sizeof(this->m_aryOptions) );
	memset(this->m_aryMaxNums, 0, sizeof(this->m_aryMaxNums) );
}
GOptionScene::~GOptionScene() {
}
// [v邩
static bool LOOPS[] = {
	true,
	false,
	false,
	false,
	false,
	false,
	false,
	false,
	true,
};

int GOptionScene::addColumn(int nDX) {
	HPLError::assertIt( 0 <= this->m_nRow && this->m_nRow < OptionType::MAX_NUM,
		"IvV̐ݒ荀ڂs:m_nRow:%d", this->m_nRow );

	this->m_aryOptions[ this->m_nRow ] += nDX;
	int nMax = this->m_aryMaxNums[ this->m_nRow ];

	if( LOOPS[this->m_nRow] ) {
		if( this->m_aryOptions[ this->m_nRow ] < 0 ) {
			this->m_aryOptions[ this->m_nRow ] = nMax - 1;
		} else if( this->m_aryOptions[ this->m_nRow ] >= nMax ) {
			this->m_aryOptions[ this->m_nRow ] = 0;
		}
	} else {	// [vȂiőlAŏlłƂ܂j
		if( this->m_aryOptions[ this->m_nRow ] < 0 ) {
			this->m_aryOptions[ this->m_nRow ] = 0;
		} else if( this->m_aryOptions[ this->m_nRow ] >= nMax ) {
			this->m_aryOptions[ this->m_nRow ] = nMax - 1;
		}

	}
	return this->m_aryOptions[ this->m_nRow ];
}

// J[\\ʒuŒ`
static int aryConsts[] = {
	0,	// level
	0,	// evigens
	0,	// no dead speak
	0,	// music volume
	0,	// se vol
	0,	// voice vol
	0,	// inmo
	0,	// guro
	-1,	// ok cancel
};
static const char ON_OFF_LABELS[][256] = {
	"OFF",
	"ON",
};

void GOptionScene::init() {
	this->m_nRow = 0;

	// current
	int aryCurrent[] = {
		getStaticData()->m_OptionData.getLevelIndex(),
		getStaticData()->m_OptionData.getMaxPlayerRemain(),
		getStaticData()->m_OptionData.getOptionFlag( OptionFlagType::NO_NEAR_DEAD_SPEAK ),

		getStaticData()->m_OptionData.getOptionFlag( OptionFlagType::MUSIC_VOLUME ),
		getStaticData()->m_OptionData.getOptionFlag( OptionFlagType::SE_VOLUME ),
		getStaticData()->m_OptionData.getOptionFlag( OptionFlagType::VOICE_VOLUME ),

		getStaticData()->m_OptionData.getOptionFlag( OptionFlagType::INMO ),
		getStaticData()->m_OptionData.getOptionFlag( OptionFlagType::GURO ),

		OKCancelType::OK,
	};
	// max
	int aryMaxes[] = {
		LevelType::MAX_NUM,
		MAX_REMAIN_NUM,
		OKCancelType::MAX_NUM,

		256,
		256,
		256,

		2,
		2,

		2
	};

	// copy from save
	for( int i = 0; i < OptionType::MAX_NUM; i ++ ) {
		this->m_aryOptions[i] = aryCurrent[i];
		this->m_aryMaxNums[i] = aryMaxes[i];
	}
}
void GOptionScene::quit() {
}
void GOptionScene::process() {
	// IvV\

	// ^Cg
	{
		const int OPTION_TITLE_X = 100;
		const int OPTION_TITLE_Y = 20;
		DrawString( OPTION_TITLE_X, OPTION_TITLE_Y,
			"Options", GetColor(255,255,0) );
	}

	// IvV
	{

		int nLeft = 100;
		int nTop = 100;

		vector<string> strings;
		
		// level
//		string strFormat = HPLString::chrToStr(
//			"     Level   : [%%%ds]",
//			MAX_LEVEL_LABEL_STRING_LENGTH );
		strings.push_back( HPLString::chrToStr(
			"     Level   : [%s]", 
			LEVEL_LABELS[m_aryOptions[OptionType::LEVEL]] ) );

		// remain num
		strings.push_back( HPLString::chrToStr(
			"     Evigens : [%5d]", 
			m_aryOptions[OptionType::MAX_PLAYER_NUM] ) );

		// no near dead speak
		strings.push_back( HPLString::chrToStr(
			"     mZt : [%s]", 
			ON_OFF_LABELS[m_aryOptions[OptionType::NO_NEAR_DEAD_SPEAK]] ) );

		// music volume
		strings.push_back( HPLString::chrToStr(
			"     Music Volume(NL) : [%3d/255]", 
			m_aryOptions[OptionType::MUSIC_VOLUME] ) );

		strings.push_back( HPLString::chrToStr(
			"     Sound Effect Volume(NL) : [%3d/255]", 
			m_aryOptions[OptionType::SE_VOLUME] ) );

		strings.push_back( HPLString::chrToStr(
			"     Voice Volume(NL) : [%3d/255]", 
			m_aryOptions[OptionType::VOICE_VOLUME] ) );

		strings.push_back( HPLString::chrToStr(
			"     Aс@@ : [%3s]", 
			ON_OFF_LABELS[m_aryOptions[OptionType::INMO]] ) );

		strings.push_back( HPLString::chrToStr(
			"     O\ : [%3s]", 
			ON_OFF_LABELS[m_aryOptions[OptionType::GURO]] ) );

		// ok cancel
		strings.push_back( string(
			"     OK               Cancel"));


		const int OPTION_INTERVAL_Y = 30;
		for( int i = 0; i < static_cast<int>( strings.size() ); i ++ ) {
			int nY = nTop + OPTION_INTERVAL_Y * i;

			// `
			DrawString( nLeft, nY, strings[i].c_str(), GetColor(255,255,255) );

		}

		// J[\\
		// TODO
		{
			int nColumn = aryConsts[ this->m_nRow ];
			if( nColumn < 0 ) {
				nColumn = this->m_aryOptions[ this->m_nRow ];
			}
			HPLSequence* lpCursorSeq = getStaticData()->getSequence(SpriteType::COMMON,
				Sequence::Common::CURSOR);
			getStaticData()->m_Basic.m_Graphics.draw(
				nLeft + nColumn * OPTION_OK_CANCEL_INTERVAL_X,
				nTop + this->m_nRow * OPTION_INTERVAL_Y,
				lpCursorSeq );

			// Aj[V
			lpCursorSeq->step();
		}

	}
	// J
	if( false == getStaticData()->isFadingOutDX() ) {
		// tF[hAEg͑s
		int nDX = 0;	// J[\ωX
		int nDY = 0;	// J[\ωY
		// J[\ړ
		{
			if( getStaticData()->getInputManager()->isPressedNow( KeyMap::UP ) ) {
				nDY = -1;
			}
			if( getStaticData()->getInputManager()->isPressedNow( KeyMap::DOWN ) ) {
				nDY = 1;
			}
			int nPreviousX = this->m_aryOptions[ this->m_nRow ];
			
			if( 0 == nDY ) {
				// cړȂꍇ̂݉ړ
				bool bIsLeft = getStaticData()->getInputManager()->isPressedNow( KeyMap::LEFT );
				bool bIsRight = getStaticData()->getInputManager()->isPressedNow( KeyMap::RIGHT );
				if( this->m_nRow == OptionType::MUSIC_VOLUME ||
					this->m_nRow == OptionType::SE_VOLUME ||
					this->m_nRow == OptionType::VOICE_VOLUME ) 
				{
					bIsLeft = getStaticData()->getInputManager()->isPressed( KeyMap::LEFT );
					bIsRight = getStaticData()->getInputManager()->isPressed( KeyMap::RIGHT );
				}

				if( bIsLeft ) {
					nDX = -1;
				}
				if( bIsRight ) {
					nDX = 1;
				}

				// ǂ͂ꂼŔf
				if( 0 != nDX ) {
					// ύXꂽ
					this->addColumn( nDX );
				}
			} else {	// 0 != nDY
				// cړꍇ͂D悷
				this->m_nRow += nDY;
				if( this->m_nRow < 0 ) {
					this->m_nRow = OptionType::MAX_NUM - 1; 
				}
				if( this->m_nRow >= OptionType::MAX_NUM ) {
					this->m_nRow = 0;
				}
			}
		}
		if( 0 == nDX && 0 == nDY ) {
			// ړƂ͋ʂi͈ړD悳j
			
			bool bIsReturnToTitle = false;	// ^Cg֖߂
			bool bIsCancel = false;			// LZ
			
			
			if( OptionType::OK_CANCEL == this->m_nRow ) {
				if( getStaticData()->getInputManager()->isPressedNow( KeyMap::B ) ) {
					// B{^LZ
					bIsCancel = true;
				}
				if( getStaticData()->getInputManager()->isPressedNow( KeyMap::A ) ) {
					// OK/Cancelɍ킹
					// L[Ŏ

					if( this->m_aryOptions[ this->m_nRow ] == 0 ) {
						// ok
						// sound
						getStaticData()->m_MixerManager.play( 
							SE::KETTEI,
							false );

						// ݒ𔽉f
						getStaticData()->m_OptionData.setLevelIndex(this->m_aryOptions[ OptionType::LEVEL ]);
						getStaticData()->m_OptionData.setMaxPlayerRemain( this->m_aryOptions[ OptionType::MAX_PLAYER_NUM ]);
						getStaticData()->m_OptionData.setOptionFlag( OptionFlagType::NO_NEAR_DEAD_SPEAK, this->m_aryOptions[ OptionType::NO_NEAR_DEAD_SPEAK ] );
						getStaticData()->m_OptionData.setOptionFlag( OptionFlagType::MUSIC_VOLUME, this->m_aryOptions[ OptionType::MUSIC_VOLUME ] );
						getStaticData()->m_OptionData.setOptionFlag( OptionFlagType::SE_VOLUME, this->m_aryOptions[ OptionType::SE_VOLUME ] );
						getStaticData()->m_OptionData.setOptionFlag( OptionFlagType::VOICE_VOLUME, this->m_aryOptions[ OptionType::VOICE_VOLUME ] );
						getStaticData()->m_OptionData.setOptionFlag( OptionFlagType::INMO, this->m_aryOptions[ OptionType::INMO ] );
						getStaticData()->m_OptionData.setOptionFlag( OptionFlagType::GURO, this->m_aryOptions[ OptionType::GURO ] );
						getStaticData()->m_OptionData.save();
						
						// ^Cg֖߂
						bIsReturnToTitle = true;
					} else {
						// cancel
						bIsCancel = true;
					}
				}

				if( bIsCancel ) {
					// sound
					getStaticData()->m_MixerManager.play( 
						SE::CANCEL,
						false );

					// ^Cg֖߂
					bIsReturnToTitle = true;
				}

				if( bIsReturnToTitle ) {
					// ^Cg֖߂鏈
					getStaticData()->startFadeOutDX( SceneType::TITLE, -1 );
				}
			}
		}
	}
}
