#include "GSave.h"

#include "GStaticData.h"

#include <fstream>
#include <iostream>
#include <hpllib/system/HPLError.h>
#include <hpllib/system/HPLFile.h>
#include <hpllib/general/HPLString.h>

using namespace std;
using namespace GEnum;

const char* const COMMAND_COMMENT = "COMMENT";
const char* const COMMAND_STAR = "STAR";
const char* const COMMAND_PLACE = "PLACE";
const char* const COMMAND_WEAPON = "WEAPON";
const char* const COMMAND_ITEM = "ITEM";
const char* const COMMAND_FLAGS = "FLAGS";
const char* const COMMAND_SUPER_POWER = "SUPER_POWER";
const char* const COMMAND_SHORT_CUTS = "SHORT_CUTS";
const char* const COMMAND_CHEST = "CHEST";
const char* const COMMAND_TANSU = "TANSU";
const char* const COMMAND_BOX_SEPARATE = "_";	// boxpZp[^

static const int NEUTRALS[] = {
	Sequence::Neutral::TAKARA_BAKO,
	Sequence::Neutral::TANSU,
};
static const int OBJECT_TYPES[] = {
	ObjectType::ITEM,
	ObjectType::ENEMY,
};
static const char OBJECT_LABELS[][32] = {
	"ITEM",
	"ENEMY",
};

static const int MAXS[] = {
	Sequence::Item::MAX_NUM,
	EnemyType::MAX_NUM,
};
static const char NEUTRAL_LABELS[][32] = {
	"CHEST",
	"TANSU"
};

GBoxFlag::GBoxFlag() {
}
/**
  */
bool GBoxFlag::isFlag( int nObjectType, int nType, int nFlagIndex ) {
	vector<int>& lpVec = this->getFlags( nObjectType );
	int nMax = lpVec.size();
	ASSERT_IT_INDEX( nType, 0, nMax);
	return HPLMath::getFlagByIndexInt(this->m_mapFlags[nObjectType][nType], nFlagIndex );
}
/**
  */
void GBoxFlag::setFlag( int nObjectType, int nType, int nFlagIndex, bool bIsOn ) {
	vector<int>& lpVec = this->getFlags( nObjectType );
	int nMax = lpVec.size();
	ASSERT_IT_INDEX( nType, 0, nMax);

	this->m_mapFlags[nObjectType][nType] = 
		HPLMath::setFlagByIndexInt( this->m_mapFlags[nObjectType][nType], nFlagIndex, bIsOn );
}

void GBoxFlag::fill( int nData ) {
	for( map<int, vector<int>>::iterator it = this->m_mapFlags.begin();
		it != this->m_mapFlags.end(); it ++ )
	{
		vector<int>& vecTypes = (vector<int>&)it->second;
		std::fill( vecTypes.begin(), vecTypes.end(), nData );
	}
}
void GBoxFlag::clearAndFill( int nObjectType, int nMaxNum, int nData ) {
	this->m_mapFlags[nObjectType].clear();
	for( int i = 0; i < nMaxNum; i ++ ) {
		this->m_mapFlags[nObjectType].push_back( nData );
	}
}
void GBoxFlag::clear(){
	this->m_mapFlags.clear();
}
vector<int>& GBoxFlag::getFlags( int nObjectType ) {
	HPLError::assertIt( this->m_mapFlags.find( nObjectType ) != this->m_mapFlags.end(),
		"%s:%s(%d):Invalid object type[%d]", __FILE__,__FUNCTION__,__LINE__,nObjectType );
	return (vector<int>&)this->m_mapFlags[nObjectType];
}
////////////////////////////////////////////////////////////////////////////////////////
//
GSave::GSave() {
	for( int nNeutralsIndex = 0; nNeutralsIndex < SIZEOF( NEUTRALS ); nNeutralsIndex ++ ) {
		GBoxFlag* lpBox = new GBoxFlag();
		// neutral type(chest,tansu)
		int nNeutralType = NEUTRALS[nNeutralsIndex];
		// setup box
		this->m_mapBoxFlags[nNeutralType] = lpBox;

		for( int nObjectsIndex = 0; nObjectsIndex < SIZEOF( OBJECT_TYPES ); nObjectsIndex ++ ) {
			// clear and fill by 0
			lpBox->clearAndFill( OBJECT_TYPES[nObjectsIndex], MAXS[nObjectsIndex], 0 );
		}
	}
}

GSave::~GSave() {
	for( map<int, GBoxFlag*>::iterator it = this->m_mapBoxFlags.begin();
		it != this->m_mapBoxFlags.end(); it ++ )
	{
		GBoxFlag* lpBox = it->second;
		// delete box
		delete lpBox;
		it->second = NULL;
	}
	this->m_mapBoxFlags.clear();
}

/*
bool GSave::isChestFlag( int nItemIndex, int nFlagIndex ) {
#ifdef _DEBUG
	ASSERT_IT_INDEX( nItemIndex, 0, Sequence::Item::MAX_NUM );
//	HPLError::assertIt( nItemIndex < Sequence::Item::MAX_NUM );
#endif
	return HPLMath::getFlagByIndexInt(
		this->m_aryChestFlag[nItemIndex], nFlagIndex );
}

void GSave::setChestFlag( int nItemIndex, int nFlagIndex, bool bFlag ) {
#ifdef _DEBUG
	ASSERT_IT_INDEX( nItemIndex, 0, Sequence::Item::MAX_NUM );
//	HPLError::assertIt( nItemIndex < Sequence::Item::MAX_NUM );
#endif
	this->m_aryChestFlag[nItemIndex] = HPLMath::setFlagByIndexInt(
		this->m_aryChestFlag[nItemIndex], nFlagIndex, bFlag);
}*/
bool GSave::setScore( long lScore ) {
	bool bIsOver = getStaticData()->m_OptionData.checkAndSetHighScore( lScore );
	this->m_lScore = lScore;
	// `FbNAchievement
	//getStaticData()->stepAndCheckAchievement(
	//	AchievementType::SCORE_HONOR, (int)lScore, true );
	return bIsOver;
}
bool GSave::addScore( long lScore ) {
	return this->setScore( this->getScore() + lScore );
}

/** Star/Placê݂ǂmF
  * @retval		-1		݂ȂStar
  * @retval		-2		݂ȂPlace
  */
int GSave::checkPlaceNum( int nStar, int nPlace ) {
	if( nStar < 0 || nStar >= StageType::MAX_NUM ) {
		return -1;
	}
	if( nPlace < 0 || nPlace >= (int)this->m_vecPlaceState[ nStar ].size() ) {
		return -2;
	}
	return 0;
}
int GSave::getPlaceState( int nStar, int nPlace ) {
	HPLError::assertIt( this->checkPlaceNum( nStar, nPlace ) >= 0,
		"݂ȂꏊQ[%d,%d]", nStar, nPlace );

	return this->m_vecPlaceState[ nStar ][ nPlace ];
}

/**
  */
void GSave::setPlaceState( int nStar, int nPlace, int nState ) {
	HPLError::assertIt( this->checkPlaceNum( nStar, nPlace ) >= 0,
		"݂ȂꏊQ[%d,%d]", nStar, nPlace );
	HPLError::assertIt( nState >= 0 && nState < PlaceType::MAX_NUM,
		"`State[%d]", nState );

	this->m_vecPlaceState[ nStar ][ nPlace ] = nState;
}


int GSave::save() {
	if( getStaticData()->m_Game.isCheat()) {
		LOG_POINT( "save cancelled because of cheating");
		return 0;
	}

	ofstream ofs( this->m_strFilePath.c_str() );
	if( ofs.is_open() == false ) {
		LOG_WARN( HPLString::chrToStr( "Failed to open file[%s]", this->m_strFilePath.c_str()).c_str());
		return 1;
	}
	// comment
	ofs << COMMAND_COMMENT << "=" << this->m_strComment << endl;
	// star
	ofs << COMMAND_STAR << "=" << HPLString::getListStringFromArray( StageType::MAX_NUM, this->m_aryStarState ) << endl;

	// Places
	for( int i = 0; i < StageType::MAX_NUM; i ++ ) {
		ofs << COMMAND_PLACE << HPLString::chrToStr("_%02d", i ) << "=" << 
			HPLString::getListStringFromArray( &this->m_vecPlaceState[i] ) << endl;
	}

	ofs << COMMAND_WEAPON << "=" << HPLString::getListStringFromArray( WeaponType::MAX_WEAPON_NUM, this->m_aryWeaponState ) << endl;

	// items
	ofs << COMMAND_ITEM << "=" << HPLString::getListStringFromArray( Sequence::Item::MAX_NUM, this->m_aryItemNum ) << endl;

	// flags
	ofs << COMMAND_FLAGS << "=" << HPLString::getListStringFromArray( MAX_FLAGS, this->m_aryFlags ) << endl;

	// super power
	ofs << COMMAND_SUPER_POWER << "=" << HPLString::getListStringFromArray( SuperPowerType::MAX_NUM, this->m_arySuperPower ) << endl;

	// short cuts
	ofs << COMMAND_SHORT_CUTS << "=" << HPLString::getListStringFromArray( //MAX_SHORTCUT_NUM, 
		getStaticData()->m_ShortCutPaletteManager.getPalettes() ) << endl;//this->m_aryShortCuts ) << endl;

	// box flags
	for( int nNeutralsIndex = 0; nNeutralsIndex < SIZEOF( NEUTRALS ); nNeutralsIndex ++ ) {
		int nNeutralType = NEUTRALS[nNeutralsIndex];
		GBoxFlag* lpBox = this->getBoxFlag( nNeutralType );
		for( int nObjectsIndex = 0; nObjectsIndex < SIZEOF( OBJECT_TYPES ); nObjectsIndex ++ ) {
			ofs << NEUTRAL_LABELS[nNeutralsIndex] << COMMAND_BOX_SEPARATE;
			ofs << OBJECT_LABELS[nObjectsIndex] << "=" << HPLString::getListStringFromArray(
				&lpBox->getFlags( OBJECT_TYPES[nObjectsIndex] )) << endl;
		}
	}
	ofs.close();
	LOG_POINT( HPLString::chrToStr("save[%s] complete without errors!", this->m_strFilePath.c_str()).c_str());
	return 0;
}
int GSave::load( const char* lpFilePath ) {
	this->m_strFilePath = string( lpFilePath );
	// ftHg
	this->setAsDefault();

	// ݃`FbN
	ifstream ifs( lpFilePath );
	if( ifs.is_open() == false ) {
		return 1;
	}

	ifs.close();


	// Rg"#"XLbv
	vector<string> vecList = HPLFile::loadStringLinesFromFile( lpFilePath, true );

	for( vector<string>::iterator it = vecList.begin(); 
		it != vecList.end();
		it ++ )
	{
		string str = *it;

		vector<string> items = HPLString::split( str, "=" );
		if( items.size() < 2 ) {
			HPLError::msg( "WARNING", MB_OK, "tH[}bgG[BXLbv܂[%s]", str.c_str());
			continue;
		}

		if( items[0].compare( COMMAND_COMMENT ) == 0 ) {
			// comment
			this->m_strComment = items[1];
		} else if( items[0].compare( COMMAND_STAR ) == 0 ) {
			// stars
			vector<string> stars = HPLString::split( items[1], "," );
			if( stars.size() != StageType::MAX_NUM ) {
				HPLError::msg( "WARNING", MB_OK, "̐ȂBȂɂ킹܂[%s]", str.c_str() );
			}
			for( int i = 0; i < (int)stars.size() && i < StageType::MAX_NUM; i ++ ) {
				this->m_aryStarState[i] = atoi( stars[i].c_str() );
			}

		} else if( items[0].compare( COMMAND_WEAPON ) == 0 ) {
			// weapons
			vector<string> weapons = HPLString::split( items[1], "," );
			if( weapons.size() != WeaponType::MAX_WEAPON_NUM ) {
				HPLError::msg( "WARNING", MB_OK, "̎ނ̐ȂBȂɂ킹܂[%s]", str.c_str() );
			}
			for( int i = 0; i < static_cast<int>( weapons.size() ) && i < WeaponType::MAX_WEAPON_NUM; i ++ ) {
				this->m_aryWeaponState[i] = atoi( weapons[i].c_str() );
			}

		} else if( items[0].compare( COMMAND_ITEM ) == 0 ) {
			// ACe
			vector<string> nums = HPLString::split( items[1], "," );
			for( int i = 0; i < static_cast<int>( nums.size() ) && i < Sequence::Item::MAX_NUM; i ++ ) {
				this->m_aryItemNum[i] = atoi( nums[i].c_str() );
			}

//		} else if( items[0].compare( COMMAND_START_TYPE ) == 0 ) {
//			this->m_nStartType = atoi( items[1].c_str() );

		} else if( items[0].compare( COMMAND_FLAGS ) == 0 ) {
			vector<string> flags = HPLString::split( items[1], "," );
			if( MAX_FLAGS < static_cast<int>( flags.size() ) ) {
				HPLError::msg( "WARNING", MB_OK, "Z[üꕔ̃tO񂪌[Flags=%d<SaveFlags=%d][%s]",
					MAX_FLAGS, flags.size(), str.c_str() );
			}
			for( int i = 0; i < MAX_FLAGS && i < (int)flags.size(); i ++ ) {
				this->m_aryFlags[i] = atoi( flags[i].c_str() );
			}
		} else if( items[0].compare( COMMAND_SUPER_POWER ) == 0 ) {
			vector<string> nums = HPLString::split( items[1], "," );
			if( nums.size() != SuperPowerType::MAX_NUM ) {
				HPLError::msg( "WARNING", MB_OK, "SuperPoweř܂BKɍ킹Ă܂B[save data num=%d != SuperPowerTypes=%d][%s]",
					nums.size(), SuperPowerType::MAX_NUM, str.c_str() );
			}
			for( int i = 0; i < static_cast<int>( nums.size() ) && i < SuperPowerType::MAX_NUM; i ++ ) {
				this->m_arySuperPower[i] = atoi( nums[i].c_str() );
			}
		} else if( items[0].compare( COMMAND_SHORT_CUTS ) == 0 ) {
			vector<string> nums = HPLString::split( items[1], "," );
			/*if( nums.size() != MAX_SHORTCUT_NUM ) {
				HPLError::msg( "WARNING", MB_OK, "V[gJbg̐܂BKɂ킹Ă܂ [save short cuts=%d != MAX_SHORTCUT_NUM",
					nums.size(), MAX_SHORTCUT_NUM );
			}*/
			for( int i = 0; i < static_cast<int>( nums.size() ) &&
				i < getStaticData()->m_OptionData.getOptionFlag( OptionFlagType::MAX_SHORTCUT_NUM); i ++ )
			{
				getStaticData()->m_ShortCutPaletteManager.setValue(i, atoi(nums[i].c_str() ) );//this->m_aryShortCuts[i] = atoi( nums[i].c_str() );
			}

		} else {
			if( items[0].find( "PLACE") == 0 ) {
				vector<string> placeNumbers = HPLString::split( items[0], "_" );
				if( placeNumbers.size() == 2 ) {
					// places
					int nNumber = atoi( placeNumbers[1].c_str() );
					if( nNumber < 0 || nNumber >= StageType::MAX_NUM ) {
						HPLError::msg( "WARNING", MB_OK, "݂Ȃ̃f[^BXLbv܂[%s]",
							str.c_str() );
						continue;
					}
					vector<string> places = HPLString::split( items[1], "," );

					// ݒ̃vCX
					int nSettingPlaceNum = getStaticData()->m_Setting.maps.getPlaceNum(nNumber);
					if( places.size() != nSettingPlaceNum ) {
						HPLError::msg( "WARNING", MB_OK, "vCXȂBȂɂ킹܂[%s]",
							str.c_str() );
					}
					for( int i = 0; i < (int)places.size() && i < nSettingPlaceNum; i ++ ) {
						this->m_vecPlaceState[nNumber][i] = atoi( places[i].c_str() );
					}
				} else {
					// unknown
					HPLError::msg( "WARNING", MB_OK, "݂Ȃ߃ZbgBXLbv܂[%s]", 
						str.c_str() );
				}
			} else {
				for( int nNeutralsIndex = 0; nNeutralsIndex < SIZEOF( NEUTRALS ); nNeutralsIndex ++ ) {
					int nNeutralType = NEUTRALS[nNeutralsIndex];

					vector<string> vItems = HPLString::split( items[0], COMMAND_BOX_SEPARATE );

					if( HPLString::equals( vItems[0], NEUTRAL_LABELS[nNeutralsIndex] ) ) {

						GBoxFlag* lpBox = this->getBoxFlag( nNeutralType );
						for( int nObjectsIndex = 0; nObjectsIndex < SIZEOF( OBJECT_TYPES ); nObjectsIndex ++ ) {
							if( HPLString::equals(vItems[1], OBJECT_LABELS[nObjectsIndex] ) == false ) {
								continue;
							}
							// object type
							int nObjectType = OBJECT_TYPES[nObjectsIndex];
							// flag list
							vector<int>& vec = lpBox->getFlags( nObjectType );
							vector<string> flags = HPLString::split( items[1], "," );
							if( flags.size() != vec.size() ) {
								HPLError::msg( "caption", MB_OK,
									"Save file flags does not equal to num of types. Box type (NeutralType):%d, ObjectType:%d, flags=%d != types=%d",
									nNeutralType, nObjectType, flags.size(), vec.size());
							}
							for( int i = 0; i < (int)flags.size() && i < (int)vec.size(); i ++ ) {
								vec[i] = atoi(flags[i].c_str());
							}
						}
					}
				}
			}
		}

	}


	return 0;
}
void GSave::setAsDefault() {
	// stars
	for( int i = 0; i < StageType::MAX_NUM; i ++ ) {
		int nType = PlaceType::SEALED;
		if( i == StageType::EVIL_STAR ) {
			nType = PlaceType::REVEALED;
		}
		// т鐯ON
		this->m_aryStarState[ i ] = nType;
	}

	// places
	this->m_vecPlaceState.clear();
	for( int nStar = 0; nStar < StageType::MAX_NUM; nStar ++ ) {
		vector<int> vecPlaces;
		for( int nPlace = 0; nPlace < getStaticData()->m_Setting.maps.getPlaceNum(nStar); nPlace ++ ) {
			vecPlaces.push_back( PlaceType::SEALED );
		}
		vecPlaces[0] = PlaceType::REVEALED;
		this->m_vecPlaceState.push_back( vecPlaces );
	}

	// weapons
	memset( this->m_aryWeaponState, 0, sizeof( this->m_aryWeaponState ) );
	// p`͗L
	this->m_aryWeaponState[WeaponType::PUNCH ] = 1;
	//this->m_aryWeaponState[0] = 1;

	// items
	memset( this->m_aryItemNum, 0, sizeof( this->m_aryItemNum ) );
	// p`
	this->m_aryItemNum[Sequence::Item::PUNCH] = 1;

	// comment
	this->m_strComment = string( "Comment" );

	// flags
	memset( this->m_aryFlags, 0, sizeof(int) * MAX_FLAGS ); 
	const int DEFAULT_START_ORB = 3;
	this->setFlag( FlagType::EVIL_ORB_MAX, DEFAULT_START_ORB );
	this->setFlag( FlagType::EVIL_ORB_NUM, DEFAULT_START_ORB );

	memset( this->m_arySuperPower, 0, sizeof(this->m_arySuperPower));

	// V[gJbg
/*	for( int i = 0; i < MAX_SHORTCUT_NUM; i ++ ) {
		if( i == 0 ) {
			this->m_aryShortCuts[i] = Sequence::Item::PUNCH;	// punch ACeƂēo^
		} else {
			this->m_aryShortCuts[i] = -1;
		}
	}*/
	getStaticData()->m_ShortCutPaletteManager.resize(
		MAX_SHORTCUT_NUM, -1, true );
	getStaticData()->m_ShortCutPaletteManager.setValue(0, Sequence::Item::PUNCH );

	for( map<int,GBoxFlag*>::iterator it = this->m_mapBoxFlags.begin(); it != this->m_mapBoxFlags.end(); it ++ ) {
		GBoxFlag* lpBox = it->second;
		lpBox->fill( 0 );
	}
	// ftHgݒ
	//this->setFlag( FlagType::EVIL_ORB_NUM, DEFAULT_MAX_EVIL_ORB_NUM );
}


/** w肵ɂPlace̐
  */
int GSave::getStarPlaceNum( int nStar ) {
	HPLError::assertIt( nStar >= 0 && nStar < StageType::MAX_NUM,
		"݂Ȃ[%d]", nStar );
	return static_cast<int>( this->m_vecPlaceState[nStar].size() );
}

/*int GSave::getFlag( int nIndex, int& nFlag ) {
	if( nIndex < 0 || nIndex >= MAX_FLAGS ) {
		HPLError::msg("error", MB_OK, "nIndex=[%d], {0, %d}", nIndex, MAX_FLAGS);
		return -1;
	}
	nFlag = this->m_aryFlags[nIndex];
	return 0;
}
int GSave::setFlag( int nIndex, int nFlag ) {
	if( nIndex < 0 || nIndex >= GEnum::MAX_FLAGS ) {
		HPLError::msg("error", MB_OK, "nIndex=[%d], {0, %d}", nIndex, MAX_FLAGS);
		return -1;
	}
	this->m_aryFlags[nIndex] = nFlag;
	return 0;
	
}*/

/**
  * @retval		l	ŏIIɐݒ肳ꂽli0艺ɂȂꍇ̏CȂǁj
  */
int GSave::setItemNum( int nIndex, int nNum ) {
	if( 0 > nNum ) {
		nNum = 0;
	}

	this->m_aryItemNum[nIndex] = nNum;

	return nNum;
}

bool GSave::getFlagBit( int nIndex, int nBitIndex ) {
	return HPLMath::getFlagByIndexInt( this->getFlag( nIndex), nBitIndex );
}

void GSave::setFlagBit( int nIndex, int nBitIndex, bool bFlag ) {
	this->setFlag( nIndex,
		HPLMath::setFlagByIndexInt( this->getFlag( nIndex), nBitIndex, bFlag ) );
		
}

void GSave::setFlag( int nIndex, int nFlag ) {
	switch( nIndex ) {
	case FlagType::EVIL_ORB_MAX:
		{
			if( nFlag < 0 ) {
				nFlag = 0;
			}
			int nNum = this->getFlag( FlagType::EVIL_ORB_NUM );
			if( nNum > nFlag ) {
				// ɂől𒴂Ă܂
				nNum = nFlag;
				this->setFlag( FlagType::EVIL_ORB_NUM, nNum );
			}
		}
		break;
	case FlagType::EVIL_ORB_NUM:
		{
			int nMax = this->getFlag( FlagType::EVIL_ORB_MAX );
			if( nFlag < 0 ) {
				nFlag = 0;
			} else if( nFlag > nMax ) {
				nFlag = nMax;
			}
		}
		break;
	}
	this->m_aryFlags[nIndex] = nFlag;
}
/** V[gJbgpbg̈ԍŏ擾
  * @retval	-1		V[gJbgpbgɂ͉o^ĂȂ
  * @retval 0ȏ	݂V[gJbgCfbNX
  *
int GSave::getFirstShortCutIndex() {
	for( int i = 0; i < getStaticData()->m_OptionData.getOptionFlag( OptionFlagType::MAX_SHORTCUT_NUM ); i ++ ) {
		if( this->getShortCut(i) >= 0 ) {
			return i;
		}
	}
	return -1;
}
*/

/** w肵ꏊɑ݂V[gJbg擾
  * 
  * @param[in]	nIndex	݂̃V[gJbgCfbNXB
  *						-1ȉMAXȏw肵ꍇAꂼ擪ƍŌɊۂ߂
  * @retval		-1		V[gJbg񂪑݂ȂiTCY0A܂̓V[gJbgS-1j
  * @retval		0ȏ	-1ȊO̒lz̃CfbNXiV[gJbgCfbNXj
  *
int GSave::getNextShortCutIndex(int nIndex ) {
	if( nIndex < 0 ) {
		nIndex = 0;
	}
	int MAX_SHORT_CUT_NUM = getStaticData()->m_OptionData.getOptionFlag( OptionFlagType::MAX_SHORTCUT_NUM );
	if( nIndex >= MAX_SHORT_CUT_NUM ) {
		nIndex = MAX_SHORT_CUT_NUM - 1;
	}
	int nOldIndex = nIndex;
	// ̃pbgT
	nIndex ++;
	if( nIndex >= MAX_SHORT_CUT_NUM ) {
		// ŌɓB
		// ̂̂TĂꍇ͈B
		nIndex = 0;
	}
	while( nOldIndex != nIndex ) {	// ɖ߂ĂI(gԂ)


		if( this->getShortCut(nIndex) >= 0 ) {	// V[gJbgɉo^Ă
			return nIndex;
		} else {
			nIndex ++;
			if( nIndex >= MAX_SHORT_CUT_NUM ) {
				// ŌɓB
				// ̂̂TĂꍇ͈B
				nIndex = 0;
			}
		}
	}
	if( this->getShortCut( nIndex ) < 0 ) { // (w肵Indexɂ)
		return -1;
	}
	return nIndex;
}

*/

