#include "GSelectStageScene.h"

#include "GStaticData.h"
#include <hpllib/general/HPLString.h>
#include <hpllib/math/HPLMath.h>
#include <algorithm>		// \[gp

#include <hpllib/action/HPLActionMoveTool.h>

using namespace GEnum;
using namespace std;
using namespace HPLMathConsts;

const int MIN_X = 100;
int MAX_X;
const int MIN_Y = 100;
int MAX_Y;

int CENTER_X = 0;
int CENTER_Y = 0;
float X_Y_RATE = 1;
float RADIUS = 1;
static char STAR_LABELS[ StageType::MAX_NUM ][256] = {
	"Evil Star",
	"Chocolate Rock",
	"Tropical Water",
	"Strawberry Fire",
	"Matcha Forest",
	"Cream Wind",
	"Kokutou Machina",
	"Tozicist",
	"Resort Star -Orupaka-"
};

GSelectStageScene::GSelectStageScene() {
	MAX_X = getStaticData()->m_Setting.windowWidth - MIN_X;
	MAX_Y = getStaticData()->m_Setting.windowHeight - MIN_Y * 2;

	CENTER_X = ( MIN_X + MAX_X ) / 2;
	CENTER_Y = ( MIN_Y + MAX_Y ) / 2;

	RADIUS = static_cast<float>( MAX_X - CENTER_X );

	X_Y_RATE = (float)( MAX_Y - MIN_Y) / (float)( MAX_X - MIN_X);
	m_nSelectedStar = 0;
	
	// ]px
	this->m_fRotateDegree = 0;

	this->m_VTOL.setType( PlayerType::TANYA );

	// opXvCg
	this->m_lpEffectSprite = getStaticData()->getSprite(SpriteType::SHOT )->clone();

	// Ȃ
	this->setEffectType( SelectStarSceneEffectType::NONE );

	// SV[hujɑtOl
	for( int i = 0; i < StageType::MAX_STORY_STAR; i ++ ) {
		this->m_nFullDestroyShield |= 2 << i;
	}

	this->m_ShotEvilBeamCounter.setWaitFrameNum(100);

}
GSelectStageScene::~GSelectStageScene() {
	delete this->m_lpEffectSprite;
}

HPLVector GSelectStageScene::getCenter( int nIndex ) {
	float fDegree = this->getDegree( nIndex );
	HPLVector vCenter(
		CENTER_X + cos( HPLMath::getRadianFromDegree( fDegree) ) * RADIUS,
		CENTER_Y + sin( HPLMath::getRadianFromDegree( fDegree) ) * RADIUS * X_Y_RATE );
	return vCenter;
}
float GSelectStageScene::getDegree( int nIndex ) {
	float fInterval = DEGREE_FULL_ROUND / (StageType::MAX_NUM - 1);
	
	return fInterval * (nIndex - 1) + this->m_fRotateDegree ;
}

void GSelectStageScene::init() {

	//vector<pair<int,int> > vOrders;
	getStaticData()->m_MixerManager.playMusic(Music::SPACE, true );

	this->m_nSelectedStar = getStaticData()->m_nSelectedStar;
	int nCounter = 0;

	// ɂۂ
	this->resetVTOL();

	// 
	for( int i = 0;i < StageType::MAX_NUM; i ++ ) {

		this->m_Stars[i].setType( i );

		this->m_Stars[i].m_nSequenceID = getStaticData()->getStarSequenceID(i );

		HPLVector vCenter;
		if( i == StageType::EVIL_STAR ) {
			vCenter.set( (float)CENTER_X, (float)CENTER_Y );
		} else {
			vCenter = this->getCenter( i );
		}
		// z[|WVݒ
		this->m_Stars[i].m_vHomePosition = vCenter;


		//struct pair<int,int> p(i, this->m_Stars[i].m_vHomePosition.getY() );
		//vOrders.push_back(p);

		// test
		//m_aryDrawOrder[i] = i;

		// z[|WV̈ʒuɈړ
		this->m_Stars[i].setCenter( this->m_Stars[i].m_vHomePosition );
	}

	// ŏ͂т邷H
	// ƂOۑĂꏊHŁB

	// test

	// IĂ鐯̏Ɉړ
	this->m_SelectPlayer.setCenter( this->m_Stars[ this->m_nSelectedStar ].getCenter() );

	// ~߂
	this->m_SelectPlayer.setVelocityAngle(0,0);

	// GtFNg
	switch( this->getEffectType() ) {
	case SelectStarSceneEffectType::BARRIER_EXPLOSION:
		// Aj[V
		this->m_lpEffectSprite->getSequence( Sequence::Shot::EXPLOSION, FaceDirection::LEFT )->resetCounter();

		// se
		getStaticData()->m_MixerManager.play( SE::EXPLOSION, false );
		break;
	}

	// Cxgǂݍ
	getStaticData()->setupEvent(NULL);

	this->m_ShotEvilBeamCounter.reset();

	// }bvǂݍ
	getStaticData()->setupMap( SpecialMapIndex::SELECT_STAR, true );

	// bgm
	getStaticData()->m_MixerManager.playMusic( Music::SPACE , true );
}
void GSelectStageScene::quit() {
}

bool order(GStar* a, GStar* b){
	return a->getCenter().getY() < b->getCenter().getY();
}

float getZoom( float fY ) {
	return fY  / ((MAX_Y - MIN_Y) );
}
void GSelectStageScene::process() {

	// wi
	//drawUniverseSpace();
	getStaticData()->commonBackgroundProcess();

	// Cxg1GANeBx[g
	getStaticData()->isEvent( EventStepType::DEFAULT, true, false);

	// 1ӏłoAj󂵂ĂVTOL
	bool bIsDrawVTOL = getStaticData()->m_Save.getFlag( FlagType::DESTROY_BARRIER ) != 0;
	
	if( getStaticData()->m_EventImage.isActivated() == false ) {

	this->m_nSelectedStar = -1;
	HPLVector vTowerCenter = this->m_Stars[StageType::EVIL_STAR].getCenter();
	const float TOWER_CENTER_DELTA_Y = -100;
	vTowerCenter.add(0,TOWER_CENTER_DELTA_Y );

	HPLVector vSubstract = HPLVector::subtract( this->m_VTOL.getCenter(), this->m_SelectPlayer.getCenter() );
	// ̏ꍇVTOL`FbN
	// E苗ȓ
	// EoAjԂł͂Ȃ
	bool bIsOnTheVTOL = bIsDrawVTOL &&
		vSubstract.getLength() <= SELECT_VTOL_THRESHOLD &&
		this->getEffectType() != SelectStarSceneEffectType::BARRIER_EXPLOSION;

	bool bIsDrawJamming = getStaticData()->m_Save.getFlag( FlagType::VTOL_JAMMING) == FlagType::JammingFlagType::JAMMING;	// JAMMING\邩ǂ
	
	{
		// \̌vZ
		std::vector<GStar*> vecStarSorted;
		
		// \
		for( int i = 0; i < StageType::MAX_NUM; i ++ ) {
			vecStarSorted.push_back( &this->m_Stars[i] );
		}

		// YقǌɂȂ悤ɂ
		sort( vecStarSorted.begin(), vecStarSorted.end(), order );

		for( std::vector<GStar*>::iterator it = vecStarSorted.begin();
			it != vecStarSorted.end(); it ++)
		{
			GStar* lpStar = *it;
			if( getStaticData()->m_Save.m_aryStarState[lpStar->getType()] != PlaceType::SEALED ) {
				// opened

				// 
				HPLVector vSelectCenter = this->m_SelectPlayer.getCenter();
				HPLVector vLength(
					lpStar->getCenter().getX() - vSelectCenter.getX(),
					lpStar->getCenter().getY() - vSelectCenter.getY() );

				if( vLength.getLength() < SELECT_STAR_THRESHOLD ) {
					// I
					this->m_nSelectedStar = lpStar->getType();
					
				}
			}
		}
		for( std::vector<GStar*>::iterator it = vecStarSorted.begin();
			it != vecStarSorted.end(); it ++)
		{
			// CfbNX
			//int nIndex = this->m_aryDrawOrder[i];
			GStar* lpStar = *it;

			// V[PX
			HPLSequence* lpSeq = getStaticData()->getSequence( SpriteType::COMMON, 
				lpStar->m_nSequenceID );


			// g嗦ϓB傤ǒԂɁA0`2{
			float fZoom = getZoom(lpStar->getCenter().getY());// / ((MAX_Y - MIN_Y) );

			HPLVector vJammingCenter = lpStar->getCenter();
			const float JAMMING_TOWER_DELTA_Y = (-100 * fZoom);
			vJammingCenter.add(0,(float)JAMMING_TOWER_DELTA_Y);
			// 
			getStaticData()->m_Basic.m_Graphics.draw( 
				static_cast<int>( lpStar->getCenter().getX() ),
				static_cast<int>( lpStar->getCenter().getY() ),
				lpSeq, FaceDirection::LEFT, 0, fZoom );

			if( lpStar->getType() != StageType::EVIL_STAR &&
				lpStar->getType() < StageType::MAX_STORY_STAR &&
				HPLMath::getFlagByIndexInt( getStaticData()->m_Save.getFlag( FlagType::DESTROY_BARRIER ), 
				lpStar->getType() ) == false )
			{
				// ́ł
				// oAuj󂵂ĂȂ

				// ->electroˁIi\j
				GStar* lpEvilStar = &this->m_Stars[StageType::EVIL_STAR];
				HPLVector vCenter = lpStar->getCenter();
				HPLVector vNextCenter = vTowerCenter;
				HPLVector vAim = HPLVector::subtract( vCenter, vNextCenter );
				float fDeg = vAim.getDegree();
				HPLVector vDelta;	// 
				vDelta.setByDegree( fDeg + 90 );

				HPLSequence* lpSeqBeam = getStaticData()->getSequence( SpriteType::COMMON,
					Sequence::Common::SELECT_BARRIER_BEAM );

				int nRoadImageHandle = getStaticData()->getImageHandle(
					SpriteType::COMMON, Sequence::Common::SELECT_BARRIER_BEAM,
					lpSeqBeam->getFrameIndex() );

				
				float fRoadHeight = lpSeqBeam->getImageInformation(0)->srcRect.getHeight();

				lpSeqBeam->step();

				DrawModiGraph(
					static_cast<int>( vCenter.getX() + vDelta.getX() * fRoadHeight / 2 ),
					static_cast<int>( vCenter.getY() + vDelta.getY() * fRoadHeight / 2 ),
					static_cast<int>( vNextCenter.getX() + vDelta.getX() * fRoadHeight / 2 ),
					static_cast<int>( vNextCenter.getY() + vDelta.getY() * fRoadHeight / 2 ),
					static_cast<int>( vNextCenter.getX() - vDelta.getX() * fRoadHeight / 2 ),
					static_cast<int>( vNextCenter.getY() - vDelta.getY() * fRoadHeight / 2 ),
					static_cast<int>( vCenter.getX() - vDelta.getX() * fRoadHeight / 2 ),
					static_cast<int>( vCenter.getY() - vDelta.getY() * fRoadHeight / 2 ),
					nRoadImageHandle, TRUE );
			}

			switch( lpStar->getType() ) {
			case StageType::EVIL_STAR:
				{
					// draw tower
					getStaticData()->m_Basic.m_Graphics.draw(
						vTowerCenter.getX(), vTowerCenter.getY(), 
						getStaticData()->getSequence( SpriteType::COMMON, Sequence::Common::SELECT_EVIL_TOWER),
						FaceDirection::LEFT, 0,1 );

					if( getStaticData()->m_Save.getFlag( FlagType::DESTROY_BARRIER ) != m_nFullDestroyShield ) {
						// SV[hSɂ͔j󂵂ĂȂ

						SetDrawBlendMode( DX_BLENDMODE_ADD, HPLMath::nextInt( 100, 200 ) );
						// ->V[h`
						getStaticData()->m_Basic.m_Graphics.draw(
							vTowerCenter.getX(), vTowerCenter.getY(), 
							getStaticData()->getSequence( SpriteType::COMMON, Sequence::Common::SELECT_EVIL_TOWER_SHIELD),
							FaceDirection::LEFT, 0,1 );

						SetDrawBlendMode( DX_BLENDMODE_NOBLEND, 255 );
					}
				}
				break;
			case StageType::REST_STAR:
				{
					if( bIsDrawJamming ) {
						// draw jamming
						getStaticData()->m_Basic.m_Graphics.draw(
							vJammingCenter.getX(), vJammingCenter.getY(), 
							getStaticData()->getSequence( SpriteType::NEUTRAL, Sequence::Neutral::JAMMING),
							FaceDirection::LEFT, 0,fZoom * 0.5f );
					}
				}
				break;
			}
			if( getStaticData()->m_Save.m_aryStarState[lpStar->getType()] != PlaceType::SEALED ) {
				// opened

				// 
			/*	HPLVector vSelectCenter = this->m_SelectPlayer.getCenter();
				HPLVector vLength(
					lpStar->getCenter().getX() - vSelectCenter.getX(),
					lpStar->getCenter().getY() - vSelectCenter.getY() );

				if( vLength.getLength() < SELECT_STAR_THRESHOLD ) {
					// I
					this->m_nSelectedStar = lpStar->getType();
					
				}*/
			} else {
				// sealed 
				// -> draw cloud over start

				HPLSequence* lpSeqCloud = getStaticData()->getSequence( SpriteType::COMMON,
					Sequence::Common::SELECT_CLOUD );
				// Cloud
				getStaticData()->m_Basic.m_Graphics.draw( 
					static_cast<int>( lpStar->getCenter().getX() ),
					static_cast<int>( lpStar->getCenter().getY() ),
					lpSeqCloud, FaceDirection::LEFT, 0, fZoom );
			}

			// I𒆂̓J[\`
			if( !bIsOnTheVTOL && this->m_nSelectedStar == lpStar->getType() ) {	// selected
				// ̉摜TCY
				HPLRect rZoomedSrcRect = lpSeq->getImageInformation()->srcRect;
				int nOrgWidth = rZoomedSrcRect.getWidth();
				int nOrgHeight = rZoomedSrcRect.getHeight();
				int nCenterX = nOrgWidth / 2;	// ΒSW
				int nCenterY = nOrgHeight / 2;
				int nNewWidth = (int)(nOrgWidth * fZoom);
				int nNewHeight = (int)(nOrgWidth * fZoom);
	//				int nNewLeft = nOrgWidth - nNewWidth / 2;
	//				int nNewTop = nOrgHeight - nNewHeight / 2;
				//rZoomedSrcRect.set( nNewLeft, nNewTop, nNewWidth, nNewHeight);

				// ΍WɕύX
				HPLVector vCenter = lpStar->getCenter();
				//rZoomedSrcRect.setX( rZoomedSrcRect.getX() + vCenter.getX() );
				//rZoomedSrcRect.setY( rZoomedSrcRect.getY() + vCenter.getY() );

				// \
				getStaticData()->drawSelectFrame(
					vCenter.getX() - nNewWidth / 2,
					vCenter.getY() - nNewHeight / 2,
					nNewWidth, nNewHeight );
			}
			
			if( lpStar->getType() == this->m_nSelectedStar &&
				this->getEffectType() == SelectStarSceneEffectType::BARRIER_EXPLOSION )
			{
				// Ă鐯
				// oAj󉉏o

				HPLVector vStarCenter = lpStar->getCenter();

				HPLSequence* lpEffectSeq = this->m_lpEffectSprite->getSequence( Sequence::Shot::EXPLOSION, FaceDirection::LEFT );
				// ʒ͔ʂ\
				getStaticData()->m_Basic.m_Graphics.draw(
					vStarCenter.getX(), vStarCenter.getY(),
					lpEffectSeq,
					FaceDirection::LEFT, 1, fZoom );

				if( lpEffectSeq->step().isFinishAnimation ) {
					// I
					this->setEffectType( SelectStarSceneEffectType::NONE );

					// r[
					HPLMath::setFlagByIndexInt( getStaticData()->m_Save.getFlag( FlagType::DESTROY_BARRIER ), 
						lpStar->getType(), true );
				}
			}
		} // rnd of for stars

		if( this->getEffectType() == SelectStarSceneEffectType::NONE ) {
			// oł͂Ȃ

			if( bIsDrawVTOL ) {
				HPLSequence* lpVTOLSeq = getStaticData()->getSequence( SpriteType::COMMON, Sequence::Common::SELECT_VTOL );
				// VTOLW
				HPLVector vCenter = this->m_VTOL.getCenter();
				
				// VTOLɂ\
				getStaticData()->m_Basic.m_Graphics.draw(
					static_cast<int>( vCenter.getX() ),
					static_cast<int>( vCenter.getY() ),
					lpVTOLSeq, (int)HPLMath::getSgn( this->m_VTOL.getVelocityAngle().getX() ),0,1);

				// Aj[V
				lpVTOLSeq->step();

				// ړ
				if( false == getStaticData()->isFadingOutDX() ) {
					vCenter.add( this->m_VTOL.getVelocityAngle() );
					if( false == getStaticData()->m_Setting.merginedGameRange.isPointInRect(
						vCenter.getX(), vCenter.getY() ) )
					{
						// reset
						this->resetVTOL();
					}
					this->m_VTOL.setCenter( vCenter );
				}
			}

		}
		if( this->getEffectType() == SelectStarSceneEffectType::BARRIER_EXPLOSION ) {
			// oAj󉉏o
			// ݂ɂ͕\Ȃ
		} else {
			// ݂ɂ\
			//if( this->m_nSelectedStar == lpStar->getType() )
			// ݂ɂ͕Kԏɕ\?
			// IĂ

			// ~jуV[PX
			int nSequenceID = this->m_SelectPlayer.m_nSequenceID;
			HPLSequence* lpEviSeq = getStaticData()->getSequence( SpriteType::COMMON, nSequenceID );

			int nFaceDirection = FaceDirection::LEFT;
			if( nSequenceID == Sequence::Common::SELECT_MINI_EVIGEN_LEFT ) {
				nFaceDirection = m_SelectPlayer.getFaceDirection();
			}
			// ~jт\
			getStaticData()->m_Basic.m_Graphics.draw(
				this->m_SelectPlayer.getCenter().getX(), this->m_SelectPlayer.getCenter().getY(),
				lpEviSeq, nFaceDirection, 0, 1 );

			// Aj[V
			lpEviSeq->step();
		}
	}

	getStaticData()->commonForegroundProcess();

	// Shot 
	getStaticData()->m_ObjectManager.m_ShotManager.process(true, true);



	if( bIsDrawVTOL && 	// VTOL
		bIsOnTheVTOL )	// on the VTOL
		{
		// VTOLɏĂ

		getStaticData()->drawDescription( "VTOL", true, 0  );

		
		HPLSequence* lpVTOLSeq = getStaticData()->getSequence( SpriteType::COMMON, Sequence::Common::SELECT_VTOL );
		HPLRect rSrcRect = lpVTOLSeq->getImageInformation()->srcRect;
		HPLVector vCenter = this->m_VTOL.getCenter();
		// \
		getStaticData()->drawSelectFrame(
			vCenter.getX() - rSrcRect.getWidth() / 2,
			vCenter.getY() - rSrcRect.getHeight() / 2,
			rSrcRect.getWidth(), rSrcRect.getHeight() );
		
	} else if ( this->m_nSelectedStar >= 0 ) {
		// ɏĂ

		getStaticData()->drawDescription( STAR_LABELS[ this->m_nSelectedStar ], true, 0  );

	}
	if( getStaticData()->isFadingOutDX() == false && this->getEffectType() == SelectStarSceneEffectType::NONE ) {
		// tF[hAEgĂȂ
		// sȃGtFNgł͂Ȃ
		// L[\
		{
			// ㉺
		/*	int nDeltaY = 0;
			if( getStaticData()->getInputManager()->isPressed( KeyMap::UP )) {
				nDeltaY = -1;
			}
			if( getStaticData()->getInputManager()->isPressed( KeyMap::DOWN )) {
				nDeltaY = 1;
			}
			if( nDeltaY != 0 ) {
				const float STAR_MOVE_SPEED = 10.f;
				for( int i = 0; i < StageType::MAX_NUM; i ++ ) {
					HPLVector vCenter = this->m_Stars[i].getCenter();
					vCenter.add(0,nDeltaY * STAR_MOVE_SPEED );
					this->m_Stars[i].setCenter( vCenter );
				}
			}*/
		}

		// ]
		{
			int nDeltaY = -1;
			if( getStaticData()->getInputManager()->isPressed( KeyMap::C )) {
				// ieXgj
				nDeltaY = -10;
			}
			/*
			if( getStaticData()->getInputManager()->isPressed( KeyMap::DOWN )) {
				nDeltaY = 1;
			}*/
			//if( nDeltaY != 0 ) {

			if( this->getEffectType() == SelectStarSceneEffectType::NONE ) {
				// oꍇ
				// ->]

				// ]px
				const float STAR_ROTATE_SPEED = 0.1f;

				// ]pis
				this->m_fRotateDegree += nDeltaY * STAR_ROTATE_SPEED;
				if( this->m_fRotateDegree > DEGREE_FULL_ROUND ) {
					// 360xz0ɂ
					this->m_fRotateDegree -= DEGREE_FULL_ROUND;

				} else if( this->m_fRotateDegree < 0 ) {
					// }CiXɂȂ360
					this->m_fRotateDegree += DEGREE_FULL_ROUND;
				}
			}

			// ʒuݒ
			for( int i = 1; i < StageType::MAX_NUM; i ++ ) {
				this->m_Stars[i].m_vHomePosition = this->getCenter( i );
				this->m_Stars[i].setCenter( this->m_Stars[i].m_vHomePosition );
			}
			//}
		}

		// Vړ
		{
			int nDY = 0;
			int nDX = 0;

			//int nID = Sequence::Common::SELECT_MINI_EVIGEN;
			
			if( getStaticData()->getInputManager()->isPressed( KeyMap::UP ) ) {
				nDY = -1;
			}
			if( getStaticData()->getInputManager()->isPressed( KeyMap::DOWN ) ) {
				nDY = 1;
			}
			if( getStaticData()->getInputManager()->isPressed( KeyMap::LEFT ) ) {
				nDX = -1;
			}
			if( getStaticData()->getInputManager()->isPressed( KeyMap::RIGHT ) ) {
				nDX = 1;
			}

			this->m_SelectPlayer.m_nSequenceID = Sequence::Common::SELECT_FUYU;

			HPLVector vCenter = m_SelectPlayer.getCenter();
			
			HPLVector vVelocityAngle = this->m_SelectPlayer.getVelocityAngle();
			// 

			const float SELECT_RESIST = 0.2f;

			float fAccel = SELECT_STAR_MOVE_ACCEL + SELECT_RESIST;
			// x
			HPLActionMoveTool::accelWithLimitX(
				&vVelocityAngle,
				HPLVector(nDX * fAccel,0),
//				HPLVector(SELECT_RESIST, 0),
				SELECT_STAR_MOVE_SPEED);

			HPLActionMoveTool::accelWithLimitY(
				&vVelocityAngle,
				HPLVector(0, nDY * fAccel),
//				HPLVector(0, SELECT_RESIST),
				SELECT_STAR_MOVE_SPEED);

			// 
			vVelocityAngle.setX(
				HPLActionMoveTool::reduceSpeed( SELECT_RESIST, vVelocityAngle.getX() ) );
			// 
			vVelocityAngle.setY(
				HPLActionMoveTool::reduceSpeed( SELECT_RESIST, vVelocityAngle.getY() ) );

			vCenter.add( vVelocityAngle );
			this->m_SelectPlayer.setVelocityAngle( vVelocityAngle );

			this->m_SelectPlayer.setCenter( vCenter );
			
			// [o
			// [ɗ甽Α֔
			{
				HPLRect rGameRect = getStaticData()->getGameRange();
				HPLRect rHitArea = this->m_SelectPlayer.getHitArea();

				bool bIsLooped = false;	// [v
				
				if( rHitArea.getX() > rGameRect.getRight() ) {
					// Ez
					// ->[ɗ
					vCenter.setX( rGameRect.getX() - ( rHitArea.getRight() - vCenter.getX() ) );
					bIsLooped = true;
				}
				if( rHitArea.getRight() < rGameRect.getX() ) {
					// z
					// ->E[ɗ
					vCenter.setX( rGameRect.getRight() + ( vCenter.getX() - rHitArea.getX() ) );
					bIsLooped = true;
				}
				if( rHitArea.getY() > rGameRect.getBottom() ) {
					// z
					// ->[ɗ
					vCenter.setY( rGameRect.getY() - ( rHitArea.getBottom() - vCenter.getY() ) );
					bIsLooped = true;
				}
				if( rHitArea.getBottom() < rGameRect.getY() ) {
					// z
					// ->[ɗ
					vCenter.setY( rGameRect.getBottom() + ( vCenter.getY() - rHitArea.getY() ) );
					bIsLooped = true;
				}

				this->m_SelectPlayer.setCenter(vCenter);

				if( bIsLooped ) {
					getStaticData()->checkAndStartEvent(FlagType::ONE_TALK, OneTalkType::LOOP, 
						SpecialScenarioIndex::LOOP_EXPLAIN );
				}
			}
			

			
			/*
			if( getStaticData()->getInputManager()->isPressedNow( KeyMap::LEFT ) ) {
				this->m_nSelectedStar --;
				if( this->m_nSelectedStar < 0 ) {
					this->m_nSelectedStar = StageType::MAX_NUM - 1;
				}
			}
			if( getStaticData()->getInputManager()->isPressedNow( KeyMap::RIGHT ) ) {
				this->m_nSelectedStar ++;
				if( this->m_nSelectedStar >= StageType::MAX_NUM) {
					this->m_nSelectedStar = 0;
				}
			}*/
		}
		//////////////////////////////////////////////////////////////////////////////////////////////////////////
		if( ( bIsOnTheVTOL || this->m_nSelectedStar >= 0 && this->m_nSelectedStar < StageType::MAX_NUM ) &&
			getStaticData()->getInputManager()->isPressedNow( KeyMap::A ) )
		{
			// VTOL̏ or ̏
			// L[
			
			// sound
			getStaticData()->m_MixerManager.play( 
				SE::KETTEI,
				false );

			if( bIsOnTheVTOL ) {	// VTOL̏
				if( getStaticData()->m_ObjectManager.m_Player.getEvilGage() >= MAX_EVIL_GAGE ) {
					// over or equals to max evil gage
					// ->VTOL_BAD2(eat tanya)
				} else {
					if( getStaticData()->m_Save.getFlagBit( FlagType::DESTROY_JAMMING, FlagType::JammingFlagType::DESTROYED) ) {
						getStaticData()->startFadeOutDX( SceneType::GAME, SpecialMapIndex::VTOL_GOOD );
					} else {
						getStaticData()->startFadeOutDX( SceneType::GAME, SpecialMapIndex::VTOL_BAD );
					}
				}
			} else {
				// StarI
				getStaticData()->m_nSelectedStar = this->m_nSelectedStar;
				getStaticData()->m_nSelectedPlace = 0;
				// save
				getStaticData()->m_Save.setFlag( FlagType::STAR, this->m_nSelectedStar );
				getStaticData()->m_Save.setFlag( FlagType::PLACE, getStaticData()->m_nSelectedPlace );
				getStaticData()->m_Save.save();

				// PlaceIցB
				getStaticData()->startFadeOutDX(SceneType::SELECT_PLACE, -1 );
				//getStaticData()->m_SceneManager.setScene( SceneType::SELECT_PLACE );
			}
		}
		if( getStaticData()->getInputManager()->isPressedNow( KeyMap::B ) ) {

			// PlaceIցB
			getStaticData()->startFadeOutDX(SceneType::TITLE, -1 );
			//getStaticData()->m_SceneManager.setScene( SceneType::SELECT_PLACE );
			// sound
			getStaticData()->m_MixerManager.play( 
				SE::CANCEL,
				false );
		}

		// --------------------- тс[ނ͂˂
		if( bIsDrawJamming &&	// jamming alive
			 m_ShotEvilBeamCounter.step().isFinishAnimation )	// step interval
		{
			m_ShotEvilBeamCounter.reset();

			GStar* lpStar = &this->m_Stars[StageType::REST_STAR];
			HPLVector vJammingCenter = lpStar->getCenter();
			float fZoom = getZoom( lpStar->getCenter().getY() );
			const float JAMMING_TOWER_DELTA_Y = (-100.f * fZoom);
			vJammingCenter.add(0,JAMMING_TOWER_DELTA_Y);
			GShot* lpShot = getStaticData()->m_ObjectManager.m_ShotManager.create(
				Sequence::Shot::EVIL_BEAM, vJammingCenter,
				lpStar->getID(), FaceDirection::LEFT, false, true );
			lpShot->setVelocityAngle(0,-EVIL_BEAM_SPEED);
			if( bIsDrawVTOL ) {
				lpShot->setTarget( &this->m_VTOL );
			} else {
				HPLVector v;
				v.setByDegree(HPLMath::nextInt(0,(int)HPLMathConsts::DEGREE_FULL_ROUND ) );
				v.multiple( EVIL_BEAM_SPEED );
				lpShot->setVelocityAngle(v);
				lpShot->setTarget( NULL);
				lpShot->m_nHomingType = HomingType::NONE;
			}
		}
	}
	}

	if(getStaticData()->m_EventImage.isActivated()){
		// draw image
		HPLSequence* lpSeq = getStaticData()->getSequence( getStaticData()->m_EventImage.spriteID,
			getStaticData()->m_EventImage.seqID );
		getStaticData()->m_Basic.m_Graphics.draw( 
			0,0, lpSeq );

	}
	getStaticData()->checkEvent(false, EventStepType::DEFAULT, true, false);

}

/** ʒuZbg
  */
void GSelectStageScene::resetVTOL() {
	const float DELTA = 10.f;

	float x = 0;
	float y = 0;
	HPLRect rMergined = getStaticData()->m_Setting.merginedGameRange;
	HPLMath::getFrameRandomPosition( rMergined.getX(), rMergined.getY(), rMergined.getWidth(), rMergined.getHeight(),
		x, y );
	this->m_VTOL.setCenter(x, y);

	const float TARGET_MERGINE = 200.f;
	HPLRect rGame = getStaticData()->m_Setting.gameRange;
	rGame.set(
		rGame.getX() + TARGET_MERGINE,
		rGame.getY() + TARGET_MERGINE,
		rGame.getWidth() - 2 * TARGET_MERGINE,
		rGame.getHeight() - 2 * TARGET_MERGINE
		);

	HPLVector vVelocityAngle( 
		static_cast<float>( HPLMath::nextInt(rGame.getX(), rGame.getRight() ) - x ),
		static_cast<float>( HPLMath::nextInt(rGame.getY(), rGame.getBottom() ) - y  ));
	vVelocityAngle.normalize();

	vVelocityAngle.multiple( SELECT_VTOL_SPEED );

	m_VTOL.setVelocityAngle( vVelocityAngle );
}


void GSelectStageScene::setEffectType( int nEffectType ) {
	this->m_nEffectType = nEffectType;
	
	int nResetSeq = -1;
	switch( nEffectType ) {
	case SelectStarSceneEffectType::BARRIER_EXPLOSION:
		nResetSeq = Sequence::Shot::EXPLOSION;
		break;
	}
	if( nResetSeq >= 0 ) {
		this->m_lpEffectSprite->getSequence( nResetSeq, FaceDirection::LEFT )->resetCounter();
	}
}
