#include "GShot.h"
#include "GStaticData.h"

#include <hpllib/action/HPLActionMoveTool.h>
#include <hpllib/action/HPLCollisionCheckTool.h>
#include <hpllib/math/HPLMath.h>

#include <vector>
using namespace std;

using namespace GEnum;

using namespace HPLMathConsts;

HPLVector HomingInformation::getControlPoint( HPLVector& vFrom, HPLVector& vTo,
											 float fControlU, 
											 float fControlImageRate )
{
	float fControlK = sqrt(
		abs( (vTo.getX()-vFrom.getX())) * 2 + abs( ( vTo.getY() - vFrom.getY() ) * 2 )
		);

	HPLVector vControl(
		( 1.f - fControlU ) * vFrom.getX() + fControlU * vTo.getX(),
		( 1.f - fControlU ) * vFrom.getY() + fControlU * vTo.getY() );

	float fControlR = atan2( vFrom.getY() - vControl.getY() , vFrom.getX() - vControl.getY() );
	float fControlImage = fControlImageRate * fControlK;

	HPLVector vControlRD = vControl;
	vControlRD.add(
		fControlImage * cos( fControlR + PI / 2 ),
		fControlImage * sin( fControlR + PI / 2 )
		);

	HPLVector vControlLD = vControl;
	vControlLD.add(
		fControlImage * cos( fControlR - PI / 2 ),
		fControlImage * sin( fControlR - PI / 2 )
		);

	fControlU = static_cast<float>( HPLMath::nextDouble(0,1.f ) );
	HPLVector vControlPoint(
		( 1.f - fControlU ) * vControlRD.getX() + fControlU * vControlLD.getX(),
		( 1.f - fControlU ) * vControlRD.getY() + fControlU * vControlLD.getY()
		);
	return vControlPoint;
}

void HomingInformation::getControlPoints( HPLVector vFrom, HPLVector vTo,
		HPLVector* lpvControlUP, HPLVector* lpvControlDW) {
	/*HPLVector vControl[2];
	float fControlU = 0.3f;
	float fControlR = 0;
	HPLVector vControlRD;
	HPLVector vControlLD;
	float fControlImage = 0;
	float fControlK = 0;*/

	//fControlK = sqrt( abs( vTo.getX() - vFrom.getX() ) * 2 + 
	//	abs( vTo.getY() - vFrom.getY() ) * 2 );

	float fBack = 1.f;

	HPLRect rRect = getStaticData()->getGameRange();

	float fKRate = 12.f;

	if( rRect.isPointInRect( vTo.getX(), vTo.getY() ) ) {
		fBack *= 3.f;
		fKRate *= 5.f;
	}

	*lpvControlUP = this->getControlPoint(
		vFrom, vTo, 0.5f, fKRate );

	*lpvControlDW = this->getControlPoint(
		vFrom, vTo, -fBack, fKRate );

}

GShot::GShot(int nType):HPLShootingObject(nType) {
	this->m_bIsUseReduceAccel = false;
	this->m_fReduceAccel = 0;

	this->setObjectType( ObjectType::SHOT );

	this->setParentWeaponType(-1);

	this->m_lpClipCounter = NULL;

//	this->m_lpIntervalCounter = NULL;

#ifdef _DEBUG
	//HPLError::assertIt( nType >= 0 && nType < Sequence::Shot::MAX_NUM);
	ASSERT_IT_INDEX( nType, 0, Sequence::Shot::MAX_NUM );
#endif
	GShotSetting* lpSetting = getStaticData()->m_ObjectManager.m_ShotManager.getShotSetting(nType);
	
	//if( nType == Sequence::Shot::DUMMY ) {
	//	this->setSprite( NULL );
	//} else {
	HPLSprite* lpSprite = new HPLSprite();
	lpSprite->setSequence( nType, getStaticData()->getSprite( SpriteType::SHOT)->getPair(nType )->clone() );
	for( int i = 0; i < (int)lpSetting->m_vecExtraSequenceID.size(); i ++ ) {
		int nSeqID = lpSetting->m_vecExtraSequenceID[i] + 
			Sequence::Shot::MAX_NUM;
		lpSprite->setSequence( nSeqID,
			getStaticData()->getSprite( SpriteType::SHOT)->getPair( nSeqID )->clone() );
	}
	this->setSprite( lpSprite );
//	this->setSprite( getStaticData()->getSprite( SpriteType::SHOT)->clone() );
	//}

	this->setObjectAttribute( ObjectAttribute::HIT_TO_NEUTRAL,
		lpSetting->m_aryFlags[ShotSettingFlagType::HIT_TO_NEUTRAL] );	// hit to neutral
	this->setObjectAttribute( ObjectAttribute::HIT_OBJECT_TO_DIE,
		lpSetting->m_aryFlags[ShotSettingFlagType::HIT_OBJECT_TO_DIE] );
	this->setObjectAttribute( ObjectAttribute::HIT_BLOCK_TO_DIE, true );
	this->setObjectAttribute( ObjectAttribute::REFRECTABLE, true );
	this->setObjectAttribute( ObjectAttribute::HIT_TO_BLOCK, true );
	this->setObjectAttribute( ObjectAttribute::HIT_TO_ENEMY, true );
	this->setObjectAttribute( ObjectAttribute::HIT_TO_SHOT,
		lpSetting->m_aryFlags[ ShotSettingFlagType::HIT_TO_SHOT ] );
	this->setObjectAttribute( ObjectAttribute::GRAVITY, 
		lpSetting->m_aryFlags[ ShotSettingFlagType::GRAVITY ]);

	// VbgNX^Cv
	this->setShotClassType( ShotClassType::DEFAULT );

	// ftHg̓蔻=ۂ̋`
	if( lpSetting->m_aryHitRect[2] > 0 ) {
		this->setHitRect(
			(float)lpSetting->m_aryHitRect[0],
			(float)lpSetting->m_aryHitRect[1],
			(float)lpSetting->m_aryHitRect[2],
			(float)lpSetting->m_aryHitRect[3]
			);//6,13,12,10);
	} else {
		this->setHitRect(
			0,0,this->getWidth(), this->getHeight() );
	}
	if( this->getSprite() ) {
		HPLRect rSrcRect = this->getSprite()->getSequence( nType ,FaceDirection::LEFT)->getImageInformation(0)->srcRect;
		this->setSize( rSrcRect.getWidth(), rSrcRect.getHeight() );
	}

	this->m_nCreateShotTypeWhenDead = -1;

	this->setState( StateType::STAND );
	
	// ftHgݒ
	this->m_bIsDrawRotateByVelocityAngle = true;	// ɍ킹ĉ]\
	this->m_bIsNoMove = false;					// ړ
	this->m_nHomingType = HomingType::NONE;	// z[~O^CvȂ
	this->m_lpHomingData = NULL;	// z[~OȂ
	this->m_nContrailType = -1;	// K^CvȂ
	this->m_lpContrailCreateInterval = NULL;	// K
//	this->m_nDamageType = 0;		//! _[W^CvȂ
	
	this->m_fRotateSpeed = 0;
	this->m_bIsClip = false;
	this->m_nHitToSolidType = SolidType::NONE;

	const float POWER_RATE = 10.f;
	this->m_fPower = (float)lpSetting->m_nPower / POWER_RATE;//POWERS[nType];

	const float BLOW_SPEED_RATE = 10.f;
	this->m_fBlowSpeed = (float)lpSetting->m_nBlowSpeed / BLOW_SPEED_RATE;//BLOW_SPEEDS[nType];

	// Geʏ
	bool bIsEnemyShot = lpSetting->m_aryFlags[ShotSettingFlagType::ENEMY_SHOT];	
	// ǊђʋʏB
	bool bIsBlockThrowShot = lpSetting->m_aryFlags[ShotSettingFlagType::THROW_BLOCK];	
									// 邯ǏȂêŁȀꍇ͐ݒ肵Ȃ

	this->m_lpPuyoPuyoCounter = NULL;
	
	float fLevel = getStaticData()->m_Game.getLevel( LEVEL_EPSILON );

	const int SLIME_CLIP_COUNTER = (int)(5.f * fLevel);	// Uقǂ[^[

	float fHPRate = fLevel;

	// łߏ񏉊
	this->setSolidType( SolidType::NONE );

	// G̒ȅꍇɁAvC[Ƀqbg邩ǂ
	bool bIsHitToPlayerWhenEnemyShot = true;
	switch( nType ) {
	case Sequence::Shot::FIRE_TRAP:
		//this->setHitRect(0,14,32,114);
		//this->setObjectAttribute( ObjectAttribute::HIT_OBJECT_TO_DIE, false );
		//bIsBlockThrowShot = true;
		//this->setObjectAttribute( ObjectAttribute::HIT_TO_PLAYER, true );

		//this->setObjectAttribute( ObjectAttribute::HIT_TO_NEUTRAL, false );	// Ȃ

		this->m_bIsNoMove = true;
		this->m_bIsDrawRotateByVelocityAngle = false;
		this->setDamageType( DamageType::FIRE, true );
		this->m_Weakness.set( DamageType::WATER, true );
		this->m_Weakness.set( DamageType::ICE, true );
		break;
	case Sequence::Shot::NATO_BULLET:
	case Sequence::Shot::MAGNUM:
	case Sequence::Shot::SNIPER:
	case Sequence::Shot::NATO_10MM:
		//this->setHitRect(6,13,12,10);
		this->setDamageType( DamageType::BULLET, true );
		break;
	case Sequence::Shot::HOMING_LASER:
	case Sequence::Shot::ELECTRIC_HOMING_LASER:
		/*||
		nType == Sequence::Shot::HOMING_LASER_TAIL ||
		nType == Sequence::Shot::HOMING_LASER_2
		)*/
		{
			this->setHitRect( 0,0, getWidth(), getHeight() );

			bIsBlockThrowShot = true;
//			this->setObjectAttribute( ObjectAttribute::HIT_BLOCK_TO_DIE, false );
			this->setObjectAttribute( ObjectAttribute::HIT_OBJECT_TO_DIE, false );
			this->setObjectAttribute( ObjectAttribute::HIT_TO_NEUTRAL, false );	// Ȃ
//			this->m_aryObjectAttribute[ObjectAttribute::HIT_BLOCK_TO_DIE] = false;	// ubNɓȂ
//			this->m_aryObjectAttribute[ObjectAttribute::HIT_OBJECT_TO_DIE] = false;	// ubNɓďȂ

			this->m_nHomingType = HomingType::BEZIE;
			
			this->m_lpHomingData = new HomingData();
			this->m_bIsNoMove = true;					// ȓ邽ߏO

			/*if( nType == Sequence::Shot::HOMING_LASER_TAIL ) {
				this->m_bIsNoMove = true;
			} else {
				this->m_nHomingType = HomingType::NORMAL;
			}*/

			/*if( nType == Sequence::Shot::HOMING_LASER_2 ) {
				this->m_vecCounters.push_back( new HPLCounter( HOMING_LASER_2_DEACCEL_FRAMES ) );
			}*/
			this->setDamageType( DamageType::LAZER, true );
			if( Sequence::Shot::ELECTRIC_HOMING_LASER == nType ) {
				this->setDamageType( DamageType::ELEC, true );
			}
			
		}
		break;
	case Sequence::Shot::HOMING_MISSILE:
	case Sequence::Shot::SPREADED_MISSILE:
		this->m_nHomingType = HomingType::MISSILE;
//		this->m_lpHomingData = new HomingData();
	case Sequence::Shot::MISSILE:
	case Sequence::Shot::NIKITA_MISSILE:
	case Sequence::Shot::SPREAD_MISSILE:
//		this->setHitRect(6,13,12,10);
		// missile contrail𔭐
		this->m_lpContrailCreateInterval = new HPLCounter( MISSILE_CONTRAIL_CREATE_INTERVAL );
		this->m_lpContrailCreateInterval->setLoop( true );
		this->m_nContrailObjectType = ObjectType::EFFECT;
		this->m_nContrailType = Sequence::Effect::MISSILE_CONTRAIL;

		this->m_nCreateShotTypeWhenDead = Sequence::Shot::EXPLOSION;
		this->setDamageType( DamageType::EXPLOSION, true );
		break;
	case Sequence::Shot::BEAM_1:	// ؂ւL (IveBbNuXgr[)
		bIsBlockThrowShot = true;
//		this->setObjectAttribute( ObjectAttribute::HIT_TO_BLOCK, false );
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_BLOCK] = false;
		this->setDamageType( DamageType::LAZER, true );
		break;
	case Sequence::Shot::PINE_APPLE_0:	// G̐؂ւL
	case Sequence::Shot::PINE_APPLE_1:
	case Sequence::Shot::SEED:
		// TODO neutral ɓƂ͔˂
		{
			this->setObjectAttribute( ObjectAttribute::HIT_OBJECT_TO_DIE, false );
			this->setObjectAttribute( ObjectAttribute::HIT_TO_ENEMY, false );
			this->setObjectAttribute( ObjectAttribute::HIT_TO_NEUTRAL, false );
			this->setObjectAttribute( ObjectAttribute::HIT_BLOCK_TO_DIE, false );
			this->setObjectAttribute( ObjectAttribute::BOUND, true );
			this->setObjectAttribute( ObjectAttribute::GRAVITY, true );
			this->setObjectAttribute( ObjectAttribute::HIT_TO_SHOT, true );
	//		this->m_aryObjectAttribute[ObjectAttribute::HIT_OBJECT_TO_DIE] = false;		//ĂȂ
	//		this->m_aryObjectAttribute[ObjectAttribute::HIT_BLOCK_TO_DIE] = false;		//ĂȂ
	//		this->m_aryObjectAttribute[ObjectAttribute::BOUND ] = true;
			switch( nType ) {
			case Sequence::Shot::SEED:
				this->m_Weakness.set( DamageType::FIRE, true );
				break;
			default:
				this->m_nCreateShotTypeWhenDead = Sequence::Shot::EXPLOSION;
			}
	//		this->m_aryObjectAttribute[ObjectAttribute::GRAVITY ] = true;
			this->m_bIsDrawRotateByVelocityAngle = false;

			this->setRotateDegree( HPLMath::nextFloat(0,HPLMathConsts::DEGREE_FULL_ROUND ) );
			this->setFaceDirection( HPLMath::nextInt(0,2) == 0 ? FaceDirection::LEFT : FaceDirection::RIGHT );
			float fRotateDegreeSpeed = (HPLMath::nextInt(0,2) * 2 - 1 ) * 
				HPLMath::nextFloat( ROCK_ROTATE_DEGREE_SPEED_MIN, ROCK_ROTATE_DEGREE_SPEED_MAX );
			// rotate deg speed
			this->setRotateDegreeSpeed( fRotateDegreeSpeed );
			// default rotate deg speed
			this->setDefaultRotateDegreeSpeed( fRotateDegreeSpeed );
			// damage type
			this->setDamageType( DamageType::HIT, true );
		}
		break;
	case Sequence::Shot::WIND_EGG:
		{
			this->setObjectAttribute( ObjectAttribute::HIT_OBJECT_TO_DIE, false );
			this->setObjectAttribute( ObjectAttribute::HIT_TO_ENEMY, false );
			this->setObjectAttribute( ObjectAttribute::HIT_TO_NEUTRAL, false );
			this->setObjectAttribute( ObjectAttribute::HIT_BLOCK_TO_DIE, false );
			this->setObjectAttribute( ObjectAttribute::BOUND, false );
			this->setObjectAttribute( ObjectAttribute::GRAVITY, true );
			this->setObjectAttribute( ObjectAttribute::HIT_TO_SHOT, false );
//			this->m_bIsDrawRotateByVelocityAngle = false;	// ]\Ȃ

//			this->setRotateDegree( HPLMath::nextFloat(0,HPLMathConsts::DEGREE_FULL_ROUND ) );
//			this->setFaceDirection( HPLMath::nextInt(0,2) == 0 ? FaceDirection::LEFT : FaceDirection::RIGHT );
//			float fRotateDegreeSpeed = (HPLMath::nextInt(0,2) * 2 - 1 ) * 
//				HPLMath::nextFloat( ROCK_ROTATE_DEGREE_SPEED_MIN, ROCK_ROTATE_DEGREE_SPEED_MAX );
			// rotate deg speed
//			this->setRotateDegreeSpeed( fRotateDegreeSpeed );
			// default rotate deg speed
//			this->setDefaultRotateDegreeSpeed( fRotateDegreeSpeed );
			// damage type
			this->setDamageType( DamageType::HIT, true );
		}
		break;
	case Sequence::Shot::TOMAHAWK:
		this->setHitRect(94,8,66,48 );
		this->m_nCreateShotTypeWhenDead = Sequence::Shot::EXPLOSION;
		this->setObjectAttribute( ObjectAttribute::GRAVITY, true );
//		this->m_aryObjectAttribute[ObjectAttribute::GRAVITY ] = true;
		this->m_bIsDrawRotateByVelocityAngle = false;
		this->setRotateDegree( HPLMath::nextFloat(0,HPLMathConsts::DEGREE_FULL_ROUND ) );
		this->m_fRotateSpeed = (HPLMath::nextInt(0,2) * 2 - 1 ) * 
			HPLMath::nextFloat( ROCK_ROTATE_DEGREE_SPEED_MIN, ROCK_ROTATE_DEGREE_SPEED_MAX );
		this->setDamageType( DamageType::EXPLOSION, true );
		break;
	case Sequence::Shot::EXPLOSION:
		this->setHitRect( 14,12,46,62);
		bIsBlockThrowShot = true;
//		this->setObjectAttribute( ObjectAttribute::HIT_TO_BLOCK, false );
		this->setObjectAttribute( ObjectAttribute::HIT_OBJECT_TO_DIE, false );
		// vC[ɂqbg
		this->setObjectAttribute( ObjectAttribute::HIT_TO_PLAYER, true );
//		this->setObjectAttribute( ObjectAttribute::HIT_TO_ENEMY, true );
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_BLOCK] = true;
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_OBJECT_TO_DIE] = false;		//ĂȂ
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_PLAYER] = true;
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_ENEMY] = true;
//		this->setHitToPlayer( true );
		this->m_bIsDrawRotateByVelocityAngle = false;
		this->setDamageType( DamageType::EXPLOSION, true );

		// SEoႤ
		getStaticData()->m_MixerManager.play( SE::EXPLOSION, false );

		// U
		getStaticData()->m_ShakeManager.startShake();
		break;
	case Sequence::Shot::NEN_EKI:
		bIsBlockThrowShot = true;
//		this->setObjectAttribute( ObjectAttribute::HIT_TO_BLOCK, false );
		this->setObjectAttribute( ObjectAttribute::HIT_TO_NEUTRAL, false );	// Ȃ
		this->m_Weakness.set( DamageType::FIRE, true );
		this->m_Weakness.set( DamageType::ICE, true );
		this->setDamageType( DamageType::WATER, true );

		bIsEnemyShot = true;
//		this->setObjectAttribute( ObjectAttribute::HIT_TO_PLAYER, true );
//		this->setObjectAttribute( ObjectAttribute::HIT_TO_ENEMY, false );
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_BLOCK] = false;
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_PLAYER] = true;
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_ENEMY] = false;
		break;
	case Sequence::Shot::FIRE_BALL:
		// G̉Ήe
		bIsEnemyShot = true;
		// this->setObjectAttribute( ObjectAttribute::HIT_TO_PLAYER, true );
		// this->setObjectAttribute( ObjectAttribute::HIT_TO_ENEMY, false );
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_PLAYER] = true;
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_ENEMY] = false;
		this->setDamageType( DamageType::FIRE, true );
		this->m_Weakness.set( DamageType::WATER, true );
		this->m_Weakness.set( DamageType::ICE, true );
		break;
	case Sequence::Shot::ICE_SPIKE:
		bIsEnemyShot = true;
		this->setDamageType( DamageType::ICE, true );
		this->setDamageType( DamageType::WATER, true );
		this->m_Weakness.set( DamageType::WATER, true );
		this->m_Weakness.set( DamageType::FIRE, true );
		break;
	case Sequence::Shot::FIRE:	// Ήˊp
		bIsBlockThrowShot = true;
		this->setObjectAttribute( ObjectAttribute::HIT_OBJECT_TO_DIE, false );
//		this->setObjectAttribute( ObjectAttribute::HIT_TO_BLOCK, false );
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_OBJECT_TO_DIE] = false;		//ĂȂ
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_BLOCK] = false;
		this->m_bIsDrawRotateByVelocityAngle = false;

		this->setDamageType( DamageType::FIRE, true );
		// vC[Vbg͓ƂǂƂ͏Ȃ
//		this->m_Weakness.set( DamageType::WATER, true );
//		this->m_Weakness.set( DamageType::ICE, true );

		//this->setObjectAttribute( ObjectAttribute::HIT_TO_SHOT, true );
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_SHOT] = true;	// ꕔ̃Vbgł
		break;
	case Sequence::Shot::DUMMY:
		this->setSize(2,2 );
		this->setHitRect( (float)(-PUNCH_HIT_RECT_SIZE),(float)(-PUNCH_HIT_RECT_SIZE_Y),
			(float)(PUNCH_HIT_RECT_SIZE * 2) ,(float)(PUNCH_HIT_RECT_SIZE_Y * 2));

		this->setObjectAttribute( ObjectAttribute::HIT_OBJECT_TO_DIE, false );
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_OBJECT_TO_DIE] = false;
		this->setDamageType( DamageType::HIT, true );
		break;
	case Sequence::Shot::SLIME_CLIP:
		{
			// ftHgJ[FF
			this->setColor( SlimeQueenColorType::YELLOW );

			bIsEnemyShot = true;
	//		this->setObjectAttribute( ObjectAttribute::HIT_TO_ENEMY, false );
	//		this->setObjectAttribute( ObjectAttribute::HIT_TO_PLAYER, true );
			bIsBlockThrowShot = true;
	//		this->setObjectAttribute( ObjectAttribute::HIT_TO_BLOCK, false );
	//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_ENEMY] = false;
	//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_BLOCK] = false;
	//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_PLAYER] = true;

			this->setObjectAttribute( ObjectAttribute::GRAVITY, true );
	//		this->m_aryObjectAttribute[ObjectAttribute::GRAVITY ] = true;

			this->setHitRect(8,8,15,15);
			this->m_DamageIntervalCounter.setWaitFrameNum(
				HPLMath::nextInt(
					(int)(SLIME_CLIP_DAMAGE_INTERVAL_MIN / fLevel),
					(int)(SLIME_CLIP_DAMAGE_INTERVAL_MAX / fLevel) ) );

			this->m_bIsClip = true;
			this->m_lpClipCounter = new HPLCounter( SLIME_CLIP_COUNTER );
		}
		break;
	case Sequence::Shot::SNAKE:
		{
			bIsEnemyShot = true;
			bIsBlockThrowShot = true;
			this->setObjectAttribute( ObjectAttribute::GRAVITY, true );	// d͗L
			this->setHitRect(5,10,50,15);
			const int SNAKE_CLIP_DAMAGE_INTERVAL_MIN = 50;
			const int SNAKE_CLIP_DAMAGE_INTERVAL_MAX = 100;
			this->m_DamageIntervalCounter.setWaitFrameNum(
				HPLMath::nextInt(
					(int)(SNAKE_CLIP_DAMAGE_INTERVAL_MIN / fLevel),
					(int)(SNAKE_CLIP_DAMAGE_INTERVAL_MAX / fLevel) ) );
			this->m_bIsClip = true;
			const int SNAKE_CLIP_COUNTER = (int)(10 * fLevel);
			this->m_lpClipCounter = new HPLCounter( SLIME_CLIP_COUNTER );
		}
		break;
	case Sequence::Shot::WATER:
		this->setHitRect( 0,0,4,4);
		this->setObjectAttribute( ObjectAttribute::GRAVITY, true );
//		this->m_aryObjectAttribute[ObjectAttribute::GRAVITY] = true;
		this->setDamageType( DamageType::WATER, true );
		this->setObjectAttribute( ObjectAttribute::HIT_TO_SHOT, true );
		break;
	case Sequence::Shot::SATELITE_UNDER:
		this->setShotClassType( ShotClassType::LASER );
		this->setObjectAttribute( ObjectAttribute::GRAVITY, false );
		bIsBlockThrowShot = true;
//		this->setObjectAttribute( ObjectAttribute::HIT_TO_BLOCK, false );
//		this->m_aryObjectAttribute[ObjectAttribute::GRAVITY] = false;
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_BLOCK] = false;
		this->setDamageType( DamageType::LAZER, true );
		break;
	case Sequence::Shot::KOGUMO_1:
	case Sequence::Shot::KOGUMO_2:
		this->setObjectAttribute( ObjectAttribute::GRAVITY, true );
//		this->m_aryObjectAttribute[ObjectAttribute::GRAVITY] = true;
		if( nType == Sequence::Shot::KOGUMO_1 ) {
			this->setHitRect(9,9,15,15);
		} else if( nType == Sequence::Shot::KOGUMO_2 ) {
			this->setHitRect(3,3,9,9);
		}
		this->setObjectAttribute( ObjectAttribute::BOUND, true );
		this->setObjectAttribute( ObjectAttribute::HIT_OBJECT_TO_DIE, false );

		bIsEnemyShot = true;
//		this->setObjectAttribute( ObjectAttribute::HIT_TO_ENEMY, false );
//		this->setObjectAttribute( ObjectAttribute::HIT_TO_PLAYER, true );
//		this->m_aryObjectAttribute[ObjectAttribute::BOUND] = true;
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_OBJECT_TO_DIE ] = false;
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_ENEMY ] = false;
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_PLAYER ] = true;
		break; 
	case Sequence::Shot::NEEDLE:
		this->setHitRect(6,10,22,12 );
		bIsEnemyShot = true;
		bIsBlockThrowShot = true;
		this->setObjectAttribute( ObjectAttribute::HIT_TO_NEUTRAL, false );	// Ȃ
//		this->setObjectAttribute( ObjectAttribute::HIT_TO_ENEMY, false );
//		this->setObjectAttribute( ObjectAttribute::HIT_TO_PLAYER, true );
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_ENEMY ] = false;
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_PLAYER ] = true;
		break;
	case Sequence::Shot::RAIL:	// rail gun
		// shake screen
		getStaticData()->m_ShakeManager.startShake();
	case Sequence::Shot::LASER:
		this->setShotClassType( ShotClassType::LASER );
		this->setHitRect(0,0,getWidth(), getHeight() );
		bIsBlockThrowShot = true;
		this->setObjectAttribute( ObjectAttribute::HIT_OBJECT_TO_DIE, false );
//		this->setObjectAttribute( ObjectAttribute::HIT_TO_BLOCK, false );
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_BLOCK] = false;
		this->setDamageType( DamageType::LAZER, true );
		this->m_bIsDrawRotateByVelocityAngle = false;
		break;
/*		this->setHitRect(0,0,getWidth(), getHeight() );
		bIsBlockThrowShot = true;
		this->setObjectAttribute( ObjectAttribute::HIT_OBJECT_TO_DIE, false );
//		this->setObjectAttribute( ObjectAttribute::HIT_BLOCK_TO_DIE, false );
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_OBJECT_TO_DIE] = false;
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_BLOCK_TO_DIE] = false;
		this->setDamageType( DamageType::LAZER, true );
		this->m_bIsDrawRotateByVelocityAngle = false;
		break;*/
	case Sequence::Shot::MAYO:
		this->setHitRect( 0,0,4,4);
		this->setObjectAttribute( ObjectAttribute::GRAVITY, true );
		this->setObjectAttribute( ObjectAttribute::HALF_GRAVITY, true );
		this->setObjectAttribute( ObjectAttribute::HIT_BLOCK_TO_DIE, false );
		this->setObjectAttribute( ObjectAttribute::HIT_OBJECT_TO_DIE, false );
//		this->m_aryObjectAttribute[ObjectAttribute::GRAVITY] = true;
//		this->m_aryObjectAttribute[ObjectAttribute::HALF_GRAVITY] = true;
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_BLOCK_TO_DIE] = false;
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_OBJECT_TO_DIE] = false;
	//	this->setDamageType( DamageType::, true );
		break;
	case Sequence::Shot::LEAF:
		this->setHitRect(13,19,36,27);
		this->setObjectAttribute( ObjectAttribute::GRAVITY, true );
		this->setObjectAttribute( ObjectAttribute::HALF_GRAVITY, true );
//		this->m_aryObjectAttribute[ObjectAttribute::GRAVITY] = true;
//		this->m_aryObjectAttribute[ObjectAttribute::HALF_GRAVITY] = true;

		bIsEnemyShot = true;
//		this->setObjectAttribute( ObjectAttribute::HIT_TO_ENEMY, false );
//		this->setObjectAttribute( ObjectAttribute::HIT_TO_PLAYER, true );
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_ENEMY] = false;
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_PLAYER ] = true;
		bIsBlockThrowShot = true;
		this->setObjectAttribute( ObjectAttribute::HIT_TO_NEUTRAL, false );	// Ȃ
//		this->setObjectAttribute( ObjectAttribute::HIT_TO_BLOCK, false );
//		this->setObjectAttribute( ObjectAttribute::HIT_BLOCK_TO_DIE, false );
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_BLOCK ] = false;
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_BLOCK_TO_DIE] = false;
		
		break;
	case Sequence::Shot::TENTACLE:
		this->setHitRect(4,1,25,255);
		// no gravity
		bIsEnemyShot = true;
//		this->setObjectAttribute( ObjectAttribute::HIT_TO_ENEMY, false );
//		this->setObjectAttribute( ObjectAttribute::HIT_TO_PLAYER, true );
		bIsBlockThrowShot = true;
		this->setObjectAttribute( ObjectAttribute::HIT_TO_NEUTRAL, false );	// Ȃ
//		this->setObjectAttribute( ObjectAttribute::HIT_TO_BLOCK, false );
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_ENEMY] = false;
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_PLAYER ] = true;
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_BLOCK ] = false;

//		this->setObjectAttribute( ObjectAttribute::HIT_BLOCK_TO_DIE, false );
//		this->setObjectAttribute( ObjectAttribute::HIT_OBJECT_TO_DIE, false );
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_BLOCK_TO_DIE] = false;
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_OBJECT_TO_DIE] = false;
		this->m_bIsDrawRotateByVelocityAngle = false;
		break;
	case Sequence::Shot::KAHUN:
		this->setHitRect(15,12,33,40 );

		bIsEnemyShot = true;
//		this->setObjectAttribute( ObjectAttribute::HIT_TO_ENEMY, false );
//		this->setObjectAttribute( ObjectAttribute::HIT_TO_PLAYER, true );
		bIsBlockThrowShot = true;
//		this->setObjectAttribute( ObjectAttribute::HIT_TO_BLOCK, false );
//		this->setObjectAttribute( ObjectAttribute::HIT_BLOCK_TO_DIE, false );
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_ENEMY] = false;
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_PLAYER ] = true;
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_BLOCK ] = false;
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_BLOCK_TO_DIE] = false;

		this->setObjectAttribute( ObjectAttribute::HIT_TO_NEUTRAL, false );	// Ȃ
		this->setObjectAttribute( ObjectAttribute::GRAVITY, true );
		this->setObjectAttribute( ObjectAttribute::HALF_GRAVITY, true );
		this->m_Weakness.set( DamageType::FIRE, true );
//		this->m_aryObjectAttribute[ObjectAttribute::GRAVITY] = true;
//		this->m_aryObjectAttribute[ObjectAttribute::HALF_GRAVITY] = true;
		break;
	case Sequence::Shot::SPIDER_NET:
		{
			this->setHitRect(11,11,24,24);
			bIsEnemyShot = true;
	//		this->setObjectAttribute( ObjectAttribute::HIT_TO_ENEMY, false );
	//		this->setObjectAttribute( ObjectAttribute::HIT_TO_PLAYER, true );
			bIsBlockThrowShot = true;
	//		this->setObjectAttribute( ObjectAttribute::HIT_TO_BLOCK, false );
	//		this->setObjectAttribute( ObjectAttribute::HIT_BLOCK_TO_DIE, false );
	//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_ENEMY] = false;
	//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_PLAYER ] = true;
	//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_BLOCK ] = true;
	//		this->m_aryObjectAttribute[ObjectAttribute::HIT_BLOCK_TO_DIE] = false;

			this->setObjectAttribute( ObjectAttribute::HIT_TO_NEUTRAL, false );	// Ȃ
			this->setObjectAttribute( ObjectAttribute::GRAVITY, true );
	//		this->m_aryObjectAttribute[ObjectAttribute::GRAVITY] = true;
			//this->m_aryObjectAttribute[ObjectAttribute::HALF_GRAVITY] = true;

			this->m_Weakness.set( DamageType::FIRE, true );

			this->m_bIsClip = true;
			const int SPIDER_NET_CLIP_COUNTER = 5;
			this->m_lpClipCounter = new HPLCounter( SPIDER_NET_CLIP_COUNTER );
		}
		break;
	case Sequence::Shot::ONPA:
		this->setHitRect(17,14,36,35);
		bIsEnemyShot = true;
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_ENEMY] = false;
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_PLAYER ] = true;
		bIsBlockThrowShot = true;
		this->setObjectAttribute( ObjectAttribute::HIT_TO_NEUTRAL, false );	// Ȃ
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_BLOCK ] = false;
//		this->m_aryObjectAttribute[ObjectAttribute::HIT_BLOCK_TO_DIE] = false;
		break;
	case Sequence::Shot::ELEC:
	case Sequence::Shot::WIND:
		{
			if( Sequence::Shot::WIND == nType ) {
				this->m_bIsDrawRotateByVelocityAngle = false;
			} else {
				this->m_bIsDrawRotateByVelocityAngle = true;
			}
			this->setHitRect(7,10,18,14);
			bIsEnemyShot = true;
//			this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_ENEMY] = false;
//			this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_PLAYER ] = true;
			bIsBlockThrowShot = true;
//			this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_BLOCK ] = false;
//			this->m_aryObjectAttribute[ObjectAttribute::HIT_BLOCK_TO_DIE] = false;
			this->setObjectAttribute( ObjectAttribute::HIT_TO_NEUTRAL, false );	// Ȃ
		}
		break;
	case Sequence::Shot::ICE:
	case Sequence::Shot::FALL_ICE:
		{
			this->setHitRect(8,8,34,34);
			const int ICE_CONTRAIL_CREATE_INTERVAL = MISSILE_CONTRAIL_CREATE_INTERVAL;
			this->m_lpContrailCreateInterval = new HPLCounter( ICE_CONTRAIL_CREATE_INTERVAL );
			this->m_lpContrailCreateInterval->setLoop( true );
			this->m_nContrailObjectType = ObjectType::EFFECT;
			this->m_nContrailType = Sequence::Effect::ICE_CONTRAIL;

			this->setDamageType( DamageType::ICE, true );
			this->m_Weakness.set( DamageType::FIRE, true );
			this->m_nHitToSolidType = SolidType::CRYO;

			if( Sequence::Shot::FALL_ICE == nType ) {
				bIsEnemyShot = true;
				bIsBlockThrowShot = true;
				//this->setObjectAttribute( ObjectAttribute::GRAVITY, true );
				//this->setObjectAttribute( ObjectAttribute::HALF_GRAVITY, true );
			}
		}
		break;
	case Sequence::Shot::SKYTHE:
		{
			this->setHitRect(5,3,39,38 );
			bIsEnemyShot = true;
			bIsBlockThrowShot = true;
			this->m_bIsDrawRotateByVelocityAngle = false;
			this->setObjectAttribute( ObjectAttribute::HIT_TO_NEUTRAL , false );

			this->setRotateDegree( HPLMath::nextFloat(0,HPLMathConsts::DEGREE_FULL_ROUND ) );
		}
		break;
	case Sequence::Shot::ROBOT_BEAM:
		{
			this->setHitRect(0,1,35,18);
			bIsEnemyShot = true;
			bIsBlockThrowShot = true;
		}
		break;
	case Sequence::Shot::ROBOT_MISSILE:
		{
			this->setHitRect(0,0,this->getWidth(), this->getHeight());
			bIsEnemyShot = true;
			bIsBlockThrowShot = true;
			this->m_bIsDrawRotateByVelocityAngle = true;
			this->m_nHomingType = HomingType::MISSILE;
		}
		break;
	case Sequence::Shot::LAVA_BALL:
		{
			this->setObjectAttribute( ObjectAttribute::HALF_GRAVITY, true );
			this->m_Weakness.set( DamageType::WATER, true );
			this->m_Weakness.set( DamageType::ICE, true );
			// 
			this->m_Immunities.set( DamageType::FIRE, true );	
			fHPRate = 15;
			this->m_bIsDrawRotateByVelocityAngle= true;

			this->m_nContrailObjectType = ObjectType::SHOT;
			this->m_nContrailType = Sequence::Shot::FIRE_PARTICLE;

			const int LAVA_PARTICLE_INTERVAL = 3;
			this->m_lpContrailCreateInterval = new HPLCounter( LAVA_PARTICLE_INTERVAL );
			this->m_lpContrailCreateInterval->setLoop( true );
		}
		break;
	case Sequence::Shot::FIRE_PARTICLE:
		{
			//this->setObjectAttribute( ObjectAttribute::HIT_TO_BLOCK, false );
			this->setObjectAttribute( ObjectAttribute::HALF_GRAVITY, true );
			this->m_Weakness.set( DamageType::WATER, true );
			this->m_Weakness.set( DamageType::ICE, true );
			this->m_Immunities.set( DamageType::FIRE, true );

			// őՓx̂Ƃ̂ݓ蔻L
			bIsHitToPlayerWhenEnemyShot = 
				getStaticData()->m_OptionData.getLevelIndex() == LevelType::MAX_NUM - 1 ;
		}
		break;
	case Sequence::Shot::SHABON:
		{
			this->m_bIsDrawRotateByVelocityAngle = false;
			this->setObjectAttribute( ObjectAttribute::HALF_GRAVITY, true );
			this->setObjectAttribute( ObjectAttribute::HIT_TO_SHOT, true );		// hit to other shot
			//this->m_Immunities.set( DamageType::WATER, true );
			this->m_Weakness.set( DamageType::ICE, true );
			this->m_Weakness.set( DamageType::FIRE, true );
			this->m_Weakness.set( DamageType::DRILL, true );
			this->m_Weakness.set( DamageType::CHAIN_SAW, true );
			this->setDamageType( DamageType::WATER, true );
			//this->setObjectAttribute( ObjectAttribute::HIT_TO_BLOCK, false );
			//bIsBlockThrowShot = true;
			const int DOUBLE_PUYO_INTERVAL = 10;
			this->m_lpPuyoPuyoCounter = new HPLCounter(DOUBLE_PUYO_INTERVAL);
			this->m_lpPuyoPuyoCounter->setLoop(true);
		}
		break;
	case Sequence::Shot::STONE_CURSE:
		this->m_bIsDrawRotateByVelocityAngle = false;
		// block trhow true
		// enemy shot
		break;
	case Sequence::Shot::EVIL_BEAM:
		// threw block,neutral,effect,shot
		// enemy shot
		this->m_bIsDrawRotateByVelocityAngle = false;
		this->m_nHomingType = HomingType::STOP_MISSILE;
		this->m_lpHomingData = new HomingData();

		break;
	case Sequence::Shot::THUNDER_BOLT:
		this->m_bIsDrawRotateByVelocityAngle = false;
		this->setDamageType( DamageType::ELEC, true );
		break;
	case Sequence::Shot::RUNNER_BOLT:
		this->setObjectAttribute(ObjectAttribute::NO_RESIST, true );
		this->setObjectAttribute( ObjectAttribute::HIT_BLOCK_TO_DIE, false );	//! n`ɂĂȂȂ
		this->m_bIsDrawRotateByVelocityAngle = false;
		this->setDamageType( DamageType::ELEC, true );
		break;
	case Sequence::Shot::CHARGE_BOLT:
		this->m_nCreateShotTypeWhenDead = Sequence::Shot::RUNNER_BOLT;
		this->m_bIsDrawRotateByVelocityAngle = false;
		this->setObjectAttribute( ObjectAttribute::NO_RESIST, true );
		this->setObjectAttribute( ObjectAttribute::HIT_BLOCK_TO_DIE, true );	
		this->setDamageType( DamageType::ELEC, true );
		break;
	case Sequence::Shot::CAR_BALL:
		this->m_bIsDrawRotateByVelocityAngle = false;
		this->m_nCreateShotTypeWhenDead = Sequence::Shot::EXPLOSION;
		break;
	case Sequence::Shot::DIRTY_DROP:
		this->m_bIsDrawRotateByVelocityAngle = false;
		// water
		this->setDamageType( DamageType::WATER, true );
		break;
	case Sequence::Shot::DIRTY_WIND:
		this->m_bIsDrawRotateByVelocityAngle = false;
		break;
	case Sequence::Shot::THUNDER_BALL:
		this->m_bIsDrawRotateByVelocityAngle = false;
		this->m_nCreateShotTypeWhenDead = Sequence::Shot::THUNDER_BOLT;
		this->setDamageType( DamageType::ELEC, true );
		break;
	case Sequence::Shot::SNOW_BALL:
		{
			this->m_bIsDrawRotateByVelocityAngle = false;

//			bool bIsHitToPlayer = false;
			bIsEnemyShot = false;
			if( getStaticData()->m_OptionData.getLevelIndex() == LevelType::MAX_NUM - 1 ) {
				// max level!
				bIsEnemyShot = true;
			}
			this->setObjectAttribute( ObjectAttribute::HIT_TO_PLAYER, bIsEnemyShot );
		}
		break;
	case Sequence::Shot::SOWRD:
		{
		}
		break;
	default:
		HPLError::assertIt(false,
			"%s:No initialize:type-%d", __FUNCTION__, nType);
		break;
	}

	// neutral Ƃ̂蔻
	// this->m_aryObjectAttribute[ ObjectAttribute::HIT_NEUTRAL_TO_DIE ] = this->m_aryObjectAttribute[ObjectAttribute::HIT_OBJECT_TO_DIE];

	// ꌂŎ
	this->setHPMax( 1 );
	this->setHP( this->getHPMax());

	this->setPoint( this->getHP() * fLevel * SHOT_POINT_DEFAULT );

	if( bIsBlockThrowShot ) {
		// ubN[
		this->setObjectAttribute( ObjectAttribute::HIT_TO_BLOCK, false );
		this->setObjectAttribute( ObjectAttribute::HIT_BLOCK_TO_DIE, false );
	}
	
#ifdef _DEBUG
	if( this->m_bIsClip ) {
		// Ђ^Cv
		// null check
		ASSERT_IT_NULL( this->m_lpClipCounter );
	}
	// null check
	if( this->m_nHomingType == HomingType::BEZIE ) {
		ASSERT_IT_NULL( this->m_lpHomingData );
	}
#endif

	if( bIsEnemyShot ) {
		// G˂e
		this->setObjectAttribute( ObjectAttribute::HIT_TO_ENEMY, false );
		this->setObjectAttribute( ObjectAttribute::HIT_TO_PLAYER, bIsHitToPlayerWhenEnemyShot );

		// ^[QbgvC[ɂ
		this->setTarget( &getStaticData()->m_ObjectManager.m_Player );
		this->setForceType( ForceType::ENEMY );
	} else {
		this->setForceType( ForceType::ALLY );
	}
//	if( nType == ShotType::BULLET ) {
		// do nothing
//	}

}

void GShot::setDamageType( int nType, bool bIsWeak ) {
//	this->m_nDamageType = HPLMath::setFlagByIndexInt(this->m_nDamageType, nType,
//		bIsWeak );
	this->m_DamageType.set( nType, bIsWeak );
}
bool GShot::getDamageType( int nType ) {
//	return HPLMath::getFlagByIndexInt( this->m_nDamageType, nType );
	return this->m_DamageType.get( nType );

}

GShot::~GShot() {
	//SAFE_DELETE(this->m_lpClipCounter );

	HPLPointerTool::HPLFree( this->m_lpPuyoPuyoCounter );
	HPLPointerTool::HPLFree( this->m_lpClipCounter );
	HPLPointerTool::HPLFree( this->m_lpContrailCreateInterval );
	HPLPointerTool::HPLFree( this->m_lpHomingData );

	for( vector<HPLCounter*>::iterator it = this->m_vecCounters.begin();
		it != this->m_vecCounters.end();
		it ++ )
	{
		delete *it;
	}
	this->m_vecCounters.clear();
}
void GShot::setDirection( int nDirection ) {
	this->m_nDirection = nDirection;

	if( this->getType() == Sequence::Shot::FIRE_TRAP ) {
		// 蔻̏C
		if( nDirection == FanDirectionType::LEFT ||
			nDirection == FanDirectionType::RIGHT)
		{
			HPLVector vCenter = this->getCenter();

			HPLRect rArea = this->getArea();
			float dx = vCenter.getX() - rArea.getX();
			float dy = vCenter.getY() - rArea.getY();
			rArea.setX( vCenter.getX() - dy );
			rArea.setY( vCenter.getY() - dx );
			this->setArea( rArea );
			this->setSize( getHeight(),getWidth());
			HPLRect rHitRect = this->getHitRect();
			this->setHitRect( rHitRect.getY(), rHitRect.getX(), rHitRect.getHeight(), rHitRect.getWidth() );
		}
	}
}

HPLVector getInterPoint( float fU, HPLVector vFrom, HPLVector vTo ) {
	HPLVector vInterPoint(
		( 1.f - fU ) * vFrom.getX() + 
		fU * vTo.getX(),
		( 1.f - fU ) * vFrom.getY() + 
		fU * vTo.getY() );
	return vInterPoint;
}

void GShot::draw(){
	// scroller
	HPLScroller* scroller = getStaticData()->getScrollManager()->getScroller(0);
	// virtual rect
	HPLRect rVir = scroller->getVirtualRect();
	// left/top
	float left = this->getArea().getX() + rVir.getX();
	float top = this->getArea().getY() + rVir.getY();
	// center
	float fCenterX = this->getCenter().getX() + scroller->getVirtualRect().getX();
	float fCenterY = this->getCenter().getY() + scroller->getVirtualRect().getY();
	float fZoom = scroller->m_fZoomRate;
	fCenterX *= fZoom;
	fCenterY *= fZoom;

	if( this->getSprite() == NULL ) {
		// S
		this->setAlive( false );
	} else {
		 
		HPLSequence* seq = this->getSequence();

		if( this->m_nHomingType == HomingType::BEZIE ) {//this->getType() == Sequence::Shot::HOMING_LASER ||
			//this->getType() == Sequence::Shot::ELECTRIC_HOMING_LASER
			//)
		//{

			bool bIsMove = getStaticData()->m_Game.isTheWorld() == false;
			SetDrawBlendMode( DX_BLENDMODE_ADD, 255 );
			if( this->m_lpHomingData->m_bIsStartExists ) {
				// 擪݂
				// 
				float fU = ( 1.f / this->m_lpHomingData->m_nDivNum ) * this->m_lpHomingData->m_nCounter;
				// `p_
				HPLVector vP01 = getInterPoint( fU,
					this->m_lpHomingData->m_aryControl[0],
					this->m_lpHomingData->m_aryControl[1] );
				HPLVector vP12 = getInterPoint( fU,
					this->m_lpHomingData->m_aryControl[1],
					this->m_lpHomingData->m_aryControl[2] );
				HPLVector vP23 = getInterPoint( fU,
					this->m_lpHomingData->m_aryControl[2],
					this->m_lpHomingData->m_aryControl[3] );

				HPLVector vP02 = getInterPoint( fU, vP01, vP12 );
				HPLVector vP13 = getInterPoint( fU, vP12, vP23 );

				HPLVector vP03 = getInterPoint( fU, vP02, vP13 );

				this->m_lpHomingData->m_aryTailRs[0] = atan2(
					vP03.getY() - this->m_lpHomingData->m_aryTailCenters[0].getY(),
					vP03.getX() - this->m_lpHomingData->m_aryTailCenters[0].getX() );

				HPLVector vLU = this->m_lpHomingData->m_vLD;	// left up
				HPLVector vRU = this->m_lpHomingData->m_vRD;	// right up

				float fDelta = 0;
				//HPLMath::getRadianFromDegree( 
				//	HPLMath::nextDouble( - 60, 60 ) );
				this->m_lpHomingData->m_vRD.set(
					this->m_lpHomingData->m_aryTailCenters[0].getX() + 6 * cos( fDelta + this->m_lpHomingData->m_aryTailRs[0] + PI / 2 ),
					this->m_lpHomingData->m_aryTailCenters[0].getY() + 6 * sin( fDelta + this->m_lpHomingData->m_aryTailRs[0] + PI / 2 )
					);
				this->m_lpHomingData->m_vLD.set(
					this->m_lpHomingData->m_aryTailCenters[0].getX() + 6 * cos( fDelta + this->m_lpHomingData->m_aryTailRs[0] - PI / 2 ),
					this->m_lpHomingData->m_aryTailCenters[0].getY() + 6 * sin( fDelta + this->m_lpHomingData->m_aryTailRs[0] - PI / 2 )
					);


				int nFirstHandle = getStaticData()->m_Basic.m_Graphics.m_ImageManager.get(
					seq->getImageInformation(0)->imageTag );
				// 擪 draw
				//
				/*DrawGraph( 
					vLU.getX() + scroller->getVirtualRect().getX(),
					vLU.getY() + scroller->getVirtualRect().getY(),
					nFirstHandle, TRUE );*/
				DrawModiGraphF( 
					vLU.getX() + scroller->getVirtualRect().getX(),
					vLU.getY() + scroller->getVirtualRect().getY(),
					vRU.getX() + scroller->getVirtualRect().getX(),
					vRU.getY() + scroller->getVirtualRect().getY(),
					this->m_lpHomingData->m_vRD.getX() + scroller->getVirtualRect().getX(),
					this->m_lpHomingData->m_vRD.getY() + scroller->getVirtualRect().getY(),
					this->m_lpHomingData->m_vLD.getX() + scroller->getVirtualRect().getX(),
					this->m_lpHomingData->m_vLD.getY() + scroller->getVirtualRect().getY(),
					nFirstHandle, TRUE );

				this->m_lpHomingData->m_nTailCount = 40;

				this->m_lpHomingData->m_aryTailCenters[0] = vP03;

				//Center擪Ɉv
				this->setCenter( vP03 );

				int DISTANCE_THRESHOLD = 16;
				if( this->getType() == Sequence::Shot::ELECTRIC_HOMING_LASER ) {
					DISTANCE_THRESHOLD = 100;
				}
				if( HPLVector::subtract( vP03, this->m_lpHomingData->m_aryControl[3]).getLength() <= DISTANCE_THRESHOLD ) {
					// ڕWn_ɓB
					// 
					this->m_lpHomingData->m_bIsStartExists = false;
					//this->setAlive( false );
				}

				if( bIsMove ) {
					this->m_lpHomingData->m_nCounter ++;
				}
			}
			
			
			if( this->m_lpHomingData->m_nTailCount >= 0 ) {
				//this->m_lpHomingData->m_nTail = 
				int nTail = this->m_lpHomingData->m_nTailCount;
				for( int i = 0; i < this->m_lpHomingData->m_nTailCount; i ++ ) {
					if( bIsMove ) {
						// ŴۂɃRs[
						this->m_lpHomingData->m_aryTailCenters[ nTail ] =
							this->m_lpHomingData->m_aryTailCenters[ nTail - 1 ];
					}

					HPLVector vRU = this->m_lpHomingData->m_vRD;
					HPLVector vLU = this->m_lpHomingData->m_vLD;
					//if( bIsMove ) {

						this->m_lpHomingData->m_vRD.set(
							this->m_lpHomingData->m_aryTailCenters[ nTail + 1 ].getX() + 6 * cos( this->m_lpHomingData->m_aryTailRs[ nTail ] + PI / 2 ),
							this->m_lpHomingData->m_aryTailCenters[ nTail + 1 ].getY() + 6 * sin( this->m_lpHomingData->m_aryTailRs[ nTail ] + PI / 2 )
							);

						this->m_lpHomingData->m_vLD.set(
							this->m_lpHomingData->m_aryTailCenters[ nTail + 1 ].getX() + 6 * cos( this->m_lpHomingData->m_aryTailRs[ nTail ] - PI / 2 ),
							this->m_lpHomingData->m_aryTailCenters[ nTail + 1 ].getY() + 6 * sin( this->m_lpHomingData->m_aryTailRs[ nTail ] - PI / 2 )
							);
					if( this->getType() == Sequence::Shot::ELECTRIC_HOMING_LASER ) {
						float D = 20;
						this->m_lpHomingData->m_vRD.add(
							static_cast<float>( HPLMath::nextDouble(-D,D) ),
							static_cast<float>( HPLMath::nextDouble(-D,D ) ) );
						this->m_lpHomingData->m_vLD.add(
							static_cast<float>( HPLMath::nextDouble(-D,D) ),
							static_cast<float>( HPLMath::nextDouble(-D,D ) ) );
					}
					//}

					if( nTail > 1 ) {
						int nHandle = getStaticData()->m_Basic.m_Graphics.m_ImageManager.get(
							seq->getImageInformation(nTail - 1)->imageTag );

						/*DrawGraph( 
							vLU.getX() + scroller->getVirtualRect().getX(),
							vLU.getY() + scroller->getVirtualRect().getY(),
							nHandle, TRUE );*/
						
						/*
						DrawModiGraphF( 
							vLU.getX() + scroller->getVirtualRect().getX(),
							vLU.getY() + scroller->getVirtualRect().getY(),
							vRU.getX() + scroller->getVirtualRect().getX(),
							vRU.getY() + scroller->getVirtualRect().getY(),
							this->m_lpHomingData->m_vRD.getX() + scroller->getVirtualRect().getX() + HPLMath::nextDouble(-D,D),
							this->m_lpHomingData->m_vRD.getY() + scroller->getVirtualRect().getY() + HPLMath::nextDouble(-D,D),
							this->m_lpHomingData->m_vLD.getX() + scroller->getVirtualRect().getX() + HPLMath::nextDouble(-D,D),
							this->m_lpHomingData->m_vLD.getY() + scroller->getVirtualRect().getY() + HPLMath::nextDouble(-D,D),
							nHandle, TRUE );*/
						DrawModiGraphF( 
							vLU.getX() + scroller->getVirtualRect().getX(),
							vLU.getY() + scroller->getVirtualRect().getY(),
							vRU.getX() + scroller->getVirtualRect().getX(),
							vRU.getY() + scroller->getVirtualRect().getY(),
							this->m_lpHomingData->m_vRD.getX() + scroller->getVirtualRect().getX(),
							this->m_lpHomingData->m_vRD.getY() + scroller->getVirtualRect().getY(),
							this->m_lpHomingData->m_vLD.getX() + scroller->getVirtualRect().getX(),
							this->m_lpHomingData->m_vLD.getY() + scroller->getVirtualRect().getY(),
							nHandle, TRUE );
					}

					if( sqrt(
						pow( this->m_lpHomingData->m_aryTailCenters[40].getX() - this->m_lpHomingData->m_vErase.getX(), 2 ) +
						pow( this->m_lpHomingData->m_aryTailCenters[40].getY() - this->m_lpHomingData->m_vErase.getY(), 2 ) ) <=
						50 )
					{
						this->m_lpHomingData->m_nTailCount = 0;
						//this->setAlive( false );
						break;
					}

					nTail --;


				}
			} else {

				// S
				this->setAlive( false );
			}

			SetDrawBlendMode( DX_BLENDMODE_NOBLEND, 255 );
			
			if( seq->step().isFinishAnimation ) {
				// S
				this->die( NULL );
			}
			///////////////////////////////////////////////
		} else {	// NOT bezier HOMING TYPE SHOTS

			if( seq->getState().isFinishAnimation ) {	// finish animation
				if( this->getType() == Sequence::Shot::PINE_APPLE_0 &&	// pineapple
					this->getState()->getState() == StateType::STAND )	// first state
				{
					// ύX to next state
					this->changeState( StateType::ATTACK );
				} else {	// other shots
					// 蔻͍sĂ̂ŁA\Ă

					// ȃVbg
					switch( this->getType() ) {
					case Sequence::Shot::SPREAD_MISSILE:
						{
							// SPREADED~TC
							int NUM = 5;
							const float SPREADED_MISSILE_SPEED = 10;
							const float MAX_DEG = 120;
							const float INTERVAL_DEG = MAX_DEG / NUM;
							for( int i = 0; i < NUM; i ++ ) {
								GShot* lpSpread = getStaticData()->m_ObjectManager.m_ShotManager.create(
									Sequence::Shot::SPREADED_MISSILE, this->getCenter(),
									this->getParentID(), this->getFaceDirection(), false, false );
								getStaticData()->m_ObjectManager.m_ShotManager.addToAdd( lpSpread );
								// same parent weapon type
								lpSpread->setParentWeaponType( this->getParentWeaponType() );
								float fDeg = this->getFaceDirection() < 0 ? 180 : 0;
								fDeg += (i - (NUM / 2)) * INTERVAL_DEG;
								HPLVector vVel;
								vVel.setByDegree( fDeg );
								vVel.multiple( SPREADED_MISSILE_SPEED );
								lpSpread->setVelocityAngle( vVel );
							}
						}
						break;
					}
					// S
					this->die( NULL );
				}
			}

			// rotate deg
			float rotateDegree = 0;
			if( this->m_bIsDrawRotateByVelocityAngle ) {	// rotate sprite 
				// L
				rotateDegree = this->getVelocityAngle().getDegree() + 180;
			} else {	// not rotate sprite
				switch( this->getType() ) {
				case Sequence::Shot::FIRE_TRAP:
					rotateDegree = static_cast<float>( (this->getDirection()) * 90 );
					break;
				case Sequence::Shot::TOMAHAWK:
				case Sequence::Shot::SKYTHE:
				case Sequence::Shot::PINE_APPLE_0:
				case Sequence::Shot::SEED:
					rotateDegree = this->getRotateDegree();
					break;
				default:
					if( this->getShotClassType() == ShotClassType::LASER &&
						this->getType() != Sequence::Shot::SATELITE_UNDER )
					{
						rotateDegree = this->getRotateDegree();
					}
				}

			}
			int alpha = 255;
			// 
			switch( this->getType() ) {
			case Sequence::Shot::WATER:
				alpha = 180;
				break;
			}
			// tF[hAEgH
			float zoom = 1;

			// 蔻\
			if( getStaticData()->getInputManager()->isPressed( KeyMap::F ) ){
				HPLRect rHitRect = this->getHitRect();
				DrawFillBox( 
					static_cast<int>( left + rHitRect.getX() ),
					static_cast<int>( top + rHitRect.getY() ),
					static_cast<int>( left + rHitRect.getX() + rHitRect.getWidth() ),
					static_cast<int>( top + rHitRect.getY() + rHitRect.getHeight() ),
					GetColor(255,0,0) );
			}
			if(getStaticData()->m_Game.isCheat()){
			}

			float rate = 1;
			float zoomx = rate;
			float zoomy = rate;

			int nBlendMode = alpha < 255 ? DX_BLENDMODE_ALPHA : DX_BLENDMODE_NOBLEND;

			// FωeŗL̏
			bool bIsChangedColor = false;

			int nR = 255;
			int nG = 255;
			int nB = 255;
			switch( this->getType() ) {
			case Sequence::Shot::MISSILE:
				{
					if( this->getObjectAttribute( ObjectAttribute::HIT_TO_PLAYER ) ) {	// hit to player
						// G̃~TC
						if( seq->getStepCounter() % 6 < 3 ) {	// Ԋu
							// Ԃ
							nG = 100;
							nB = 100;
						}
					}
				}
				break;
			case Sequence::Shot::SLIME_CLIP:
				{
					bIsChangedColor = true;
					nR = SlimeQueenColors::COLORS[this->getColor()].r;
					nG = SlimeQueenColors::COLORS[this->getColor()].g;
					nB = SlimeQueenColors::COLORS[this->getColor()].b;
				}
				break;
			case Sequence::Shot::SATELITE_UNDER:

				{
					alpha = 200;
					// add
					nBlendMode = DX_BLENDMODE_ADD;

					// r[`
					HPLSequence* lpBeam = this->getSprite()->getSequence( Sequence::Shot::SATELITE, FaceDirection::LEFT );
					HPLRect rSrc = lpBeam->getImageInformation(0)->srcRect;
					SetDrawBlendMode( nBlendMode, alpha );
					int nX = (int)( fCenterX - rSrc.getWidth() / 2 );
					DrawExtendGraph( 
						nX, 0,
						(int)(nX + rSrc.getWidth()), (int)fCenterY, 
						getStaticData()->getImageHandle(
						SpriteType::SHOT, Sequence::Shot::SATELITE, lpBeam->getFrameIndex() ), 
						TRUE);

				}
				break;
			case Sequence::Shot::LASER:
				{
					alpha = 100;
					nBlendMode = DX_BLENDMODE_ADD;
				}
				break;
			case Sequence::Shot::NIKITA_MISSILE:
				// draw guided
				{
					HPLSequence* lpGuidedSeq = getStaticData()->getSequence( SpriteType::COMMON,
					Sequence::Common::GUIDED );
					SetDrawBlendMode( DX_BLENDMODE_ALPHA, 200 );

					getStaticData()->m_Basic.m_Graphics.draw(
						(int)fCenterX,
						(int)fCenterY,
						lpGuidedSeq,
						FaceDirection::LEFT,0,1 );
					SetDrawBlendMode( DX_BLENDMODE_NOBLEND, 255);

					lpGuidedSeq->step();

				}

				break;
			case Sequence::Shot::FIRE_PARTICLE:
				{
					alpha = (int)(255.f * (1.f - seq->getLoopNum() /(float)seq->getLoopNumMax() ));
				}
				break;
			case Sequence::Shot::LEAF:
			case Sequence::Shot::KAHUN:
				{
					bIsChangedColor = true;
					nR = 255;
					nG = 200;
					nB = 200;
				}
				break;
			}

			if( bIsChangedColor  ) {
				SetDrawBright( nR, nG, nB );
			}
			if( this->m_lpPuyoPuyoCounter ) {
				const int MAX_DELTA_Y = 10;
				int nMax = this->m_lpPuyoPuyoCounter->getWaitFrameNum();
				int nStep =this->m_lpPuyoPuyoCounter->getStepCounter();

				int nCur = nStep >= nMax / 2 ? nMax - nStep : nStep;

				float fDelta = (float)MAX_DELTA_Y * nCur / (nMax / 2 ) * 2;

				int nNewHeight = (int)(seq->getImageInformation()->srcRect.getHeight() +
					fDelta );
				DrawExtendGraph( (int)( left ), (int)( fCenterY - nNewHeight / 2 ),
					(int)( left + seq->getImageInformation()->srcRect.getWidth() ),
					(int)( top + nNewHeight ),
					getStaticData()->getImageHandle(
					SpriteType::SHOT, this->getSequenceID() ), TRUE );
			} else {
				getStaticData()->m_Basic.m_Graphics.draw(
					static_cast<int>( fCenterX ),
					static_cast<int>( fCenterY ), seq,
					this->m_bIsDrawRotateByVelocityAngle ?
					FaceDirection::LEFT : this->getFaceDirection(),
					rotateDegree, fZoom,
					alpha, alpha != 255 ? nBlendMode : DX_BLENDMODE_NOBLEND);
			}


			if( bIsChangedColor ) {
				SetDrawBright( 255,255,255);
			}
			if(getStaticData()->m_Game.isTheWorld() == false){
				seq->step();
				switch( this->getType() ) {
				case Sequence::Shot::TOMAHAWK:
					this->setRotateDegree( this->getRotateDegree() + this->m_fRotateSpeed );
					break;
				}
				if( this->m_lpPuyoPuyoCounter ) {
					this->m_lpPuyoPuyoCounter->step();
				}
			}
		}
	}
}

int GShot::getSequenceID() {
	int seqID = getType();
	int st = this->getState()->getState();

	switch( getType() ) {
	case Sequence::Shot::PINE_APPLE_0:
		break;
	case Sequence::Shot::SPIDER_NET:
		switch( st ) {
		case StateType::ATTACK:
			seqID = Sequence::Shot::SPIDER_NET_BIG;
			break;
		}
		break;
	}
	return seqID;
}
HPLSequence* GShot::getSequence(){

	int nSeqID = this->getSequenceID();
	HPLSequence* seq = 
		this->getSprite()->getPair(nSeqID)->left;
	return seq;

}

// override
void GShot::changeState(int st){
	int oldSt = this->getState()->getState();

	if(oldSt != st){
		this->getSequence()->resetCounter();
	}

	this->getState()->setState(st);

}

/*
bool GShot::collision(HPLMatrix<int>* lpBlockMatrix,
			   std::map<int, HPLBlock*>* lpBlockMap )
{
	// TODO


	const float RESIST = getStaticData()->getSetting()->resist;
	const float GRAVITY = getStaticData()->getSetting()->gravity;
	// Rő呬xiCRƂ̂肠j
	const float FALL_MAX_SPEED = getStaticData()->getSetting()->fallMaxSpeed;

	
	int tileSize = getStaticData()->getSetting()->tileSize;
	
	HPLRect hitArea = getHitArea();
	HPLVector velocityAngle = getVelocityAngle();
	HPLVector center = getCenter();

	bool hitBlock = false;

	bool bIsNeedCheckCollision =// this->m_bIsHitToBlock ||
		this->m_aryObjectAttribute[ ObjectAttribute::HIT_BLOCK_TO_DIE ] ||
		this->m_aryObjectAttribute[ ObjectAttribute::HIT_TO_BLOCK ];

	float dx = center.getX() - hitArea.getX();
	float dy = center.getY() - hitArea.getY();

	if( bIsNeedCheckCollision ) {

		{
	//if( this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_BLOCK] ) {//isHitToBlock() ){
	//	{
			CollisionState stateX = HPLCollisionTool::adjustCollisionX(
				hitArea, velocityAngle,
				lpBlockMatrix, tileSize, *lpBlockMap);

			bool isTouch = lpBlockMap->find( stateX.m_nTouchBlockID )->second->isEmpty() == false;
			
			this->getTrigger()->setTrigger(TriggerType::HIT_TO_THE_LEFT,
				isTouch && stateX.isHitX == HitDirectionX::LEFT);
			this->getTrigger()->setTrigger(TriggerType::HIT_TO_THE_RIGHT,
				isTouch && stateX.isHitX == HitDirectionX::RIGHT);
			///////
			// ɂԂ
			if(isTouch){
				center.setX( stateX.m_nNewX + dx);
				hitArea.setX( stateX.m_nNewX );

				//if(this->isBoundWhenHitToBlock()){
				if( this->m_aryObjectAttribute[ObjectAttribute::BOUND] ) {	
					float fRate = 1;

					velocityAngle.setX(- velocityAngle.getX() * fRate);

				}

				//if(isHitToBlock() && isDieWhenHitToBlock()){
				if( this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_BLOCK ] && //isHitToBlock() &&
					this->m_aryObjectAttribute[ObjectAttribute::HIT_BLOCK_TO_DIE] )
				{
					setAlive(false);
				}
				hitBlock = true;
			}else{
			}
			//
			///////
		}
		{
			
			CollisionState stateY = HPLCollisionTool::adjustCollisionY(
				hitArea, velocityAngle,
				lpBlockMatrix, tileSize, 
				*lpBlockMap);
			// C
			bool isTouch = lpBlockMap->find( stateY.m_nTouchBlockID )->second->isEmpty() == false;
			this->getTrigger()->setTrigger(TriggerType::ON_THE_GROUND,
				isTouch && stateY.isHitY == HitDirectionY::BOTTOM);
			this->getTrigger()->setTrigger(TriggerType::HIT_TO_THE_CEILING,
				isTouch && stateY.isHitY == HitDirectionY::TOP);

			if(isTouch){
				// GĂ
				center.setY( stateY.m_nNewY + dy);
				hitArea.setY( stateY.m_nNewY);


				//if(this->isBoundWhenHitToBlock()){
				if( this->m_aryObjectAttribute[ObjectAttribute::BOUND] ) {
					const float EPS = 0.7f;
					velocityAngle.setY((float)(-fabs(velocityAngle.getY()) * EPS));
				}
				{
					if(stateY.isHitY == HitDirectionY::BOTTOM){
						//velocityAngle.setY(0);
						////
						// (X)

						float fRate = 1.f;

						velocityAngle.setX(HPLActionMoveTool::reduceSpeed(
							RESIST * fRate,
							velocityAngle.getX()));

					}else{
						velocityAngle.setY(0);
					}
				}

				if( this->m_aryObjectAttribute[ObjectAttribute::HIT_TO_BLOCK] &&//isHitToBlock() &&
					this->m_aryObjectAttribute[ObjectAttribute::HIT_BLOCK_TO_DIE] )
				{
					setAlive(false);
				}
				hitBlock = true;

			}else{

				/////
				// Ȓeďd͂̉e͎󂯂Ȃ
				if(this->isGravity() == true){
					velocityAngle.setY(velocityAngle.getY() + GRAVITY);
				}else{
				}
			}
		}
	}

	if( hitBlock ) {
	}
	center.add(velocityAngle);

	if( this->m_bIsNoMove == false ) {
		// ړL
		
		setCenter(center);
		setVelocityAngle(velocityAngle);
	}


	//HPLActionMoveTool::checkGroundAtTheFaceDirection((HPLShootingObject*)this,
	//	getStaticData()->getBlockManager(), tileSize);
	
	return hitBlock;
}
*/
void GShot::die(HPLShootingObject* damageSource){
	if( this->m_nHomingType == HomingType::BEZIE ) {//this->getType() == Sequence::Shot::HOMING_LASER ||
		//this->getType() == Sequence::Shot::ELECTRIC_HOMING_LASER
		//)
	//{
		this->m_lpHomingData->m_bIsStartExists = false;	// 擪

		// 擪W
		this->m_lpHomingData->m_vErase = this->getCenter();

		// Ȃ
		this->setObjectAttribute( ObjectAttribute::HIT_TO_ENEMY ,false );
//		this->m_aryObjectAttribute[ ObjectAttribute::HIT_TO_ENEMY ] = false;

	} else {
		this->setAlive( false );

		// 
		switch( this->getType() ) {
		case Sequence::Shot::SEED:	// seed makes flower
			{
				HPLVector vCenter( 0, - 100 );
				vCenter.add( this->getCenter() );
				// neutral ally flower zu
				GNeutral* lpNeutral = getStaticData()->m_ObjectManager.m_NeutralManager.create(
					Sequence::Neutral::ALLY_FLOWER, vCenter,-1,-1 );
			}
			break;
		case Sequence::Shot::WIND_EGG:	// wind egg makes fan
			{
//				HPLVector vCenter( 0, 50 );
//				vCenter.add( this->
				HPLVector vCenter;
				vCenter.add( this->getCenter());
				GNeutral* lpNeutral = getStaticData()->m_ObjectManager.m_NeutralManager.create(
					Sequence::Neutral::FAN, vCenter, -1, -1);
				lpNeutral->setColor(1);
			}
			break;
		}
	}

	// TODO 
	
	GPlayer* lpPlayer = &getStaticData()->m_ObjectManager.m_Player;
	if( damageSource != NULL &&		// _[W\[X
		damageSource->getParentID() == lpPlayer->getID())	// vC[̃Vbg
	{
		const float POWER_SCORE_RATE = 1.f;	// 100ƂxȂ̂ŁÂ܂܂ł悢
		long lScore = (long)( this->m_fPower * POWER_SCORE_RATE );
		if( lScore <= 0 ) {
			lScore = 1;
		}
		getStaticData()->m_Save.addScore( lScore );
	}
}

void GShot::damage(float dmg, HPLShootingObject* damageSource){
	int nOld = this->getHP();
	this->setHP( nOld - dmg );
	if( nOld > 0 && this->getHP() <= 0 ) {
		this->die( damageSource );
	}

}

bool GShot::isHomingLaser() {
	return this->getType() == Sequence::Shot::HOMING_LASER ||
			this->getType() == Sequence::Shot::ELECTRIC_HOMING_LASER;
}


/** NbvJE^X^[g
  */
void GShot::startClipCounter() {
#ifdef _DEBUG
	ASSERT_IT_NULL( this->m_lpClipCounter );
#endif
	this->m_lpClipCounter->reset();
}
/** Ԍo߁A܂͂ӂقǂɂJE^i
  */
HPLSequenceState& GShot::stepClipCounter() {
#ifdef _DEBUG
	ASSERT_IT_NULL( this->m_lpClipCounter );
#endif
	return this->m_lpClipCounter->step();
}

bool GShot::isUnique( int nType ) {
	switch( nType ) {
	case Sequence::Shot::NIKITA_MISSILE:
		return true;
	}
	return false;
}

