#include "GStaticData.h"
#include <hpllib/action/HPLActionMoveTool.h>
#include <hpllib/math/HPLMath.h>
#include <hpllib/system/HPLFile.h>
#include <hpllib/system/HPLLogger.h>
#include <hpllib/general/HPLString.h>

#include "GGameScene.h"
#include "GTitleScene.h"
#include "GSelectStageScene.h"
#include "GSelectPlaceScene.h"
#include "GStoryScene.h"
#include "GOptionScene.h"
#include "GAchievementScene.h"
#include "GContinueScene.h"
#include "GGameOverScene.h"
#include "GSpecialDeadScene.h"
#include "GLibraryScene.h"
#include "GEventCheckScene.h"

GStaticData staticData;
#include "GEnum.h"
using namespace std;
using namespace GEnum;

GStaticData::GStaticData() : AbstractDXStaticData () {
	this->m_ScreenFormat.width = this->getSetting()->windowWidth;
	this->m_ScreenFormat.height = this->getSetting()->windowHeight;
	this->m_ScreenFormat.colorBitPerPixel = this->getSetting()->bitPerPixel;
	this->m_ScreenFormat.videoFlags = 1;

	this->getBlockManager()->setBlockSize(this->getSetting()->tileSize);

	this->m_nSelectedStar = 0;
	this->m_nSelectedPlace = 0;
	//this->m_lpFadeOutSequence = NULL;
	this->m_nPlayerPlaceIndex = 0;

	// tF[hAEgJE^
	this->m_FadeOutCounter.setWaitFrameNum(40 );


	//this->m_ShakeManager.

	this->m_nSelectIndex = 0;	// Iʒu

	// setup key
/*	for( int i = 0; i < MAX_SHORTCUT_NUM; i ++ ) {
		this->m_aryPrevShortCutKeyStates[i] = 0;
	}*/

//	this->m_nShortCutIndex = 0;
	this->setCOMPlayer( false );

	this->m_EventFadeCounter.end();

	this->m_nAfterBossScenarioID = -1;
	this->m_AfterBossWaitCounter.setWaitFrameNum( 30 );

	this->m_nAfterFadeOutDXProcessType = AfterFadeOutDXProcessType::NONE;

	HPLScroller* lpBGScroller = new HPLScroller(
		HPLRect(0,0,this->m_Setting.windowWidth, this->m_Setting.windowHeight ),
		HPLRect(0,0,this->m_Setting.windowWidth, this->m_Setting.windowHeight ));
	this->m_ScrollManager.add( 1,  lpBGScroller );

	using namespace HPL::MapData::Item;

	// check Chest parameters
	map<int,HPLMapDataItem*>::iterator it = this->m_MapData.getObjectMap()->begin();
	for( ; it != this->m_MapData.getObjectMap()->end(); it ++ ) {
		HPLMapDataItem* item = it->second;
		if( item->getType() == MapDataItemType::NEUTRAL ) {
			switch( item->getObjectData()->getType() ) {
			case Sequence::Neutral::TAKARA_BAKO:
			case Sequence::Neutral::TANSU:
				{
					if( item->getParameterSize() < ParameterType::Neutral::FLAG_INDEX ) {
						// less than need
						HPLError::msg("",MB_OK,"[ID:%d][Type:%d]Too few arguments[%d<%d]",
							it->first, item->getObjectData()->getType(), item->getParameterSize(),
							ParameterType::Neutral::FLAG_INDEX );
#ifdef _DEBUG
						HPLError::abort();
#endif
					}
				}
				break;
			}
		}
	}
}

GStaticData::~GStaticData() {
	/*if( this->m_lpFadeOutSequence != NULL ) {
		delete this->m_lpFadeOutSequence;
	}*/
//	delete this->m_lpReadyGoSequence;
//	delete this->m_lpGameSetSequence;
	if( this->m_lpReadyGoSequence ) {
		delete this->m_lpReadyGoSequence;
		this->m_lpReadyGoSequence = NULL;
	}
	if( this->m_lpGameSetSequence ) {
		delete this->m_lpGameSetSequence;
		this->m_lpGameSetSequence = NULL;
	}
}

void GStaticData::startAfterBossWait( int nScenarioID, int nWaitFrameNum  ) {
	this->m_nAfterBossScenarioID = nScenarioID;
	this->m_AfterBossWaitCounter.setWaitFrameNum(
		nWaitFrameNum >= 0 ? nWaitFrameNum : AFTER_BOSS_DEFAULT_WAIT_FRAME_NUM );

	this->m_AfterBossWaitCounter.reset();
}
bool GStaticData::processAfterBossWait() {
	if( this->m_nAfterBossScenarioID < 0 ) {
		return true;
	}
	if( this->m_AfterBossWaitCounter.step().isFinishAnimation ) {
		// ViI
		this->startEvent( this->m_nAfterBossScenarioID, true );
		this->m_nAfterBossScenarioID = -1;
		return true;
	}
	return false;
}
void GStaticData::stopAfterBossWait() {
	this->m_AfterBossWaitCounter.end();
	this->m_nAfterBossScenarioID = -1;
}

void GStaticData::loadSave( int nIndex ) {
	// load save
	{
		//
		string str = HPLString::chrToStr(
			SAVE_PATH_FORMAT, nIndex+1 );
		// [h
		if( this->m_Save.load( str.c_str() ) != 0 ) {
			// s

			// Z[u
			if( this->m_Save.save() != 0 ) {
				// s
				HPLError::msg("WARNING", MB_OK, "Couldn't save datas[%s]", str.c_str());
			}
		}

		// ŏ̐
		this->m_nSelectedStar = this->m_Save.getFlag( FlagType::STAR );
		this->m_nSelectedPlace = this->m_Save.getFlag( FlagType::PLACE );
		
		// ŏ̕I
//		getStaticData()->setFirstShortCutIndex( &getStaticData()->m_ObjectManager.m_Player);
		this->selectFirstWeapon( &getStaticData()->m_ObjectManager.m_Player );
	}
}
/** Eŏ̕I
  * EȂꍇ̓p`o^
  * EȊOIiV[gJbgɕςȂ̂
  */
void GStaticData::selectFirstWeapon(GPlayer* lpPlayer ) {
	// ŏ̃V[gJbgCfbNXT
	int nFirstIndex = getStaticData()->m_ShortCutPaletteManager.getFirstRegisteredIndex();
	// num of palettes
	int nPaletteNum = getStaticData()->m_ShortCutPaletteManager.getPaletteNum();
	if( nFirstIndex < 0 ) {	// Ȃ
		// p`ŏɓo^
		getStaticData()->m_ShortCutPaletteManager.setValue(0,
			Sequence::Item::PUNCH );
		// ŏ̃V[gJbgip`jI
		getStaticData()->m_ShortCutPaletteManager.setSelectedIndex( 0 );
	} else {	// 
		// I
		getStaticData()->m_ShortCutPaletteManager.setSelectedIndex(
			nFirstIndex );
	}
	// 킩ǂ`FbNiłɕ̎ގ擾j
	int nWeaponType = this->checkItemIsWeapon( getStaticData()->m_ShortCutPaletteManager.getSelectedValue() );
	if( nWeaponType >= 0 ) {	// ł
		// I
		lpPlayer->setWeaponType( nWeaponType );
	}
	// save
	getStaticData()->m_Save.save();
}

void GStaticData::loadTexture() {

	this->m_nCurrentCount = GetNowCount();
	{
		this->m_AchievementSave.load();
	}

	// load game options
	{
		if( this->m_OptionData.load() != 0 ) {
			// error to load!

			// try to save default
			if( this->m_OptionData.save() != 0) {
				HPLError::msg( "Error", MB_OK, "Q[IvṼZ[uɎs܂");
			}
		}
	}
	// setup shortcut
	this->m_ShortCutPaletteManager.setup( this->m_OptionData.getOptionFlag( OptionFlagType::MAX_SHORTCUT_NUM ),
		-1, 0 );
	// load images
	this->m_Basic.m_Graphics.m_SpriteContainer.createImageSetContainer( "data/image/ImageSetFileList.dat" );


	this->m_SceneManager.putScene( SceneType::GAME,
		new GGameScene() );
	this->m_SceneManager.putScene( SceneType::TITLE,
		new GTitleScene() );
	this->m_SceneManager.putScene( SceneType::SELECT_STAGE,
		new GSelectStageScene() );
	this->m_SceneManager.putScene( SceneType::SELECT_PLACE,
		new GSelectPlaceScene() );
	this->m_SceneManager.putScene( SceneType::STORY,
		new GStoryScene() );
	this->m_SceneManager.putScene( SceneType::OPTION,
		new GOptionScene() );
	this->m_SceneManager.putScene( SceneType::ACHIEVEMENT,
		new GAchievementScene() );
	this->m_SceneManager.putScene( SceneType::CONTINUE,
		new GContinueScene() );
	this->m_SceneManager.putScene( SceneType::GAME_OVER,
		new GGameOverScene() );
	this->m_SceneManager.putScene( SceneType::SPECIAL_DEAD,
		new GSpecialDeadScene() );
	this->m_SceneManager.putScene( SceneType::LIBRARY,
		new GLibraryScene() );
	this->m_SceneManager.putScene( SceneType::EVENT_CHECK,
		new GEventCheckScene() );

	// tiles
	HPLSprite* lpSprite = this->getSprite( SpriteType::COMMON );
	int nIndex = lpSprite->getSequence( Sequence::Common::TILE, FaceDirection::LEFT )->getImageInformation(0)->imageTag;
	int nHandle = this->m_Basic.m_Graphics.m_ImageManager.get( nIndex );
	this->getBlockManager()->setupBlock("data/maps/Block000.txt",
		lpSprite->clone(), nHandle );

	// objects setup
	this->m_ObjectManager.setup();

	// scroller
	this->m_ScrollManager.getScroller(0)->screenRect.set(
		0,0,
		static_cast<float>( this->m_Setting.windowWidth), 
		static_cast<float>( this->m_Setting.windowHeight ) );

	// fade out
	//this->m_lpFadeOutSequence = this->getSequence( SpriteType::COMMON, Sequence::Common::FADE_OUT )->clone();
	//this->m_lpFadeOutSequence->end();
	this->m_FadeOutCounter.end();

	// mixer
	vector<string> vecSEFiles = HPLFile::loadStringLinesFromFile( SE_LIST_FILE_PATH,
		true);
	
	// sound
	for( int i = 0; i < (int)vecSEFiles.size(); i ++ ) {
		string strPath = string( "data/se/" ) + vecSEFiles[i];
		int nHandle = this->m_MixerManager.add( strPath.c_str() );
		HPLError::assertIt( nHandle >= 0,
			"%s:invalid handle:%d", __FUNCTION__, nHandle);

		// ʒ
		if( i >= SE::VOICE_START && i <= SE::VOICE_END ) {
			ChangeVolumeSoundMem( this->m_OptionData.getOptionFlag( OptionFlagType::VOICE_VOLUME ), nHandle );
		} else {
			ChangeVolumeSoundMem( this->m_OptionData.getOptionFlag( OptionFlagType::SE_VOLUME ), nHandle );
		}
	}
	int nMusicNum = this->m_MixerManager.getSoundHandleNum();
	int nMusicMax = SE::MAX_NUM;
	HPLError::assertIt( nMusicNum == nMusicMax,
		"%s:musics loading failed:loaded=%d,max=%d", __FUNCTION__,
		nMusicNum, nMusicMax);

	// musics
	vector<string> vecMusicFiles = HPLFile::loadStringLinesFromFile( MUSIC_LIST_FILE_PATH,
		true );

	this->m_MixerManager.setMusicVolume( this->m_OptionData.getOptionFlag( OptionFlagType::MUSIC_VOLUME ) );
	//SetVolumeMusic(this->m_OptionData.getOptionFlag( OptionFlagType::MUSIC_VOLUME ) );
	// bgm
	for( vector<string>::iterator it = vecMusicFiles.begin(); it != vecMusicFiles.end(); it ++ ) {
		string strPath = string( "data/music/" ) + *it;
		int nHandle = this->m_MixerManager.addMusic( strPath.c_str() );

		// ʒ
	//	ChangeVolumeSoundMem( this->m_OptionData.getOptionFlag( OptionFlagType::MUSIC_VOLUME ), nHandle );
	}

	// ready go
	this->m_lpReadyGoSequence = this->getSequence( SpriteType::COMMON,
		Sequence::Common::START );
	this->m_lpReadyGoSequence->end();

	// game set
	this->m_lpGameSetSequence = this->getSequence( SpriteType::COMMON,
		Sequence::Common::STAGE_CLEAR );
	this->m_lpGameSetSequence->end();

	// show enemy name setup
	this->m_ShowEnemyName.setup( ShowType::ENEMY_NAME, 
		getStaticData()->getSequence( SpriteType::COMMON, Sequence::Common::ENEMY_NAME )->clone() );

	// show item name setup
	this->m_ShowItemName.setup( ShowType::ITEM_NAME,
		getStaticData()->getSequence( SpriteType::COMMON, Sequence::Common::ITEM_NAME )->clone());

	// setup valuables
	{
		HPLRect rSrcRect = this->getSequence( SpriteType::EFFECT, Sequence::Effect::NUMBER )->getImageInformation(0)->srcRect;
		const int NUM_MAX_NUM = 10;	// ̉摜ɕ`Ă鐔̐
		this->m_ObjectManager.m_EffectManager.NUM_WIDTH = rSrcRect.getWidth();
		this->m_ObjectManager.m_EffectManager.NUM_HEIGHT = rSrcRect.getHeight();
	}
}

HPLSetting* GStaticData::getSetting() {
	return &this->m_Setting;
}

void GStaticData::setupEvent( const char* lpszEventFilePath) {
	// ViI
	// EventPackǂݍ
	m_EventPackManager.load( lpszEventFilePath == NULL ? COMMON_EVENET_PACK_FILE_PATH : lpszEventFilePath);

	// C[W
	this->m_EventImage.deactivate();

	// CxgpbN
	m_lpEventPack = NULL;
}

void GStaticData::setupMap(int mapIndex, bool bIsLoadEvent ){
	///////////

	// }bvǂݍ
	HPLMapManager* mapManager = this->getMapManager();

	// vtBbNX擾(*.emp"*")
	std::string mapFilePathBase = mapManager->getMapFilePath(mapIndex);
	
	// Cxg{bNX񂪖ꍇ̃ftHgf[^
	std::string eventBoxFilePath("data/maps/NULL_EventBox.txt");

	// Cxg{bNX()
	std::string checkEventBoxFileExists = mapFilePathBase + std::string(".txt");
	if(HPLFile::isExists(checkEventBoxFileExists.c_str()) == true){
		eventBoxFilePath = checkEventBoxFileExists;

	}

	// empt@C
	string strEMP = mapFilePathBase + string( ".emp");
	
	// { csv
	std::string mapCSV = mapFilePathBase + std::string(".csv");
	// IuWFNgzu
	// zupf[^t@CpX
	std::string objectPlaceDataFilePath = mapFilePathBase + std::string("-object.csv");

	
	if( HPLFile::isExists( strEMP.c_str()) ) {
		vector<string> vecLines = HPLFile::loadStringFromFile(
			strEMP.c_str(), true );
		for( int i = 0; i < (int)vecLines.size(); i ++ ) {
			string strLine = vecLines[i];
			vector<string> vecEquals = HPLString::split( strLine, "=");

			if( vecEquals.size() < 2 ) {
				LOG_WARN( HPLString::chrToStr("Invalid emp file[%s]:%s",
					strEMP.c_str(), strLine.c_str()).c_str());
				continue;
			}
			if( HPLString::equals( vecEquals[0], "PlayerStartIndex")) {
				if( this->m_nPlayerPlaceIndex >= 0 ) {	// ɐݒ肳Ă
					// OŎw肳Ă΁AX̏͐؂̂Ă
					// do nothing
				} else {
					int nIndex = -1;
					try {
						nIndex = atoi( vecEquals[1].c_str());
					} catch( ... ) {
						LOG_WARN( HPLString::chrToStr( "Failed to convert or invalid player place index[%s]:%s:%d"),
							strEMP.c_str(), strLine.c_str(), nIndex );
						continue;
					}
					if( nIndex >= 0 ) {
						this->m_nPlayerPlaceIndex = nIndex;
					}
				}

			} else if( HPLString::equals( vecEquals[0], "Size")){
				// TODO use
			} else if( HPLString::equals( vecEquals[0], "Terrain")) {
				string strPath = vecEquals[1];
				if( HPLFile::isExists( strPath.c_str())) {
					mapCSV = strPath;
				}
			} else if( HPLString::equals( vecEquals[0], "Objects")) {
				string strPath = vecEquals[1];
				if( HPLFile::isExists( strPath.c_str()) ) {
					objectPlaceDataFilePath = strPath;
				}
			} else{
				LOG_WARN( HPLString::chrToStr( "Unknown setting found[%s]:%s",
					strEMP.c_str(), strLine.c_str()).c_str());
				continue;
			}
		}
	} else {
		LOG_POINT( HPLString::chrToStr( "No emp file found. Loading by default name pattern.[%s]",
			strEMP.c_str()).c_str());
	}
	if( this->m_nPlayerPlaceIndex < 0 ) {
		this->m_nPlayerPlaceIndex = 0;
	}
	// player index

	// obejct file
	// map file

	// load emp file:
	//  -default start place index

	// load event( and initialize)
	if( bIsLoadEvent ) {
		this->setupEvent( );

		// load event boxes
		m_EventBoxManager.load(eventBoxFilePath.c_str() );

	}
	// IuWFNgNA
	this->m_ObjectManager.clear();

	// ubNZbg
	AbstractBlockManager* blockManager = this->getBlockManager();

	/////////
	// }bv}gNX[h
	blockManager->loadMatrix(mapCSV.c_str());

	// ^CTCY
	int tileSize = this->getSetting()->tileSize;

	////////////////////
	// set scroller
	{
		HPLRect virRect(0,0,
			(float)(blockManager->getBlockMatrix()->getColumnSize() * tileSize),
			(float)(blockManager->getBlockMatrix()->getRowSize() * tileSize));

		HPLScroller* scroller = this->getScrollManager()->getScroller(0);
		scroller->setVirtualRect(virRect);
	}

	// reset game param
	this->m_Game.setGameOver(false);
	this->m_Game.setMenu(false);
	this->m_Game.setScenario(false);
	this->m_Game.setTheWorld(false);

	// IuWFNgzupf[^
	AbstractBlockManager placerBlockManager;

	// IuWFNgzuǂݍ
	placerBlockManager.loadMatrix(objectPlaceDataFilePath.c_str());

	// IuWFNgzu
	this->m_ObjectPlacer.loadAndPlace(placerBlockManager.getBlockMatrix(),
		this->m_Setting.tileSize,
		this->getMapData(),
		mapIndex);

	// zu-1ɂ
	this->m_nPlayerPlaceIndex = -1;
	// j
	int nStar = 0;
	int nPlace = 0;
	if( this->m_Setting.maps.getStarAndPlaceFromMapIndex( mapIndex, nStar, nPlace ) == 0 ) {	// 擾
		// ->SelectPlaceSceneɕ\}bvł

		// ꏊ̏Ԃ擾
		int nPlaceState = this->m_Save.getPlaceState( nStar, nPlace );

		// Starł̓{X|ĂH(true=yes)
		bool bIsDestroyBoss = 
			HPLMath::getFlagByIndexInt( this->m_Save.getFlag( FlagType::DESTROY_BOSS ), nStar );

		if( nPlaceState == PlaceType::SEALED ||	// sealed
			nPlaceState == PlaceType::REVEALED	// revealed
			//||
			//this->m_Save.getPlaceState( nStar, nPlace ) == PlaceType::REVEALED
			)
		{
			// ->not cleared or in a vail

			// next state of place
			int nNextState = PlaceType::REVEALED;

			if( bIsDestroyBoss ) {	// {Xj
				// set cleared
				nNextState = PlaceType::CLEARED;
			}

			// 
			this->m_Save.setPlaceState( nStar, nPlace, nNextState );
		}

		// IԂƂĐݒ肷
		this->setStarAndPlace( nStar, nPlace );

	}


	int nBGType = BGType::EVIL_STAR;
	// bgݒ
	switch( this->m_nSelectedStar ) {
	case StageType::FOREST:		nBGType = BGType::FOREST;		break;
	case StageType::WATER:		nBGType = BGType::SEA;			break;
	case StageType::FIRE:		nBGType = BGType::VOLCANO;		break;
	case StageType::WIND:		nBGType = BGType::CLOUD;		break;
	}
	// XN[ύX
	{
		this->m_BGManager.setBGImage(nBGType );
	}

//	for( int i = 0; i < GEnum::PLAYER_MAX_NUM; i ++ ) {
	{
		// vC[
		GPlayer* player = &this->m_ObjectManager.m_Player;//&this->m_ObjectManager.m_aryPlayerList[i];
		// _[Wʏ
//		player->setStoredDamage( 0 );
//		player->m_bIsNaked = false;

		/////
		// vC[Ԉُ

		player->resetPlace();

		GPlayer* lpCOMPlayer = &this->m_ObjectManager.m_COMPlayer;
		lpCOMPlayer->resetPlace();

		// XN[̒SWvC[ƈꏏɂ
		HPLScroller* lpScroller = this->getScrollManager()->getScroller(0);
		lpScroller->m_CurrentVirtualCenter = player->getCenter();

	}
	// ŏ̃Cxg̓XLbv
//	this->setSkipFirst( true );

	/////
	// `FWO
//	this->m_Game.setChangingMap(false);

	// vC[ݒuʒû߂ɃZbgĂ
//	this->m_nPlayerPlaceIndex = -1;

	// bgm
	int nNextMusicIndex = -1;
	switch( this->getMapType() ) {
	case MapType::BOSS:
		nNextMusicIndex = Music::BOSS;
		break;
//	case // XN[
	}

	// bgm set
	if( nNextMusicIndex >= 0 ) {
		getStaticData()->m_MixerManager.playMusic(nNextMusicIndex, true );

		// BGMł΁Â܂ܑ̂ŁAL̊֐ŗǂB
/*		if( nNextMusicIndex != this->m_MixerManager.getCurrentMusicIndex() ) {
	//		this->m_MixerManager.stopMusic();
			this->m_MixerManager.playMusic(nNextMusicIndex, true );
		} else {	// same bgm
			// do nothing ( continue playing current music )
		}
		*/
	} else if( this->m_SceneManager.getSceneType() == SceneType::SPECIAL_DEAD ) {
		this->m_MixerManager.playMusic( Music::BAD_END , true );
	} else { //if(mapIndex != SpecialMapIndex::SELECT_STAR) {
		// stage normal
//		this->m_MixerManager.stopMusic();
		int nBGMType = Music::STAGE_NORMAL;
		switch( this->m_nSelectedStar ) {
		case StageType::FIRE:		nBGMType = Music::FIRE;			break;
		case StageType::FOREST:		nBGMType = Music::FOREST;			break;
		case StageType::WIND:		nBGMType = Music::CLOUD;			break;
		}
		this->m_MixerManager.playMusic( nBGMType , true );
	//if( nNextMusicIndex < 0 ) {
	//	this->m_MixerManager.stopMusic();
	//} else	
	}

	switch( this->m_SceneManager.getSceneType() ) {
	case SceneType::GAME:
	case SceneType::LIBRARY:
		{
			////////
			// XN[ʒuvC[ɍ킹
			GPlayer* lpPlayer = &this->m_ObjectManager.m_Player;
			HPLScroller* scroller = this->getScrollManager()->getScroller(0);
			// XN[́AvC[̏ʒuɋړ
			scroller->m_CurrentVirtualCenter = lpPlayer->getCenter();
			this->adjustScreen( scroller );

			///////
			switch( getStaticData()->getMapIndex() ) {
			case SpecialMapIndex::WIND_BOSS_1:	// wind boss1
				{
					vector<int> vTypes;
					vTypes.push_back( EnemyType::CHASER );
					this->m_ObjectManager.m_EnemyManager.setupCountTargets( vTypes );
				}
				break;
			}
		}
		break;
	}

	// reset darken
	HPLFadeOut* lpFade = getStaticData()->m_BGManager.getFadeOut();
	lpFade->stop();
/*
	// set play time
	getStaticData()->m_OptionData.setPlayTime(
		getStaticData()->m_OptionData.getPlayTime() + getStaticData()->m_TimeRecorder.getElapseTime(true) );
//	getStaticData()->m_OptionData.save();

	// achievement
	GAchievementItem* lpItem = getStaticData()->m_AchievementSave.getAchievementItem( AchievementType::PLAY_LONG );
	lpItem->setCounter( getStaticData()->m_TimeRecorder.getElapseTime(false) );
	*/
	// save
	this->m_Save.save();
}

void GStaticData::changeState(HPLShootingObject* obj, int newState){
	int ot = obj->getObjectType();
	if(ot == ObjectType::PLAYER){
		((GPlayer*)obj)->changeState(newState);
	}else if(ot == ObjectType::ENEMY){
//		((GEnemy*)obj)->changeState(newState);
	}else if(ot == ObjectType::NEUTRAL){
//		((GNeutral*)obj)->changeState(newState);
	}else if(ot == ObjectType::SHOT){
		((GShot*)obj)->changeState(newState);
	}
}

int GStaticData::getEvigenSequenceID(HPLShootingObject* obj){
	// state
	int st = obj->getState()->getState();

	// id(return value)
	int id = GEnum::Sequence::Player::STAND;

	// wear state
	int nWearState = WearStateType::NORMAL;
	bool bIsNaked = nWearState == WearStateType::NAKED;

	if( obj->getObjectType() == ObjectType::PLAYER &&
		obj->getType() == PlayerType::EVIGEN  ) {
		// vC[ł

		nWearState = ((GPlayer*)obj)->m_nWearState;
	}

	if( bIsNaked ) {
		id = GEnum::Sequence::Player::STAND_NAKED;
	}

	switch( st ) {
	case StateType::WALK:
		if( bIsNaked ) {
			id = GEnum::Sequence::Player::WALK_NAKED;
		} else {
			id = GEnum::Sequence::Player::WALK;
		}
		break;
	case StateType::FLYDOWN:
		if( bIsNaked ) {
			id = GEnum::Sequence::Player::FLYDOWN_NAKED;
		} else {
			id = GEnum::Sequence::Player::FLYDOWN;
		}
		break;
	case StateType::CROUCH:
		if( bIsNaked ) {
			id = GEnum::Sequence::Player::CROUCH_NAKED;
		} else {
			id = GEnum::Sequence::Player::CROUCH;
		}

		break;
	case StateType::FLYUP:
		if( bIsNaked ) {
			id = GEnum::Sequence::Player::FLYUP_NAKED;
		} else {
			id = GEnum::Sequence::Player::FLYUP;
		}
		break;

	case StateType::ATTACK:
	case StateType::ATTACK_PUNCH:
	case StateType::ATTACK_PINEAPPLE:

		if( st == StateType::ATTACK &&
			obj->getTrigger()->getTrigger( TriggerType::ON_THE_GROUND ) == false )
		{
			id = GEnum::Sequence::Player::JATTACK;
		} else {
			if( bIsNaked ) {
				id = GEnum::Sequence::Player::ATTACK_PUNCH_NAKED;
			} else {
				id = GEnum::Sequence::Player::ATTACK;
			}
		}
		break;

	case StateType::ATTACK_2:
		id = GEnum::Sequence::Player::ATTACK_2;
		break;
	case StateType::DAMAGE:
		if( WearStateType::NAKED == nWearState ) {
			id = GEnum::Sequence::Player::DAMAGE_NAKED;
		} else {
			id = this->getDamageSequenceID(obj);
		}
		break;
	case StateType::DAMAGE_FLYDOWN:
		
		if( bIsNaked ) {
			id = GEnum::Sequence::Player::DAMAGE_FLYDOWN_NAKED;
		} else {
			id = GEnum::Sequence::Player::DAMAGE_FLYDOWN;
		}
		break;
	case StateType::DAMAGE_FLYDOWN_REVERSE:
		if( bIsNaked ) {
			id = GEnum::Sequence::Player::DAMAGE_FLYDOWN_REVERSE_NAKED;
		} else {
			id = GEnum::Sequence::Player::DAMAGE_FLYDOWN_REVERSE;
		}
		break;
	case StateType::DAMAGE_ELEC:
		id = GEnum::Sequence::Player::DAMAGE_ELEC;
		break;
	case StateType::DEAD:
	case StateType::LYING:
		
		if( bIsNaked ) {
			id = GEnum::Sequence::Player::DEAD_NAKED;
		} else {
			id = GEnum::Sequence::Player::DEAD;
		}
		break;
	case StateType::DEAD_FLYDOWN:
		id = GEnum::Sequence::Player::DEAD_FLYDOWN;
		break;
	case StateType::DEAD_FLYUP:
		id = GEnum::Sequence::Player::DEAD_FLYUP;
		break;
	case StateType::HOLD_DOWN:
	case StateType::JII:
		{
			
			if( bIsNaked ) {
				id = GEnum::Sequence::Player::HOLD_DOWN_NAKED;
			} else {
				id = GEnum::Sequence::Player::DAMAGE_HOLD_DOWN;
			}

		}
		break;
	case StateType::ATTACK_GUN:
		
		if( bIsNaked ) {
			id = GEnum::Sequence::Player::ATTACK_GUN_NAKED;
		} else {
			id = GEnum::Sequence::Player::ATTACK_GUN;
		}
		break;
	case StateType::HOLD:
		if( bIsNaked ) {
			id = GEnum::Sequence::Player::DAMAGE_HOLD_NAKED;
		} else {
			id = GEnum::Sequence::Player::DAMAGE_HOLD;
		}
		break;

	case StateType::BIND_DAMAGE_ELEC:
		if( bIsNaked ) {
			id = Sequence::Player::BIND_DAMAGE_ELEC_NAKED;
		} else {
			id = GEnum::Sequence::Player::BIND_DAMAGE_ELEC;
		}
		break;
	case StateType::BIND:
		if( bIsNaked ) {
			id = Sequence::Player::BIND_NAKED;
		} else {
			id = GEnum::Sequence::Player::BIND;
		}
		break;

	case StateType::EATENING:
		id = GEnum::Sequence::Player::EATENING;
		if( obj->getObjectType() == ObjectType::PLAYER ) {
			switch( ((GPlayer*)obj)->getHoldEnemyType() ) {
			case EnemyType::MUTANT_GEL:
				id = GEnum::Sequence::Player::EATENING_2;
				break;
			}
		}
		break;
	case StateType::DISAPPEAR:
		id = GEnum::Sequence::Player::EATEN;
		break;
	case StateType::BIND_DAMAGE:
		if( bIsNaked ) {
			id = Sequence::Player::BIND_DAMAGE_NAKED;
		} else {
			id = GEnum::Sequence::Player::BIND_DAMAGE;
		}
		break;
	case StateType::BROKEN:
		id = GEnum::Sequence::Player::BROKEN;
		break;
	case StateType::SWIM:
		
		if( bIsNaked ) {
			id = GEnum::Sequence::Player::SWIM_NAKED;
		} else {
			id = GEnum::Sequence::Player::SWIM;
		}
		break;
/*	case StateType::CAKE && this->m_vecPlayerSequenceNumbers[obj->getType()] >= Sequence::Player::TAUNT:
		// TAUNTÃV[PX
		id = GEnum::Sequence::Player::TAUNT;*/

	case StateType::WALK_GUN:
		id = Sequence::Player::WALK_GUN;
		break;
	case StateType::JUMP_GUN:
		id = Sequence::Player::JUMP_GUN;
		break;
	case StateType::DAMAGE_FLAME:
		id = Sequence::Player::DAMAGE_FIRE;
		break;
	case StateType::CHECK:
		id = Sequence::Player::CHECK;
		break;
	case StateType::FLYUP_2:
		id = Sequence::Player::DADAKKO;
		break;
	case StateType::BORING:
		id = Sequence::Player::STAND_BORED ;
		break;
	case StateType::BORING_CROUCH:
		id = Sequence::Player::NERU;
		break;
	case StateType::CHONPA_STAND:
		id = Sequence::Player::CHONPA_STAND;
		break;
	case StateType::CHONPA_FALL:
		id = Sequence::Player::CHONPA_FALL;
		break;
	case StateType::CHONPA_DEAD:
		id = Sequence::Player::CHONPA_DEAD;
		break;
	case StateType::FLYUP_3:
		id = Sequence::Player::DOUBLE_JUMP;
		break;
	case StateType::STONE:
		id = Sequence::Player::STONE;
		break;
	}

	return id;
}

int GStaticData::getDamageSequenceID(HPLShootingObject* player){
	int st = player->getState()->getState();

	int id = GEnum::Sequence::Player::DAMAGE;
	return id;
}

void GStaticData::setDamageFlyingState(HPLShootingObject* obj){
	int nNextState = HPLActionMoveTool::setDamageFlyingState( obj );

	if(nNextState >= 0){
		if(obj->getObjectType() == ObjectType::PLAYER){
			((GPlayer*)obj)->changeState(nNextState);
		} else {
			HPLError::assertIt( false, "" );
			//((TTEnemy*)obj)->changeState(nextState);
		}

	}

}
void GStaticData::rollBack( HPLShootingObject* lpObject ) {
	if( lpObject->getObjectType() == ObjectType::PLAYER) {
		((GPlayer*)lpObject)->rollBackState();
	} else {
		//this->changeState( lpObject, StateType::STAND );
		// error
		HPLError::assertIt( false, "" );
	}
}
void GStaticData::hitToPlayer( HPLShootingObject* lpDamageSource, GPlayer* lpTargetPlayer, float fSpeed, float fDamage ) {

//	fSpeed += lpTargetPlayer->getStoredDamage() / 10.f;

//	this->blowAway( lpDamageSource, lpTargetPlayer, fSpeed );
	HPLActionMoveTool::blowAway( lpDamageSource, lpTargetPlayer, fSpeed );

	// _[W܂
//	lpTargetPlayer->addDamage( fDamage );
	
	// _[WԂɂȂ
	lpTargetPlayer->changeState( StateType::DAMAGE );
}
GEffect* GStaticData::createEffect(int nType, const HPLVector& vFrom, const HPLVector& vTo,
	int nCreateEnemyType, int nForceID )
{
	HPLVector vecEffectCenter = vFrom;
	vecEffectCenter.add( vTo );
	vecEffectCenter.multiple( 0.5f );
		
	GEffect* lpEffect = this->m_ObjectManager.m_EffectManager.create(
		nType, vecEffectCenter, true);//, nCreateEnemyType, nForceID );
	return lpEffect;
}

int GStaticData::getBlockID( HPLVector& v ) {
	int TILE_SIZE = this->getSetting()->tileSize;

	return this->getBlockManager()->getBlockMatrix()->get(
						static_cast<int>( (v.getX()) / TILE_SIZE ),
						static_cast<int>( (v.getY()) / TILE_SIZE ) );
}

void GStaticData::startFadeOutDX(int nNextScene, int nNextMap, int nAfterFadeOutDXProcessType) {
	this->m_FadeOutCounter.reset();
	this->m_nNextScene = nNextScene;
	this->m_nNextMap = nNextMap;
	this->m_nAfterFadeOutDXProcessType = nAfterFadeOutDXProcessType;

	this->updateStarAndPlace( nNextMap );
}
void GStaticData::updateStarAndPlace( int nMapIndex ) {
	int nStar = 0;
	int nPlace = 0;
	if( this->m_Setting.maps.getStarAndPlaceFromMapIndex( nMapIndex, nStar, nPlace ) >= 0 ) {
		// 擾

		// ݒ肷
		this->setStarAndPlace( nStar, nPlace );
	}
}


bool GStaticData::processFadeOutDX( bool bIsAutoChange ) {
	//this->m_lpFadeOutSequence->step();
	this->m_FadeOutCounter.step();

	bool bIsFading = this->isFadingOutDX();

	if( bIsAutoChange && bIsFading == false ) {
		// ؂ւ
		if( this->m_nNextScene >= 0 ) {
			this->m_SceneManager.setScene( this->m_nNextScene );
		}
		if( this->m_nNextMap >= 0 ) {
			this->setupMap( this->m_nNextMap, true );
		}

		this->m_nNextScene = -1;
		this->m_nNextMap = -1;
	}

	return bIsFading;
}

void GStaticData::drawFadeOutDX() {
	const int WIDTH = 100;
	const int HEIGHT = 50;
//	HPLImageInformation* lpInfo = this->m_lpFadeOutSequence->getImageInformation(0);
	
	int nStep = ( this->m_FadeOutCounter.getStepCounter() );
	float fMax = this->m_FadeOutCounter.getWaitFrame() / 3.f * 2.f;

	float fRate = (float)nStep / fMax;

	int nYCounter = 0;
	for( int y = 0; y < this->m_Setting.windowHeight; y += HEIGHT ) {
		int nXCounter = 0;
		for( int x = 0; x < this->m_Setting.windowWidth; x += WIDTH ) {
			bool bEven = false;
			if( (nYCounter % 2) == 0 ) {
				bEven = (nXCounter % 2) == 1;
			} else {
				bEven = (nXCounter % 2) == 0;
			}
			if( bEven ) {
				DrawBox( x, y, (int)(x + WIDTH * fRate), y+HEIGHT, 
					GetColor(0,0,0), TRUE);
			} else {
				DrawBox( (int)( x + WIDTH * (1-fRate) ) , y, x + WIDTH, y+HEIGHT, 
					GetColor(0,0,0), TRUE);
			}

			nXCounter ++;
		}

		nYCounter ++;
	}

	if( nStep >= fMax -1  ) {
		// [fBOʕ\
		HPLSequence* lpSeq = getStaticData()->getSequence(SpriteType::COMMON, Sequence::Common::LOADING );
		getStaticData()->m_Basic.m_Graphics.draw(
			0,0,lpSeq );
	}
}

/** 
  *
  */
CollisionState GStaticData::checkCollisionObjectX( HPLShootingObject* lpObject, 
	HPLShootingObject* lpBlockObject )
{

	HPLRect rHitArea = lpObject->getHitArea();
	HPLRect rBlockHitArea = lpBlockObject->getHitArea();

	float dx = lpObject->getVelocityAngle().getX();

	// level
	float fLevel = this->m_Game.getLevel( LEVEL_EPSILON );

	CollisionState state;

	const int COLLISION_OBJECT_HIT_AREA_DELTA = 0;

	// ͈͒
	rHitArea.setX( rHitArea.getX() - COLLISION_OBJECT_HIT_AREA_DELTA );
	rHitArea.setWidth( rHitArea.getWidth() + COLLISION_OBJECT_HIT_AREA_DELTA * 2 );

	/*if( dx == 0 ) {
		// qbgmF
		if( rHitArea.isShare( rBlockHitArea ) ) {
		}
	} else*/ {
		rHitArea.setX( rHitArea.getX() + dx );

		if( rHitArea.isShare( rBlockHitArea ) ) {

			if( /*dx >= 0 && */lpObject->getCenter().getX() <= rBlockHitArea.getX()){//rBlockHitArea.getX()) {
				// EɂԂ
				state.isHitX = HitDirectionX::RIGHT;

				if( lpObject->getTrigger()->getTrigger( TriggerType::ON_THE_GROUND ) &&
					lpObject->getTrigger()->getTrigger( TriggerType::HIT_TO_THE_CEILING ) )
				{
					// ɋ܂Ă
					// do nothing
					
				} else if( lpObject->getTrigger()->getTrigger( TriggerType::HIT_TO_THE_LEFT ) ) {
					// ɍɐڐGĂ
					// ܂ꂽ
					// _[W(x␳LAGADAMAGE)
					this->damageObject( lpObject, NULL, BLOCK_BITE_DAMAGE * fLevel, false, StateType::DAMAGE );
				} else {
					// ͈͂ɖ߂
					rHitArea.setX( rHitArea.getX() + COLLISION_OBJECT_HIT_AREA_DELTA );
					rHitArea.setWidth( rHitArea.getWidth() - COLLISION_OBJECT_HIT_AREA_DELTA * 2 );

					// center.x ~ hitArea.right̋

					float fDeltaX = rHitArea.getRight() - lpObject->getCenter().getX();

					HPLVector vCenter = lpObject->getCenter();

					// ubN̍[ɒ
					vCenter.setX( rBlockHitArea.getX() - fDeltaX );

					// ʒu
					lpObject->setCenter( vCenter );

					// x0
					HPLVector vVelocityAngle = lpObject->getVelocityAngle();
					vVelocityAngle.setX(0);
					lpObject->setVelocityAngle( vVelocityAngle );
				}

				// ڐGgK
				lpObject->getTrigger()->setTrigger( TriggerType::HIT_TO_THE_RIGHT, true );


			} else if( /*dx <= 0 && */lpObject->getCenter().getX() > rBlockHitArea.getRight()){//rBlockHitArea.getRight()){

				// ɂԂ
				state.isHitX = HitDirectionX::LEFT;

				if( lpObject->getTrigger()->getTrigger( TriggerType::ON_THE_GROUND ) &&
					lpObject->getTrigger()->getTrigger( TriggerType::HIT_TO_THE_CEILING ) )
				{
					// ɋ܂Ă
					// do nothing
					
				} else if( lpObject->getTrigger()->getTrigger( TriggerType::HIT_TO_THE_RIGHT ) ) {
					// ɑ̂̂ɂԂĂ
					// ܂ꂽ
					// _[W(x␳LAGADAMAGE)
					this->damageObject( lpObject, NULL, BLOCK_BITE_DAMAGE * fLevel, false, StateType::DAMAGE );
				} else {
					rHitArea.setX( rHitArea.getX() + COLLISION_OBJECT_HIT_AREA_DELTA );
					rHitArea.setWidth( rHitArea.getWidth() - COLLISION_OBJECT_HIT_AREA_DELTA * 2 );

					// center.x ~ hitArea.left̋
					float fDeltaX = lpObject->getCenter().getX() - rHitArea.getX();

					HPLVector vCenter = lpObject->getCenter();

					// ubN̉E[ɒ
					vCenter.setX( rBlockHitArea.getRight() + fDeltaX);

					// ʒu
					lpObject->setCenter( vCenter );

					HPLVector vVelocityAngle = lpObject->getVelocityAngle();
					vVelocityAngle.setX(0);
					lpObject->setVelocityAngle( vVelocityAngle );
				}

				// ڐGgK
				lpObject->getTrigger()->setTrigger( TriggerType::HIT_TO_THE_LEFT, true );

			}

			// ~߂
			/*HPLVector velocityAngle = lpObject->getVelocityAngle();
			velocityAngle.setX(0);
			lpObject->setVelocityAngle( velocityAngle );*/
		}
	}
	return state;
}
CollisionState GStaticData::checkCollisionObjectY( HPLShootingObject* lpObject, 
	HPLShootingObject* lpBlockObject, int nMountType )
{
	HPLRect rHitArea = lpObject->getHitArea();

	const int COLLISION_OBJECT_HIT_AREA_DELTA = 0;

	// ͈͒
	rHitArea.setY( rHitArea.getY() - COLLISION_OBJECT_HIT_AREA_DELTA );

	rHitArea.setHeight( rHitArea.getHeight() + COLLISION_OBJECT_HIT_AREA_DELTA * 2  + COLLISION_OBJECT_HIT_AREA_DELTA_BOTTOM);


	HPLRect rBlockHitArea = lpBlockObject->getHitArea();

	float dy = lpObject->getVelocityAngle().getY();

	CollisionState state;

	// level
	float fLevel = this->m_Game.getLevel(LEVEL_EPSILON);

	/*
	if( dy == 0 ) {
		// qbgmF
		if( rHitArea.isShare( rBlockHitArea ) ) {
		}
	} else*/
	{
		bool bIsHitPrevious = rHitArea.isShare( rBlockHitArea );   // Ȍ

		// ړ
		rHitArea.setY( rHitArea.getY() + dy );

		if( rHitArea.isShare( rBlockHitArea ) ) {
			if( nMountType == MountType::THROW && ( bIsHitPrevious ) && dy <= 0  ) {
				// X[^CvłA~ȊOAOɐڐGĂȂ
				// Ȃ

			} else if( /*dy >= 0 && */ lpObject->getCenter().getY() <= rBlockHitArea.getY() ){//rBlockHitArea.getY() ) {

				// ubN̏ɍڂ
				state.isHitY = HitDirectionY::BOTTOM;

				if( lpObject->getTrigger()->getTrigger( TriggerType::HIT_TO_THE_LEFT ) &&
					lpObject->getTrigger()->getTrigger( TriggerType::HIT_TO_THE_RIGHT ) )
				{
					// ɋ܂Ă
					// do nothing
					
				} else {
					// Eɂ͋܂ĂȂ

					// iqW
					int nTileCoordX = (int)((rHitArea.getX() + rHitArea.getWidth() / 2 ) / this->m_Setting.tileSize);
					int nTileCoordY = (int)((rHitArea.getY() ) / this->m_Setting.tileSize );
					
					// ɂԂĂ邩H
					bool bIsCeiling = lpObject->getTrigger()->getTrigger( TriggerType::HIT_TO_THE_CEILING );

					// ubN̎
					int nTargetTileID = this->getBlockManager()->getBlockMatrix()->get(nTileCoordX, nTileCoordY );
					map<int,HPLBlock*>::iterator bit = this->getBlockManager()->getBlockMap()->find(nTargetTileID);

#ifdef _DEBUG
					HPLError::assertIt( bit != this->getBlockManager()->getBlockMap()->end(),
						"%s:BlockData is not found:%d", __FUNCTION__, nTargetTileID );
#endif
					if( this->getBlockManager()->getBlockMatrix()->isValidPosition(nTileCoordX, nTileCoordY ) &&
						bit->second->isEmpty() == false )
					{
						// ͈͓̃ubNΏ
						// Emptył͂Ȃ
						// ->ԂĂ
						bIsCeiling = true;
					}


					if( bIsCeiling ) {
						// ɑ̂̂ɂԂĂ
						// ܂ꂽ
						// _[W(x␳LAGALying)
						this->damageObject( lpObject, NULL, BLOCK_BITE_DAMAGE * fLevel, true, StateType::LYING );
					} else {
					}

					// ͈͏C
					rHitArea.setY( rHitArea.getY() + COLLISION_OBJECT_HIT_AREA_DELTA );
					rHitArea.setHeight( rHitArea.getHeight() - COLLISION_OBJECT_HIT_AREA_DELTA * 2 -
						COLLISION_OBJECT_HIT_AREA_DELTA_BOTTOM  );

					// center.y ~ hitArea.bottom̋
					float fDeltaY = rHitArea.getBottom() - lpObject->getCenter().getY();

					HPLVector vCenter = lpObject->getCenter();

					// ubN̒[ɒ
					vCenter.setY( rBlockHitArea.getY() - fDeltaY );
					//vCenter.setX( vCenter.getX() + lpBlockObject->getVelocityAngle().getX() );

					// ʒu
					lpObject->setCenter( vCenter );
					
					//HPLVector vVelocityAngle = lpObject->getVelocityAngle();
					//vVelocityAngle.setY(lpBlockObject->getVelocityAngle().getY());
					//lpObject->setVelocityAngle( vVelocityAngle );
				}

				// ڐGgK
				lpObject->getTrigger()->setTrigger( TriggerType::ON_THE_GROUND, true );

				//bool bIsGetVel = true;	// x炤ǂ

				/*if( lpObject->getObjectType() == ObjectType::PLAYER ) {
					// vC[ł
					GPlayer* lpPlayer = ((GPlayer*)lpObject);
					if( lpPlayer->isGetRideSpeed() ) {
						// ɂĂ
						// ->x͂Ȃ
						bIsGetVel = false;

					} else {
						// ܂ĂȂ
						// ->x炤B̃t[܂łȂî鏈͍sj
					//	lpPlayer->setGetRideSpeed( bIsGetVel );
					//}
				}*/
				//if( bIsGetVel ) {
					
					// x炤(Y)
					HPLVector vCenter = lpObject->getCenter();
					// X݂̂̒ǉ͑ʖڂ
					HPLVector vAddVel = lpBlockObject->getVelocityAngle();
					// lŏoĂ݂
					//vAddVel.set((int)vAddVel.getX(),(int)vAddVel.getY() );
					vCenter.add( vAddVel );
					lpObject->setCenter( vCenter );
				//}

			} else if( /*dy <= 0 && */lpObject->getCenter().getY() > rBlockHitArea.getBottom() ){//rBlockHitArea.getBottom()){
				// IuWFNgɓԂ

				if( lpObject->getTrigger()->getTrigger( TriggerType::HIT_TO_THE_LEFT ) &&
					lpObject->getTrigger()->getTrigger( TriggerType::HIT_TO_THE_RIGHT ) )
				{
					// ɋ܂Ă
					// do nothing
					
				} else {
					
					if( lpObject->getTrigger()->getTrigger( TriggerType::ON_THE_GROUND ) ) {
						// ɑ̂̂ɂԂĂ
						// ܂ꂽ
						// _[W(x␳LAGLALying)
						this->damageObject( lpObject, NULL, BLOCK_BITE_DAMAGE * fLevel, true, StateType::LYING );
					} else {
					}

					rHitArea.setY( rHitArea.getY() + COLLISION_OBJECT_HIT_AREA_DELTA );
					rHitArea.setHeight( rHitArea.getHeight() - COLLISION_OBJECT_HIT_AREA_DELTA * 2 );

					// ɂԂ
					state.isHitY = HitDirectionY::TOP;

					// center.y ~ hitArea.left̋
					float fDeltaY = lpObject->getCenter().getY() - rHitArea.getY();

					HPLVector vCenter = lpObject->getCenter();

					// ubN̉[ɒ
					vCenter.setY( rBlockHitArea.getBottom() + fDeltaY);

					// ʒu
					lpObject->setCenter( vCenter );

					// x0
					HPLVector vVelocityAngle = lpObject->getVelocityAngle();
					vVelocityAngle.setY(0);
					lpObject->setVelocityAngle( vVelocityAngle );
				}

				// ڐGgK
				lpObject->getTrigger()->setTrigger( TriggerType::HIT_TO_THE_CEILING, true );

			}

			// ~߂
			//HPLVector velocityAngle = lpObject->getVelocityAngle();
			//velocityAngle.setY(0);
			//lpObject->setVelocityAngle( velocityAngle );

/*			//if( dy > 0 ) {
				// ɍڂĂ

			//}*/
		}
	}
	return state;
}

/** vC[ǂʂă_[W^
  * @param[out]		lpObject		_[W󂯂IuWFNg
  * @param[out]		lpDamageSource	_[W^IuWFNgBNULL
  * @param[in]		fDamage			_[WʁBx␳ς
  * @param[in]		nNextState		_[W󂯂IuWFNgɎXe[g
  */
void GStaticData::damageObject( HPLShootingObject* lpObject, HPLShootingObject* lpDamageSource, 
	float fDamage, bool bIsInvinsible, int nNextState )
{
	switch( lpObject->getObjectType() ) {
	case ObjectType::PLAYER:
		{
			if( lpObject->isInvinsible() == false ) {
				dynamic_cast<GPlayer*>( lpObject )->damage( fDamage, lpDamageSource, bIsInvinsible, nNextState );
			}
		}
		break;
	case ObjectType::ENEMY:
		{
			if( lpObject->isInvinsible() == false ) {
				dynamic_cast<GEnemy*>( lpObject)->damage( fDamage, lpDamageSource );
			}
		}
		break;
	default:
		HPLError::assertIt(false, "Ή");
	}
}


/*void GStaticData::shotBullet(
	int nType,// HPLShootingObject* lpParent,
	int nFaceDirection,
	int nWeaponType,
	HPLVector vShotCenter,
	HPLVector vVelocityAngle)
{
	int sht = nType;



	int d = 0;
	if( nType == Sequence::Shot::HOMING_LASER ||
		nType == Sequence::Shot::ELECTRIC_HOMING_LASER)
	{
		d = -30;
	}

	if(lpParent->getFaceDirection() == FaceDirection::LEFT){
		d = -10;
		if( nType == Sequence::Shot::HOMING_LASER ||
		nType == Sequence::Shot::ELECTRIC_HOMING_LASER )
		{
			d = 20;
		}
	}

	int dy = 22;
	if(nWeaponDirection == WeaponDirection::FRONT_UP ){
		dy = 18;

		d = -2;
		if(lpParent->getFaceDirection() == FaceDirection::LEFT){
			d = -3;
		}
		//dy = 3;
	} else if( nWeaponDirection == WeaponDirection::FRONT_DOWN ) {
		d = -5;
		if(lpParent->getFaceDirection() == FaceDirection::LEFT){
			d = 0;
		}
	}

	// ˈʒu

	HPLVector c(
		lpParent->getCenter().getX() + 
		lpParent->getFaceDirection() * 10.f +
		(float)HPLMath::nextDouble(-1,1) * 5.f +
		d,

		lpParent->getCenter().getY() +
		dy +
		(float)HPLMath::nextDouble(-1,1) * 5.f);

	GShot* shot = this->m_ObjectManager.m_ShotManager.create(
		sht, c,
		lpParent->getID(), lpParent->getFaceDirection());
	// xƔ˕
	float delta = 0;
	if( nWeaponDirection == WeaponDirection::FRONT_UP ){
		delta = - (float)lpParent->getFaceDirection() * 45;
	} else if( nWeaponDirection == WeaponDirection::FRONT_DOWN ){
		delta =  (float)lpParent->getFaceDirection() * 45;
	}

	// 
	HPLVector v;
	if( nType == Sequence::Shot::NATO_BULLET ) {
		v.set(
			(float)lpParent->getFaceDirection(), 
			(float)(HPLMath::nextDouble(-1,1)* GATLING_ERR));
		v.setByDegree(v.getDegree() + delta);
		v.multiple(GATLING_SPD);
	} else if( nType == Sequence::Shot::HOMING_LASER ||
		nType == Sequence::Shot::ELECTRIC_HOMING_LASER )
	{
		// 
		//v.set(HPLVector(
		//	-1.f * (float)lpParent->getFaceDirection(), 
		//	(float)(HPLMath::nextDouble(-1,1)* 5.f)));
		////v.multiple(GATLING_SPD);
	}
	shot->setVelocityAngle(v);
}
*/
HPLVector GStaticData::getEmotionIconPosition( HPLShootingObject* lpObject ) {
	HPLVector vEffectCenter = this->m_ObjectManager.m_Player.getCenter();
	vEffectCenter.add( static_cast<float>( EMOTION_ICON_DELTA_X ),
		static_cast<float>( EMOTION_ICON_DELTA_Y ) );
	return vEffectCenter;
}

void GStaticData::createEffectEmotionIcon(HPLShootingObject* lpObject, int nEffectType) {
	// GtFNg
	// TODO eXgőSޔ
	
	GEffect* lpEffect = this->m_ObjectManager.m_EffectManager.create(
		nEffectType, this->getEmotionIconPosition( lpObject ), true );
		

}
HPLShootingObject* GStaticData::createPlayerEmotionIcon( int nEffectType ) {
	// vC[ĂG[VLɂH
	GPlayer& player = this->m_ObjectManager.m_Player;
	player.m_EmotionIcon.setType( nEffectType );
	player.m_EmotionIcon.setAlive( true );

	// V[PX
	player.m_EmotionIcon.getSequence()->resetCounter();
	return &player.m_EmotionIcon;
}

/** 蔻`FbNiʏj+ړ
  * Enemy/Neutral/Shot/Item̑΃ubN蔻
  * ɉLsBړ⌸܂Ŋ܂܂Ă̂Œ
  * - ړ
  * - oEh(oEhΏۂ̂)
  * - (ڐGA^Cvł邱ƂK{)
  *   @󒆂ł̌͂ł͍sȂ
  * - d͉
  * - Rɍ|̃`FbN
  */
void GStaticData::collisionObject( HPLShootingObject* lpObject,
								  CollisionState& stateX,
								  CollisionState& stateY)
{

	// CRl
	float RESIST = this->getSetting()->resist;

	// d
	const float GRAVITY = this->getSetting()->gravity;

	// Rő呬xiCRƂ̂肠j
	float FALL_MAX_SPEED = this->getSetting()->fallMaxSpeed;
	switch( lpObject->getObjectType() ) {
	case ObjectType::SHOT:
		FALL_MAX_SPEED *= 2;
		break;
	}

	
	// ^CTCY
	int tileSize = this->getSetting()->tileSize;
	
	// IuWFNĝ蔻
	HPLRect hitArea = lpObject->getHitArea();

	// VxxNg
	HPLVector vNewVelocityAngle = lpObject->getVelocityAngle();

	// VʒuxNg
	HPLVector vNewCenter = lpObject->getCenter();

	// 蔻̍[/[璆SWւ̋
	float dx = vNewCenter.getX() - hitArea.getX();
	float dy = vNewCenter.getY() - hitArea.getY();

	// block map
	map<int, HPLBlock*>* blockMap = this->getBlockManager()->getBlockMap();

	// block matrix
	HPLMatrix<int>* matrix = this->getBlockManager()->getBlockMatrix();

	// ubNɂԂ
	bool bIsHitToBlock = false;

	// ˌ
	const float BOUNCE_DECELERATION_RATE_X = 0.5f;
	const float BOUNCE_DECELERATION_RATE_Y = 0.5f;

	if( lpObject->getObjectAttribute( ObjectAttribute::HIT_TO_BLOCK ) ) {	// hit to block object
		// ÕX[v
		int nSlopePreviousTileCoordX = lpObject->getSlopePreviousTileCoordX();
		int nSlopePreviousTileCoordY = lpObject->getSlopePreviousTileCoordY();

		// object̑xxNg
		HPLVector velocityAngle = lpObject->getVelocityAngle();

		//HPLVector center = lpObject->getCenter();

		// `FbN
		stateX = HPLCollisionTool::adjustCollisionX(
			hitArea, velocityAngle,
			matrix, tileSize, 
			*blockMap,
			nSlopePreviousTileCoordX, nSlopePreviousTileCoordY);

		map<int,HPLBlock*>::iterator bit = blockMap->find( stateX.m_nTouchBlockID );
		HPLError::assertIt( bit != blockMap->end(),
			"%s:BlockData is not found:%d", __FUNCTION__, stateX.m_nTouchBlockID );

		// ڐGubNǂ
		bool isTouch = bit->second->getBlockType() == BlockType::BLOCK;
		
		// trigger 
		lpObject->getTrigger()->setTrigger(TriggerType::HIT_TO_THE_LEFT,
			isTouch && stateX.isHitX == HitDirectionX::LEFT);
		lpObject->getTrigger()->setTrigger(TriggerType::HIT_TO_THE_RIGHT,
			isTouch && stateX.isHitX == HitDirectionX::RIGHT);

		///////
		// ɂԂ
		if( isTouch && //this->m_bIsHitToBlock ){
			lpObject->getObjectAttribute(ObjectAttribute::HIT_TO_BLOCK) )
		{
			// ʒu
			vNewCenter.setX(stateX.m_nNewX + dx);
			// TODO {ɍĂ̂HEɐڐGȂA
			// dx = -(HitAreảE[-CenterX)ȂH

			// ubNɐڐG
			bIsHitToBlock = true;

			if( lpObject->getObjectAttribute( ObjectAttribute::BOUND ) ||
				lpObject->getObjectAttribute( ObjectAttribute::BOUND_X ))
			{
				// oEhL

				// oEh
				vNewVelocityAngle.setX( 
					HPLActionMoveTool::getBoundedSpeed(
					velocityAngle.getX(), BOUNCE_DECELERATION_RATE_X,
					stateX.isHitX ) );
			} else {
				//hitArea.setX(stateX.m_nNewX);

				// ~
				vNewVelocityAngle.setX(0);

			}

		}
		//
		///////
	} else {	// no hit to block
		// in the air
		lpObject->getTrigger()->setTrigger( TriggerType::ON_THE_GROUND, false );
	}

	if( lpObject->isAlive() ){ // ĂFrŎłΉLi㉺j͍̏sȂ

		bool bIsGravity = false;	// d͂󂯂\

		// reset trigger
		lpObject->getTrigger()->setTrigger( TriggerType::ON_THE_GROUND, false );
		lpObject->getTrigger()->setTrigger( TriggerType::HIT_TO_THE_CEILING, false );

		if( lpObject->getObjectAttribute( ObjectAttribute::HIT_TO_BLOCK ) ) {	// ubNɂIuWFNgł

			// vel
			HPLVector velocityAngle = lpObject->getVelocityAngle();

			// previous slope info
			int nSlopePreviousTileCoordX = lpObject->getSlopePreviousTileCoordX();
			int nSlopePreviousTileCoordY = lpObject->getSlopePreviousTileCoordY();

			// c`FbN
			stateY = HPLCollisionTool::adjustCollisionY(
				hitArea, velocityAngle,
				matrix, tileSize, 
				*blockMap,
				nSlopePreviousTileCoordX, nSlopePreviousTileCoordY);

			// X[vڐGL^
			lpObject->setSlopePreviousTileCoordX( nSlopePreviousTileCoordX );
			lpObject->setSlopePreviousTileCoordY( nSlopePreviousTileCoordY );

			// ڐG
			bool isTouch = blockMap->find( stateY.m_nTouchBlockID )->second->isEmpty() == false;

			// update trigger
			lpObject->getTrigger()->setTrigger(
				TriggerType::ON_THE_GROUND,
				isTouch && stateY.isHitY == HitDirectionY::BOTTOM);
			lpObject->getTrigger()->setTrigger(
				TriggerType::HIT_TO_THE_CEILING,
				isTouch && stateY.isHitY == HitDirectionY::TOP);

			if( isTouch )// && // ڐG
				//lpObject->getObjectAttribute(ObjectAttribute::HIT_TO_BLOCK) // HIT_TO_BLOCK==falsȅꍇɂ͂Ȃ
			//	this->m_bIsHitToBlock
//			)
			{
				vNewCenter.setY(stateY.m_nNewY + dy);	// TODO ʒu{ɂŗǂmF
				//hitArea.setY(stateY.m_nNewY);

				// ubNƐڐG
				bIsHitToBlock = true;

				if( lpObject->getObjectAttribute( ObjectAttribute::BOUND ) ) {
					// oEh^Cvł

					//if(stateY.isHitY == HitDirectionY::BOTTOM){
					//const float RATE = 0.7f;
					//vNewVelocityAngle.setY( - (float)(fabs(velocityAngle.getY()) * RATE));

					// oEhvelݒ
					vNewVelocityAngle.setY( 
						HPLActionMoveTool::getBoundedSpeed(
						velocityAngle.getY(), BOUNCE_DECELERATION_RATE_Y,
						stateY.isHitY ) );

					//}
					// resist
					if( lpObject->getObjectAttribute(ObjectAttribute::NO_RESIST) == false ) {
						// XɌ
						// CRɂ錸
						//const float RESIST_X_FOR_BOUND = 0.1f;
						vNewVelocityAngle.setX( HPLActionMoveTool::reduceSpeed(
							RESIST, velocityAngle.getX() ) );
					}

				} else {	// not bound
					// stop y axis of vel
					vNewVelocityAngle.setY(0);
					if(stateY.isHitY == HitDirectionY::BOTTOM){
						// ɂԂ

						// ڐG
						lpObject->getTrigger()->setTrigger( TriggerType::ON_THE_GROUND, true );
						
						if( false == lpObject->getObjectAttribute( ObjectAttribute::NO_RESIST ) ) {
							// 

							// x
							float fResistXRate = 1.f;

							if( lpObject->getObjectAttribute( ObjectAttribute::BOUND ) ){ // bound object
								// reduce resist
								const float RESIST_BOUND_X_RATE = 2.f;
								fResistXRate = RESIST_BOUND_X_RATE;
							}
							// 
							vNewVelocityAngle.setX(
								HPLActionMoveTool::reduceSpeed(
									RESIST * fResistXRate,
									vNewVelocityAngle.getX() ) );
						}

						/*if(this->getType() == EffectType::){
							const float EPS = 0.7f;
							velocityAngle.setY((float)(-fabs(velocityAngle.getY()) * EPS));
						}*/
					}else{
						// stop y axis of vel
						vNewVelocityAngle.setY(0);

						lpObject->getTrigger()->setTrigger( TriggerType::HIT_TO_THE_CEILING, true );
					}
				}

		/*		if(isHitToBlock()){
					setAlive(false);
				}*/

			}else{
				// d͂󂯂\
				bIsGravity = true;
			}
		} else {
			// ubNɂ͓ȂAd͎͂󂯂\
			bIsGravity = true;
		}

		if( bIsGravity ) {	// d͉
			if( lpObject->getObjectAttribute(ObjectAttribute::GRAVITY) ) {	// d͉L̃IuWFNg
				float fGravity = GRAVITY;	// d͉x
				float fMaxFallSpeed = FALL_MAX_SPEED;	// ő呬x

				if( lpObject->getObjectAttribute(ObjectAttribute::HALF_GRAVITY) ) {	// 
					fGravity *= 0.25f;
//					fMaxFallSpeed *= 0.5f;
				} else if( lpObject->getObjectAttribute(ObjectAttribute::DOUBLE_GRAVITY) ) {	// 2{
					fGravity *= 2.f;
					fMaxFallSpeed *= 2;
				}

				// vel += g
				vNewVelocityAngle.setY(vNewVelocityAngle.getY() + fGravity);

				if( vNewVelocityAngle.getY() > fMaxFallSpeed ) {	// reach to max fall speed
					vNewVelocityAngle.setY( fMaxFallSpeed );
				}
			}
		}
	}

	if( bIsHitToBlock && lpObject->getObjectAttribute(ObjectAttribute::HIT_BLOCK_TO_DIE ) ) {
		// ubNɂԂƎS

		int nObjectType = lpObject->getObjectType();
		switch( nObjectType ) {
		case ObjectType::SHOT:
			((GShot*)lpObject)->die( NULL );
			break;
		case ObjectType::EFFECT:
			((GEffect*)lpObject)->die( NULL );
			break;
		default:
			LOG_WARN( HPLString::chrToStr("Invalid object type[%d]", nObjectType).c_str());
		}/* else if( lpObject->getObjectType() == ObjectType::NEUTRAL ) {
			((GShot*)lpObject)->die( NULL );
		} else if( lpObject->getObjectType() == ObjectType::SHOT ) {
			((GShot*)lpObject)->die( NULL );
		} else */
	}
	// center += vel
	vNewCenter.add(vNewVelocityAngle);

	// update center
	lpObject->setCenter(vNewCenter);

	// update vel
	lpObject->setVelocityAngle(vNewVelocityAngle);

	// R`FbN
	HPLActionMoveTool::checkGroundAtTheFaceDirection( lpObject,
		this->getBlockManager(), tileSize);
}

void GStaticData::setStarAndPlace( int nStar, int nPlace ) {

	// Im
	if( nStar >= 0 ) {
		this->m_nSelectedStar = nStar;
		this->m_Save.setFlag( FlagType::STAR, this->m_nSelectedStar );
	}
	if( nPlace >= 0 ) {
		this->m_nSelectedPlace = nPlace;

		this->m_Save.setFlag( FlagType::PLACE, this->m_nSelectedPlace );
	}
	// Z[u
	this->m_Save.save();

}

int GStaticData::getMapIndexFromStarAndPlace( ) {
	// }bvǂݍ
	return this->m_Setting.maps.getMapIndexFromStarAndPlace(
		this->m_nSelectedStar, this->m_nSelectedPlace );
}



/** ^CgʂȂǂŋʂ̓
  */
void GStaticData::commonBackgroundProcess() {
	///////////////////////////// 
	{
		// draw stars
		this->m_ObjectManager.m_EffectManager.drawBack();// true, true );


		
	}

}
void GStaticData::commonForegroundProcess() {
	{
		HPLRect rGameRect = this->m_Setting.gameRange;
		HPLRect rMerginedRect = this->m_Setting.merginedGameRange;
		float fStartX = rGameRect.getRight() + (rMerginedRect.getRight() - rGameRect.getRight()) / 2;

		for( int i = 0; i < HPLMath::nextInt( CURE_STAR_NUM_MIN, CURE_STAR_NUM_MAX ); i ++ ) {
			// EgO
			HPLVector vCenter( fStartX, HPLMath::nextFloat(rGameRect.getY(), rGameRect.getBottom() ) );
			// 
			this->createEffect( Sequence::Effect::CURE_STAR,
				vCenter, vCenter, -1, -1 );
		}

		this->m_ObjectManager.m_EffectManager.process( true, true );
	}
}

/** vC[̏Ԃɂăv`\ACRSequenceID擾
  * @param[in]	lpPlayer	vC[
  */
int GStaticData::getFaceSeqIDFromState( int st, int nWearState ) {
	// 11őΉĂ^Cv
//	static int FACE_SEQUENCE_ID_MAP[][3] = {
//		{ StateType::BORING, Sequence::Common::TALK_EVIGEN_DAMEGEN, -1 },
//		{ StateType::BORING_CROUCH, Sequence::Common::TALK_EVIGEN_DAMEGEN, -1 },
//	};
	
	if( st == StateType::BORING || st == StateType::BORING_CROUCH ) {
		// ̂قق
		return Sequence::Common::TALK_EVIGEN_DAMEGEN;

	} else if( HPLState::isDamageState(st) ) {
		// _[W

		if( nWearState == WearStateType::NAKED ) {
			return Sequence::Common::TALK_EVIGEN_NAKED_SHAMED;
		} else {
			return Sequence::Common::TALK_EVIGEN_SHAMED;
		}
	} else {
		// ʏ
		if( nWearState == WearStateType::NAKED ) {
			// 㵒p
			return Sequence::Common::TALK_EVIGEN_NAKED_SHAMED;
		} else {
			return Sequence::Common::TALK_EVIGEN_NORMAL;
		}
	}
}


void GStaticData::drawDescription( const char* lpczDescription, bool bIsDefault, int nTop ) {
	// bt[\
	int nTextTop = this->drawTalkFrame(bIsDefault, nTop);

	int nFontWidth = 6;
	
	nTextTop += this->m_Setting.windowHeight * 3 / 8 - nFontWidth / 2;
	int nLeft = 50;
	// \
	DrawString(
		//this->m_Setting.windowWidth / 2 - strlen( lpczDescription ) * nFontWidth ,
		nLeft ,
		nTextTop ,
		lpczDescription,
		GetColor( 255,255,255) );
}

int GStaticData::getButtonBright( int nCurrentStepCounter, int nMaxStepCounter ) {
	if( nCurrentStepCounter < nMaxStepCounter / 2 ) {
		// do nothing
	} else {
		// 㔼
		// ->]
		nCurrentStepCounter = nMaxStepCounter - nCurrentStepCounter;
	}
	float fCurrentRate = static_cast<float>(nCurrentStepCounter) /
		nMaxStepCounter;
	float fBrightRate = 0.5f;
	return static_cast<int>(
		155.f * ( fCurrentRate * fBrightRate + (1-fBrightRate))  ) + 100;
}

float GStaticData::getLimitUnder() {
	return (EVIGEN_LIMIT_AREA_UNDER_BOTTOM_MERGINE + this->m_Setting.tileSize * 
		this->getBlockManager()->getBlockMatrix()->getRowSize() );
}

int GStaticData::getStarSequenceID(int nStarType) {
	int nStarSeqID = Sequence::Common::SELECT_EVIL_STAR + nStarType;
	if( nStarType == StageType::REST_STAR ) {
		nStarSeqID = Sequence::Common::SELECT_REST_STAR;
	}
	return nStarSeqID;
}


/** ACe̎ނ畐̃^Cv𓾂
  * @retval	̎
  * @retval	-1				ȊÕACełꍇ
  *
  */
int GStaticData::getWeaponTypeFromItemType( int nItemType ) {
	return this->m_ObjectManager.m_WeaponManager.getWeaponTypeFromItemType( nItemType);

/*#ifdef _DEBUG
	HPLError::assertIt(m_mapItem2WeaponType.find(nItemType) != m_mapItem2WeaponType.end(),
		"%s:invalid item type:%d", __FUNCTION__, nItemType);
#endif
	return m_mapItem2WeaponType[nItemType];*/
}
int GStaticData::getItemTypeFromWeaponType( int nWeaponType ) {
	return this->m_ObjectManager.m_WeaponManager.getItemTypeFromWeaponType( nWeaponType );
/*	for( map<int,int>::iterator it = this->m_mapItem2WeaponType.begin();
		it != this->m_mapItem2WeaponType.end(); it ++ )
	{
		if( it->second == nWeaponType ) {
			return it->first;
		}
	}
#ifdef _DEBUG
	HPLError::assertIt(false,
		"%s:invalid weapon type:%d", __FUNCTION__, nWeaponType);
#endif
	return -1;*/
}

int GStaticData::getItemClassType( int nItemType ) {
	// class type
/*	const int CLASS_TYPE[] = {
		ItemClassType::USE_ITEM,	//! kokemomo
		ItemClassType::USE_ITEM,	//! meat
		ItemClassType::USE_ITEM,	//! evil drink
		ItemClassType::USE_ITEM,	//! battery
		ItemClassType::WEAPON,		//! gatling

		ItemClassType::STOCK_ITEM,	//! kapibara
		ItemClassType::STOCK_ITEM,	//! alpaca
		ItemClassType::STOCK_ITEM,	//! delta
		ItemClassType::STOCK_ITEM,	//! lamda
		ItemClassType::STOCK_ITEM,	//! kappa

		ItemClassType::STOCK_ITEM,	//! coin
		ItemClassType::STOCK_ITEM,	//! blue coin
		ItemClassType::WEAPON,		//! pine apple
		ItemClassType::WEAPON,		//! missile launcher
		ItemClassType::WEAPON,		//! shot gun

		ItemClassType::WEAPON,		//! rail gun
		ItemClassType::WEAPON,		//! frame thrower
		ItemClassType::USE_ITEM,	//! 
		ItemClassType::USE_ITEM,	//! 
		ItemClassType::WEAPON,		//! chainsaw

		ItemClassType::WEAPON,		//! punch
		ItemClassType::WEAPON,		//! drill
		ItemClassType::WEAPON,		//! 
		ItemClassType::WEAPON,		//! 
		ItemClassType::WEAPON,		//! sniper

		ItemClassType::WEAPON,		//! 
		ItemClassType::WEAPON,		//! 
		ItemClassType::WEAPON,		//! 
		ItemClassType::WEAPON,		//! 
		ItemClassType::WEAPON,		//! mayo

		ItemClassType::WEAPON,		//! laser
		ItemClassType::STOCK_ITEM,	//! missile
		ItemClassType::WEAPON,		//! handgun
		ItemClassType::WEAPON,		//! cryolator
		ItemClassType::WEAPON,		//! pile

		ItemClassType::WEAPON,		// nikita
		ItemClassType::USE_ITEM,	// ebi
		ItemClassType::USE_ITEM,	// aji
		ItemClassType::USE_ITEM,	// steak
		ItemClassType::WEAPON,		// Queen

		ItemClassType::WEAPON,		// seed
		ItemClassType::USE_ITEM,	// boss ability
		ItemClassType::USE_ITEM,	// dummy
		ItemClassType::WEAPON,		// spread missile launcher

	};
	int nTypeNum = SIZEOF( CLASS_TYPE );
	int nMax = Sequence::Item::MAX_NUM;
	HPLError::assertIt( nTypeNum == nMax,
		"%s:item setting is not match:setting=%d, item types=%d", __FUNCTION__,nTypeNum, nMax
		);
	*/
	GItemSetting* lpSetting = this->m_ObjectManager.m_ItemManager.getSetting(
		nItemType);
#ifdef _DEBUG
	ASSERT_IT_NULL( lpSetting );
#endif
	int nClassType = lpSetting->m_nClassType;
	return nClassType;//CLASS_TYPE[nItemType];
}

bool GStaticData::hitRKey() {
	// RL[

	GPlayer* lpPlayer = &this->m_ObjectManager.m_Player;

	if( lpPlayer->getRemainNum() > 0 ) {
		// c@


		// fadeout X^[g
		this->startFadeOutDX( SceneType::GAME, -1, AfterFadeOutDXProcessType::RESPAWN );

		return true;
	} else {
		// c@[
		// 
		this->startFadeOutDX( SceneType::GAME_OVER, -1 );

		return false;
	}
}


/** ݂̃}bvǂȃ^Cv
  */
int GStaticData::getMapType() {
	
	switch( this->getMapIndex() ) {
	case SpecialMapIndex::FOREST_FORCE_SCROLL:
		return MapType::FORCE_SCROLL_X;
	case SpecialMapIndex::WIND_HIGH_SPEED:
		return MapType::FORCE_SCROLL_X_FASTER;
		
	case SpecialMapIndex::SLIME_QUEEN:
	case SpecialMapIndex::ALRAUNE:
	case SpecialMapIndex::MERAKO:
	case SpecialMapIndex::WIND_BOSS_1:
	case SpecialMapIndex::WIND_BOSS_2:
		return MapType::BOSS;
		
	case SpecialMapIndex::BEE:
	case SpecialMapIndex::FOREST_ARYONA:
	case SpecialMapIndex::WIND_ARYONA:
	case SpecialMapIndex::WIND_FUU_RAI:
		return MapType::MID_BOSS;
	}
	return MapType::NORMAL;
}

/** HP\
  */
void GStaticData::drawHP( HPLShootingObject* lpObject ) {
	// XN[
	HPLScroller* scroller = this->getScrollManager()->getScroller(0);

	float left = lpObject->getArea().getX() + scroller->getVirtualRect().getX();
	float top = lpObject->getArea().getY() + scroller->getVirtualRect().getY();

	HPLVector vCen = lpObject->getCenter();
	float fCenterX = vCen.getX() + scroller->getVirtualRect().getX();
	float fCenterY = vCen.getY() + scroller->getVirtualRect().getY();
	float fZoom = scroller->m_fZoomRate;

	// frame
	HPLSequence* lpHPFrame = this->getSequence(
		SpriteType::COMMON, Sequence::Common::ENEMY_HP_FRAME );

	HPLRect rHitArea = lpObject->getHitArea();
	
	HPLRect rHPFrameSrcRect = lpHPFrame->getImageInformation(0)->srcRect;

	HPLRect rArea = lpObject->getArea();

	int x = (int)(fCenterX - rHPFrameSrcRect.getWidth() / 2);
	int y = (int)(( rArea.getY() + scroller->getVirtualRect().getY() ) * fZoom -
		ENEMY_HP_METER_DELTA_Y);
	
	this->m_Basic.m_Graphics.draw(
		x, y, lpHPFrame );

	// meter
	float fHPRate = lpObject->getHP() / lpObject->getHPMax();
	
	// ꕔ\
	DrawRectGraph(
		static_cast<int>( x ),static_cast<int>( y ),
		0, 0,
		static_cast<int>( rHPFrameSrcRect.getWidth() * fHPRate ),
		static_cast<int>( rHPFrameSrcRect.getHeight() ),
		this->getImageHandle(
			SpriteType::COMMON, Sequence::Common::ENEMY_HP ),
			TRUE, FALSE);
}

int GStaticData::getTankSequenceID( HPLShootingObject* obj ) {
	return 0;
}

int GStaticData::getVTOLSequenceID( HPLShootingObject* obj ) {
	int id = Sequence::VTOLnya::STAND;

	int st = obj->getState()->getState();
	switch( st ) {
	case StateType::WALK:
		id = Sequence::VTOLnya::WALK;
		break;
	case StateType::ATTACK:
		id = Sequence::VTOLnya::ATTACK_GATLING;
	case StateType::ATTACK_2:
		id = Sequence::VTOLnya::ATTACK_MISSILE;
	}
	return id;
}

/** XN[
  * wiłgp
  * SHAKE͖
  */
void GStaticData::adjustScroll( HPLScroller* scroller ) {
	
	GPlayer* lpPlayer = &this->m_ObjectManager.m_Player;

	int nMapType = this->getMapType();
	switch( nMapType ) {
	case MapType::FORCE_SCROLL_X_FASTER:
	case MapType::FORCE_SCROLL_X:
		{
			const float FORCE_SCROLL_SPEED_DEFAULT = 3;
			const float FORCE_SCROLL_SPEED_FASTER = 5;
			float fScrollSpeed = FORCE_SCROLL_SPEED_DEFAULT;

			switch( nMapType) {
			case MapType::FORCE_SCROLL_X_FASTER:
				fScrollSpeed = FORCE_SCROLL_SPEED_FASTER;
				break;
			case MapType::FORCE_SCROLL_X:
				break;
			default:
				//HPLError::assertIt(false);
				ASSERT_IT_TYPE( nMapType, "MapType");
			}

			HPLVector vVelocity(1,0);
			vVelocity.multiple(fScrollSpeed );

			scroller->m_CurrentVirtualCenter.add(vVelocity );

			// c͂킹
			// x͋ɔ
			HPLVector v(0,lpPlayer->getCenter().getY() - scroller->m_CurrentVirtualCenter.getY());
			float fSpeed = v.getLength() *		SCROLL_LENGTH_TO_SPEED_RATE;
			
			// 1ɂ
			v.normalize();

			// x|
			v.multiple( fSpeed );

			scroller->m_CurrentVirtualCenter.add( v );
		}
		break;
	default:
		{
			HPLVector vGoalPoint( lpPlayer->getCenter() );

			// ㉺͊Sǐɂ
			scroller->m_CurrentVirtualCenter.setY( vGoalPoint.getY() );

			const int SCROLL_FORWARD_DELTA = 150;
			// Oɂ
			vGoalPoint.setX( vGoalPoint.getX() + lpPlayer->getFaceDirection() * SCROLL_FORWARD_DELTA );

			// ݂̒SWS[ւ̋
			HPLVector vDirection(
				vGoalPoint.getX() - scroller->m_CurrentVirtualCenter.getX(),
				vGoalPoint.getY() - scroller->m_CurrentVirtualCenter.getY() );

			// E𒴂狗l߂邩
			bool bIsScrollLimit = true;	// CxgȂǂŗꂽƂփXN[ꍇ͂͂

			if( bIsScrollLimit && vDirection.getLength() > SCROLL_LENGTH_LIMIT ) {
				// E𒴂

				// K苗ɂ
				// 1ɂ
				vDirection.normalize();

				// K苗ɂ
				vDirection.multiple( - SCROLL_LENGTH_LIMIT );

				// ʒuڕWK苗ꂽꏊɂ
				scroller->m_CurrentVirtualCenter = vGoalPoint;
				scroller->m_CurrentVirtualCenter.add( vDirection );

			} else {

				// x͋ɔ
				float fSpeed = vDirection.getLength() * SCROLL_LENGTH_TO_SPEED_RATE;
				
				// 1ɂ
				vDirection.normalize();

				// x|
				vDirection.multiple( fSpeed );

				scroller->m_CurrentVirtualCenter.add( vDirection );
			}
		}
	}
	// ɂ
	//setVectorInt(scroller->m_CurrentVirtualCenter );

	this->adjustScreen( scroller );
}

void GStaticData::adjustScreen( HPLScroller* scroller ) {
	// SƂȂXN[W
	HPLVector scrPoint(
		this->getSetting()->windowWidth / 2.f / scroller->m_fZoomRate,
		this->getSetting()->windowHeight / 2.f / scroller->m_fZoomRate);
	// ɂ
	//setVectorInt(scrPoint );

	// ʔ͈͂𒴂邱Ƃ͖i[͍킹j
	bool isAdjust = true;
	// tag
	scroller->matchPoint(
		scrPoint, scroller->m_CurrentVirtualCenter,
		isAdjust);
}

/** уJEgi߂
  * nStep=0łAOBɂȂĂȂBƂ
  */
bool GStaticData::stepAndCheckAchievement( int nAchievementType, int nStep, bool bIsSave ) {

	GAchievementItem* lpItem = getStaticData()->m_AchievementSave.getAchievementItem(
		nAchievementType );
	if( lpItem->isFinish() == false && lpItem->setCounter( lpItem->getCounter() + nStep ) ) {
		// B

		// Achievement
		GAchievement* lpAchievement = getStaticData()->m_ObjectManager.m_AchievementManager.create(
			nAchievementType );

		// ۑ
		if( bIsSave ) {
			getStaticData()->m_AchievementSave.save();
		}
		return true;
	}

	return false;
}


int GStaticData::getWeaponTypeBySuperPower(int nSuperPowerType ) {
	switch( nSuperPowerType ) {
	case SuperPowerType::QUEEN_AUTHORITY:
		return WeaponType::SLIME_GUN;
	case SuperPowerType::SEED:
		return WeaponType::SEED;
	case SuperPowerType::WIND_WIZARD:
		return WeaponType::WIND_WIZARD;
	}

	// 肦ȂcH
	//HPLError::assertIt( false );
	ASSERT_IT_TYPE( nSuperPowerType, "Super Power Type");
	return -1;
}

//#define X_FIRST

/** IuWFNg̃IuWFNgɏƂ̏
  * @param[in,out]	lpRider	鑤BPlayerȂ
  * @param[in,out]	lpRided	鑤BGNeutralȂ
  * @param[out]		stateX,stateY ʁBRider̔ɂȂB
  */
bool GStaticData::rideOnObject( HPLShootingObject* lpRider, HPLShootingObject* lpRided,
	CollisionState& stateX, CollisionState& stateY)
{
#ifdef X_FIRST
	CollisionState stateX = getStaticData()->checkCollisionObjectX(lpPlayer, lpNeutral );
	CollisionState stateY;
	if( stateX.isHitX == HitDirectionX::NONE ) {
		stateY = getStaticData()->checkCollisionObjectY(lpPlayer, lpNeutral );
	}
#else
	stateY = getStaticData()->checkCollisionObjectY(lpRider, lpRided,
		lpRided->getObjectType() == ObjectType::NEUTRAL ? ((GNeutral*)lpRided)->m_nMountType : MountType::BLOCK);

	bool bIsSkipX = false;
	if( lpRided->getObjectType() == ObjectType::NEUTRAL &&
		((GNeutral*)lpRided)->m_nMountType != MountType::BLOCK )
	{
		// neutralł
		// block^CvȊO(none or `FbNȂ^Cv)ł
		bIsSkipX = true;
	}
	if( !bIsSkipX ) {
		// `FbN
		//if( stateY.isHitY == HitDirectionY::NONE ) {
			stateX = getStaticData()->checkCollisionObjectX(lpRider, lpRided );
		//}
	}
#endif

	bool bIsRide = false;
	if( stateY.isHitY == HitDirectionY::BOTTOM ) {
		// vC[ɏ

		bIsRide = true;

		// ꂽtOON
		
		if( lpRided->getObjectType() == ObjectType::NEUTRAL ) {
			((GNeutral*)lpRided)->m_bIsPlayerMountOn = true;
		}

		// tOON
		if( lpRider->getObjectType() == ObjectType::PLAYER ) {
			((GPlayer*)lpRider)->m_bIsLandOnObject = true;
		}
		lpRider->getTrigger()->setTrigger( TriggerType::ON_THE_GROUND, true );
		//lpPlayer->changeState( StateType::STAND );
	}
	return bIsRide;
}

/** vC[̌ݑI𒆂̃V[gJbgo
  * @retval	-1	Ȃ
  *
int GStaticData::getShortCutIndex( int nWeaponType ) {
	// 
	// V[gJbg̐
	int nMax = this->m_OptionData.getOptionFlag(
		OptionFlagType::MAX_SHORTCUT_NUM );

	// ACe^Cv擾

	int nItemType = getItemTypeFromWeaponType(nWeaponType );

	for( int i = 0; i < nMax; i ++ ) 
	{
		if( this->m_Save.getShortCut( i ) == nItemType ) 
		{	// IĂ
			// IĂV[gJbgCfbNXԋp
			return i;
		}
	}

	// Ȃ
	for( int i = 0; i < nMax; i ++ ) {
		if( this->m_Save.getShortCut(i) < 0 ) {
			// 
			// ->Ɍ߂I
			return i;
		}
	}
	// Ȃ-1
	return -1;
}

*/
bool GStaticData::checkPoint( int nNextMapIndex ) {

	bool bHasChanged = false;

	int nStar = 0;
	int nPlace = 0;
	getStaticData()->m_Setting.maps.getStarAndPlaceFromMapIndex( nNextMapIndex, nStar, nPlace );
	getStaticData()->updateStarAndPlace( nNextMapIndex );

	// ̃}bvLɂ
	if( getStaticData()->m_Save.getPlaceState( nStar, nPlace ) == PlaceType::SEALED ) {
		getStaticData()->m_Save.setPlaceState( nStar, nPlace, PlaceType::REVEALED );
//		bIsSave = true;
		bHasChanged = true;
	}
	// tF[hAEgJn
	getStaticData()->startFadeOutDX( SceneType::SELECT_PLACE, nNextMapIndex );

	return bHasChanged;
}

/** 1Ԗڂ`FbNāAo^Ăŏ̃pbg̕I
  * o^ĂȂ΃p`ɂȂ
  *
int GStaticData::setFirstShortCutIndex(GPlayer* lpPlayer) {

	// ŏ̃V[gJbgTĐݒ肷
	int nIndex = this->m_Save.getFirstShortCutIndex();

	if( nIndex < 0 ) {
		// o^ĂȂ
		// p`o^
		this->m_Save.setShortCut( 0, Sequence::Item::PUNCH );//WeaponType::PUNCH );
		// indexŏɂ
		nIndex = 0;
	}
	this->m_nShortCutIndex = nIndex;

	// set weapon
	lpPlayer->setWeaponType( this->getWeaponTypeFromItemType( this->m_Save.getShortCut( nIndex ) ) );
	
	return nIndex;
}
*/

/** 킩ǂmFB
  * @retval	-1		ȊO
  * @retval	0ȏ	̎
  */
int GStaticData::checkItemIsWeapon( int nItemType ) {
	return this->m_ObjectManager.m_WeaponManager.checkItemIsWeapon( nItemType );
/*	std::map<int,int>::iterator it = this->m_mapItem2WeaponType.find( nItemType );
	if( it == this->m_mapItem2WeaponType.end() ) {
		return -1;
	}
	return it->second;
*/
}


/** w͈͂őIJ[\\
  */
void GStaticData::drawSelectFrame( int nLeft, int nTop, int nWidth, int nHeight ) {
	HPLSequence* lpSeq = this->getSequence( SpriteType::COMMON, Sequence::Common::SELECT_FRAME );
	HPLRect rSrcRect = lpSeq->getImageInformation()->srcRect;

	// left-top
	this->m_Basic.m_Graphics.draw( nLeft, nTop, lpSeq );
	// right-top
	this->m_Basic.m_Graphics.draw( nLeft + nWidth - rSrcRect.getWidth()/2, nTop + rSrcRect.getHeight() / 2.f,
		lpSeq, FaceDirection::RIGHT, 0, 1 );
	// left-bottom
	this->m_Basic.m_Graphics.draw(
		nLeft + rSrcRect.getWidth()/2, nTop + nHeight - rSrcRect.getHeight() / 2.f,
		lpSeq, FaceDirection::LEFT, 270, 1 );
	// right-bottom
	this->m_Basic.m_Graphics.draw(
		nLeft + nWidth - rSrcRect.getWidth()/2, nTop + nHeight - rSrcRect.getHeight() / 2.f,
		lpSeq, FaceDirection::RIGHT, 90, 1 );
}

/** mapɃACeƕ̑΂o^
  *
void GStaticData::registItemAndWeaponSet( int nItemType, int nWeaponType ) {
/*	HPLError::assertIt(
		this->m_mapItem2WeaponType.find( nItemType ) == this->m_mapItem2WeaponType.end(),
		"%s:%s(%d):Item[%d] already exists(registered weapon:%s,new weapon:%s)",
		__FILE__, __FUNCTION__, __LINE__, nItemType,
		this->m_ObjectManager.m_WeaponManager.getWeaponSetting(this->m_mapItem2WeaponType[nItemType])->m_strName.c_str(),
		this->m_ObjectManager.m_WeaponManager.getWeaponSetting( nWeaponType)->m_strName.c_str());*
	this->m_mapItem2WeaponType[nItemType] = nWeaponType;
}
*/

/**
  * @param[in]	lpPlayer		L[vC[f[^
  * @retval		ꂽ
  */
int GStaticData::getScrambleKeyCount( GPlayer* lpPlayer, bool bIsOnlyOne) {
	int nCount = 0;
	for( int i = 0; i < KeyMap::MAX_KEY; i ++ ) {
		if( lpPlayer->isPressedNow( i ) ) {
			nCount ++;
			if( bIsOnlyOne ) {
				return nCount;
			}
		}
	}
	return nCount;
}

