#include "GTitleScene.h"

#include "GStaticData.h"
#include <hpllib/general/HPLString.h>
#include <hpllib/math/HPLMath.h>

#include <shlwapi.h>

using namespace GEnum;

static const int SAVE_NUM = 3;
namespace TitleType {
	enum {
		MAIN,
		SELECT_SAVE,
		DELETE_SAVE,	//! 폜Z[uI폜
		CONFIRM,		//! 폜ėǂ̊mF
	};
}
namespace ConfirmType {
	enum {
		OK,
		CANCEL,
		MAX_NUM
	};
}

//#define DRAW_LIGHT

class ImageData {
public:
	int x;
	int y;
	HPLSequence* seq;
	ImageData(){
		x = 0;
		y = 0;
		seq = NULL;
	}
};

namespace ImageIndex {
	enum {
		MINIGUN,
		TITLE,
		TWO,
		ESCAPE,
		DR_TANYA,
		MAX_NUM
	};
}

namespace ButtonType {
	enum {
		START,
		OPTION,
		ACHIEVEMENT,
		LIBRARY,
	};
}

typedef struct ButtonInfo_ {
	int m_nNextScene;
	int m_nSequenceID;
}ButtonInfo;
static ButtonInfo BUTTON_INFOS[] = {
	{SceneType::SELECT_STAGE,	Sequence::Common::TITLE_BUTTON_START},	// start	(s̓Z[uԂɂĕω)
	{SceneType::OPTION,			Sequence::Common::TITLE_BUTTON_OPTION},	// option
	{SceneType::ACHIEVEMENT,	Sequence::Common::TITLE_BUTTON_ACHIEVEMENT},	// achievement
	{SceneType::LIBRARY,		Sequence::Common::TITLE_BUTTON_LIBRARY}	// library
};

static ImageData IMAGES[ImageIndex::MAX_NUM];

static int getButtonNum() {
	return sizeof(BUTTON_INFOS)/sizeof(BUTTON_INFOS[0]);
}
GTitleScene::GTitleScene() {
	int nWindowWidth = getStaticData()->m_Setting.windowWidth;
	int nWindowHeight = getStaticData()->m_Setting.windowHeight;

	// "Evigen"
	IMAGES[ImageIndex::TITLE].seq = getStaticData()->m_Basic.m_Graphics.getSequence(
		SpriteType::COMMON, Sequence::Common::TITLE );
	IMAGES[ImageIndex::TITLE].x = static_cast<int>( nWindowWidth /2 - IMAGES[ImageIndex::TITLE].seq->getImageInformation(0)->srcRect.getWidth() / 2 - 50 );
	IMAGES[ImageIndex::TITLE].y = 100;

	// "2"
	IMAGES[ImageIndex::TWO].seq = getStaticData()->m_Basic.m_Graphics.getSequence(
		SpriteType::COMMON, Sequence::Common::TITLE_2 );
	IMAGES[ImageIndex::TWO].x = 
		static_cast<int>( IMAGES[ImageIndex::TITLE].x + IMAGES[ImageIndex::TITLE].seq->getImageInformation(0)->srcRect.getWidth()  -
		IMAGES[ImageIndex::TWO].seq->getImageInformation(0)->srcRect.getWidth() + 100 );
	IMAGES[ImageIndex::TWO].y = IMAGES[ImageIndex::TITLE].y;

	// "Escape from ... or Dr Tanya"
	IMAGES[ImageIndex::ESCAPE].seq = getStaticData()->m_Basic.m_Graphics.getSequence(
		SpriteType::COMMON, Sequence::Common::TITLE_ESCAPE );
	IMAGES[ImageIndex::ESCAPE].x =
		static_cast<int>( IMAGES[ImageIndex::TITLE].x + IMAGES[ImageIndex::TITLE].seq->getImageInformation(0)->srcRect.getWidth()  -
		IMAGES[ImageIndex::ESCAPE].seq->getImageInformation(0)->srcRect.getWidth()+50 );
	IMAGES[ImageIndex::ESCAPE].y = 
		static_cast<int>( IMAGES[ImageIndex::TITLE].y + IMAGES[ImageIndex::TITLE].seq->getImageInformation(0)->srcRect.getHeight() -
		IMAGES[ImageIndex::ESCAPE].seq->getImageInformation(0)->srcRect.getHeight() );

	IMAGES[ImageIndex::DR_TANYA].seq = getStaticData()->m_Basic.m_Graphics.getSequence(
		SpriteType::COMMON, Sequence::Common::TITLE_DR_TANYA );
	IMAGES[ImageIndex::DR_TANYA].x = 
		static_cast<int>( IMAGES[ImageIndex::TITLE].x + IMAGES[ImageIndex::TITLE].seq->getImageInformation(0)->srcRect.getWidth()  -
		IMAGES[ImageIndex::DR_TANYA].seq->getImageInformation(0)->srcRect.getWidth()+50 );
	IMAGES[ImageIndex::DR_TANYA].y =
		static_cast<int>( IMAGES[ImageIndex::TITLE].y + IMAGES[ImageIndex::TITLE].seq->getImageInformation(0)->srcRect.getHeight() -
		IMAGES[ImageIndex::DR_TANYA].seq->getImageInformation(0)->srcRect.getHeight() );

	// "Minigun
	IMAGES[ImageIndex::MINIGUN].seq = getStaticData()->m_Basic.m_Graphics.getSequence(
		SpriteType::COMMON, Sequence::Common::TITLE_MINIGUN );
	IMAGES[ImageIndex::MINIGUN].x =
		static_cast<int>( IMAGES[ImageIndex::TITLE].x + IMAGES[ImageIndex::TITLE].seq->getImageInformation(0)->srcRect.getWidth()  -
		IMAGES[ImageIndex::MINIGUN].seq->getImageInformation(0)->srcRect.getWidth() );
	IMAGES[ImageIndex::MINIGUN].y = IMAGES[ImageIndex::TITLE].y - 50;

	this->m_nIndex = 0;
	this->m_nSaveIndex = 0;
	this->m_nTitleType = TitleType::MAIN;
	this->m_nConfirmIndex = 1;
}
GTitleScene::~GTitleScene() {
}
void GTitleScene::init() {
	getStaticData()->m_MixerManager.playMusic(Music::TITLE, true );
//	m_nUseSubTitleID = HPLMath::nextInt(0,2);

	// wi
	SetBackgroundColor(0,0,0);


	// XN[
	HPLScroller* lpScroll = getStaticData()->getScrollManager()->getScroller(0);
	lpScroll->setVirtualRect( getStaticData()->m_Setting.gameRange );

	// F2Ŗ߂ĂpɃZbg
	GPlayer* lpPlayer = &getStaticData()->m_ObjectManager.m_Player;
	lpPlayer->setHP( lpPlayer->getHPMax() );
	lpPlayer->getSequence()->resetCounter();
	lpPlayer->m_nWearState = WearStateType::NORMAL;

	this->m_nTitleType = TitleType::MAIN;
}
void GTitleScene::quit() {
}

static void drawLight( int nLightX ) {
	static HPLSequence* lpLightSeq = getStaticData()->getSequence(SpriteType::COMMON, 
		Sequence::Common::TITLE_LIGHT);
	// CgAbv
	getStaticData()->m_Basic.m_Graphics.draw( nLightX, IMAGES[ImageIndex::TITLE].y,
		lpLightSeq );
}

void GTitleScene::process() {
	static int nLightX = 0;

	getStaticData()->commonBackgroundProcess();

	// ^Cg
	{
		for( int i = 0; i < sizeof(IMAGES)/sizeof(IMAGES[ImageIndex::TITLE]); i ++ ) {
			/*if( i == ImageIndex::ESCAPE && m_nUseSubTitleID == 0 ) {
				continue;
			}*/
			if( i == ImageIndex::DR_TANYA ){//&& m_nUseSubTitleID == 1 ) {
				continue;
			}

			if( i == ImageIndex::TITLE ) {
			}
			getStaticData()->m_Basic.m_Graphics.draw( IMAGES[i].x, IMAGES[i].y,
				IMAGES[i].seq );

			if( i == ImageIndex::TWO ) {
#ifdef DRAW_LIGHT
				SetDrawBlendMode( DX_BLENDMODE_ADD, 100  );
				drawLight(nLightX);

				SetDrawBlendMode( DX_BLENDMODE_NOBLEND, 255  );
#endif

			}

			IMAGES[i].seq->step();
		}
	}


	//////////////////////////////////////////////// {^̑
	switch( this->m_nTitleType ) {
	case TitleType::MAIN:
		this->procMain();
		break;
	case TitleType::SELECT_SAVE:
		this->procSelectSave();
		break;
	case TitleType::DELETE_SAVE:
		this->procDeleteSave();
		break;
	case TitleType::CONFIRM:
		this->procConfirm();
		break;
	default:
		ASSERT_IT_TYPE( m_nTitleType, "Title type");
//		HPLError::assertIt( false );
	}

	getStaticData()->commonForegroundProcess();

#ifdef DRAW_LIGHT
	const int LIGHT_SPEED = 10;
	int nDeltaX = 0;
	if( getStaticData()->getInputManager()->isPressed( KeyMap::LEFT ) ) {
		nDeltaX = -1;
	}
	if( getStaticData()->getInputManager()->isPressed( KeyMap::RIGHT ) ) {
		nDeltaX = 1;
	}
	if( nDeltaX != 0 ) {
		nLightX += nDeltaX * LIGHT_SPEED;
	}
#endif

	if( getStaticData()->isFadingOutDX() == false ) {
		// tF[hAEg͑łȂ
		if( getStaticData()->getInputManager()->isPressedNow( KeyMap::UP ) ||
			getStaticData()->getInputManager()->isPressedNow( KeyMap::DOWN ))
		{
			// J[\
			
			// TODO sound
			switch( this->m_nTitleType ) {
			case TitleType::MAIN:
				this->procMainOnArrow();
				break;
			case TitleType::SELECT_SAVE:
				this->procSelectSaveOnArrow();
				break;
			case TitleType::DELETE_SAVE:
				this->procDeleteSaveOnArrow();
				break;
			case TitleType::CONFIRM:
				this->procConfirmOnArrow();
				break;
			}
		} else if( getStaticData()->getInputManager()->isPressedNow( KeyMap::A ) ) {
			// A(ZL[)

			bool bIsPlaySound = false;

			switch( this->m_nTitleType ) {
			case TitleType::MAIN:
				bIsPlaySound = this->procMainOnA();
				break;
			case TitleType::SELECT_SAVE:
				bIsPlaySound = this->procSelectSaveOnA();
				break;
			case TitleType::DELETE_SAVE:
				bIsPlaySound = this->procDeleteSaveOnA();
				break;
			case TitleType::CONFIRM:
				bIsPlaySound = this->procConfirmOnA();
				break;
			}
			// sound
			if( bIsPlaySound ) {
				getStaticData()->m_MixerManager.play( 
					SE::KETTEI,
					false );
			}
		} else if( getStaticData()->getInputManager()->isPressedNow( KeyMap::B ) ) {
			// B(XL[)

			bool bIsPlayCancelSound = false;

			switch( this->m_nTitleType ) {
			case TitleType::MAIN:
				// do nothing
				break;
			case TitleType::SELECT_SAVE:
				bIsPlayCancelSound = true;
				this->procSelectSaveOnB();
				break;
			case TitleType::DELETE_SAVE:
				bIsPlayCancelSound = true;
				this->procDeleteSaveOnB();
				break;
			case TitleType::CONFIRM:
				bIsPlayCancelSound = true;
				this->procConfirmOnB();
				break;
			}
			if( bIsPlayCancelSound ) {
				// sound
				getStaticData()->m_MixerManager.play( 
					SE::CANCEL,
					false );

			}
/*		} else if( CheckHitKey( KEY_INPUT_I ) ){
			// IL[
			// CxgeXgV[ցB
			getStaticData()->startFadeOutDX( SceneType::EVENT_CHECK, -1 );*/
		}
	}

}

static const int BUTTON_INTERVAL_Y = 50;
static const int BUTTON_TOP = 10;

void GTitleScene::procMain() {
	for( int i = 0; i < getButtonNum(); i ++ ) {
		HPLSequence* lpButtonStartSeq = getStaticData()->getSequence( SpriteType::COMMON,
			BUTTON_INFOS[ i ].m_nSequenceID );
		//getStaticData()->getSequence(SpriteType::COMMON, 
		//	Sequence::Common::TITLE_BUTTON_START);

		bool bIsSelectedButton = this->m_nIndex == i;
		int BUTTON_START_X = 
			(int)( getStaticData()->m_Setting.windowWidth / 2 - 
			lpButtonStartSeq->getImageInformation(0)->srcRect.getWidth() / 2 );

		int nBright = DARK_BUTTON_BRIGHT;
		if( bIsSelectedButton ) {
			nBright = getStaticData()->getButtonBright(
				lpButtonStartSeq->getStepCounter(),
				lpButtonStartSeq->getImageInformation(0)->waitFrame );


		}
		SetDrawBright(nBright,nBright,nBright);

		// {^\
		getStaticData()->m_Basic.m_Graphics.draw( 
			static_cast<int>( BUTTON_START_X ),
			static_cast<int>( IMAGES[ImageIndex::TITLE].y +
				IMAGES[ImageIndex::TITLE].seq->getImageInformation(0)->srcRect.getHeight() +
				BUTTON_TOP + i * BUTTON_INTERVAL_Y ),
			lpButtonStartSeq );

		SetDrawBright(255,255,255);
		if( bIsSelectedButton ) {
			// IĂ{^
			// ɖ߂

			// Xebv
			lpButtonStartSeq->step();

			/*if( getStaticData()->isFadingOutDX() ) {
				//for( int d = 0; d < BUTTON_BRINK_SPEEDUP_NUM; d ++ ) {
					// 莞͑߂
					lpButtonStartSeq->step();
				//}
			}*/
		}
	}
}

void GTitleScene::procMainOnArrow() {
	// ݂̃V[PXZbg
	getStaticData()->getSequence( SpriteType::COMMON,
		BUTTON_INFOS[ this->m_nIndex ].m_nSequenceID )->resetCounter();

	this->commononArrow( this->m_nIndex,
		0, getButtonNum() );
}
bool GTitleScene::procMainOnA() {
	switch( this->m_nIndex ) {
	case ButtonType::START:
		// Z[uI
		this->m_nTitleType = TitleType::SELECT_SAVE;

		break;

	case ButtonType::OPTION:
		// option scene
		getStaticData()->startFadeOutDX( SceneType::OPTION, -1 );
		break;
	case ButtonType::ACHIEVEMENT:
		getStaticData()->startFadeOutDX( SceneType::ACHIEVEMENT, -1 );
		break;
	case ButtonType::LIBRARY:
		getStaticData()->startFadeOutDX( SceneType::LIBRARY, -1 );
		break;
	default:
		break;
	} // switch m_nIndex
	return true;
}

////////////////////////////////////////////////////////
void GTitleScene::procSelectSave() {
	const int SAVE_LABEL_X = 300;
	const int CURSOR_X = 200;
	for( int i = 0; i < SAVE_NUM; i ++ ) {
		int y = IMAGES[ImageIndex::TITLE].y +
					IMAGES[ImageIndex::TITLE].seq->getImageInformation(0)->srcRect.getHeight() +
					BUTTON_TOP + i * BUTTON_INTERVAL_Y;
		
		string strText = HPLString::chrToStr( "Save%d", i+1);

		DrawString(
			SAVE_LABEL_X, y,
			strText.c_str(), GetColor(255,255,255 ) );

		// save file path
		string strSaveFilePath( HPLString::chrToStr(SAVE_PATH_FORMAT, i+1));
		if( PathFileExists( strSaveFilePath.c_str()) == false ) {	
			DrawString(
				SAVE_LABEL_X + strText.length() * FONT_WIDTH + 20, y,
				"[NEW!]", GetColor(255,255,0 ) );
		}
		if( i == this->m_nSaveIndex ) {
			// draw cursor 
			getStaticData()->m_Basic.m_Graphics.draw(
				CURSOR_X, y, getStaticData()->getSequence( SpriteType::COMMON, Sequence::Common::CURSOR) );
		}
	}
	int y = IMAGES[ImageIndex::TITLE].y +
					IMAGES[ImageIndex::TITLE].seq->getImageInformation(0)->srcRect.getHeight() +
					BUTTON_TOP + SAVE_NUM * BUTTON_INTERVAL_Y;
	DrawString( SAVE_LABEL_X, y,
		"Delete save", GetColor(255,255,255));
	if( this->m_nSaveIndex == SAVE_NUM ) {
		getStaticData()->m_Basic.m_Graphics.draw(
			CURSOR_X, y, getStaticData()->getSequence( SpriteType::COMMON, Sequence::Common::CURSOR) );
	}
}
void GTitleScene::procSelectSaveOnArrow() {
	this->commononArrow( this->m_nSaveIndex,
		0, SAVE_NUM+1 );
}
bool GTitleScene::procSelectSaveOnA() {
	if( this->m_nSaveIndex == SAVE_NUM ) {	// ŏIsiSaveꗗ{Pj
		// Delete[hֈڍs
		this->m_nTitleType = TitleType::DELETE_SAVE;
		this->m_nSaveIndex = 0;
		return true;
	}
	// Z[uǂݍ
	getStaticData()->loadSave( this->m_nSaveIndex );

	// tOǂݍ
	int nFlag = getStaticData()->m_Save.getFlag( FlagType::START );

	switch( nFlag ) {
	case StartType::OPENING:			// I[vjO
		{
			// Cxgǂݍ
			getStaticData()->m_EventPackManager.load("data/event/EventPack.txt");
			// I[vjOĐ
			getStaticData()->startEvent( 14, false );
			getStaticData()->startFadeOutDX(SceneType::STORY, -1 );
		}
		break;
	case StartType::PRACTICE:			// vNeBX
		{
			// 
//			getStaticData()->m_EventPackManager.load("data/event/EventPack.txt");
			// vNeBX̃Xe[W
			getStaticData()->startFadeOutDX(SceneType::GAME, SpecialMapIndex::PRACTICE );
		}
		break;
	case StartType::EVIL_HOUSE_BED:		// xbhB(OɏoĂȂj
	case StartType::UNTIL_LOBBY:
		{
			// PlaceI
			getStaticData()->setStarAndPlace( StageType::EVIL_STAR, 0 );
			
			// Z[u
			// ̏ԁi΂ׂ܂œBĂȂjɕύX
			//getStaticData()->m_Save.setFlag( FlagType::START, StartType::UNTIL_BAVEL );
			//getStaticData()->m_Save.save();

			// Q[ֈړB
			getStaticData()->startFadeOutDX(SceneType::GAME,
				getStaticData()->getMapIndexFromStarAndPlace());
		}
		break;
	case StartType::UNTIL_BAVEL:		// ΂ׂ܂
	case StartType::UNTIL_FUYU_EVIL:	// VGr߂܂܂
		{
			// I
			getStaticData()->setStarAndPlace( StageType::EVIL_STAR, -1 );

			// Iʂֈړ
			getStaticData()->startFadeOutDX( SceneType::SELECT_PLACE, -1 );
		}
		break;
/*	case StartType::UNTIL_BAVEL:		// ΂ׂ܂
	case StartType::UNTIL_FUYU_EVIL:	// VGr߂܂܂
		{
			// PlaceI
			getStaticData()->setStarAndPlace( StageType::EVIL_STAR, 1 );

			// Q[ֈړ
			getStaticData()->startFadeOutDX( SceneType::GAME, 
				SpecialMapIndex::EVIL_HOUSE_FRONT );

			// player place
			()->m_nPlayerPlaceIndex = 1;
		}
		break;*/
	/*	{
			// Q[֒ڈړB
			getStaticData()->startFadeOutDX(SceneType::GAME,
				getStaticData()->getMapIndexFromStarAndPlace());
		}
		break;*/

	/*	{
			// PlaceI
			//getStaticData()->changeStarAndPlace( StageType::EVIL_STAR, 0 );

			// Q[ֈړB
			getStaticData()->startFadeOutDX(SceneType::GAME,
				getStaticData()->getMapIndexFromStarAndPlace());
		}
		break;*/

	default:							// ̑
		{
			// J
			getStaticData()->startFadeOutDX(SceneType::SELECT_STAGE, -1 );
		}
		break;
	}// end of flag
	return true;
}
void GTitleScene::procSelectSaveOnB() {
	this->m_nTitleType = TitleType::MAIN;
}

void GTitleScene::commononArrow( int& nIndex, int nMin, int nMax ) {//, bool bIsWrap ) {
	if( getStaticData()->getInputManager()->isPressedNow( KeyMap::UP ) ) {
		nIndex --;
		if( nIndex < nMin ) {
			nIndex = nMax - 1;
		}
	} else if( getStaticData()->getInputManager()->isPressedNow( KeyMap::DOWN ) ) {
		nIndex ++;
		if( nIndex >= nMax ) {
			nIndex = nMin;
		}
	}
}

/////////////////////////////////////
void GTitleScene::procDeleteSave() {
	const int SAVE_LABEL_X = 300;
	const int CURSOR_X = 200;
	for( int i = 0; i < SAVE_NUM; i ++ ) {
		int y = IMAGES[ImageIndex::TITLE].y +
					IMAGES[ImageIndex::TITLE].seq->getImageInformation(0)->srcRect.getHeight() +
					BUTTON_TOP + i * BUTTON_INTERVAL_Y;
		// color
		int nR = 255, nG = 255, nB = 255;

		string strSaveFilePath( HPLString::chrToStr(SAVE_PATH_FORMAT, i+1));
		if( PathFileExists( strSaveFilePath.c_str()) == false ) {	// no save file
			// gray
			nR = 150;
			nG = 150;
			nB = 150;
		}
		DrawString(
			SAVE_LABEL_X, y,
			HPLString::chrToStr( "Delete save%d", i+1 ).c_str(), GetColor(nR,nG,nB ) );

		if( i == this->m_nSaveIndex ) {
			// draw cursor 
			getStaticData()->m_Basic.m_Graphics.draw(
				CURSOR_X, y, getStaticData()->getSequence( SpriteType::COMMON, Sequence::Common::CURSOR) );
		}
	}
}
void GTitleScene::procDeleteSaveOnArrow() {
	// ʒuύX
	this->commononArrow( this->m_nSaveIndex,
		0, SAVE_NUM );
}
bool GTitleScene::procDeleteSaveOnA() {
	string strSaveFilePath( HPLString::chrToStr(SAVE_PATH_FORMAT, this->m_nSaveIndex +1));
	if( PathFileExists( strSaveFilePath.c_str()) == false ) {	// no save file
		// NG 
		getStaticData()->m_MixerManager.play( 
			SE::BUZZ, false );
		// without selection se
		return false;
	}

	// move to confirm
	this->m_nTitleType = TitleType::CONFIRM;
	this->m_nConfirmIndex = ConfirmType::CANCEL;//cancel

	return true;
}
void GTitleScene::procDeleteSaveOnB() {
	// back to select save
	this->m_nTitleType = TitleType::SELECT_SAVE;
}
void GTitleScene::procConfirm() {
	const int SAVE_LABEL_X = 300;
	const int CURSOR_X = 200;
	char LABELS[][20] = {
		"Delete save!",
		"Cancel",
	};
	for( int i = 0; i < SIZEOF(LABELS); i ++ ) {
		int y = IMAGES[ImageIndex::TITLE].y +
					IMAGES[ImageIndex::TITLE].seq->getImageInformation(0)->srcRect.getHeight() +
					BUTTON_TOP + i * BUTTON_INTERVAL_Y;
		DrawString( SAVE_LABEL_X, y,
			LABELS[i], GetColor(255,255,255 ) );

		if( i == this->m_nConfirmIndex ) {
			// draw cursor 
			getStaticData()->m_Basic.m_Graphics.draw(
				CURSOR_X, y, getStaticData()->getSequence( SpriteType::COMMON, Sequence::Common::CURSOR) );
		}
	}
}
void GTitleScene::procConfirmOnArrow() {
	this->commononArrow( this->m_nConfirmIndex,
		0, ConfirmType::MAX_NUM );
}
bool GTitleScene::procConfirmOnA() {
	if( this->m_nConfirmIndex == ConfirmType::CANCEL ) {
		// cancel
		this->m_nTitleType = TitleType::DELETE_SAVE;
		getStaticData()->m_MixerManager.play( 
			SE::CANCEL, false );
		return false;
	}
	// delete!
	string strFilePath = HPLString::chrToStr( SAVE_PATH_FORMAT, this->m_nSaveIndex+1 );
	try {
		if( DeleteFile( strFilePath.c_str() ) == FALSE ) {
			string strMsg = HPLString::chrToStr( "File delete error![%s]", strFilePath.c_str());
			HPLError::msg( "error", MB_OK, (char*)strMsg.c_str());
		}
	} catch(... ) {
		string strMsg = HPLString::chrToStr( "File delete unknown error![%s]", strFilePath.c_str());
		HPLError::msg( "error", MB_OK, (char*)strMsg.c_str());
	}
	// change to select save type
	this->m_nTitleType = TitleType::SELECT_SAVE;

	return true;
}
void GTitleScene::procConfirmOnB() {
	// cancel
	this->m_nTitleType = TitleType::DELETE_SAVE;
}
