#include "GWeaponManager.h"

#include <hpllib/system/HPLError.h>
#include <hpllib/system/HPLFile.h>
#include <hpllib/general/HPLString.h>
#include "GStaticData.h"
#include "GShotSetting.h"

using namespace std;

namespace GEnum {
	namespace WeaponSettingItemType {
		enum WeaponSettingItemType_ {
			NAME,
			ID,
			DIRECTION_TYPE,
			SHOT_INTERVAL,
			NEXT_PLAYER_STATE,
			WEAPON_CLASS_TYPE,
			SHOT_TYPE,
			MAX_BULLET_NUM,
			MIN_BULLET_NUM,
			SHOT_DX_LEFT,
			SHOT_DX_RIGHT,
			CENTER_ERROR,
			FLAGS,
			ITEM_TYPE,
			MAX_NUM
		};
	}
}
using namespace GEnum;

GWeaponSetting* GWeaponManager::getWeaponSetting( int nIndex ) {
#ifdef _DEBUG
	ASSERT_IT_INDEX( nIndex, 0, WeaponType::MAX_WEAPON_NUM );
//	HPLError::assertIt( nIndex >= 0 && nIndex < WeaponType::MAX_WEAPON_NUM );
#endif

	return &this->m_aryWeaponSettings[nIndex];
}

GWeaponManager::GWeaponManager() {
}
int GWeaponManager::loadWeaponSettings( const char* lpFilePath ) {
	vector<string> vecLines;
	try {
		HPLFile::loadStringFromFile( vecLines, lpFilePath, false );
	} catch( exception& e ) {
		HPLError::msg( "error", MB_OK, (char*)e.what() );
		return -1;
	}

	// 
	int nLoadedIndexBit = 0;

	// ƃ[hłސ
	int nLoadedCounter = 0;

	for( int i = 0; i < (int)vecLines.size(); i ++ ) {
		string strLine = vecLines[i];
		if( HPLFile::isCommentLine( &strLine ) ) {
			continue;
		}

		int nIndex = -1;	//! ݒ肵CfbNX
		int nRet = this->parseSettingStringLine( strLine, nIndex, i );
		if( nRet < 0 ) {	// s
			return nRet;
		}
		if( nIndex < 0 ) {
			// minus
			HPLError::msg( "Syntax error", MB_OK, "(Line:%d)Invalid weapon index[%s]",
				i, nIndex, strLine.c_str() );
			return -2;
		}
		int nIndexBit = 1 << nIndex;
		if( (nLoadedIndexBit & nIndexBit ) != 0 ) {
			// duplicated
			HPLError::msg( "Duplicate error", MB_OK,
				"(Line:%d)Index[%d] is already registered.[%s]", i, nIndex,
				strLine.c_str() );
			return -2;
		}
		nLoadedIndexBit |= nIndexBit;
		nLoadedCounter ++;

	}
	// 킹
	HPLError::assertIt( nLoadedCounter == WeaponType::MAX_WEAPON_NUM,
		"Weapon setting count and weapon type num do not match(Setting count = [%d], Weapon type num = [%d]",
		nLoadedCounter, WeaponType::MAX_WEAPON_NUM);
	return 0;
}

int GWeaponManager::parseSettingStringLine( std::string& strLine, int& nIndex, int nLineIndex ) {
	vector<string> aryItems = HPLString::split( strLine, ",");
	if( aryItems.size() < WeaponSettingItemType::MAX_NUM ) {
		HPLError::msg( "Error", MB_OK, "(Line:%5d)Syntax error( too few arguments ):%s", nLineIndex, 
			strLine.c_str() );
		return -2;
	}

	nIndex = -1;
	// ݒp
	GWeaponSetting temp;

	for( int i = 0; i < (int)aryItems.size(); i ++ ) {
		string strItem = aryItems[i];
		try {
			switch( i ) {
			case WeaponSettingItemType::NAME:
				{
					temp.m_strName = strItem;
				}
				break;
			case WeaponSettingItemType::ID:
				{
					nIndex = atoi( strItem.c_str() );
					if( nIndex < 0 ) {
						HPLError::msg( "Syntax error", MB_OK,
							"(Line:%5d)Invalid id[%d] in [%s]", nLineIndex, temp.m_nID, strLine );
						return -2;
					}
					temp.m_nID = nIndex;
				}
				break;
			case WeaponSettingItemType::DIRECTION_TYPE:
				{
					// trim
					string str = HPLString::trim( strItem );
					int nDirectionType = -1;
					if( HPLString::equals( str, "STATIC") ) {
						nDirectionType = WeaponDirType::STATIC;
					} else if( HPLString::equals( str, "SIX") ) {
						nDirectionType = WeaponDirType::SIX;
					} else if( HPLString::equals( str, "EIGHT")) {
						nDirectionType = WeaponDirType::EIGHT;
					}
					if( nDirectionType < 0 ) {
						HPLError::msg( "Invalid argument", MB_OK, 
							"(Line:%5d)Invalid direction type[%s] in [%s]",
							nLineIndex, strItem.c_str(), strLine.c_str() );
						return -2;
					}

					temp.m_nDirectionType = nDirectionType;
				}
				break;
			case WeaponSettingItemType::SHOT_INTERVAL:
				{
					temp.m_nShotInterval = atoi( strItem.c_str() );
				}
				break;
			case WeaponSettingItemType::NEXT_PLAYER_STATE:
				{
					// trim
					string str = HPLString::trim( strItem );
					int nNextPlayerState = -1;
					if( HPLString::equals( str, "ATTACK") ) {
						nNextPlayerState = StateType::ATTACK;
					} else if( HPLString::equals( str, "ATTACK_GUN") ) {
						nNextPlayerState = StateType::ATTACK_GUN;
					}
					if( nNextPlayerState < 0 ) {
						HPLError::msg( "Invalid argument", MB_OK, 
							"(Line:%5d)Invalid NextPlayerState[%s] in [%s]",
							nLineIndex, strItem.c_str(), strLine.c_str() );
						return -2;
					}
					temp.m_nNextPlayerState = nNextPlayerState;
				}
				break;
			case WeaponSettingItemType::WEAPON_CLASS_TYPE:
				{
					string str = HPLString::trim( strItem );
					int nClassType = -1;
					if( HPLString::equals( str, "CON_SHOT") ) {
						nClassType = WeaponClassType::CON_SHOT;
					} else if( HPLString::equals( str, "ONE_SHOT") ) {
						nClassType = WeaponClassType::ONE_SHOT;
					} else if( HPLString::equals( str, "THROW") ) {
						nClassType = WeaponClassType::THROW;
					} else if( HPLString::equals( str, "CHARGE") ) {
						nClassType = WeaponClassType::CHARGE;
					} else if( HPLString::equals( str, "LASER_SHOT") ) {
						nClassType = WeaponClassType::LASER_SHOT;
					}
					if( nClassType < 0 ) {
						HPLError::msg("Invalid argument", MB_OK,
							"(Line:%5d)Specified weapon class type is invalid:[%s] in [%s]",
							nLineIndex, strItem.c_str(), strLine.c_str());
						return -2;
					}
					temp.m_nWeaponClassType = nClassType;
				}
				break;
			case WeaponSettingItemType::SHOT_TYPE:
				{
					// Vbg^Cv猟
					string str = HPLString::trim( strItem );
					int nShotType = -1;
					for( int nS = 0; nS < Sequence::Shot::MAX_NUM; nS ++ ) {
						GShotSetting* lpShotSetting = getStaticData()->m_ObjectManager.m_ShotManager.getShotSetting(nS );
						if( HPLString::equals( str, lpShotSetting->m_strName ) ) {
							nShotType = nS;
							break;
						}
					}
					if( nShotType < 0 ) {
						HPLError::msg("Invalid argument", MB_OK,
							"(Line:%5d)Shot type not found:[%s] in [%s]", nLineIndex, strItem.c_str(), strLine.c_str());
						return -2;
					}
					temp.m_nShotType = nShotType;
				}
				break;
			case WeaponSettingItemType::MAX_BULLET_NUM:
				{
					temp.m_nMaxBulletNum = atoi( strItem.c_str() );
				}
				break;
			case WeaponSettingItemType::MIN_BULLET_NUM:
				{
					temp.m_nMinBulletNum = atoi( strItem.c_str() );
				}
				break;
			case WeaponSettingItemType::SHOT_DX_LEFT:
				{
					temp.m_nShotDXLeft = atoi( strItem.c_str() );
				}
				break;
			case WeaponSettingItemType::SHOT_DX_RIGHT:
				{
					temp.m_nShotDXRight = atoi( strItem.c_str() );
				}
				break;
			case WeaponSettingItemType::CENTER_ERROR:
				{
					temp.m_nCenterError = atoi( strItem.c_str() );
				}
				break;
			case WeaponSettingItemType::FLAGS:
				{
					// "/"ŕ
					vector<string> vecFlags = HPLString::split( strItem, "/");
					if( vecFlags.size() < WeaponFlagType::MAX_NUM ) {
						HPLError::msg( "Syntax error", MB_OK, 
							"(Line:%5d)Too few flags:[%s] in [%s]", strItem.c_str(), strLine.c_str() );
						return -2;
					}
					for( int i = 0; i < (int)vecFlags.size() && i < WeaponFlagType::MAX_NUM; i ++ ) {
						// trim
						string str = HPLString::trim( vecFlags[i] );
						bool bFlag = false;
						if( HPLString::equals( str, "TRUE") ) {
							bFlag = true;
						} else if( HPLString::equals( str, "FALSE") ) {
							// false
						} else {
							HPLError::msg( "Syntax error", MB_OK,
								"(Line:%5d)Invalid flag string(set TRUE or FALSE):[%s] in [%s]",
								nLineIndex, str.c_str(), strLine.c_str());
							return -2;
						}
						temp.m_aryFlags[i] = bFlag;
					}
				}
				break;
			case WeaponSettingItemType::ITEM_TYPE:
				{
					string strTrim = HPLString::trim( strItem );
					int nIndex = -1;
					for( int j = 0; j < Sequence::Item::MAX_NUM; j ++ ) {
						GItemSetting* lpSetting = getStaticData()->m_ObjectManager.m_ItemManager.getSetting( j );
						ASSERT_IT_NULL( lpSetting );
						if( HPLString::equals( lpSetting->m_strName, strTrim ) ) {
							nIndex = j;
							break;
						}
					}
					if( nIndex >= 0 && nIndex < Sequence::Item::MAX_NUM ) {
						//getStaticData()->registItemAndWeaponSet( nIndex, temp.m_nID );
						this->m_mapItem2WeaponType[nIndex]=temp.m_nID;
					} else {
						HPLError::msg("Error", MB_OK, "(Line:%5d)Invalid item name:[%s] in [%s]",
								nLineIndex, strTrim.c_str(), strLine.c_str());
						return -2;
					} 
				}
				break;
			default:
				HPLError::msg( "Syntax error", MB_OK,
					"(Line:%5d)Unknown index:Index[%d][%s] in [%s]", nLineIndex, i, strItem.c_str(), strLine.c_str() );
				return -2;
			}
		} catch( exception& e ) {
			HPLError::msg( "Syntax error", MB_OK, 
				"(Line:%5d)Convertion failed:Index[%d][%s] in [%s]:%s",
				nLineIndex, i, strItem.c_str(), strLine.c_str(), e.what() );
			return -2;
		}
	}
	if( nIndex < 0 ) {
		// ݒ肳Ȃ
		HPLError::msg( "Invalid argument", MB_OK, "(Line:%5d)ID does not exist[%s]",
			nLineIndex, strLine.c_str());
		return -2;
	}
	this->m_aryWeaponSettings[nIndex] = temp;
	return 0;
}

